/* Copyright 2009 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro etlcdim(soletl_nowDTTM=, soletl_instanceId=);

%global soletl_error;
%global soletl_errormsg;
%global soletl_statusID;

%let soletl_error = ;
%let soletl_errormsg = ;
%let soletl_statusID = 1; /* Will be changed to 0 if program runs successful */

%let entry = etlcdim;
%let putlog = Y;

%let soletl_nowDTTM = %unquote(&soletl_nowDTTM);

%let _where1=((VALID_FROM_DTTM <= &soletl_nowDTTM)
          and (VALID_TO_DTTM >= &soletl_nowDTTM or VALID_TO_DTTM = .));

%let curr_rec_dttm = '01JAN5999:00:00'DT;

/* Check if datetime passed in is valid */
%if %datatyp(&soletl_nowDTTM) ne NUMERIC %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=1
          , msgTxt1=<soletl_nowDTTM : &soletl_nowDTTM>, msgPutlog=&putlog);
  %goto exit ;
%end;

/* Check for
           102 : Metaserver not defined
           103 : Libref not found
           104 : Access denied (somekind of security issue on the metadata user)
*/

%bldLibNm(sesetl_sasLibRef=_CONFLIB
        , sesetl_OMRSASLibraryObjName=&FMdatatier_folder./&CONFLIBOBJ./&CONFLIBOBJ.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%if &sysrc ne 0 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&sysrc
          , msgTxt1=<_CONFLIB>, msgPutlog=&putlog);
  %goto exit ;
%end;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%if &sysrc ne 0 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&sysrc
          , msgTxt1=<_SDMLIB>, msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpDSN = _CONFLIB.CODE_LANGUAGE;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=3
          , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;


/*
 * The Locale codes are pass in on a tab delimited macro variable.  Parse
 * the list into a data set
 */
data WORK.LOCALES_TO_IMPORT
           (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_CD) ;

   set _SDMLIB.sas_etl_member(obs=0 keep=LOCALE_CD)
       &_tmpDSN(obs = 1 keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD);

   do n = 1 to 9999999999 ;
     LOCALE_CD = strip( kscan("&soletl_localeCodes", n, "09"x) ) ;
     if LOCALE_CD eq "" then leave ;

     LOCALE_LANGUAGE_CD = "";
     LOCALE_COUNTRY_CD = "";
     LOCALE_VARIANT_CD = "";
     if LOCALE_CD ne "" then do;
        LOCALE_LANGUAGE_CD = strip( kscan(LOCALE_CD, 1, '_') ) ;
        NLS_COUNT1 = count(LOCALE_CD, '_') ;
        if NLS_COUNT1 eq 0 then do;
           LOCALE_COUNTRY_CD = '';
           LOCALE_VARIANT_CD = '';
        end;
        else do;
           NLS_COUNT2 = count(LOCALE_CD, '__') ;
           if NLS_COUNT2 eq 1 then do;
              LOCALE_COUNTRY_CD = '';
              LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 2, '_') );
           end;
           else do;
              LOCALE_COUNTRY_CD = strip( kscan(LOCALE_CD, 2, '_') );
              LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 3, '_') );
           end;
        end;
     end;

     output;
   end ;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sort in=_CONFLIB.CODE_LANGUAGE out=WORK.SAS_CODE_LANG;
  by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
  where &_where1;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sort in=WORK.LOCALES_TO_IMPORT;
  by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.LOCALE_MAP(keep=LANGUAGE_CD LOCALE_CD);
  merge SAS_CODE_LANG(in=nSAS) LOCALES_TO_IMPORT(in=nImport);
  by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
  if nSAS and nImport;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;


/* Check for the existence of records */
%let _tmpDSN = WORK.LOCALE_MAP;
proc sql noprint;
   select count(*) into :_rec_cnt
   from &_tmpDSN
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgPutlog=Y);
  %goto exit ;
%end;

data WORK.LOCALE_FORMAT(keep=START LABEL FMTNAME HLO);
  retain FMTNAME "$LCLMAP";
  set WORK.LOCALE_MAP(rename=(LANGUAGE_CD=START LOCALE_CD=LABEL)) end=nEnd;
  HLO = "";
  output;
  if nEnd then do;
     START = "";
     LABEL = "*";
     HLO = "O";
     output;
  end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=WORK.LOCALE_FORMAT library=WORK;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;


%let _tmpDSN = _CONFLIB.APP_DIMENSION;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=3
          , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

/* Check for the existence of records */
proc sql noprint;
   select count(*) into :_rec_cnt
   from &_tmpDSN
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=4
          , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

data WORK.SAS_ETL_DIMENSION
       (keep=ETL_INSTANCE_ID DIM_TYPE_CD DIMENSION_CD DIMENSION_NM DIMENSION_DESC
             LOCALE_CD LOCALE_ID VALID_FROM_DTTM VALID_TO_DTTM);

   set _SDMLIB.SAS_ETL_DIMENSION(obs=0)
     &_tmpDSN(where=&_where1);

   LOCALE_CD = put(LANGUAGE_CD, $LCLMAP.);
   if LOCALE_CD NE "*" then do ;
     ETL_INSTANCE_ID = symgetn("soletl_instanceId");
     DIM_TYPE_CD = DIMENSION_TYPE_CD;
     DIMENSION_CD = APP_DIM_ADK;
     DIMENSION_NM = APP_DIM_NM;
     DIMENSION_DESC = APP_DIM_DESC;
     LOCALE_ID = 0;
     VALID_FROM_DTTM = 0;
     VALID_TO_DTTM = 0;
     output;
   end ;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_DIMENSION data=WORK.SAS_ETL_DIMENSION &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let soletl_statusID = 0; /* Program ran successful */

%exit:

%if &soletl_statusID ne 0 %then %do;
   %let soletl_error = &sesgmsg_msgCodeReturn;
   %let soletl_errormsg = &sesgmsg_msgReturn;
%end;

%mend etlcdim;
