%macro etlcprex(resultCode=, exportLib=, ruleType=);

%global fmsetl_error;
%global fmsetl_errormsg;
%global fmsetl_statusId;

%let fmsetl_error = ;
%let fmsetl_errormsg = ;
%let fmsetl_statusId = 1 ; /* Will be changed to 0 if program runs successful */

%let entry = etlcprex;
%let putlog = Y;

%global today_dttm;
%let today_dttm=%qsysfunc(datetime());

%startsln(FMS);

/******************************************************************************************/
/*                                  Assign Librefs                                        */
/******************************************************************************************/
%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.,
          sesetl_useType=CON
        ) ;

/******************************************************************************************/
/*                            Retrieve the model id                                       */
/******************************************************************************************/
%let _modelFound = 0;

proc sql noprint;
   select count(*) into :_modelFound
   from _SDMLIB.sas_vcube
   where vcube_cd = "&resultCode";
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not &_modelFound %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=3
          , msgTxt1=<&resultCode>, msgPutlog=&putlog);
  %goto exit;
%end;

proc sql noprint;
   select vcube_id into :rules_vcube_id
   from _SDMLIB.sas_vcube
   where vcube_cd = "&resultCode";
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*          Creation of the hash table to be used for converting member ids to codes.     */
/******************************************************************************************/
%let _tmpSdmTbl=_SDMLIB.sas_member;

proc sql noprint;
  create table work.memberHashTable as select distinct member_id, member_cd
     from &_tmpSdmTbl
     where valid_from_dttm <= &today_dttm and valid_to_dttm >= &today_dttm;
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*   Creation of the hash table to be used for converting dimension type ids to codes.    */
/******************************************************************************************/
%let _tmpSdmTbl=_SDMLIB.sas_dim_type;

proc sql noprint;
  create table work.dimensionTypeHashTable as select dim_type_id, dim_type_cd
     from &_tmpSdmTbl
     where valid_from_dttm <= &today_dttm and valid_to_dttm >= &today_dttm;
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*         Export rules to app_cell_protection_rule data set in output location.          */
/******************************************************************************************/
%let _tmpSdmTbl=_SDMLIB.sas_cell_protection_rule;

proc sql noprint;
   create table work._tmp_sdm_cpr as 
   select *
   from &_tmpSdmTbl
   where vcube_id = &rules_vcube_id and level in(&ruleType); /* subset model rules (1 and 8) */
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

data work._tmp_CPR;
   length cell_protection_rule_id 8;
   length model_cd $ %eval(32*&FM_DBCSMULT);
   length rule_order_no 8;
   length rule_type 8;
   length protect_type 8;
   set work._tmp_sdm_cpr;
   cell_protection_rule_id = id;
   model_cd = "&resultCode";
   rule_order_no = rule_order;
   keep cell_protection_rule_id model_cd rule_order_no rule_type protect_type;
run;
%checkErr;
%if &_tmpRC %then %goto exit;

%let _tmpDSN=&exportLib..app_cell_protection_rule;

%if %sysfunc( exist(&_tmpDSN) ) %then %do;
   %let protectType=1;
   %if "&ruleType" eq "8" %then %let protectType=2;
   proc sql noprint;
      delete from &_tmpDSN
      where model_cd = "&resultCode" and protect_type=&protectType;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit;
%end;

proc append base=&_tmpDSN data=work._tmp_CPR &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*        Export rules to app_dim_type_member_selector data set in output location.       */
/******************************************************************************************/
%let _tmpSdmTbl=_SDMLIB.sas_dim_type_member_selector;

proc sql noprint;
   create table work._tmp_export_dtms as 
   select * 
   from &_tmpSdmTbl(dbmax_text=32767)
   where cell_protection_rule_id in (select cell_protection_rule_id from work._tmp_cpr);
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

data work._tmp_DTMS;
   length cell_protection_rule_id 8;
   length member_id 8;
   length member_cd $ %eval(32*&FM_DBCSMULT);
   length dim_type_id 8;
   length dim_type_cd $ %eval(32*&FM_DBCSMULT);
   length member_selection_rule 8;
   length virtual_child_flg $1;
   length i 8;
   length selector_string $ 32;
   set work._tmp_export_dtms;
   if _n_ = 1 then do;
      declare hash mht(dataset:"work.memberHashTable");
      mht.defineKey("member_id");
      mht.defineData("member_cd");
      mht.defineDone();
      declare hash dtht(dataset:"work.dimensionTypeHashTable");
      dtht.defineKey("dim_type_id");
      dtht.defineData("dim_type_cd");
      dtht.defineDone();
   end;
   dim_type_id = dimension_type_id;
   rc = dtht.find();
   if rc = 0 then do;
      i = 1;
      selector_string = strip(scan(selector,i,','));
      do until (selector_string = '');
         virtual_child_flg = 'N';
         member_id = scan(selector_string,1,';');
         if member_id < 0 then do;
            member_id = abs(member_id);
            virtual_child_flg = 'Y';
         end;
         rc = mht.find();
         if rc = 0 then do;
            member_selection_rule = scan(selector_string,2,';');
            output;
         end;
         i+1;
         selector_string = strip(scan(selector,i,','));
      end;
   end;
   keep cell_protection_rule_id dim_type_cd member_cd member_selection_rule virtual_child_flg;
run;
%checkErr;
%if &_tmpRC %then %goto exit;

%let _tmpDSN=&exportLib..app_dim_type_member_selector;

%if %sysfunc( exist(&_tmpDSN) ) %then %do;
   proc sql noprint;
      delete from &_tmpDSN
      where cell_protection_rule_id in (select distinct cell_protection_rule_id
                                        from work._tmp_DTMS);
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit;
%end;

proc append base=&_tmpDSN data=work._tmp_DTMS &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
%let fmsetl_statusId = 0; /* Program ran successful */

%exit:

%if &fmsetl_statusID ne 0 %then %do;
   %let fmsetl_error = &sesgmsg_msgCodeReturn;
   %let fmsetl_errormsg = &sesgmsg_msgReturn;
%end;

%mend etlcprex;
