/* Copyright 2009 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro etldim (dimtype=, soletl_processedDTTM=, soletl_instanceId=, soletl_includeUserData=,
               soletl_includeSecurityData=);

/*************************************************************************************/
/*                             Initialization                                        */
/*************************************************************************************/
%global soletl_error;
%global soletl_errormsg;
%global soletl_statusID;
%global soletl_locale_missing;
%global soletl_locale_no_data_found;
%global _dupsFound;

%let soletl_error = ;
%let soletl_errormsg = ;
%let soletl_statusID = 1; /* Will be changed to 0 if program runs successful */
%let soletl_locale_missing = ;
%let soletl_locale_no_data_found = ;

%let entry = etldim;
%let putlog = Y;

%let curr_rec_dttm = %sysfunc( inputn('01JAN5999:00:00:00', datetime18.) );

%macro _validateDups(_param=, _ds=, _format=,
                     _where=, _cols=, _byVars=, _byVars1=);

  data WORK._UPCASECODES;
    set &_ds;
    if &_param ne '';
    original = &_param;
    &_param = upcase(&_param);
  run;
  %checkErr;
  %if &_tmpRC %then %goto error ;

  proc sort data = WORK._UPCASECODES(&_where keep=&_param original &_cols)
             dupout = WORK._TMPDUPCODES (keep = &_param original )
             NODUPKEY
                ;
     by &_byVars;
  run;
  %checkErr;
  %if &_tmpRC %then %goto error ;

  %if "&_byVars1" ne "" %then %do;
     proc sort data = WORK._UPCASECODES
                   dupout = WORK._TMPDUPCODES (keep = &_param original )
                   NODUPKEY
                      ;
        by &_byVars1;
     run;
     %checkErr;
     %if &_tmpRC %then %goto error ;

  %end;

  proc sql noprint;
    create table WORK._DUPCODES as
    select strip(d.original) as _tmpDupCode1, strip(o.original) as _tmpDupCode
    from WORK._TMPDUPCODES d, WORK._UPCASECODES o
    where upcase(o.original) eq upcase(d.original)
        and o.original ne d.original
    ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto error ;

  data _null_;
    length _command $1024;
    set WORK._DUPCODES ;
    call symput('_dupsFound', 1);

    _command = '%sesgmsg(msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=16,
              msgTxt1=<' || strip(_tmpDupCode) || '>, msgTxt2=<' || strip(_tmpDupCode1) || '>,
              msgTxt3=&_ds, msgTxt4=&_param, msgPutlog=&putlog)';
    call execute(_command);
  run;
  %checkErr;
  %if &_tmpRC %then %goto error ;
  %if (&_dupsFound eq 1) %then %goto error ;

  %if "&_format" ne "" %then %do;
    data WORK._DUPCODES(keep=_tmpDupCode _tmpDupCode1);
      length _tmpDupCode _tmpDupCode1 $ %eval(32*&FM_DBCSMULT);
      set WORK._UPCASECODES;
      _tmpDupCode = strip(original);
      _tmpDupCode1 = strip(put(upcase(strip(original)), $FMT_&_format..));
      if _tmpDupCode eq _tmpDupCode1 then ;
      else if upcase(put(upcase(_tmpDupCode), $FMT_&_format..))
                                 eq upcase(_tmpDupCode) then output;
    run;
    %checkErr;
    %if &_tmpRC %then %goto error ;

    data _null_;
      length _command $1024;
      set WORK._DUPCODES ;
      call symput('_dupsFound', 1);

      _command = '%sesgmsg(msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=61,
              msgTxt1=<' || strip(_tmpDupCode) || '>, msgTxt2=<' || strip(_tmpDupCode1) || '>,
              msgTxt3=&_ds, msgTxt4=&_param, msgTxt5=&_format, msgPutlog=&putlog)';
      call execute(_command);
    run;
    %checkErr;
    %if &_tmpRC %then %goto error ;
    %if (&_dupsFound eq 1) %then %goto error ;
  %end;

  %if "&_param" eq "PROPERTY_CD" %then %do;
     data WORK._DUPCODESPROPS(keep=_tmpDupCode _tmpDupCode1);
       length _tmpDupCode _tmpDupCode1 $ %eval(32*&FM_DBCSMULT);
       set WORK._UPCASECODES;
       _tmpDupCode = strip(original);
       _tmpDupCode1 = strip(put(upcase(strip(original)), $FMT_PROPS.));
       if _tmpDupCode eq _tmpDupCode1 then ;
       else if upcase(put(upcase(_tmpDupCode), $FMT_PROPS.))
                                  eq upcase(_tmpDupCode) then output;
     run;
     %checkErr;
     %if &_tmpRC %then %goto error ;

     data _null_;
       length _command $1024;
       set WORK._DUPCODESPROPS ;
       call symput('_dupsFound', 1);

       _command = '%sesgmsg(msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=61,
               msgTxt1=<' || strip(_tmpDupCode) || '>, msgTxt2=<' || strip(_tmpDupCode1) || '>,
               msgTxt3=&_ds, msgTxt4=&_param, msgTxt5=SAS_PROPERTY, msgPutlog=&putlog)';
       call execute(_command);
     run;
     %checkErr;
     %if &_tmpRC %then %goto error ;
     %if (&_dupsFound eq 1) %then %goto error ;

  %end;

  %let _dupsFound = 0;
  %goto noerror;

  %error:
  %let _dupsFound = 1;

  %noerror:;

%mend;

%sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgPutlog=N);

%let _lib = _StgDDS;

%let _where1=((VALID_FROM_DTTM <= &soletl_processedDTTM)
          and (VALID_TO_DTTM >= &soletl_processedDTTM or VALID_TO_DTTM = .));

%let _where2=(VALID_TO_DTTM >= &soletl_processedDTTM or VALID_TO_DTTM = .);

%let byHierarchy = HIERARCHY_CD;

/* Check if datetime passed in is valid */
%if %datatyp(&soletl_processedDTTM) ne NUMERIC %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=1
          , msgTxt1=<soletl_processedDTTM : &soletl_processedDTTM>
          , msgPutlog=&putlog);
  %goto exit ;
%end;

/* Check for
           102 : Metaserver not defined
           103 : Libref not found
           104 : Access denied (somekind of security issue on the metadata user)
*/

%bldLibNm(sesetl_sasLibRef=&_lib
        , sesetl_OMRSASLibraryObjName=&FMdatatier_folder./&CONFLIBOBJ./&CONFLIBOBJ.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%if &sysrc ne 0 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&sysrc
          , msgTxt1=<_STGDDS>, msgPutlog=&putlog);
  %goto exit ;
%end;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%if &sysrc ne 0 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&sysrc
          , msgTxt1=<_SDMLIB>, msgPutlog=&putlog);
  %goto exit ;
%end;

/*************************************************************************************/
/* Verify that there is 1 and only 1 default language record identified in the       */
/* CODE_LANGUAGE table in the Stage.                                                 */
/*************************************************************************************/
%let _tmpDSN = &_lib..CODE_LANGUAGE;

proc sql noprint;
   select count(*) into :_dfltCount
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y" and &_where1
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if &_dfltCount < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=38
          , msgPutlog=&putlog);
  %goto exit;
%end;

%if &_dfltCount > 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=39
          , msgPutlog=&putlog);
  %goto exit;
%end;

proc sql noprint;
   select LANGUAGE_CD into :_dfltLangCd
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y" and &_where1
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/* Sort the _StgDDS.CODE_LANGUAGE table by LOCALE_LANGUAGE, LOCALE_COUNTRY and       */
/* LOCALE_VARIANT and write to _CODE_LANG_SORT table.  This table will be used in    */
/* conjunction with the inputted SDM locales to create mapping to the locales in the */
/* stage.                                                                            */
/*************************************************************************************/
%let _tmpWhere=where=(&_where1);

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._CODE_LANG_SORT(keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD
                       LOCALE_VARIANT_CD LANGUAGE_CD DEFAULT_LANGUAGE_FLG)
           NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/* Extract the language codes from the soletl_localeCodes macro variable and place   */
/* in a temporary dataset.                                                           */
/*************************************************************************************/
data _INPUT_LOCALES
           (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_CD) ;

   set _SDMLIB.sas_etl_member(obs=0 keep=LOCALE_CD)
       &_tmpDSN(obs = 1 keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD);

   do n = 1 to 9999999999 ;
     LOCALE_CD = strip( kscan("&soletl_localeCodes", n, "09"x) ) ;
     if LOCALE_CD eq "" then leave ;

     LOCALE_LANGUAGE_CD = "";
     LOCALE_COUNTRY_CD = "";
     LOCALE_VARIANT_CD = "";
     if LOCALE_CD ne "" then do;
        LOCALE_LANGUAGE_CD = strip( kscan(LOCALE_CD, 1, '_') ) ;
        NLS_COUNT1 = count(LOCALE_CD, '_') ;
        if NLS_COUNT1 eq 0 then do;
           LOCALE_COUNTRY_CD = '';
           LOCALE_VARIANT_CD = '';
        end;
        else do;
           NLS_COUNT2 = count(LOCALE_CD, '__') ;
           if NLS_COUNT2 eq 1 then do;
              LOCALE_COUNTRY_CD = '';
              LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 2, '_') );
           end;
           else do;
              LOCALE_COUNTRY_CD = strip( kscan(LOCALE_CD, 2, '_') );
              LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 3, '_') );
           end;
        end;
     end;

     output;
   end ;
run ;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpDSN = WORK._INPUT_LOCALES;

proc sort data = &_tmpDSN
           NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/***********************************************************************************/
/* Create mapping table, WORK._INPUT_LOCALES, between the inputted locales and the */
/* locales in the stage area. Check that records were found in the CODE_LANGUAGE   */
/* table of the STAGE for all of the inputted locales. If not, populate the        */
/* soletl_locale_missing macro variable.                                           */
/***********************************************************************************/
data &_tmpDSN
       (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_CD
             LANGUAGE_CD DEFAULT_LANGUAGE_FLG);

   merge &_tmpDSN (in = inA)
         WORK._CODE_LANG_SORT (in = inB);
      by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;

   if (inA) then do;
      output;
      if (not inB) then do;
         if symget("soletl_locale_missing") = "" then
            call symput("soletl_locale_missing", LOCALE_CD);
         else
            call symput("soletl_locale_missing", strip(symget("soletl_locale_missing"))
                        || "09"x || strip(LOCALE_CD));
      end;
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;


proc sql noprint;
   select count(*) into :_dfltFound
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y"
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;


%if &_dfltFound < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=40
          , msgTxt1=<&_dfltLangCd>, msgPutlog=&putlog);
  %goto exit;
%end;

/***********************************************************************************/
/* Create a temporary sdm locale table with the locales sorted by language,        */
/* country and variant.                                                            */
/***********************************************************************************/
%let _tmpDSN = _SDMLIB.SAS_DATA_LOCALE;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=30
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

data WORK._SDM_LOCALES(keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_ID);
   set &_tmpDSN;
   LOCALE_LANGUAGE_CD = ISO639_LANGUAGE_CD;
   LOCALE_COUNTRY_CD = ISO3166_COUNTRY_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpDSN = WORK._SDM_LOCALES;

proc sort data = &_tmpDSN NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/***********************************************************************************/
/* Create temporary table containing mapping between the locales in the stage      */
/* sdm using language, country and variant as sorting variables.                   */
/***********************************************************************************/
data WORK._STG_SDM_LOCALE_MAP
    (keep=LANGUAGE_CD LOCALE_ID LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD
          LOCALE_VARIANT_CD DEFAULT_LANGUAGE_FLG);

   merge WORK._CODE_LANG_SORT (in = inA)
         &_tmpDSN (in = inB);
      by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
   if (inA and inB) then
      output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/* Check that the sas default locale exists in the SDM.                              */
/*************************************************************************************/
%let _tmpDSN = WORK._STG_SDM_LOCALE_MAP;

proc sql noprint;
   select count(*) into :_sdmDfltCnt
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y"
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if &_sdmDfltCnt < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=41
          , msgTxt1=<&_dfltLangCd>, msgPutlog=&putlog);
  %goto exit;
%end;

/***********************************************************************************/
/* Create locale format, FMT_LOCALE, using the WORK._STG_SDM_LOCALE_MAP table.     */
/***********************************************************************************/
%let _tmpFormat=FMT_LOCALE;
%let _tmpStart=LANGUAGE_CD;
%let _tmpLabel=LOCALE_ID;
%let _tmpWhere=;

proc sort data = &_tmpDSN
           out = WORK._TMPSORT (keep=&_tmpStart &_tmpLabel)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   set WORK._TMPSORT(rename=(&_tmpStart=START &_tmpLabel=LABEL));
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/***********************************************************************************/
/* Create a temporary sdm property format with existing codes                      */
/***********************************************************************************/
%let _tmpDSN = _SDMLIB.SAS_PROPERTY;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpFormat=FMT_PROPS;
%let _tmpStart=PROPERTY_CD;
%let _tmpWhere=;

proc sort data = &_tmpDSN
           out = WORK._TMPSORT (keep=&_tmpStart)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain LABEL ;
   retain HLO "";
   if _N_ = 1 then do;;
     HLO = "O";
     output;
     HLO = "";
   end;
   set WORK._TMPSORT(rename=(&_tmpStart=START )) ;
   LABEL = START;
   START = upcase(START);
   output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/***********************************************************************************/
/* Create a temporary sdm dimension member format with existing codes              */
/***********************************************************************************/

%let _tmpDSN = _SDMLIB.SAS_MEMBER;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpFormat=FMT_SAS_MEMBER;
%let _tmpStart=MEMBER_CD;

%let _internalDimId = 0;
proc sql noprint;
  select internal_dimension_id into :_internalDimId
  from _SDMLIB.SAS_DIMENSION
  where &_where2 and dimension_cd eq "&dimcode"
  ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpWhere=where=(&_where2 and INTERNAL_DIMENSION_ID eq &_internalDimId);

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._TMPSORT (keep=&_tmpStart)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain LABEL ;
   retain HLO "";
   if _N_ = 1 then do;;
     HLO = "O";
     output;
     HLO = "";
   end;
   set WORK._TMPSORT(rename=(&_tmpStart=START )) ;
   LABEL = START;
   START = upcase(START);
   output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/***********************************************************************************/
/* Create a temporary sdm dimension hierarchy format with existing codes           */
/***********************************************************************************/

%let _tmpDSN = _SDMLIB.SAS_HIERARCHY;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpFormat=FMT_SAS_HIERARCHY;
%let _tmpStart=HIERARCHY_CD;

%let _tmpWhere=where=(&_where2 and INTERNAL_DIMENSION_ID eq &_internalDimId);

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._TMPSORT (keep=&_tmpStart)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain LABEL ;
   retain HLO "";
   if _N_ = 1 then do;;
     HLO = "O";
     output;
     HLO = "";
   end;
   set WORK._TMPSORT(rename=(&_tmpStart=START )) ;
   LABEL = START;
   START = upcase(START);
   output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/***********************************************************************************/
/* Create a temporary sdm dimension type format with existing codes                */
/***********************************************************************************/

%let _tmpDSN = _SDMLIB.SAS_DIM_TYPE;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpFormat=FMT_SAS_DIM_TYPE;
%let _tmpStart=DIM_TYPE_CD;
%let _tmpWhere=where=(&_where1);

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._TMPSORT (keep=&_tmpStart)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain LABEL;
   set WORK._TMPSORT(rename=(&_tmpStart=START )) end=eof;
   LABEL = START;
   START = upcase(START);
   output;
   if eof then do;
     HLO = "O";
     START = "";
     output;
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/*************************************************************************************/
/*                Check if dimension_type table exists in the sas.                   */
/*************************************************************************************/
%let _tmpDSN = &_lib..DIMENSION_TYPE;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=27
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

/*************************************************************************************/
/*  Check that the dimension type entered can be found in the dimension_type table.  */
/*************************************************************************************/
proc sql noprint;
   select count(*) into :_dTypeCount
   from &_tmpDSN
   where upcase(DIMENSION_TYPE_CD) = upcase("&dimtype")
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if &_dtypeCount < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=3
          , msgTxt1=<&dimtype>, msgTxt2=dimension_type_cd, msgTxt3=dimension_type
          , msgPutlog=&putlog);
  %goto exit;
%end;

/*************************************************************************************/
/* Retrieve the names of the member, hierarchy, child/parent relationship and nls    */
/* tables for the dimension type and verify their existence.                         */
/*************************************************************************************/
data _null_;
   set &_lib..DIMENSION_TYPE;
   if (DIMENSION_TYPE_CD = "&dimtype") then do;
      call symput('prefix', %unquote(strip(TABLE_NM)));
      call symput('assoctype', %unquote(strip(ASSOC_TYPE_TABLE_NM)));
      call symput('assoc', %unquote(strip(ASSOC_TABLE_NM)));
      call symput('nls', %unquote(strip(NLS_TABLE_NM)));
   end;
run;

/* Check for the existence of the members table. */
%let _tableCheck = "&prefix";
%if (&_tableCheck = " ") %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=4
           , msgTxt1=TABLE_NM, msgTxt2=<&dimtype>
           , msgPutlog=&putlog);
   %goto exit;
%end;

%let _tmpDSN = &_lib..&prefix;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=5
           , msgTxt1=&prefix, msgTxt2=<&dimtype>
           , msgPutlog=&putlog);
   %goto exit;
%end;

/* Check for the existence of the hierarchy table. */
%let _tableCheck = "&assoctype";
%if (&_tableCheck = " ") %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=4
           , msgTxt1=ASSOC_TYPE_TABLE_NM, msgTxt2=<&dimtype>
           , msgPutlog=&putlog);
   %goto exit;
%end;

%let _tmpDSN = &_lib..&assoctype;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=6
           , msgTxt1=&assoctype, msgTxt2=<&dimtype>
           , msgPutlog=&putlog);
   %goto exit;
%end;

/* Check for the existence of the child/parent relationship table. */
%let _tableCheck = "&assoc";
%if (&_tableCheck = " ") %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=4
           , msgTxt1=ASSOC_TABLE_NM, msgTxt2=<&dimtype>
           , msgPutlog=&putlog);
   %goto exit;
%end;

%let _tmpDSN = &_lib..&assoc;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=7
           , msgTxt1=&assoc, msgTxt2=<&dimtype>
           , msgPutlog=&putlog);
   %goto exit;
%end;

/* Check for the existence of the nls table. */

%if ((&dimtype ne CURRENCY) and (&dimtype ne ITEMCAT)) %then %do;
   %let _tableCheck = "&nls";
   %if (&_tableCheck = " ") %then %do;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=4
              , msgTxt1=NLS_TABLE_NM, msgTxt2=<&dimtype>
              , msgPutlog=&putlog);
      %goto exit;
   %end;

   %let _tmpDSN = &_lib..&nls;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=7
              , msgTxt1=&nls, msgTxt2=<&dimtype>
              , msgPutlog=&putlog);
      %goto exit;
   %end;
%end;

/*************************************************************************************/
/*                    Set macros based on dimension type.                            */
/*************************************************************************************/

%if (&dimtype eq ACCOUNT) or (&dimtype eq ANALYSIS) or (&dimtype eq COSTCTR) or
    (&dimtype eq CURRENCY) or (&dimtype eq EXTORG) or (&dimtype eq INTORG) or
    (&dimtype eq ITEMCAT) or (&dimtype eq PROFITCTR) or (&dimtype eq TIME) %then %do;
   %let dimid=&prefix._ID;
   %let dimnm=&prefix._NM;
   %let dimdesc=&prefix._DESC;
   %let dimParentid=PARENT_&prefix._ID;
   %let assoctypecd=&prefix._ASSOC_TYPE_CD;
   %let assoctypedesc=&prefix._ASSOC_TYPE_DESC;
   %let dfltmemcd=DEFAULT_&prefix._ID;
   %if (&dimtype eq CURRENCY) %then %do;
       %let dimid=CURRENCY_CD;
       %let dimnm=CURRENCY_CD;
       %let dimdesc=CURRENCY_DESC;
       %let dimParentid=PARENT_CURRENCY_CD;
       %let dfltmemcd=DEFAULT_&prefix._CD;
   %end;
   %if (&dimtype eq INTORG) %then %do;
     %let dimnm=ORGANIZATION_NM;
     %let dimdesc=ORGANIZATION_DESC;
   %end;
   %if (&dimtype eq ITEMCAT) %then %do;
     %let dimid=ITEM_CATEGORY_CD;
     %let dimnm=ITEM_CATEGORY_CD;
     %let dimParentid=PARENT_ITEM_CATEGORY_CD;
     %let dfltmemcd=DEFAULT_&prefix._CD;
   %end;
   %if (&dimtype eq EXTORG) %then %do;
     %let dimnm=ORGANIZATION_NM;
     %let dimdesc=ORGANIZATION_DESC;
   %end;
%end;
%else %do;
   %let dimid=MEMBER_ID;
   %let dimnm=MEMBER_NM;
   %let dimdesc=MEMBER_DESC;
   %let dimParentid=PARENT_MEMBER_ID;
   %let assoctypecd=MEMBER_ASSOC_TYPE_CD;
   %let assoctypedesc=MEMBER_ASSOC_TYPE_DESC;
   %let dfltmemcd=DEFAULT_MEMBER_ID;
%end;

/*************************************************************************************/
/*        Check that records exist in the member table for the dimension type.       */
/*************************************************************************************/
%let _tmpDSN = &_lib..&prefix;

proc sql noprint;
   select count(*) into :_rec_cnt
   from &_tmpDSN
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=8
          , msgTxt1=<&dimtype>, msgPutlog=&putlog);
  %let soletl_statusID = 0; /* No updates */
  %let soletl_error = &sesgmsg_msgCodeReturn;
  %let soletl_errormsg = &sesgmsg_msgReturn;
  %goto exit ;
%end;

/*************************************************************************************/
/*           This is used when this macro is called from etldimch.sas.               */
/*************************************************************************************/
%cont:

/*************************************************************************************/
/* Verify that for the internal org dimension there exists an active ALL and EXT     */
/* member.  Also, verify that a currency dimension has already been imported.        */
/*************************************************************************************/
%if (&dimtype eq INTORG) %then %do;

   /* Verify the existence of an ALL and EXT member */

   %let _tmpDSN = &_lib..&prefix;
   proc sql noprint;
      select count(*) into :all_found
      from &_tmpDSN
      where INTERNAL_ORG_ID = 'ALL' and &_where1;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if &all_found < 1 %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=15,
               msgTxt1=<ALL>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   proc sql noprint;
      select count(*) into :ext_found
      from &_tmpDSN
      where INTERNAL_ORG_ID = 'EXT' and &_where1;
   quit;

   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if &ext_found < 1 %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=15,
               msgTxt1=<EXT>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   /* Verify the existence of a currency dimension  */

   %let _tmpDSN = _SDMLIB.SAS_DIM_TYPE;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=32
             , msgPutlog=N);
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
             , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   proc sql noprint;
      select DIM_TYPE_ID into :_dTypeId
      from &_tmpDSN
      where DIM_TYPE_ROLE = 5 ;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %let _tmpDSN = _SDMLIB.SAS_DIMENSION;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=33
             , msgPutlog=N);
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   proc sql noprint;
      select count(*) into :_numCurrDims
      from &_tmpDSN
      where DIM_TYPE_ID = &_dTypeId;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if &_numCurrDims < 1 %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=14,
               msgPutlog=&putlog);
     %goto exit ;
   %end;
%end;

/*************************************************************************************/
/*               Create member records for the imported dimension.                   */
/*************************************************************************************/

/* Verify that no duplicate codes exist in the stage member table  */

%let _tmpDSN = &_lib..&prefix;
%let _tmpWhere=where=(&_where1);

%if ((&dimtype eq CURRENCY) or (&dimtype eq ITEMCAT)) %then %do;
  %_validateDups(_param=&dimid, _ds=&_tmpDSN, _format=SAS_MEMBER,
               _where=&_tmpWhere, _cols=VALID_FROM_DTTM VALID_TO_DTTM LANGUAGE_CD,
               _byVars=&dimid LANGUAGE_CD);
%end;
%else %do;
  %_validateDups(_param=&dimid, _ds=&_tmpDSN, _format=SAS_MEMBER,
              _where=&_tmpWhere, _cols=VALID_FROM_DTTM VALID_TO_DTTM, _byVars=&dimid);
%end;

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

%if ((&dimtype eq CURRENCY) or (&dimtype eq ITEMCAT)) %then %do;
   data WORK.SAS_ETL_MEMBER;
      set _SDMLIB.sas_etl_member(obs=0);
      stop;
   run;
   /* Determine the number of locales to import. */
   %let _tmpDSN = WORK._INPUT_LOCALES;
   proc sql noprint;
      select count(*) into :_locCnt
      from &_tmpDSN
      ;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   %do i = 1 %to &_locCnt;
      data _null_;
         set &_tmpDSN (obs = &i);
         call symput("_tmpCode", LANGUAGE_CD);
         call symput("_tmpLocale", strip(LOCALE_CD));
      run;
      %let _where3=&_where1 and (LANGUAGE_CD = "&_tmpCode");
      data WORK._TMP_MEMBER
           (keep=ETL_INSTANCE_ID MEMBER_TYPE_ENUM MEMBER_CD MEMBER_NM MEMBER_DESC DIMENSION_CD
              HIDDEN_FLG LOCALE_CD LOCALE_ID VALID_FROM_DTTM VALID_TO_DTTM);

         set _SDMLIB.sas_etl_member(obs=0)
             &_lib..&prefix(where=(&_where3));

         ETL_INSTANCE_ID = symgetn("soletl_instanceId");
         MEMBER_TYPE_ENUM = 1;
         MEMBER_CD = &dimid;
         MEMBER_NM = &dimnm;
         MEMBER_DESC = &dimdesc;
         LOCALE_ID = input(LANGUAGE_CD, FMT_LOCALE.);
         if HIDDEN_FLG = . then
            HIDDEN_FLG = 0;
         if VALID_FROM_DTTM = . then
            VALID_FROM_DTTM = &soletl_processedDTTM.;
         if VALID_TO_DTTM = . then
            VALID_TO_DTTM = &curr_rec_dttm.;
      run;
      proc sql noprint;
         select count(*) into :_recCount
         from WORK._TMP_MEMBER ;
      quit;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
      data _null_;
         if &_recCount < 1 then do;
            if strip("&soletl_locale_no_data_found") = "" then
               call symput("soletl_locale_no_data_found", strip("&_tmpLocale"));
            else
               call symput("soletl_locale_no_data_found", strip(symget("soletl_locale_no_data_found"))
                        || "09"x || strip("&_tmpLocale")) ;
         end;
      run;
      proc append base=WORK.SAS_ETL_MEMBER data=WORK._TMP_MEMBER &FM_PROCAPPENDFORCE;
      run;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
   %end;
%end;
%else %do;
   %let _tmpDSN = &_lib..&prefix;
   /* Extract the non-nls values from the members table */
   %if &dimtype eq ACCOUNT %then %do;
      data WORK._TMP_MEMBER
           (keep=ETL_INSTANCE_ID MEMBER_TYPE_ENUM MEMBER_CD HIDDEN_FLG
                 VALID_FROM_DTTM VALID_TO_DTTM);

         set _SDMLIB.sas_etl_member(obs=0)
             &_tmpDSN(where=(&_where1));

         ETL_INSTANCE_ID = symgetn("soletl_instanceId");
         if (ROLL_UP_TO_PARENT_FLG = 'N') or (GL_ACCOUNT_TYPE_CD = 'NonFrequency') then
            MEMBER_TYPE_ENUM = 2;
         else
            MEMBER_TYPE_ENUM = 1;
         MEMBER_CD = &dimid;
         if HIDDEN_FLG = . then
            HIDDEN_FLG = 0;
         if VALID_FROM_DTTM = . then
            VALID_FROM_DTTM = &soletl_processedDTTM.;
         if VALID_TO_DTTM = . then
            VALID_TO_DTTM = &curr_rec_dttm.;
      run;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
   %end;
   %else %do;
      data WORK._TMP_MEMBER
           (keep=ETL_INSTANCE_ID MEMBER_TYPE_ENUM MEMBER_CD HIDDEN_FLG
                 VALID_FROM_DTTM VALID_TO_DTTM);

         set _SDMLIB.sas_etl_member(obs=0)
             &_tmpDSN(where=(&_where1));

         ETL_INSTANCE_ID = symgetn("soletl_instanceId");
         if ROLL_UP_TO_PARENT_FLG = 'N' then
            MEMBER_TYPE_ENUM = 2;
         else
            MEMBER_TYPE_ENUM = 1;
         MEMBER_CD = &dimid;
         if HIDDEN_FLG = . then
            HIDDEN_FLG = 0;
         if VALID_FROM_DTTM = . then
            VALID_FROM_DTTM = &soletl_processedDTTM.;
         if VALID_TO_DTTM = . then
            VALID_TO_DTTM = &curr_rec_dttm.;
      run;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
   %end;

   /* Extract the nls values for the members */
   data WORK._TMP_NLS;
      set _SDMLIB.sas_etl_member(obs=0
          keep=MEMBER_CD MEMBER_NM MEMBER_DESC DIMENSION_CD LOCALE_CD LOCALE_ID);
      stop;;
   run;

   /* Determine the number of locales to import. */
   %let _tmpDSN = WORK._INPUT_LOCALES;
   proc sql noprint;
      select count(*) into :_locCnt
      from &_tmpDSN
      ;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   %do i = 1 %to &_locCnt;
      data _null_;
         set &_tmpDSN (obs = &i);
         call symput("_tmpCode", LANGUAGE_CD);
         call symput("_tmpLocale", strip(LOCALE_CD));
      run;
      %if ("&_tmpCode" = "&_dfltLangCd") %then %do;
         data WORK._HOLD_NLS
            (keep=MEMBER_CD MEMBER_NM MEMBER_DESC DIMENSION_CD LOCALE_CD LOCALE_ID);
            set _SDMLIB.sas_etl_member(obs=0)
                &_lib..&prefix(where=(&_where1));
            MEMBER_CD = &dimid;
            MEMBER_NM = &dimnm;
            MEMBER_DESC = &dimdesc;
            LOCALE_ID = input("&_dfltLangCd", FMT_LOCALE.);
         run;
      %end;
      %else %do;
         %let _where3=&_where1 and (LANGUAGE_CD = "&_tmpCode");
         data WORK._HOLD_NLS
            (keep=MEMBER_CD MEMBER_NM MEMBER_DESC DIMENSION_CD LOCALE_CD LOCALE_ID);
            set _SDMLIB.sas_etl_member(obs=0)
                &_lib..&nls(where=(&_where3));
            MEMBER_CD = &dimid;
            MEMBER_NM = &dimnm;
            MEMBER_DESC = &dimdesc;
            LOCALE_ID = input(LANGUAGE_CD, FMT_LOCALE.);
         run;
      %end;
      proc sql noprint;
         select count(*) into :_recCount
         from WORK._HOLD_NLS ;
      quit;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
      data _null_;
         if &_recCount < 1 then do;
            if strip("&soletl_locale_no_data_found") = "" then
               call symput("soletl_locale_no_data_found", strip("&_tmpLocale"));
            else
               call symput("soletl_locale_no_data_found", strip(symget("soletl_locale_no_data_found"))
                        || "09"x || strip("&_tmpLocale")) ;
         end;
      run;
      proc append base=WORK._TMP_NLS data=WORK._HOLD_NLS &FM_PROCAPPENDFORCE;
      run;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
   %end;
%end;

/**********************************************************************************/
/* Merge the member table and it's corresponding nls table together and check     */
/* if any of the members only exist in the nls table.                             */
/**********************************************************************************/
%if ((&dimtype ne CURRENCY) and (&dimtype ne ITEMCAT)) %then %do;
   proc sort data = WORK._TMP_MEMBER;
      by MEMBER_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   proc sort data = WORK._TMP_NLS;
      by MEMBER_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   %let _tmpMissingCdErr = 0;
   %let _tmpCode = ;
   data WORK.SAS_ETL_MEMBER
      (keep=ETL_INSTANCE_ID MEMBER_TYPE_ENUM MEMBER_CD MEMBER_NM MEMBER_DESC DIMENSION_CD
           HIDDEN_FLG LOCALE_CD LOCALE_ID VALID_FROM_DTTM VALID_TO_DTTM);

      merge WORK._TMP_MEMBER (in = inA)
            WORK._TMP_NLS (in = inB);
         by MEMBER_CD;
      if (inB) then do;
         if (inA) then
            output;
         else do;
            call symput("_tmpMissingCdErr", 1);
            call symput("_tmpCode", strip(MEMBER_CD));
            stop;
         end;
      end;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   %if (&_tmpMissingCdErr) %then %do;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=43
              , msgTxt1=<&_tmpCode>, msgTxt2=&nls, msgTxt3=&prefix
              , msgPutlog=&putlog);
      %goto exit;
   %end;
%end;

proc append base=_SDMLIB.SAS_ETL_MEMBER data=WORK.SAS_ETL_MEMBER &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/*              Create hierarchy records for the imported dimension.                 */
/*************************************************************************************/

%let _tmpDSN = &_lib..&assoctype;
%let _tmpWhere=where=(&_where1);

/* Verify that no duplicate codes exist in the stage member assoc type table  */
%_validateDups(_param=&assoctypecd, _ds=&_tmpDSN, _format=SAS_HIERARCHY, _where=&_tmpWhere,
               _cols=&assoctypecd VALID_FROM_DTTM VALID_TO_DTTM LANGUAGE_CD,
               _byVars=&assoctypecd LANGUAGE_CD);

%if (&_dupsFound eq 1) %then %goto exit ;

%_validateDups(_param=&dfltmemcd, _ds=&_tmpDSN, _format=SAS_HIERARCHY, _where=&_tmpWhere,
               _cols=&dfltmemcd VALID_FROM_DTTM VALID_TO_DTTM LANGUAGE_CD,
               _byVars=&dfltmemcd LANGUAGE_CD);

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._HIER_SORT;
   by LANGUAGE_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sort data = WORK._INPUT_LOCALES NODUPKEY;
   by LANGUAGE_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK._sdmlib_sas_etl_hierarchy;
   length LANGUAGE_CD $32;
   set _SDMLIB.sas_etl_hierarchy(obs=0);
   stop;
run;

data WORK.SAS_ETL_HIERARCHY
     (keep=ETL_INSTANCE_ID HIERARCHY_CD HIERARCHY_NM HIERARCHY_DESC DIMENSION_CD
           HIDDEN_FLG LOCALE_CD LOCALE_ID VALID_FROM_DTTM VALID_TO_DTTM);

   merge WORK._sdmlib_sas_etl_hierarchy(obs=0)
         WORK._HIER_SORT (in = inA)
         WORK._INPUT_LOCALES (in = inB);
      by LANGUAGE_CD;

   ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   HIERARCHY_CD = &assoctypecd;
   HIERARCHY_NM = &assoctypedesc;
   HIERARCHY_DESC = &assoctypedesc;
   HIDDEN_FLG = 0;
   if VALID_FROM_DTTM = . then
      VALID_FROM_DTTM = &soletl_processedDTTM.;
   if VALID_TO_DTTM = . then
      VALID_TO_DTTM = &curr_rec_dttm.;
   if (inA) and (inB) then do;
      LOCALE_ID = input(LANGUAGE_CD, FMT_LOCALE.);
      output;
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_HIERARCHY data=WORK.SAS_ETL_HIERARCHY &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpDSN = &_lib..&assoc;
%let _errCount = 0;

%if ((&dimtype eq CURRENCY) or (&dimtype eq ANALYSIS)) %then %do;
   proc sql noprint;
      select count(*) into :_errCount
      from &_tmpDSN
      where &dimid ne &dimParentid
      ;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
%end;

%if (&_errCount > 0) %then %do;
   %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=44
           , msgTxt1=&dimtype , msgTxt2=&assoc, msgPutlog=&putlog);
   %goto exit ;
%end;

/*************************************************************************************/
/*            Create child_parent records for the imported dimension                 */
/*************************************************************************************/
%if (&dimtype eq CURRENCY) %then %do;
   %let _tmpDSN = &_lib..&assoc;
   data WORK.SAS_ETL_HIERARCHY_MEMBER
       (keep=CHILD_MEMBER_CD PARENT_MEMBER_CD MEMBER_RANK HIERARCHY_CD
           VALID_FROM_DTTM VALID_TO_DTTM ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_hierarchy_member(obs=0)
          &_tmpDSN(where=(&_where1));

      CHILD_MEMBER_CD = &dimid;
      PARENT_MEMBER_CD = PARENT_CURRENCY_CD;
      MEMBER_RANK = CHILD_INDX;
      CHILD_INDX + 1;
      HIERARCHY_CD = &assoctypecd;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
%end;
%else %if (&dimtype eq ITEMCAT) %then %do;
   %let _tmpDSN = &_lib..&assoc;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=26
              , msgTxt1=<&_tmpDSN>, msgPutlog=&putlog);
      %goto exit ;
   %end;
   proc sort data=&_tmpDSN
             out = WORK._STGDDS_ASSOC;
      by ORDER_NO;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.TEMPASSOC;
      length RECORD_ID 8;
      set WORK._STGDDS_ASSOC;
      RECORD_ID = _n_;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK._STGDDS_ASSOC;
      set WORK._STGDDS_ASSOC;
      CHILD_MEMBER_CD = &dimid;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.TEMPASSOC1
        WORK.TEMPASSOC2;
      set WORK.TEMPASSOC;
      if (&dimid = &dimParentid) then
         output WORK.TEMPASSOC1;
      else
         output WORK.TEMPASSOC2;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.TEMPASSOC1;
      by &assoctypecd ORDER_NO;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.TEMPASSOC2;
      by &assoctypecd &dimParentid ORDER_NO;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.PARENT
       (keep=RECORD_ID CHILD_MEMBER_CD PARENT_MEMBER_CD MEMBER_RANK HIERARCHY_CD
           VALID_FROM_DTTM VALID_TO_DTTM ETL_INSTANCE_ID);

      length RECORD_ID 8;
      retain HOLD_PARENT;
      retain PARENT_INDX 0;
      set  _SDMLIB.sas_etl_hierarchy_member(obs=0)
           WORK.TEMPASSOC1(where=(&_where1));

      if _N_ = 1 then do;
        HOLD_PARENT = PARENT_MEMBER_CD;
        HOLD_PARENT = '';
      end;

      if (&dimParentid = HOLD_PARENT) then
         MEMBER_RANK = PARENT_INDX - 1;
      else do;
         MEMBER_RANK = PARENT_INDX;
         PARENT_INDX + 1;
         HOLD_PARENT = &dimParentid;
      end;
      CHILD_MEMBER_CD = &dimid;
      PARENT_MEMBER_CD = &dimParentid;
      HIERARCHY_CD = &assoctypecd;
      HOLD_PARENT = &dimParentid;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.CHILD
       (keep=RECORD_ID CHILD_MEMBER_CD PARENT_MEMBER_CD MEMBER_RANK HIERARCHY_CD
           VALID_FROM_DTTM VALID_TO_DTTM ETL_INSTANCE_ID);

      length RECORD_ID 8;
      retain HOLD_PARENT;
      retain HOLD_CHILD;
      retain CHILD_INDX 0;
      set  _SDMLIB.sas_etl_hierarchy_member(obs=0)
           WORK.TEMPASSOC2(where=(&_where1));

      if _N_ = 1 then do;
        HOLD_PARENT = PARENT_MEMBER_CD;
        HOLD_PARENT = '';
        HOLD_CHILD = CHILD_MEMBER_CD;
        HOLD_CHILD = '';
      end;

      if (&dimid = HOLD_CHILD) then
         MEMBER_RANK = CHILD_INDX - 1;
      else do;
         if (&dimParentid ^= HOLD_PARENT) then do;
            MEMBER_RANK = 0;
            CHILD_INDX = 1;
            HOLD_PARENT = &dimParentid;
         end;
         else do;
            MEMBER_RANK = CHILD_INDX;
            CHILD_INDX + 1;
         end;
         HOLD_CHILD = &dimid;
      end;
      CHILD_MEMBER_CD = &dimid;
      PARENT_MEMBER_CD = &dimParentid;
      HIERARCHY_CD = &assoctypecd;
      HOLD_PARENT = &dimParentid;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.SAS_ETL_HIERARCHY_MEMBER1;
      set WORK.PARENT WORK.CHILD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.SAS_ETL_HIERARCHY_MEMBER1
             out = WORK.SAS_ETL_HIERARCHY_MEMBER(drop=RECORD_ID);
      by RECORD_ID;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;
%else %do;
   %let _tmpDSN = &_lib..&prefix;
   %if (&dimtype eq ACCOUNT) %then %do;
     /* Create format FMT_BEHAVIOR */
     %let _tmpFormat=FMT_BEHAVIOR;
     %let _tmpStart=GL_ACCOUNT_ID;
     %let _tmpWhere=where=(&_where2);
     proc sort data = &_tmpDSN(&_tmpWhere
                    keep=&_tmpStart GL_ACCOUNT_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM)
                out = work._tmpSort(drop = VALID_FROM_DTTM VALID_TO_DTTM)
                NODUPKEY;
        by &_tmpStart;
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit ;

     data work.&_tmpFormat(keep=START LABEL FMTNAME TYPE);
        length LABEL $ 32;
        retain FMTNAME "&_tmpFormat";
        retain TYPE "C";
        set work._tmpSort(rename=(&_tmpStart=START));
        if (GL_ACCOUNT_TYPE_CD in ('Asset', 'Equity', 'Liability', 'StatisticalBalance')) then
           LABEL = 'Balance';
        else if (GL_ACCOUNT_TYPE_CD in ('Expense', 'Revenue', 'StatisticalFlow')) then
           LABEL = 'Flow';
        else if GL_ACCOUNT_TYPE_CD = 'CTA' then
           LABEL = 'CTA';
        else if GL_ACCOUNT_TYPE_CD = 'RetainedEarnings' then
           LABEL = 'Hybrid';
        else if GL_ACCOUNT_TYPE_CD = 'NonFrequency' then
           LABEL = 'NonFrequency';
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit ;

     proc format cntlin=work.&_tmpFormat library=work;
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit ;

     %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
       %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=12
               , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
       %goto exit ;
     %end;
   %end;

   %let _tmpDSN = &_lib..&assoc;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=26
              , msgTxt1=<&_tmpDSN>, msgPutlog=&putlog);
      %goto exit ;
   %end;

   %let _tmpBehaviorErr = 0;

   %if (&dimtype eq ACCOUNT) %then %do;
      data _null_;
         length CHILD_BEHAVIOR $32;
         length PARENT_BEHAVIOR $32;
         set &_tmpDSN(where=(&_where2)); ;
         CHILD_BEHAVIOR = put(&dimid, $FMT_BEHAVIOR.);
         PARENT_BEHAVIOR = put(&dimParentid, $FMT_BEHAVIOR.);
         if PARENT_BEHAVIOR eq 'Balance' then do;
            if CHILD_BEHAVIOR not in ('Balance', 'Hybrid', 'CTA') then do;
               CHILD_MEMBER_CD = &dimid;
               PARENT_MEMBER_CD = &dimParentid;
               call symput("_tmpBehaviorErr", 1);
               call symput("_tmpChildCode", strip(CHILD_MEMBER_CD));
               call symput("_tmpParentCode", strip(PARENT_MEMBER_CD));
               call symput("_tmpChildBehavior", strip(CHILD_BEHAVIOR));
               stop;
            end;
         end;
         else if PARENT_BEHAVIOR ne CHILD_BEHAVIOR then do;
            CHILD_MEMBER_CD = &dimid;
            PARENT_MEMBER_CD = &dimParentid;
            call symput("_tmpBehaviorErr", 2);
            call symput("_tmpChildCode", strip(CHILD_MEMBER_CD));
            call symput("_tmpParentCode", strip(PARENT_MEMBER_CD));
            call symput("_tmpChildBehavior", strip(CHILD_BEHAVIOR));
            call symput("_tmpParentBehavior", strip(PARENT_BEHAVIOR));
            stop;
         end;
      run;
      %if &_tmpBehaviorErr = 1 %then %do;
        %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=59
                , msgTxt1=<&_tmpParentCode>, msgTxt2=<&_tmpChildCode>
                , msgTxt3=&_tmpChildBehavior, msgPutlog=&putlog);
        %goto exit ;
      %end;
      %if &_tmpBehaviorErr = 2 %then %do;
        %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=52
                , msgTxt1=<&_tmpChildCode>, msgTxt2=&_tmpChildBehavior
                , msgTxt3=<&_tmpParentCode>, msgTxt4=&_tmpParentBehavior
                , msgPutlog=&putlog);
        %goto exit ;
      %end;
   %end;

   proc sort data=&_tmpDSN
             out = WORK._STGDDS_ASSOC;
      by ORDER_NO;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.TEMPASSOC;
      length RECORD_ID 8;
      set WORK._STGDDS_ASSOC;
      RECORD_ID = _n_;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK._STGDDS_ASSOC;
      set WORK._STGDDS_ASSOC;
      CHILD_MEMBER_CD = &dimid;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.TEMPASSOC1
        WORK.TEMPASSOC2;
      set WORK.TEMPASSOC;
      if (&dimid = &dimParentid) then
         output WORK.TEMPASSOC1;
      else
         output WORK.TEMPASSOC2;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.TEMPASSOC1;
      by &assoctypecd ORDER_NO;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.TEMPASSOC2;
      by &assoctypecd &dimParentid ORDER_NO;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

  data WORK.PARENT
       (keep=RECORD_ID CHILD_MEMBER_CD PARENT_MEMBER_CD MEMBER_RANK HIERARCHY_CD
           VALID_FROM_DTTM VALID_TO_DTTM ETL_INSTANCE_ID);

      length RECORD_ID 8;
      retain HOLD_PARENT;
      retain PARENT_INDX 0;
      set  _SDMLIB.sas_etl_hierarchy_member(obs=0)
           WORK.TEMPASSOC1(where=(&_where1));

      if _N_ = 1 then do;
        HOLD_PARENT = PARENT_MEMBER_CD;
        HOLD_PARENT = '';
      end;

      if (&dimParentid = HOLD_PARENT) then
         MEMBER_RANK = PARENT_INDX - 1;
      else do;
         MEMBER_RANK = PARENT_INDX;
         PARENT_INDX + 1;
      end;
      CHILD_MEMBER_CD = &dimid;
      PARENT_MEMBER_CD = &dimParentid;
      HIERARCHY_CD = &assoctypecd;
      HOLD_PARENT = &dimParentid;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.CHILD
       (keep=RECORD_ID CHILD_MEMBER_CD PARENT_MEMBER_CD MEMBER_RANK HIERARCHY_CD
           VALID_FROM_DTTM VALID_TO_DTTM ETL_INSTANCE_ID);

      length RECORD_ID 8;
      retain HOLD_PARENT;
      retain HOLD_CHILD;
      retain CHILD_INDX 0;
      set  _SDMLIB.sas_etl_hierarchy_member(obs=0)
           WORK.TEMPASSOC2(where=(&_where1));

      if _N_ = 1 then do;
        HOLD_PARENT = PARENT_MEMBER_CD;
        HOLD_PARENT = '';
        HOLD_CHILD = CHILD_MEMBER_CD;
        HOLD_CHILD = '';
      end;

      if (&dimid = HOLD_CHILD) then
         MEMBER_RANK = CHILD_INDX - 1;
      else do;
         if (&dimParentid ^= HOLD_PARENT) then do;
            MEMBER_RANK = 0;
            CHILD_INDX = 1;
         end;
         else do;
            MEMBER_RANK = CHILD_INDX;
            CHILD_INDX + 1;
         end;
         HOLD_CHILD = &dimid;
      end;
      CHILD_MEMBER_CD = &dimid;
      PARENT_MEMBER_CD = &dimParentid;
      HIERARCHY_CD = &assoctypecd;
      HOLD_PARENT = &dimParentid;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.SAS_ETL_HIERARCHY_MEMBER1;
      set WORK.PARENT WORK.CHILD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.SAS_ETL_HIERARCHY_MEMBER1
             out = WORK.SAS_ETL_HIERARCHY_MEMBER(drop=RECORD_ID);
      by RECORD_ID;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;


/* Validate that all members within a hierarchy have only 1 parent defined. */

%let _tmpDSN = WORK.SAS_ETL_HIERARCHY_MEMBER;

%let _tmpWhere=where=(&_where2);
proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._TMPPARENTS (keep=HIERARCHY_CD CHILD_MEMBER_CD VALID_FROM_DTTM
                                   VALID_TO_DTTM) ;
   by &byHierarchy CHILD_MEMBER_CD ;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpMultParents = 0;
data _null_;
   set WORK._TMPPARENTS;
   by &byHierarchy CHILD_MEMBER_CD ;
   if (first.CHILD_MEMBER_CD ne last.CHILD_MEMBER_CD) then do;
      call symput("_tmpMultParents", 1);
      call symput("_tmpCode", strip(CHILD_MEMBER_CD));
      call symput("_tmpHier", strip(HIERARCHY_CD));
      stop;
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if (&_tmpMultParents) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=17,
            msgTxt1=<&_tmpCode>, msgTxt2=&assoc, msgTxt3=<&_tmpHier>,
            msgPutlog=&putlog);
  %goto exit ;
%end;

/* Validate that all parent members within a hierarchy exist as a child member */
/* as well within the hierarchy.                                               */

proc sql;
create table WORK.MISSING_CHILDREN as
select distinct PARENT_MEMBER_CD, HIERARCHY_CD as PARENT_HIERARCHY from &_tmpDSN
where PARENT_MEMBER_CD not in (select CHILD_MEMBER_CD from &_tmpDSN
                               where HIERARCHY_CD = PARENT_HIERARCHY);
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sql noprint;
   select count(*) into :_errCount
   from WORK.MISSING_CHILDREN;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if &_errCount > 0 %then %do;
   %do i = 1 %to &_errCount;
      data _null_;
         set WORK.MISSING_CHILDREN (obs = &i);
         call symput("_tmpCode", strip(PARENT_MEMBER_CD));
         call symput("_tmpHier", strip(PARENT_HIERARCHY));
      run;
      %checkErr;
      %if &_tmpRC %then %goto exit ;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=53,
                msgTxt1=<&_tmpCode>, msgTxt2=&assoc, msgTxt3=<&_tmpHier>,
                msgPutlog=&putlog);
   %end;
   %goto exit ;
%end;

proc append base=_SDMLIB.SAS_ETL_HIERARCHY_MEMBER data=WORK.SAS_ETL_HIERARCHY_MEMBER &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpFormat=FMT_DIMTYPE;
proc format library=work;
  invalue &_tmpFormat
    'ACCOUNT'     = 1
    'ANALYSIS'    = 3
    'COSTCTR'     = 4
    'CURRENCY'    = 5
    'EXTORG'      = 6
    'INTORG'      = 7
    'ITEMCAT'     = 8
    'PROFITCTR'   = 10
    'TIME'        = 11
    'TRADER'      = 12
    'SOURCE'      = 14
    'RATE_TYPE'   = 15
    'TO_CURRENCY' = 16
    OTHER         = 99
  ;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

proc sql noprint;
   select count(*) into :hierCnt from WORK.SAS_ETL_HIERARCHY;
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

/*************************************************************************************/
/*           Create dflt_hier_member record(s) for the imported dimension            */
/*************************************************************************************/
%if (&hierCnt < 1) %then %do;
   data WORK.SAS_ETL_HIERARCHY_DFLT_MBR
      (keep=HIERARCHY_CD DIM_TYPE_ROLE DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
           ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0);
      stop;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
%end;
%else %if ((&dimtype eq CURRENCY) or (&dimtype eq ITEMCAT)) %then %do;
   data WORK._TEMP_ASSOCTYPE
      (keep=HIERARCHY_CD DIM_TYPE_ROLE DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
           ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0)
          &_lib..&assoctype;

      HIERARCHY_CD = &assoctypecd;
      DIM_TYPE_ROLE = input(symgetc("dimtype"), FMT_DIMTYPE.);
      DFLT_MEMBER_CD = &dfltmemCD;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;

   proc sort data = WORK._TEMP_ASSOCTYPE NODUPKEY;
      by HIERARCHY_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.SAS_ETL_HIERARCHY_MEMBER (rename=(CHILD_MEMBER_CD = HOLD_DFLT_CD))
              out = WORK._TEMP_DFLT_MEM (keep=HIERARCHY_CD HOLD_DFLT_CD)
              NODUPKEY;
      by HIERARCHY_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %let _tmpMsngDfltMem = 0;

   data WORK.SAS_ETL_HIERARCHY_DFLT_MBR
      (keep=HIERARCHY_CD DIM_TYPE_ROLE DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
           ETL_INSTANCE_ID);

      merge _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0)
            WORK._TEMP_ASSOCTYPE
            WORK._TEMP_DFLT_MEM;
         by HIERARCHY_CD;

      DIM_TYPE_ROLE = input(symgetc("dimtype"), FMT_DIMTYPE.);
      if (DFLT_MEMBER_CD = '') then do;
         if (HOLD_DFLT_CD = '') then do;
            call symput("_tmpMsngDfltMem", 1);
            call symput("_tmpHier", strip(HIERARCHY_CD));
            stop;
         end;
         else
            DFLT_MEMBER_CD = HOLD_DFLT_CD;
      end;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if (&_tmpMsngDfltMem) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=19
             , msgTxt1=&dfltmemCD, msgTxt2=<&_tmpHier>
             , msgTxt3=&assoctype, msgPutlog=&putlog);
     %goto exit ;
   %end;
%end;
%else %if (&dimtype eq INTORG) %then %do;
   %let _tmpDSN = &_lib..&prefix;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
      %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=26
              , msgTxt1=<&_tmpDSN>, msgPutlog=&putlog);
      %goto exit ;
   %end;

   data WORK.TEMPASSOC
       (keep=HIERARCHY_CD DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
             ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0)
          WORK.TEMPASSOC(where=((&dimParentid = 'ALL')
        and (&dimid ^= 'ALL') and (&dimid ^= 'EXT')
        and &_where1));
      HIERARCHY_CD = &assoctypecd;
      DFLT_MEMBER_CD = &dimid;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.TEMPASSOC NODUPKEY;
      by &byHierarchy;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = &_lib..&assoctype(where=(&_where1) rename=(&assoctypecd = HIERARCHY_CD))
              out = WORK._TMPDFLTMEM (keep=&byHierarchy &dfltmemcd)
              NODUPKEY;
      by &byHierarchy;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %let _tmpInvDfltMem = 0;
   %let _tmpMsngDfltMem = 0;

   data WORK.SAS_ETL_HIERARCHY_DFLT_MBR
      (keep=HIERARCHY_CD DIM_TYPE_ROLE DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
           ETL_INSTANCE_ID);

      merge _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0)
            WORK.TEMPASSOC
            WORK._TMPDFLTMEM;
         by HIERARCHY_CD;

      HOLD_DFLT_ID = &dfltmemcd;
      if (HOLD_DFLT_ID = 'ALL' or HOLD_DFLT_ID = 'EXT') then do;
         call symput("_tmpInvDfltMem", 1);
         if (HOLD_DFLT_ID = 'ALL') then
            call symput("_tmpDfltCode", 'ALL');
         else
            call symput("_tmpDfltCode", 'EXT');
         call symput("_tmpHier", strip(HIERARCHY_CD));
         stop;
      end;
      if (HOLD_DFLT_ID = '') then do;
         if (DFLT_MEMBER_CD = '') then do;
            call symput("_tmpMsngDfltMem", 1);
            call symput("_tmpHier", strip(HIERARCHY_CD));
            stop;
         end;
      end;
      else
         DFLT_MEMBER_CD = HOLD_DFLT_ID;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
      DIM_TYPE_ROLE = input(symgetc("dimtype"), FMT_DIMTYPE.);
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if (&_tmpInvDfltMem) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=18
             , msgTxt1=<&_tmpDfltCode>, msgTxt2=<&_tmpHier>
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   %if (&_tmpMsngDfltMem) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=19
             , msgTxt1=&dfltmemcd, msgTxt2=<&_tmpHier>
             , msgTxt3=&assoctype, msgPutlog=&putlog);
     %goto exit ;
   %end;

   data WORK.DEFMEMTEMP;
      set WORK.SAS_ETL_HIERARCHY_DFLT_MBR;
      DIM_TYPE_ROLE = input('TRADER', FMT_DIMTYPE.);
      DFLT_MEMBER_CD = 'ALL';
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.SAS_ETL_HIERARCHY_DFLT_MBR;
      set WORK.SAS_ETL_HIERARCHY_DFLT_MBR WORK.DEFMEMTEMP;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;
%else %do;

   data WORK.TEMPASSOC
       (keep=HIERARCHY_CD DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
             ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0)
          WORK.TEMPASSOC(where=((&dimid = &dimParentid)
            and &_where1));

      HIERARCHY_CD = &assoctypecd;
      DFLT_MEMBER_CD = &dimid;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = WORK.TEMPASSOC NODUPKEY;
      by &byHierarchy;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc sort data = &_lib..&assoctype(where=(&_where1) rename=(&assoctypecd = HIERARCHY_CD))
              out = WORK._TMPDFLTMEM (keep=&byHierarchy &dfltmemcd)
              NODUPKEY;
      by &byHierarchy;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %let _tmpMsngDfltMem = 0;

   data WORK.SAS_ETL_HIERARCHY_DFLT_MBR
      (keep=HIERARCHY_CD DIM_TYPE_ROLE DFLT_MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
           ETL_INSTANCE_ID);

      merge _SDMLIB.sas_etl_hierarchy_dflt_mbr(obs=0)
            WORK.TEMPASSOC
            WORK._TMPDFLTMEM;
         by HIERARCHY_CD;

      DIM_TYPE_ROLE = input(symgetc("dimtype"), FMT_DIMTYPE.);
      HOLD_DFLT_ID = &dfltmemcd;
      if (HOLD_DFLT_ID = '') then do;
         if (DFLT_MEMBER_CD = '') then do;
            call symput("_tmpMsngDfltMem", 1);
            call symput("_tmpHier", strip(HIERARCHY_CD));
            stop;
         end;
      end;
      else
         DFLT_MEMBER_CD = HOLD_DFLT_ID;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if (&_tmpMsngDfltMem) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=19
             , msgTxt1=&dfltmemcd, msgTxt2=<&_tmpHier>
             , msgTxt3=&assoctype, msgPutlog=&putlog);
     %goto exit ;
   %end;

%end;

/*************************************************************************************/
/*      Validate that the designated default member exists in the hierarchy          */
/*************************************************************************************/
proc sql noprint;
   select count(*) into :hierCnt from WORK.SAS_ETL_HIERARCHY;
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

%if (&hierCnt > 0) %then %do;
   %let _tmpMemNotFound = 0;
   %if ((&dimtype eq CURRENCY) or (&dimtype eq ITEMCAT)) %then %do;
     proc sql noprint;
       select 1, strip(HIERARCHY_CD), strip(DFLT_MEMBER_CD)
         into :_tmpMemNotFound, :_tmpHierCode, :_tmpMemCode
       from WORK.SAS_ETL_HIERARCHY_DFLT_MBR a
       where not exists(select b.CHILD_MEMBER_CD
                        from WORK.SAS_ETL_HIERARCHY_MEMBER b
                        where a.HIERARCHY_CD eq b.HIERARCHY_CD
                          and a.DFLT_MEMBER_CD eq b.CHILD_MEMBER_CD
                        )
       ;
     quit;
   %end;
   %else %do;
     proc sql noprint;
       select 1, strip(HIERARCHY_CD), strip(DFLT_MEMBER_CD)
         into :_tmpMemNotFound, :_tmpHierCode, :_tmpMemCode
       from WORK.SAS_ETL_HIERARCHY_DFLT_MBR a
       where not exists(select b.CHILD_MEMBER_CD
                        from WORK._STGDDS_ASSOC b
                        where a.HIERARCHY_CD eq b.&assoctypecd
                          and a.DFLT_MEMBER_CD eq b.CHILD_MEMBER_CD
                        )
       ;
     quit;
   %end;

  %if (&_tmpMemNotFound) %then %do;
    %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=etldim, msgCode=20
            , msgTxt1=<%cmpres(&_tmpMemCode)>, msgTxt2=<%cmpres(&_tmpHierCode)>
            , msgTxt3=&assoctype, msgTxt4=&assoc, msgPutlog=Y);
    %goto exit ;
  %end;
%end;

proc sort data = WORK.SAS_ETL_HIERARCHY_DFLT_MBR NODUPKEY;
   by &byHierarchy DIM_TYPE_ROLE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_HIERARCHY_DFLT_MBR data=WORK.SAS_ETL_HIERARCHY_DFLT_MBR &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/***********************************************************************************/
/*                                                                                 */
/*                       START OF MEMBER PROPERTY CODE                             */
/*                                                                                 */
/***********************************************************************************/
data WORK.SAS_ETL_PROPERTY;
   set _SDMLIB.sas_etl_property(obs=0 dbmax_text=32767);
   stop;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.SAS_ETL_MEMBER_PROPERTY;
   set _SDMLIB.sas_etl_member_property(obs=0);
   stop;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/* Check if _StgDDS.APP_MEMBER_PROPERTY_MAP exists */
%let _tmpDSN = &_lib..APP_MEMBER_PROPERTY_MAP;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=34
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgTxt1=app_member_property_map, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpWhere=where=(DIMENSION_TYPE_CD = "&dimtype" and &_where2);
%let _tmpPropCd=PROPERTY_CD;

proc sort data = &_tmpDSN(&_tmpWhere
                 keep=&_tmpPropCd TABLE_NM COLUMN_NM DIMENSION_TYPE_CD
                      VALID_TO_DTTM)
           out = WORK._TMPDIMPROPS(drop = DIMENSION_TYPE_CD VALID_TO_DTTM)
           NODUPKEY;
   by &_tmpPropCd;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sql noprint;
   select count(*) into :fmtCnt from WORK._TMPDIMPROPS;
quit;
%checkErr;
%if &_tmpRC %then %goto exit;

%if (&fmtCnt > 0) %then %do;

   /* Verify that no duplicate codes exist in the stage property map table  */
   %let _tmpWhereDups=where=(&_where2);
   %_validateDups(_param=PROPERTY_CD, _ds=&_tmpDSN, _where=&_tmpWhereDups,
                  _cols=PROPERTY_CD VALID_FROM_DTTM VALID_TO_DTTM DIMENSION_TYPE_CD,
                  _byVars=PROPERTY_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;

   /* Verify that no duplicate codes exist in the stage property table  */
   %let _tmpWhereDups=where=(&_where2);
   %_validateDups(_param=PROPERTY_CD, _ds=&_lib..APP_PROPERTY, _where=&_tmpWhereDups,
                   _cols=PROPERTY_CD VALID_FROM_DTTM VALID_TO_DTTM LANGUAGE_CD,
                   _byVars=PROPERTY_CD LANGUAGE_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;
   /* Verify duplicate codes end */

   proc sort data = WORK._TMPDIMPROPS;
      by &_tmpPropCd;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   proc sort data = &_lib..APP_PROPERTY
              out = WORK._TMPAPPPROPS;
      by LANGUAGE_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   proc sort data = WORK._STG_SDM_LOCALE_MAP;
      by LANGUAGE_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   data WORK._TMPAPPPROPS
       (keep=PROPERTY_CD LANGUAGE_CD VALID_FROM_DTTM VALID_TO_DTTM PROPERTY_DESC
           PROPERTY_NM PROPERTY_TYPE_NM ENUM_VALUES MIN_VALUE MAX_VALUE PROPERTY_OPTIONS);

      merge WORK._TMPAPPPROPS (in = inA)
            WORK._STG_SDM_LOCALE_MAP (in = inB);
         by LANGUAGE_CD;
      if (inA and inB) then
         output;
   run;

   proc sort data = WORK._TMPAPPPROPS;
      by &_tmpPropCd;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   %let _tmpReservedProp = 0;
   %let _tmpInvPropType = 0;
   %let _tmpWrongDateValue = 0;
   data WORK._TMPUPDPROP
       (keep= ETL_INSTANCE_ID PROPERTY_CD PROPERTY_NM PROPERTY_DESC PROPERTY_TYPE_CD
           LOCALE_CD LOCALE_ID ENUM_VALUES MIN_VALUE MAX_VALUE PROPERTY_OPTIONS);

      merge _SDMLIB.sas_etl_property(obs=0 dbmax_text=32767)
            WORK._TMPDIMPROPS (in = inA)
            WORK._TMPAPPPROPS;
         by &_tmpPropCd;

      ETL_INSTANCE_ID = symgetn("soletl_instanceId");

      /* Need to check that user-defined property codes don't clash with the required */
      /* property codes.                                                              */
      CODE_TEST = upcase(PROPERTY_CD);
      if (CODE_TEST in ('ACCOUNTTYPE','ACCOUNTBEHAVIOR','ENDDATE','BASISDATA',
                        'ROLLFORWARDMETHOD','TOTALAFTERIMPORT','INTERCOMPANY',
                        'REPORTINGENTITY','LEVEL','SOURCEACCOUNTS','BALANCETYPE',
                        'FUNCTIONALCURRENCY','STARTDATE','EXCHANGERATETYPE',
                        'FORMULAPRECEDENCE')) then do;
         call symput("_tmpReservedProp", 1);
         call symput("_tmpProp", strip(PROPERTY_CD));
         stop;
      end;

      if (PROPERTY_TYPE_NM = 'none') then
         PROPERTY_TYPE_CD = '1';
      else if (PROPERTY_TYPE_NM = 'boolean') then
         PROPERTY_TYPE_CD = '2';
      else if (PROPERTY_TYPE_NM = 'integer') then
         PROPERTY_TYPE_CD = '3';
      else if (PROPERTY_TYPE_NM = 'string') then
         PROPERTY_TYPE_CD = '4';
      else if (PROPERTY_TYPE_NM = 'double') then
         PROPERTY_TYPE_CD = '5';
      else if (PROPERTY_TYPE_NM = 'date') then
         PROPERTY_TYPE_CD = '6';
      else do;
         call symput("_tmpInvPropType", 1);
         call symput("_tmpProp", strip(PROPERTY_CD));
         stop;
      end;
      LOCALE_ID = input(LANGUAGE_CD, FMT_LOCALE.);

      if MIN_VALUE = . then
        MIN_VALUE = symget('soletl_noInput');
      else if (PROPERTY_TYPE_NM = 'date'
                        and MIN_VALUE ne symget('soletl_noInput') ) then do;
        TMP_MIN_VALUE = MIN_VALUE;
        MIN_VALUE = input(put(TMP_MIN_VALUE, yymmddn8.), best8.);
        call symput("_tmpMinValue", strip(MIN_VALUE));
        if MIN_VALUE eq . or TMP_MIN_VALUE < "01JAN1600"d then do;
          call symput("_tmpWrongDateValue", 1);
          call symput("_tmpProp", strip(PROPERTY_CD));
        end;
      end;

      if MAX_VALUE = . then
         MAX_VALUE = symget('soletl_noInput');
      else if (PROPERTY_TYPE_NM = 'date'
                        and MAX_VALUE ne symget('soletl_noInput') ) then do;
        TMP_MAX_VALUE = MAX_VALUE;
        MAX_VALUE = input(put(TMP_MAX_VALUE, yymmddn8.), best8.);
        call symput("_tmpMaxValue", strip(MAX_VALUE));
        if MAX_VALUE eq . or TMP_MAX_VALUE > "01JAN5999"d then do;
          call symput("_tmpWrongDateValue", 1);
          call symput("_tmpProp", strip(PROPERTY_CD));
        end;
      end;

      if PROPERTY_OPTIONS = . then
         PROPERTY_OPTIONS = 0;

      if symget("_tmpWrongDateValue") eq 1 then
        stop;

      if (inA) then
         output;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   %if (&_tmpReservedProp) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=21
             , msgTxt1=<&_tmpProp>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %if (&_tmpInvPropType) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=48
             , msgTxt1=<&_tmpProp>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %if (&_tmpWrongDateValue) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=60
             , msgTxt1=<&_tmpProp>, msgTxt2=<&_tmpMinValue>
             , msgTxt3=<&_tmpMaxValue>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   data WORK.SAS_ETL_PROPERTY;
      set WORK.SAS_ETL_PROPERTY
          WORK._TMPUPDPROP;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   proc sort data = WORK._TMPAPPPROPS
              out = WORK._TMPAPPPROPS1
              nodupkey;
      by &_tmpPropCd;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit;

   %let _tmpFormat=FMT_PROPTYPE;

   data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE);
      retain FMTNAME "&_tmpFormat";
      retain TYPE "C";
      retain HLO "";
      set WORK._TMPAPPPROPS1(rename=(&_tmpPropCd=START PROPERTY_TYPE_NM=LABEL)) end=last;
      output;
      if last then do;
         START = 0;
         HLO = 'O';
         LABEL = '*** ERROR ***';
         output;
      end;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc format cntlin=work.&_tmpFormat library=WORK;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
             , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit ;
   %end;

%end;

%macro load_mp (_tmpProp=, _tmpTable=, _tmpColumn=);

%let _fmt = $FMT_;

data WORK._TMPUPDMP
   (keep=ETL_INSTANCE_ID PROPERTY_CD MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM
          PROPERTY_VALUE LOCALE_ID);

   set _SDMLIB.sas_etl_member_property(obs=0)
       &_lib..&_tmpTable(where=(&_where1));

   ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   PROPERTY_CD = symgetc("_tmpProp");
   MEMBER_CD = &dimid;
   if VALID_FROM_DTTM = . then
      VALID_FROM_DTTM = &soletl_processedDTTM.;
   if VALID_TO_DTTM = . then
      VALID_TO_DTTM = &curr_rec_dttm.;
   PROPERTY_VALUE = &_tmpColumn;

   _error=0;
   if put(PROPERTY_CD, $FMT_PROPTYPE.) ne 'string' then do;
     if strip(PROPERTY_VALUE) ne "." then do;

       if put(PROPERTY_CD, $FMT_PROPTYPE.) eq 'boolean' then do;
         tmpPROPERTY_VALUE = PROPERTY_VALUE;
         PROPERTY_VALUE = propcase(PROPERTY_VALUE);
         if PROPERTY_VALUE not in("False" "True" "") then
           _error = 55;
       end;
       else if put(PROPERTY_CD, $FMT_PROPTYPE.) eq 'date' then do;
         tmpPROPERTY_VALUE = PROPERTY_VALUE;
         PROPERTY_VALUE = strip(put(input(strip(PROPERTY_VALUE), best26.), yymmddn8.));

         if PROPERTY_VALUE eq "."
            or (input(strip(PROPERTY_VALUE), best26.) < 16000101
            or  input(strip(PROPERTY_VALUE), best26.) > 59990101) then do;
              tmpPROPERTY_VALUE = quote(strip(PROPERTY_VALUE))
                         !! ' (' !! strip(tmpPROPERTY_VALUE) !! ')' ;
              _error = 56;
         end;
       end;
       else if put(PROPERTY_CD, $FMT_PROPTYPE.) eq 'integer' then do;
         tmpPROPERTY_VALUE = PROPERTY_VALUE;
         PROPERTY_VALUE = strip(put(input(strip(PROPERTY_VALUE), best26.), best26.));

         /* Integer range is from -2147483648 to 2147483647, but we're using min_value
            -2147483648 to test for existing MPVs when exporting so we test for min_value
            -2147483647 instead when importing.
          */
         if PROPERTY_VALUE eq "."
            or (input(strip(PROPERTY_VALUE), best26.) < -2147483647
            or  input(strip(PROPERTY_VALUE), best26.) > 2147483647) then
           _error = 57;

       end;
       else if put(PROPERTY_CD, $FMT_PROPTYPE.) eq 'double' then do;
         tmpPROPERTY_VALUE = PROPERTY_VALUE;
         PROPERTY_VALUE = strip(put(input(strip(PROPERTY_VALUE), best26.), best26.));

         if PROPERTY_VALUE eq "." then
           _error = 58;
       end;

     end;
   end;

   LOCALE_ID = 0;

   call symput('_error', _error);
   if put(PROPERTY_CD, $FMT_PROPTYPE.) eq 'string' or _error eq 0 then
     output;
   else if _error ne 0 then do;
     call symput('_tmpPropValue', strip(tmpPROPERTY_VALUE));
     call symput('_tmpMemberCd', strip(MEMBER_CD));
     stop;
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sort data = WORK._TMPUPDMP NODUPKEY;
   by PROPERTY_CD MEMBER_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit;

proc append base=WORK.SAS_ETL_MEMBER_PROPERTY
            data=WORK._TMPUPDMP &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%exit:

%mend load_mp;

%let ERROR = %str();
%do i = 1 %to &fmtCnt;
   %let _error=0;
   data _null_;
      set _tmpDimProps (firstobs = &i obs = &i);
      put PROPERTY_CD=;
      call symput("_tmpProp", strip(PROPERTY_CD));
      call symput("_tmpTable", strip(TABLE_NM));
      call symput("_tmpColumn", strip(COLUMN_NM));
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %let _tmpPropValue=;
   %let _tmpMemberCd=;
   %load_mp(_tmpProp=&_tmpProp, _tmpTable=&_tmpTable, _tmpColumn=&_tmpColumn);
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if &_error ne 0 %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&_error
             , msgTxt1=<&_tmpProp>, msgTxt2=<&_tmpPropValue>
             , msgTxt3=<&_tmpMemberCd>, msgPutlog=&putlog);
     %let ERROR=Error;
   %end;
%end;

%if %str(&ERROR) ne %str() %then %do;
  %goto exit ;
%end;

/* Load the special case member_property records for the imported dimension */
%if (&dimtype eq ACCOUNT) %then %do;
   /* MEMBER PROPERTY RECORDS FOR ACCOUNT */

   /* Create format FMT_ACCT_TYPE */
   %let _tmpDSN = &_lib..GL_ACCOUNT;
   %let _tmpFormat=FMT_ACCT_TYPE;
   %let _tmpStart=GL_ACCOUNT_ID;
   %let _tmpLabel=GL_ACCOUNT_TYPE_CD;
   %let _tmpWhere=where=(&_where2);
   proc sort data = &_tmpDSN(&_tmpWhere
                  keep=&_tmpStart &_tmpLabel VALID_FROM_DTTM VALID_TO_DTTM)
              out = work._tmpSort(drop = VALID_FROM_DTTM VALID_TO_DTTM)
              NODUPKEY;
      by &_tmpStart;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data work.&_tmpFormat(keep=START LABEL FMTNAME TYPE);
      retain FMTNAME "&_tmpFormat";
      retain TYPE "C";
      set work._tmpSort(rename=(&_tmpStart=START &_tmpLabel=LABEL));
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc format cntlin=work.&_tmpFormat library=work;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=12
             , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %let _tmpDSN = &_lib..GL_ACCOUNT;
   %let _tmpWhere=where=(&_where1);

   %let _tmpBasisErr = 0;
   %let _tmpCTAErr = 0;
   %let _tmpXrateErr = 0;
   %let _tmpATypeErr = 0;

   data WORK._TMP_ETL_MP1
      (keep=PROPERTY_CD MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM PROPERTY_VALUE LOCALE_ID
           ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_member_property(obs=0)
          &_tmpDSN(&_tmpWhere);

      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      LOCALE_ID = 0;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");

      /* BasisData */
      if (GL_ACCOUNT_TYPE_CD = 'RetainedEarnings') then do;
         PROPERTY_CD = 'BasisData';
         MEMBER_CD = GL_ACCOUNT_ID;
         TYPE_CNT = 0;
         if (RETAINED_EARN_IMPORT_DATA_FLG = 'Y') then do;
            PROPERTY_VALUE = 'TotalAfterImport';
            TYPE_CNT = TYPE_CNT + 1;
         end;
         if (RETAINED_EARN_FORM_DATA_FLG = 'Y') then do;
            if (TYPE_CNT > 0) then
               PROPERTY_VALUE = strip(PROPERTY_VALUE) || ',BaseForm';
            else
               PROPERTY_VALUE = 'BaseForm';
            TYPE_CNT = TYPE_CNT + 1;
         end;
         if (RETAINED_EARN_ADJ_FLG = 'Y') then do;
            if (TYPE_CNT > 0) then
               PROPERTY_VALUE = strip(PROPERTY_VALUE) || ',Adj';
            else
               PROPERTY_VALUE = 'Adj';
            TYPE_CNT = TYPE_CNT + 1;
         end;
         if (RETAINED_EARN_ELIM_ADJ_FLG = 'Y') then do;
            if (TYPE_CNT > 0) then
               PROPERTY_VALUE = strip(PROPERTY_VALUE) || ',Elim';
            else
               PROPERTY_VALUE = 'Elim';
            TYPE_CNT = TYPE_CNT + 1;
         end;
         output;
      end;

      /* AccountType */
      PROPERTY_CD = 'AccountType';
      MEMBER_CD = GL_ACCOUNT_ID;
      if (GL_ACCOUNT_TYPE_CD not in ('Asset', 'CTA', 'Equity', 'Expense', 'Liability',
                                     'RetainedEarnings', 'Revenue', 'StatisticalBalance',
                                     'StatisticalFlow', 'NonFrequency')) then do;
         call symput("_tmpATypeErr", 1);
         call symput("_tmpErrCd", strip(MEMBER_CD));
         stop;
      end;
      PROPERTY_VALUE = GL_ACCOUNT_TYPE_CD;
      output;

      /* AccountBehavior */
      PROPERTY_CD = 'AccountBehavior';
      MEMBER_CD = GL_ACCOUNT_ID;
      PROPERTY_VALUE = put(GL_ACCOUNT_ID, $FMT_BEHAVIOR.);
      output;

      /* RollForwardMethod */
      if (GL_ACCOUNT_TYPE_CD = 'RetainedEarnings') then do;
         PROPERTY_CD = 'RollForwardMethod';
         MEMBER_CD = GL_ACCOUNT_ID;
         if (RETAINED_EARN_ROLL_FWD_CD = '') then
            PROPERTY_VALUE = 'YTD';
         else
            PROPERTY_VALUE = RETAINED_EARN_ROLL_FWD_CD;
         output;
      end;

      /* Intercompany */
      PROPERTY_CD = 'Intercompany';
      MEMBER_CD = GL_ACCOUNT_ID;
      If (INTERCOMPANY_ACCOUNT_FLG = 'Y') then
         PROPERTY_VALUE = 'True';
      else
         PROPERTY_VALUE = 'False';
      output;

      /* BalanceType */
      PROPERTY_CD = 'BalanceType';
      MEMBER_CD = GL_ACCOUNT_ID;
      if (NORMAL_BALANCE_CD = 'D') then
         PROPERTY_VALUE = 'Debit';
      else
         PROPERTY_VALUE = 'Credit';
      output;

      /* ExchangeRateType */

      /* Need to verify that an exchange rate type is defined for all accounts */

      PROPERTY_CD = 'ExchangeRateType';
      MEMBER_CD = GL_ACCOUNT_ID;

      if (GL_ACCOUNT_TYPE_CD = 'CTA' and EXCHANGE_RATE_TYPE_CD ne 'PeriodClose') then do;
         call symput("_tmpCTAErr", 1);
         call symput("_tmpErrCd", strip(MEMBER_CD));
         call symput("_tmpXrateType", strip(EXCHANGE_RATE_TYPE_CD));
         stop;
      end;

      if (GL_ACCOUNT_TYPE_CD = 'RetainedEarnings') then
         PROPERTY_VALUE = 'None';
      else
         PROPERTY_VALUE = EXCHANGE_RATE_TYPE_CD;
      output;

      if (PROPERTY_VALUE not in ('None', 'Historic', 'Derived', 'PeriodAverage',
                                 'PeriodClose', 'PeriodOpen', 'Custom1', 'Custom2')) then do;
         call symput("_tmpXrateErr", 1);
         call symput("_tmpErrCd", strip(MEMBER_CD));
         stop;
      end;

      /* CTAElimBehavior */
      PROPERTY_CD = 'CTAElimBehavior';
      MEMBER_CD = GL_ACCOUNT_ID;

      if GL_ACCOUNT_TYPE_CD = 'CTA' then do;
         if CTA_ELIM_BEHAVIOR_CD = 'ReportingCurrency' then do;
            PROPERTY_VALUE = 'ReportingCurrency';
         end;
         else do;
            PROPERTY_VALUE = 'TransactionCurrency';
         end;
         output;
      end;

   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if (&_tmpATypeErr) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=45
             , msgTxt1=<&_tmpErrCd>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %if (&_tmpCTAErr) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=49
             , msgTxt1=<&_tmpErrCd>, msgTxt2=&_tmpXrateType, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %if (&_tmpXrateErr) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=23
             , msgTxt1=<&_tmpErrCd>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   /* Check if _StgDDS.SOURCE_GL_ACCOUNT exists */
   %let _tmpDSN = &_lib..SOURCE_GL_ACCOUNT;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=36
             , msgPutlog=N);
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
             , msgTxt1=source_gl_account, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   %let _tmpInvAccount = 0;
   %let _tmpInvSrcAcct = 0;

   data WORK._TMP_ETL_MP2
      (keep=PROPERTY_CD MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM PROPERTY_VALUE LOCALE_ID
           ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_member_property(obs=0)
          &_tmpDSN(where=(&_where1));

      /* SourceAccounts */
      PROPERTY_CD = 'SourceAccounts';
      MEMBER_CD = GL_ACCOUNT_ID;
      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      PROPERTY_VALUE = SOURCE_GL_ACCOUNT_ID;
      LOCALE_ID = 0;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");

      ASSOC_ACCT_TYPE = put(GL_ACCOUNT_ID, $FMT_ACCT_TYPE.);
      if ASSOC_ACCT_TYPE = 'StatisticalBalance' then do;
         call symput("_tmpInvAccount", 1);
         call symput("_tmpAcctCd", strip(MEMBER_CD));
         stop;
      end;
      SOURCE_ACCT_TYPE = put(SOURCE_GL_ACCOUNT_ID, $FMT_ACCT_TYPE.);
      if ASSOC_ACCT_TYPE = 'CTA' and
         SOURCE_ACCT_TYPE in ('Revenue','Expense') then do;
         call symput("_tmpInvSrcAcct", 1);
         call symput("_tmpAcctType", strip(SOURCE_ACCT_TYPE));
         call symput("_tmpSrcAcctCd", strip(PROPERTY_VALUE));
         call symput("_tmpAcctCd", strip(MEMBER_CD));
         stop;
      end;

   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if (&_tmpInvAccount) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=50
             , msgTxt1=<&_tmpAcctCd>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %if (&_tmpInvSrcAcct) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=51
             , msgTxt1=&_tmpAcctType, msgTxt2=<&_tmpSrcAcctCd>, msgTxt3=<&_tmpAcctCd>,
               msgPutlog=&putlog);
     %goto exit ;
   %end;

   data WORK._TMPUPDMP;
      set WORK._TMP_ETL_MP1
          WORK._TMP_ETL_MP2;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.SAS_ETL_MEMBER_PROPERTY;
      set WORK.SAS_ETL_MEMBER_PROPERTY
          WORK._TMPUPDMP;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;
%else %if (&dimtype eq INTORG) %then %do;

   /* Create format FMT_CURRENCY */
   proc sql noprint;
      select DIM_TYPE_ID into :dim_type_id from _SDMLIB.SAS_DIM_TYPE
      where DIM_TYPE_CD = 'CURRENCY';
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   /* Check if _SDMLIB.SAS_MEMBER exists */
   %let _tmpDSN = _SDMLIB.SAS_MEMBER;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=37
             , msgPutlog=N);
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
             , msgTxt1=member, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
   %end;
   proc sort data = _SDMLIB.SAS_MEMBER out = WORK._SDMLIB_MEMBER;
      by INTERNAL_DIMENSION_ID;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.CURRENCY_DIMS(keep=INTERNAL_DIMENSION_ID);
      set _SDMLIB.SAS_DIMENSION;
      if (DIM_TYPE_ID = &dim_type_id) then
         output;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   proc sort data = WORK.CURRENCY_DIMS;
      by INTERNAL_DIMENSION_ID;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.CURRENCY_MEMS(keep=MEMBER_CD MEMBER_ID VALID_FROM_DTTM VALID_TO_DTTM);
      merge WORK._SDMLIB_MEMBER
            WORK.CURRENCY_DIMS (in = inB);
         by INTERNAL_DIMENSION_ID;
      if (inB) then
         output;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %let _tmpDSN=WORK.CURRENCY_MEMS;
   %let _tmpFormat=FMT_CURRENCY;
   %let _tmpStart=MEMBER_CD;
   %let _tmpLabel=MEMBER_ID;
   %let _tmpWhere=where=(&_where1);
   proc sort data = &_tmpDSN(&_tmpWhere
                    keep=&_tmpStart &_tmpLabel VALID_FROM_DTTM VALID_TO_DTTM)
              out = WORK._TMPSORT(drop = VALID_FROM_DTTM VALID_TO_DTTM)
              NODUPKEY;
      by &_tmpStart;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
      retain FMTNAME "&_tmpFormat";
      retain TYPE "I";
      retain HLO "";
      set WORK._TMPSORT(rename=(&_tmpStart=START &_tmpLabel=LABEL)) end=last;
      output;
      if (last) then do;
         HLO = 'O';
         START = .;
         LABEL = .;
         output;
      end;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc format cntlin=WORK.&_tmpFormat library=WORK;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12
             , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %let _tmpDSN = &_lib..INTERNAL_ORG;

   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=26
             , msgTxt1=<&_tmpDSN>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   %let _tmpInvFC = 0;

   /* MEMBER_PROPERTY RECORDS FOR ORG */
   data WORK._TMPUPDMP
      (keep=PROPERTY_CD MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM PROPERTY_VALUE
           LOCALE_ID ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_member_property(obs=0)
          &_tmpDSN(where=(&_where1));

      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      LOCALE_ID = 0;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");

      /* FunctionalCurrency */
      PROPERTY_CD = 'FunctionalCurrency';
      MEMBER_CD = INTERNAL_ORG_ID;
      if (REPORTING_CURRENCY_CD = '') and (MEMBER_CD = 'ALL' or MEMBER_CD = 'EXT') then
         PROPERTY_VALUE = '';
      else do;
         PROPERTY_VALUE = strip(input(REPORTING_CURRENCY_CD, FMT_CURRENCY.));
         if (PROPERTY_VALUE = .) then do;
            call symput("_tmpInvFC", 1);
            call symput("_tmpFCMem", strip(MEMBER_CD));
            call symput("_tmpFCCurr", strip(REPORTING_CURRENCY_CD));
            stop;
         end;
      end;
      output;

      /* ReportingEntity */
      PROPERTY_CD = 'ReportingEntity';
      MEMBER_CD = INTERNAL_ORG_ID;
      if  (LEGAL_ENTITY_FLG = 'Y') and (INTERNAL_ORG_ID ^= 'ALL') and (EXTERNAL_TRADER_FLG ^= 'Y') then
         PROPERTY_VALUE = 'True';
      else
         PROPERTY_VALUE = 'False';
      output;

   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   %if (&_tmpInvFC) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=25
             , msgTxt1=<&_tmpFCCurr>, msgTxt2=<&_tmpFCMem>
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   data WORK.SAS_ETL_MEMBER_PROPERTY;
      set WORK.SAS_ETL_MEMBER_PROPERTY
          WORK._TMPUPDMP;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;
%else %if (&dimtype eq TIME) %then %do;

   /* Check if _StgDDS.TIME_PERIOD exists */
   %let _tmpDSN = &_lib..TIME_PERIOD;
   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=26
             , msgTxt1=<&_tmpDSN>, msgPutlog=&putlog);
     %goto exit ;
   %end;

   /* MEMBER_PROPERTY RECORDS FOR TIME */
   data WORK._TMPUPDMP
      (keep=PROPERTY_CD MEMBER_CD VALID_FROM_DTTM VALID_TO_DTTM PROPERTY_VALUE
           LOCALE_ID ETL_INSTANCE_ID);

      set _SDMLIB.sas_etl_member_property(obs=0)
          &_tmpDSN(where=(&_where1));

      if VALID_FROM_DTTM = . then
         VALID_FROM_DTTM = &soletl_processedDTTM.;
      if VALID_TO_DTTM = . then
         VALID_TO_DTTM = &curr_rec_dttm.;
      LOCALE_ID = 0;
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
      FORMAT VALID_FROM_DTTM;
      FORMAT VALID_TO_DTTM;

      /* Level */
      PROPERTY_CD = 'Level';
      MEMBER_CD = TIME_PERIOD_ID;
      if (PERIOD_TYPE_CD = 'ALL') then
         PROPERTY_VALUE = 'AllYears';
      else if (PERIOD_TYPE_CD = 'YR') then
         PROPERTY_VALUE = 'Year';
      else if (PERIOD_TYPE_CD = 'HYR') then
         PROPERTY_VALUE = 'HalfYear';
      else if (PERIOD_TYPE_CD = 'QTR') then
         PROPERTY_VALUE= 'QuarterYear';
      else if (PERIOD_TYPE_CD in ('MO','MTH')) then
         PROPERTY_VALUE = 'Month';
      else if (PERIOD_TYPE_CD = 'WK') then
         PROPERTY_VALUE = 'Week';
      else if (PERIOD_TYPE_CD = 'DAY') then
         PROPERTY_VALUE = 'Day';
      output;

      /* StartDate */
      PROPERTY_CD = 'StartDate';
      MEMBER_CD = TIME_PERIOD_ID;
      PROPERTY_VALUE = put(datepart(START_DTTM),yymmddn8.);
      output;

      /* EndDate */
      PROPERTY_CD = 'EndDate';
      MEMBER_CD = TIME_PERIOD_ID;
      PROPERTY_VALUE = put(datepart(END_DTTM),yymmddn8.);
      output;

   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.SAS_ETL_MEMBER_PROPERTY;
      set WORK.SAS_ETL_MEMBER_PROPERTY
          WORK._TMPUPDMP;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;

proc append base=_SDMLIB.SAS_ETL_PROPERTY(dbmax_text=32767) data=WORK.SAS_ETL_PROPERTY &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_MEMBER_PROPERTY data=WORK.SAS_ETL_MEMBER_PROPERTY &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/*              Import User_X_Member Records for the dimension members               */
/*************************************************************************************/
%let _where4=(DIMENSION_TYPE_CD = "&dimtype");
%let _tmpWhereDups=where=((upcase(DIMENSION_TYPE_CD) = upcase("&dimtype")));

%if (&soletl_includeUserData = 1) and (&dimtype ne ANALYSIS) and (&dimtype ne CURRENCY) and
    (&dimtype ne TIME) %then %do;
   %let _tmpDSN = &_lib..APP_USER_X_MEMBER;

   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
             , msgTxt1=&_tmpDSN, msgTxt2=''
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   /* Verify that no duplicate codes exist in the stage user x member table  */
   %_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
                   _where=&_tmpWhereDups,
                   _cols=DIMENSION_TYPE_CD, _byVars=DIMENSION_TYPE_CD original,
                   _byVars1=DIMENSION_TYPE_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;

   %_validateDups(_param=DIMENSION_MEMBER_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
                   _where=&_tmpWhereDups,
                   _cols=DIMENSION_TYPE_CD,
                   _byVars=DIMENSION_MEMBER_CD original,
                   _byVars1=DIMENSION_MEMBER_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;
   /* Verify duplicate codes end */

   data _TMP_USER_X_MEMBER;
      set &_tmpDSN(where=(&_where4));
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   data WORK.SAS_ETL_USER_X_MEMBER
      (keep=ETL_INSTANCE_ID USER_NM DIM_TYPE_CD DIMENSION_CD MEMBER_CD SOURCE_SYSTEM_CD);

      set _SDMLIB.sas_etl_user_x_member(obs=0)
          WORK._TMP_USER_X_MEMBER;

      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
      DIM_TYPE_CD = "&dimtype";
      MEMBER_CD = DIMENSION_MEMBER_CD;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc append base=_SDMLIB.SAS_ETL_USER_X_MEMBER data=WORK.SAS_ETL_USER_X_MEMBER &FM_PROCAPPENDFORCE;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;

/*************************************************************************************/
/*              Import User Actions records for                                      */
/*                 1) dimension type                                                 */
/*                 2) dimension                                                      */
/*                 3) dimension hieracrhies                                          */
/*                 4) dimension members                                              */
/*                 5) custom member properties                                       */
/*************************************************************************************/
%if (&soletl_includeSecurityData = 1) %then %do;
   %let _tmpDSN = &_lib..APP_USER_ACTIONS;

   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
             , msgTxt1=&_tmpDSN, msgTxt2=''
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   /* Verify that no duplicate codes exist in the stage user actions table  */
   %_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
               _where=&_tmpWhereDups,
               _cols=DIMENSION_TYPE_CD, _byVars=DIMENSION_TYPE_CD original,
               _byVars1=DIMENSION_TYPE_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;

   %_validateDups(_param=OBJECT_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
               _where=&_tmpWhereDups,
               _cols=OBJECT_CD DIMENSION_TYPE_CD OBJECT_TYPE_CD,
               _byVars=OBJECT_TYPE_CD OBJECT_CD original,
               _byVars1=OBJECT_TYPE_CD OBJECT_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;
   /* Verify duplicate codes end */

   data WORK.SAS_ETL_USER_ACTIONS(keep=ETL_INSTANCE_ID--UPDATEDATA_FLG);
      set _SDMLIB.SAS_ETL_USER_ACTIONS(obs=0)
           &_tmpDSN(where=(&_where4)
             rename=(
              CREATE_FLG=CREATE_FLG_SAS
              READ_METADATA_FLG=READ_METADATA_FLG_SAS
              UPDATE_METADATA_FLG=UPDATE_METADATA_FLG_SAS
              DELETE_FLG=DELETE_FLG_SAS
              NEGATE_FLG=NEGATE_FLG_SAS
              ADMIN_FLG=ADMIN_FLG_SAS
              READ_DATA_FLG=READ_DATA_FLG_SAS
              UPDATE_DATA_FLG=UPDATE_DATA_FLG_SAS
                   )
             );
      array flg{*} 8 create_flg readmetadata_flg updatemetadata_flg delete_flg
                     negate_flg admin_flg readdata_flg updatedata_flg;
      array flg_sas{*} $1 create_flg_sas read_metadata_flg_sas update_metadata_flg_sas
                          delete_flg_sas negate_flg_sas admin_flg_sas
                          read_data_flg_sas update_data_flg_sas;

      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
      DIM_TYPE_CD = "&dimtype";

      do i = 1 to dim(flg_sas);
        if flg_sas(i) = 'Y' then
          flg(i) = 1;
        else
          flg(i) = 0;
      end;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc append base=_SDMLIB.SAS_ETL_USER_ACTIONS data=WORK.SAS_ETL_USER_ACTIONS &FM_PROCAPPENDFORCE;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;

/*************************************************************************************/
/*              Import Group Actions Records for                                     */
/*                 1) dimension type                                                 */
/*                 2) dimension                                                      */
/*                 3) dimension hieracrhies                                          */
/*                 4) dimension members                                              */
/*                 5) custom member properties                                       */
/*************************************************************************************/
%if (&soletl_includeSecurityData = 1) %then %do;
   %let _tmpDSN = &_lib..APP_GROUP_ACTIONS;

   %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
             , msgTxt1=&_tmpDSN, msgTxt2=''
             , msgPutlog=&putlog);
     %goto exit ;
   %end;

   /* Verify that no duplicate codes exist in the stage group actions table  */
   %_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
               _where=&_tmpWhereDups,
               _cols=DIMENSION_TYPE_CD, _byVars=DIMENSION_TYPE_CD original,
               _byVars1=DIMENSION_TYPE_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;

   %_validateDups(_param=OBJECT_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
               _where=&_tmpWhereDups,
               _cols=OBJECT_CD DIMENSION_TYPE_CD OBJECT_TYPE_CD,
               _byVars=OBJECT_TYPE_CD OBJECT_CD original,
               _byVars1=OBJECT_TYPE_CD OBJECT_CD);

   %if (&_dupsFound eq 1) %then %goto exit ;
   /* Verify duplicate codes end */

   data WORK.SAS_ETL_GROUP_ACTIONS(keep=ETL_INSTANCE_ID--UPDATEDATA_FLG);
      set _SDMLIB.SAS_ETL_GROUP_ACTIONS(obs=0)
           &_tmpDSN(where=(&_where4)
             rename=(
              CREATE_FLG=CREATE_FLG_SAS
              READ_METADATA_FLG=READ_METADATA_FLG_SAS
              UPDATE_METADATA_FLG=UPDATE_METADATA_FLG_SAS
              DELETE_FLG=DELETE_FLG_SAS
              NEGATE_FLG=NEGATE_FLG_SAS
              ADMIN_FLG=ADMIN_FLG_SAS
              READ_DATA_FLG=READ_DATA_FLG_SAS
              UPDATE_DATA_FLG=UPDATE_DATA_FLG_SAS
                   )
             );
      array flg{*} 8 create_flg readmetadata_flg updatemetadata_flg delete_flg
                     negate_flg admin_flg readdata_flg updatedata_flg;
      array flg_sas{*} $1 create_flg_sas read_metadata_flg_sas update_metadata_flg_sas
                          delete_flg_sas negate_flg_sas admin_flg_sas
                          read_data_flg_sas update_data_flg_sas;

      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
      DIM_TYPE_CD = "&dimtype";

      do i = 1 to dim(flg_sas);
        if flg_sas(i) = 'Y' then
          flg(i) = 1;
        else
          flg(i) = 0;
      end;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

   proc append base=_SDMLIB.SAS_ETL_GROUP_ACTIONS data=WORK.SAS_ETL_GROUP_ACTIONS &FM_PROCAPPENDFORCE;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;

%end;

/*************************************************************************************/
/*               _STGDDS.APP_FORMULA_TARGET ==> _SDMLIB.SAS_ETL_FORMULA               */
/*************************************************************************************/
%let _tmpDSN = &_lib..APP_FORMULA_TARGET;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgTxt1=&_tmpDSN, msgTxt2=
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _where5=(DIMENSION_TYPE_CD = "&dimtype") and &_where1;

/* Verify that no duplicate codes exist in the stage formula table  */
%let _tmpWhereDups=where=((upcase(DIMENSION_TYPE_CD) = upcase("&dimtype")) and &_where1);

%_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
                _where=&_tmpWhereDups,
                _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                _byVars=DIMENSION_TYPE_CD original,
                _byVars1=DIMENSION_TYPE_CD);

%if (&_dupsFound eq 1) %then %goto exit ;

%_validateDups(_param=DIMENSION_MEMBER_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
                   _where=&_tmpWhereDups,
                   _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                   _byVars=DIMENSION_MEMBER_CD original,
                   _byVars1=DIMENSION_MEMBER_CD);

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

%let _tmpInvFType = 0;
%let _tmpPlanErr = 0;

data WORK._TMP_FORMULA
    (keep=DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO FORMULA_TYPE_CD PRECEDENCE_CLASS
        VALID_FROM_DTTM VALID_TO_DTTM TARGET_DESC);

   set &_tmpDSN(where=(&_where5));

   /****************************************************************************/
   /* Prior to hard-coding 99999 into the _tmpFTarget macro value, the formula */
   /* target rk was being passed.  Error messages 46 & 47 need to be reworked. */
   /****************************************************************************/

   if (FORMULA_TYPE_CD not in ('0','2','3')) then do;
      call symput("_tmpInvFType", 1);
      call symput("_tmpFTarget", 99999);
      call symput("_tmpInvCode", strip(FORMULA_TYPE_CD));
      stop;
   end;
   else if (DIMENSION_TYPE_CD ne 'ACCOUNT' and FORMULA_TYPE_CD in ('0','3')) then do;
      call symput("_tmpPlanErr", 1);
      call symput("_tmpInvCode", strip(FORMULA_TYPE_CD));
      call symput("_tmpFTarget", 99999);
      stop;
   end;
   PRECEDENCE_CLASS = PRECEDENCE_ORDER_NO;
   if VALID_FROM_DTTM = . then
      VALID_FROM_DTTM = &soletl_processedDTTM.;
   if VALID_TO_DTTM = . then
      VALID_TO_DTTM = &curr_rec_dttm.;

   TARGET_DESC = APP_FORMULA_TARGET_DESC;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if (&_tmpInvFType) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=46
          , msgTxt1=&_tmpFTarget, msgTxt2=&_tmpInvCode
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%if (&_tmpPlanErr) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=47
          , msgTxt1=&_tmpInvCode, msgTxt2=&dimtype, msgTxt3=&_tmpFTarget
          , msgPutlog=&putlog);
  %goto exit ;
%end;

data WORK.SAS_ETL_FORMULA
   (keep=ETL_INSTANCE_ID TARGET_MEMBER_CD FORMULA_TYPE_CD PRECEDENCE_CLASS VALID_FROM_DTTM
        VALID_TO_DTTM ORDINAL TARGET_DESC);

   set _SDMLIB.SAS_ETL_FORMULA(obs=0)
       WORK._TMP_FORMULA;

   ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   TARGET_MEMBER_CD = DIMENSION_MEMBER_CD;
   ORDINAL = ORDINAL_NO;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_FORMULA data=WORK.SAS_ETL_FORMULA &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/*                _StgDDS.APP_FORMULA ==> _SDMLIB.SAS_ETL_FORMULA_TEXT               */
/*************************************************************************************/

%let _tmpDSN = &_lib..APP_FORMULA;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgTxt1=&_tmpDSN, msgTxt2=
          , msgPutlog=&putlog);
  %goto exit ;
%end;

/* Verify that no duplicate codes exist in the stage formula table  */
%_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
                _where=&_tmpWhereDups,
                _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                _byVars=DIMENSION_TYPE_CD original,
                _byVars1=DIMENSION_TYPE_CD);

%if (&_dupsFound eq 1) %then %goto exit ;

%_validateDups(_param=DIMENSION_MEMBER_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
                   _where=&_tmpWhereDups,
                   _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                   _byVars=DIMENSION_MEMBER_CD original,
                   _byVars1=DIMENSION_MEMBER_CD);

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

proc sort data = WORK._TMP_FORMULA NODUPKEY;
   by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sort data = &_tmpDSN
           out = WORK._TMP_FORMTEXT;
   by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data work._sdmlib_SAS_ETL_FORMULA_TEXT;
   set _SDMLIB.SAS_ETL_FORMULA_TEXT(obs=0 dbmax_text=32767)
       WORK._TMP_FORMULA
       WORK._TMP_FORMTEXT;

   stop;
run;

data WORK.SAS_ETL_FORMULA_TEXT
   (keep=ETL_INSTANCE_ID TARGET_MEMBER_CD ORDINAL FORMULA_TEXT ORDER_NO);

   merge work._sdmlib_SAS_ETL_FORMULA_TEXT(obs=0)
         WORK._TMP_FORMULA (in = inA)
         WORK._TMP_FORMTEXT;
      by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;

   ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   TARGET_MEMBER_CD = DIMENSION_MEMBER_CD;
   ORDINAL = ORDINAL_NO;
   FORMULA_TEXT = FORMULA_TXT;
   ORDER_NO = LINE_NO;
   if (inA) then
      output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_FORMULA_TEXT(dbmax_text=32767) data=WORK.SAS_ETL_FORMULA_TEXT &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/*       _StgDDS.APP_FORMULA_WRITE_MEMBER ==> _SDMLIB.SAS_ETL_FORMULA_LHS_SCOPE      */
/*************************************************************************************/

%let _tmpDSN = &_lib..APP_FORMULA_WRITE_MEMBER;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgTxt1=&_tmpDSN, msgTxt2=
          , msgPutlog=&putlog);
  %goto exit ;
%end;

/* Verify that no duplicate codes exist in the stage formula table  */
%_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
                _where=&_tmpWhereDups,
                _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                _byVars=DIMENSION_TYPE_CD original,
                _byVars1=DIMENSION_TYPE_CD);

%if (&_dupsFound eq 1) %then %goto exit ;

%_validateDups(_param=DIMENSION_MEMBER_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
                   _where=&_tmpWhereDups,
                   _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                   _byVars=DIMENSION_MEMBER_CD original,
                   _byVars1=DIMENSION_MEMBER_CD);

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

proc sort data = &_tmpDSN
           out = WORK._TMP_FORM_WRITE;
   by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data work._sdmlib_SAS_ETL_FORMULA_LHS;
   set _SDMLIB.SAS_ETL_FORMULA_LHS_SCOPE(obs=0 dbmax_text=32767)
       WORK._TMP_FORMULA
       WORK._TMP_FORM_WRITE;

   stop;
run;

data WORK.SAS_ETL_FORMULA_LHS_SCOPE
   (keep=ETL_INSTANCE_ID TARGET_MEMBER_CD ORDINAL FORMULA_LHS_SCOPE ORDER_NO);

   merge work._sdmlib_SAS_ETL_FORMULA_LHS(obs=0)
         WORK._TMP_FORMULA (in = inA)
         WORK._TMP_FORM_WRITE (in = inB);
      by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;

   ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   FORMULA_LHS_SCOPE = WRITE_MEMBER_SCOPE_TXT;
   TARGET_MEMBER_CD = DIMENSION_MEMBER_CD;
   ORDINAL = ORDINAL_NO;
   ORDER_NO = LINE_NO;
   if (inA and inB) then
      output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_FORMULA_LHS_SCOPE(dbmax_text=32767) data=WORK.SAS_ETL_FORMULA_LHS_SCOPE &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/*        _StgDDS.APP_FORMULA_READ_MEMBER ==> _SDMLIB.SAS_ETL_FORMULA_RHS_SCOPE      */
/*************************************************************************************/

%let _tmpDSN = &_lib..APP_FORMULA_READ_MEMBER;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=2
          , msgTxt1=&_tmpDSN, msgTxt2=
          , msgPutlog=&putlog);
  %goto exit ;
%end;

/* Verify that no duplicate codes exist in the stage formula table  */
%_validateDups(_param=DIMENSION_TYPE_CD, _ds=&_tmpDSN, _format=SAS_DIM_TYPE,
                _where=&_tmpWhereDups,
                _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                _byVars=DIMENSION_TYPE_CD original,
                _byVars1=DIMENSION_TYPE_CD);

%if (&_dupsFound eq 1) %then %goto exit ;

%_validateDups(_param=DIMENSION_MEMBER_CD, _ds=&_tmpDSN, _format=SAS_MEMBER,
                   _where=&_tmpWhereDups,
                   _cols=DIMENSION_MEMBER_CD DIMENSION_TYPE_CD VALID_FROM_DTTM VALID_TO_DTTM,
                   _byVars=DIMENSION_MEMBER_CD original,
                   _byVars1=DIMENSION_MEMBER_CD);

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

proc sort data = &_tmpDSN
           out = WORK._TMP_FORM_READ;
   by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data work._sdmlib_SAS_ETL_FORMULA_RHS;
   set _SDMLIB.SAS_ETL_FORMULA_RHS_SCOPE(obs=0 dbmax_text=32767)
       WORK._TMP_FORMULA
       WORK._TMP_FORM_READ;

   stop;
run;

data WORK.SAS_ETL_FORMULA_RHS_SCOPE
    (keep=ETL_INSTANCE_ID TARGET_MEMBER_CD ORDINAL FORMULA_RHS_SCOPE ORDER_NO);

   merge work._sdmlib_SAS_ETL_FORMULA_RHS(obs=0)
         WORK._TMP_FORMULA (in = inA)
         WORK._TMP_FORM_READ (in = inB);
      by DIMENSION_TYPE_CD DIMENSION_MEMBER_CD ORDINAL_NO;

   ETL_INSTANCE_ID = symgetn("soletl_instanceId");
   FORMULA_RHS_SCOPE = READ_MEMBER_SCOPE_TXT;
   TARGET_MEMBER_CD = DIMENSION_MEMBER_CD;
   ORDINAL = ORDINAL_NO;
   ORDER_NO = LINE_NO;
   if (inA and inB) then
      output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc append base=_SDMLIB.SAS_ETL_FORMULA_RHS_SCOPE(dbmax_text=32767) data=WORK.SAS_ETL_FORMULA_RHS_SCOPE &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
%let soletl_statusID = 0; /* Program ran successful */

%exit:

%if &soletl_statusID ne 0 %then %do;
   %let soletl_error = &sesgmsg_msgCodeReturn;
   %let soletl_errormsg = &sesgmsg_msgReturn;
%end;

%mend etldim;
