/* Copyright 2009 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro etldtype (soletl_processedDTTM=, soletl_instanceId=);

/*************************************************************************************/
/*                             Initialization                                        */
/*************************************************************************************/
%global soletl_error;
%global soletl_errormsg;
%global soletl_statusID;
%global soletl_locale_missing;
%global soletl_locale_no_data_found;

%let soletl_error = ;
%let soletl_errormsg = ;
%let soletl_statusID = 1; /* Will be changed to 0 if program runs successful */
%let soletl_locale_missing = ;
%let soletl_locale_no_data_found = ;

%let entry = etldtype;
%let putlog = Y;

%let curr_rec_dttm = %sysfunc( inputn('01JAN5999:00:00:00', datetime18.) );

%let _where1=((VALID_FROM_DTTM <= &soletl_processedDTTM)
          and (VALID_TO_DTTM   >= &soletl_processedDTTM or VALID_TO_DTTM = .));

/* Check if datetime passed in is valid */
%if %datatyp(&soletl_processedDTTM) ne NUMERIC %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=1
          , msgTxt1=<soletl_processedDTTM : &soletl_processedDTTM>, msgPutlog=&putlog);
  %goto exit ;
%end;

/* Check for
           102 : Metaserver not defined
           103 : Libref not found
           104 : Access denied (somekind of security issue on the metadata user)
*/

%bldLibNm(sesetl_sasLibRef=_CONFLIB
        , sesetl_OMRSASLibraryObjName=&FMdatatier_folder./&CONFLIBOBJ./&CONFLIBOBJ.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%if &sysrc ne 0 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&sysrc
          , msgTxt1=<_CONFLIB>, msgPutlog=&putlog);
  %goto exit ;
%end;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%if &sysrc ne 0 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=&sysrc
          , msgTxt1=<_SDMLIB>, msgPutlog=&putlog);
  %goto exit ;
%end;

/*************************************************************************************/
/* Verify that there is 1 and only 1 default language record identified in the       */
/* CODE_LANGUAGE table in the stageDDS.                                              */
/*************************************************************************************/
%let _tmpDSN = _CONFLIB.CODE_LANGUAGE;

proc sql noprint;
   select count(*) into :_dfltCount
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y" and &_where1
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if &_dfltCount < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=2
          , msgPutlog=&putlog);
  %goto exit;
%end;

%if &_dfltCount > 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=3
          , msgPutlog=&putlog);
  %goto exit;
%end;

proc sql noprint;
   select LANGUAGE_CD into :_dfltLangCd
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y" and &_where1
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/* Sort the _CONFLIB.CODE_LANGUAGE table by LOCALE_LANGUAGE, LOCALE_COUNTRY and      */
/* LOCALE_VARIANT and write to _CODE_LANG_SORT table.  This table will be used in    */
/* conjunction with the inputted SDM locales to create mapping to the locales in the */
/* stageDDS                                                                          */
/*************************************************************************************/
%let _tmpWhere=where=(&_where1);

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._CODE_LANG_SORT(keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD
                       LOCALE_VARIANT_CD LANGUAGE_CD DEFAULT_LANGUAGE_FLG)
           NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/* Extract the language codes from the soletl_localeCodes macro variable and place   */
/* in a temporary dataset.                                                           */
/*************************************************************************************/
data _INPUT_LOCALES
           (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_CD) ;

   set _SDMLIB.sas_etl_member(obs=0 keep=LOCALE_CD)
       &_tmpDSN(obs = 1 keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD);

   do n = 1 to 9999999999 ;
     LOCALE_CD = strip( kscan("&soletl_localeCodes", n, "09"x) ) ;
     if LOCALE_CD eq "" then leave ;

     LOCALE_LANGUAGE_CD = "";
     LOCALE_COUNTRY_CD = "";
     LOCALE_VARIANT_CD = "";
     if LOCALE_CD ne "" then do;
        LOCALE_LANGUAGE_CD = strip( kscan(LOCALE_CD, 1, '_') ) ;
        NLS_COUNT1 = count(LOCALE_CD, '_') ;
        if NLS_COUNT1 eq 0 then do;
           LOCALE_COUNTRY_CD = '';
           LOCALE_VARIANT_CD = '';
        end;
        else do;
           NLS_COUNT2 = count(LOCALE_CD, '__') ;
           if NLS_COUNT2 eq 1 then do;
              LOCALE_COUNTRY_CD = '';
              LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 2, '_') );
           end;
           else do;
              LOCALE_COUNTRY_CD = strip( kscan(LOCALE_CD, 2, '_') );
              LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 3, '_') );
           end;
        end;
     end;

     output;
   end ;
run ;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpDSN = WORK._INPUT_LOCALES;

proc sort data = &_tmpDSN
           NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/***********************************************************************************/
/* Create mapping table, WORK._INPUT_LOCALES, between the inputted locales and the */
/* locales in the stage area. Check that records were found in the CODE_LANGUAGE   */
/* table of the STAGE for all of the inputted locales. If not, populate the        */
/* soletl_locale_missing macro variable.                                           */
/***********************************************************************************/
data &_tmpDSN
       (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_CD
             LANGUAGE_CD DEFAULT_LANGUAGE_FLG);

   merge &_tmpDSN (in = inA)
         WORK._CODE_LANG_SORT (in = inB);
      by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;

   if (inA) then do;
      output;
      if (not inB) then do;
         if symget("soletl_locale_missing") = "" then
            call symput("soletl_locale_missing", LOCALE_CD);
         else
            call symput("soletl_locale_missing", strip(symget("soletl_locale_missing"))
                        || "09"x || strip(LOCALE_CD));
      end;
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;


proc sql noprint;
   select count(*) into :_dfltFound
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y"
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;


%if &_dfltFound < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=4
          , msgTxt1=<&_dfltLangCd>, msgPutlog=&putlog);
  %goto exit;
%end;

/***********************************************************************************/
/* Create a temporary sdm locale table with the locales sorted by language,        */
/* country and variant.                                                            */
/***********************************************************************************/
%let _tmpDSN = _SDMLIB.SAS_DATA_LOCALE;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=5
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=6
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

data WORK._SDM_LOCALES
        (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_ID);
   set &_tmpDSN;
   LOCALE_LANGUAGE_CD = ISO639_LANGUAGE_CD;
   LOCALE_COUNTRY_CD = ISO3166_COUNTRY_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let _tmpDSN = WORK._SDM_LOCALES;

proc sort data = &_tmpDSN NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/***********************************************************************************/
/* Create temporary table containing mapping between the locales in the dds and    */
/* sdm using language, country and variant as sorting variables.                   */
/***********************************************************************************/
data WORK._DDS_SDM_LOCALE_MAP
     (keep=LANGUAGE_CD LOCALE_ID LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD
           LOCALE_VARIANT_CD DEFAULT_LANGUAGE_FLG);
   merge WORK._CODE_LANG_SORT (in = inA)
         &_tmpDSN (in = inB);
      by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;

   if (inA and inB) then
      output;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

/*************************************************************************************/
/* Check that the dds default locale exists in the SDM.                              */
/*************************************************************************************/
%let _tmpDSN = WORK._DDS_SDM_LOCALE_MAP;

proc sql noprint;
   select count(*) into :_sdmDfltCnt
   from &_tmpDSN
   where upcase(DEFAULT_LANGUAGE_FLG) = "Y"
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if &_sdmDfltCnt < 1 %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=7
          , msgTxt1=<&_dfltLangCd>, msgPutlog=&putlog);
  %goto exit;
%end;

/***********************************************************************************/
/* Create locale format, FMT_LOCALE, using the WORK._DDS_SDM_LOCALE_MAP table.     */
/***********************************************************************************/
%let _tmpFormat=FMT_LOCALE;
%let _tmpStart=LANGUAGE_CD;
%let _tmpLabel=LOCALE_ID;
%let _tmpWhere=;

proc sort data = &_tmpDSN
           out = WORK._TMPSORT (keep=&_tmpStart &_tmpLabel)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   set WORK._TMPSORT(rename=(&_tmpStart=START &_tmpLabel=LABEL));
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=8
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/***********************************************************************************/
/* Create dimension type format, FMT_DIMTYPE.                                      */
/***********************************************************************************/
%let _tmpFormat=FMT_DIMTYPE;

proc format library=work;
  invalue &_tmpFormat
    'ACCOUNT'     = 1
    'ANALYSIS'    = 3
    'COSTCTR'     = 4
    'CURRENCY'    = 5
    'EXTORG'      = 6
    'INTORG'      = 7
    'ITEMCAT'     = 8
    'PROFITCTR'   = 10
    'TIME'        = 11
    'TRADER'      = 12
    'SOURCE'      = 14
    'RATE_TYPE'   = 15
    'TO_CURRENCY' = 16
    OTHER         = 99
  ;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=8
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;

/***********************************************************************************/
/* Create a temporary sdm dimension type format with existing codes                */
/***********************************************************************************/

%let _tmpDSN = _SDMLIB.SAS_DIM_TYPE;

%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=locale, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;

%let _tmpFormat=FMT_DIMTYPES;
%let _tmpStart=DIM_TYPE_CD;
%let _tmpWhere=where=(&_where1);

proc sort data = &_tmpDSN(&_tmpWhere)
           out = WORK._TMPSORT (keep=&_tmpStart)
           NODUPKEY;
   by &_tmpStart;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain LABEL;
   set WORK._TMPSORT(rename=(&_tmpStart=START )) end=eof;
   LABEL = START;
   START = upcase(START);
   output;
   if eof then do;
     HLO = "O";
     START = "";
     output; 
   end;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc format cntlin=work.&_tmpFormat library=work;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
  %sesgmsg( msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=8
          , msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
  %goto exit ;
%end;


/***********************************************************************************/
/*                    Import the dimension_type records                            */
/***********************************************************************************/
data WORK.SAS_ETL_DIM_TYPE;
   set _SDMLIB.SAS_ETL_DIM_TYPE(obs=0);
   stop;
run;

%let _tmpDSN = _CONFLIB.dimension_type;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=9
          , msgPutlog=N);
  %sesgmsg( msgDsn=sashelp.solmsgs, msgEntry=&entry, msgCode=10
          , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
  %goto exit ;
%end;


/* Verify that no duplicate codes exist in the stage dimension type table  */
%let _dupsFound = 0;
%let dimtypeid = DIMENSION_TYPE_CD;
%let _tmpWhere=where=(&_where1);

data WORK._UPCASECODES;
  set &_tmpDSN;
  if &dimtypeid ne '';
  original = &dimtypeid;
  &dimtypeid = upcase(&dimtypeid);
run;  
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sort data = WORK._UPCASECODES(&_tmpWhere
                 keep=&dimtypeid original VALID_FROM_DTTM VALID_TO_DTTM LANGUAGE_CD)
           out = WORK._TMPCODES
        dupout = WORK._TMPDUPCODES (keep = &dimtypeid original)
        NODUPKEY
            ;
  by &dimtypeid LANGUAGE_CD; 
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

proc sql noprint;
  create table WORK._DUPCODES as
  select strip(d.original) as _tmpDupCode1, strip(o.original) as _tmpDupCode
  from WORK._TMPDUPCODES d, WORK._UPCASECODES o
  where upcase(o.original) eq upcase(d.original)
      and o.original ne d.original
  ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

data _null_;
  length _command $1024;
  set WORK._DUPCODES ;
  call symput('_dupsFound', 1);

  _command = '%sesgmsg(msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=11,
            msgTxt1=<' || strip(_tmpDupCode) || '>, msgTxt2=<' || strip(_tmpDupCode1) || '>,
            msgTxt3=&_tmpDSN, msgTxt4=&dimtypeid, msgPutlog=&putlog)';
  call execute(_command);  
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%if (&_dupsFound eq 1) %then %goto exit ;

data WORK._DUPCODES(keep=_tmpDupCode _tmpDupCode1);
  length _tmpDupCode _tmpDupCode1 $ %eval(32*&FM_DBCSMULT);
  set WORK._UPCASECODES;
  _tmpDupCode = strip(original);
  _tmpDupCode1 = strip(put(upcase(strip(original)), $FMT_DIMTYPES.));
  if _tmpDupCode eq _tmpDupCode1 then ;
  else if upcase(put(upcase(_tmpDupCode), $FMT_DIMTYPES.)) 
                             eq upcase(_tmpDupCode) then output;
run; 

data _null_;
  length _command $1024;
  set WORK._DUPCODES ;
  call symput('_dupsFound', 1);

  _command = '%sesgmsg(msgDsn=SASHELP.SOLMSGS, msgEntry=&entry, msgCode=12,
          msgTxt1=<' || strip(_tmpDupCode) || '>, msgTxt2=<' || strip(_tmpDupCode1) || '>,
          msgTxt3=&_tmpDSN, msgTxt4=&dimtypeid, msgTxt5=SAS_DIM_TYPE, msgPutlog=&putlog)';
  call execute(_command);        
run;

%if (&_dupsFound eq 1) %then %goto exit ;
/* Verify duplicate codes end */

/* Determine the number of locales to import. */
proc sql noprint;
   select count(*) into :_locCnt
   from WORK._INPUT_LOCALES
   ;
quit;
%checkErr;
%if &_tmpRC %then %goto exit ;

%do i = 1 %to &_locCnt;
   data _null_;
      set WORK._INPUT_LOCALES (obs = &i);
      call symput("_tmpCode", strip(LANGUAGE_CD));
      call symput("_tmpLocale", strip(LOCALE_CD));
   run;
   data WORK._TMP_DIM_TYPE
        (keep=ETL_INSTANCE_ID DIM_TYPE_CD DIM_TYPE_NM DIM_TYPE_DESC LOCALE_CD
              LOCALE_ID DIM_TYPE_ROLE VALID_FROM_DTTM VALID_TO_DTTM);
      set _SDMLIB.SAS_ETL_DIM_TYPE(obs=0)
          &_tmpDSN( where=((upcase(LANGUAGE_CD) = upcase("&_tmpCode"))
             and &_where1));
      ETL_INSTANCE_ID = symgetn("soletl_instanceId");
      DIM_TYPE_CD = DIMENSION_TYPE_CD;
      DIM_TYPE_NM = DIMENSION_TYPE_NM;
      DIM_TYPE_DESC = DIMENSION_TYPE_DESC;
      LOCALE_ID = input(LANGUAGE_CD, FMT_LOCALE.);
      DIM_TYPE_ROLE = input(strip(DIMENSION_TYPE_CD), FMT_DIMTYPE.);
      if (VALID_TO_DTTM >= &soletl_processedDTTM) then
         VALID_TO_DTTM = &curr_rec_dttm;
      FORMAT VALID_FROM_DTTM;
      FORMAT VALID_TO_DTTM;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   proc sql noprint;
      select count(*) into :_recCount
      from WORK._TMP_DIM_TYPE ;
   quit;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
   data _null_;
      if &_recCount < 1 then do;
         if strip("&soletl_locale_no_data_found") = "" then
            call symput("soletl_locale_no_data_found", strip("&_tmpLocale"));
         else
            call symput("soletl_locale_no_data_found", strip(symget("soletl_locale_no_data_found"))
                        || "09"x || strip("&_tmpLocale")) ;
      end;
   run;
   proc append base=WORK.SAS_ETL_DIM_TYPE data=WORK._TMP_DIM_TYPE &FM_PROCAPPENDFORCE;
   run;
   %checkErr;
   %if &_tmpRC %then %goto exit ;
%end;

proc append base=_SDMLIB.SAS_ETL_DIM_TYPE data=WORK.SAS_ETL_DIM_TYPE &FM_PROCAPPENDFORCE;
run;
%checkErr;
%if &_tmpRC %then %goto exit ;

%let soletl_statusID = 0; /* Program ran successful */

%exit:

%if &soletl_statusID ne 0 %then %do;
   %let soletl_error = &sesgmsg_msgCodeReturn;
   %let soletl_errormsg = &sesgmsg_msgReturn;
%end;

%mend etldtype;
