%macro fcstload;

  /****************************************************************************************/
  /* This macro will load the forecast data generated by fcstrun into the SDM.            */
  /****************************************************************************************/

  %let cTmpRemoveROFactSDM_Nm     = &fmsetl_tempRemoveROFactTable;
  %let cTmpRemoveRWFactSDM_Nm     = &fmsetl_tempRemoveRWFactTable;
  %let cSDMStatusDS               = &fmsetl_tempStatusTable;
  %let nActionType                = &fmsetl_actionType;
  %let nSdmROFactSetId            = &fmsetl_readOnlyFactSetId;
  %let nSdmRWFactSetId            = &fmsetl_readWriteFactSetId;
  %let cDLObjTypeId               = &fmsetl_dataLoadObjectTypeId;
  %let cETL_instanceID            = &fmsetl_instanceId;
  %let cTmpFactSDM_Nm             = &fmsetl_tempCreateFactTable;

    /*  Assign constants  */
  %let cStatusDS                  = work._factstatus;
  %let cTmpFactWork_Nm            = work.sas_etl_base_fact;

  %global lcycledimtypes;
  %let lcycledimtypes             = work.cycleDTypes;

  /***************************************************************************************/
  data _null_;
     length tmp_by_dim $ %eval(32*&FM_DBCSMULT);
     length tmp_dtype $ %eval(32*&FM_DBCSMULT);
     if &by_vars_dimension_codes_count > 0 then do;
        %do i = 1 %to &BY_VARS_DIMENSION_CODES_COUNT;
           tmp_by_dim = "&&tmp_by_var_dim&i";
           tmp_dtype = put(tmp_by_dim, $fmt_dim_to_dtype.);
           %let _by_vars_dtype&i = ;
           call symput("_by_vars_dtype&i", strip(tmp_dtype));
        %end;
     end;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /*******************************************************************************************/
  /* Create a table of distinct date values that will be loaded in ascending order and will  */
  /* be used later as a hash table mapping the date values to time member codes.             */
  /*******************************************************************************************/
  proc sql noprint;
    create table date_values as select distinct date_value from fcst_output;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sort data = date_values;
    by date_value;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data work.date_value_to_time;
     set work.date_values;
     set &cSdmLib..&forecast_time_member_table;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /* Create a forecast table from the forecast output table.                                 */
  /* Extract the value in the predict column of the output forecast record as the fact_value */;
  /* and use the &tmp_analysis_forecast_member as the analysis member.                       */;

  %let cd=_CD;
  %let code = _CODE;

  data work.forecast;
     %let fcstDtypeCd=%sysfunc(strip(&forecast_dtype))&cd;
     %let fcstcode=%sysfunc(strip(&forecast_dtype))&code;
     length analysis_cd $ %eval(32*&FM_DBCSMULT);
     length "&fcstDtypeCd"n $ %eval(32*&FM_DBCSMULT);
     set work.fcst_output;
     analysis_cd = "&tmp_analysis_forecast_mbr";
     fact_value = predict;
     "&fcstDtypeCd"n = "&fcstcode"n;
     if &by_vars_dimension_codes_count > 0 then do;
        %do i = 1 %to &by_vars_dimension_codes_count ;
           %let byVarCode=%sysfunc(strip(&&_by_vars_dtype&i))&code;
           length "&&_by_vars_dtype&i..&cd"n $ %eval(32*&FM_DBCSMULT);
           "&&_by_vars_dtype&i..&cd"n = "&byVarCode"n;
           drop "&byVarCode"n;
        %end;
     end;
     drop "%sysfunc(strip(&forecast_dtype))"n;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /* If confidence interval values are wanted then create lower and upper confidence         */;
  /* interval tables.                                                                        */;

  /* The lower confidence inverval table will use the value in the lower column as the       */;
  /* fact_value and use the &tmp_analysis_lower_member as the analysis member.               */;

  /* The upper confidence inverval table will use the value in the upper column as the       */;
  /* fact_value and use the &tmp_analysis_upper_member as the analysis member.               */;

  %if &include_confidence_intervals %then %do;
     data work.forecast_lower;
        %let fcstDtypeCd=%sysfunc(strip(&forecast_dtype))&cd;
        %let fcstcode=%sysfunc(strip(&forecast_dtype))&code;
        length analysis_cd $ %eval(32*&FM_DBCSMULT);
        length "&fcstDtypeCd"n $ %eval(32*&FM_DBCSMULT);
        set work.fcst_output;
        analysis_cd = "&tmp_analysis_lower_mbr";
        fact_value = lower;
        "&fcstDtypeCd"n = "&fcstcode"n;
        if &by_vars_dimension_codes_count > 0 then do;
           %do i = 1 %to &by_vars_dimension_codes_count ;
              %let byVarCode=%sysfunc(strip(&&_by_vars_dtype&i))&code;
              length "&&_by_vars_dtype&i..&cd"n $ %eval(32*&FM_DBCSMULT);
              "&&_by_vars_dtype&i..&cd"n = "&byVarCode"n;
              drop "&byVarCode"n;
           %end;
        end;
        drop "%sysfunc(strip(&forecast_dtype))"n;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     data work.forecast_upper;
        %let fcstDtypeCd=%sysfunc(strip(&forecast_dtype))&cd;
        %let fcstcode=%sysfunc(strip(&forecast_dtype))&code;
        length analysis_cd $ %eval(32*&FM_DBCSMULT);
        length "&fcstDtypeCd"n $ %eval(32*&FM_DBCSMULT);
        set work.fcst_output;
        analysis_cd = "&tmp_analysis_upper_mbr";
        fact_value = upper;
        "&fcstDtypeCd"n = "&fcstcode"n;
        if &by_vars_dimension_codes_count > 0 then do;
           %do i = 1 %to &by_vars_dimension_codes_count ;
              %let byVarCode=%sysfunc(strip(&&_by_vars_dtype&i))&code;
              length "&&_by_vars_dtype&i..&cd"n $ %eval(32*&FM_DBCSMULT);
              "&&_by_vars_dtype&i..&cd"n = "&byVarCode"n;
              drop "&byVarCode"n;
           %end;
        end;
        drop "%sysfunc(strip(&forecast_dtype))"n;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  /* Create Slicer dimension type string and member string macro variables. */
  data _null_;
     length tmp_slicer $ %eval(32*&FM_DBCSMULT);
     length tmp_dtype $ %eval(32*&FM_DBCSMULT);
     if &slicer_dim_codes_count > 0 then do;
        %do i = 1 %to &slicer_dim_codes_count;
           tmp_slicer = "&&tmp_slicer_dim&i";
           tmp_dtype = put(tmp_slicer, $fmt_dim_to_dtype.);
           %let _slicer_dtype&i = ;
           call symput("_slicer_dtype&i", strip(tmp_dtype));
        %end;
     end;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %let cd = _CD;

  /* Add slicer members to the forecast tables. */
  data work.forecast;
     length date_value 8;
     length time_cd $ %eval(32*&FM_DBCSMULT);
     length intorg_cd $ %eval(32*&FM_DBCSMULT);
     length currency_cd $ %eval(32*&FM_DBCSMULT);
     set work.forecast;
     if _n_ = 1 then do;
        declare hash fcht(dataset:"work.org_to_currency");
        fcht.defineKey("intorg_cd");
        fcht.defineData("currency_cd");
        fcht.defineDone();
        declare hash dtht(dataset:"work.date_value_to_time");
        dtht.defineKey("date_value");
        dtht.defineData("time_cd");
        dtht.defineDone();
     end;
     if &slicer_dim_codes_count > 0 then do;
        %do i = 1 %to &slicer_dim_codes_count;
           length "&&_slicer_dtype&i..&cd"n $ %eval(32*&FM_DBCSMULT);
           "&&_slicer_dtype&i..&cd"n = "&&tmp_slicer_write_mem&i";
        %end;
     end;
     rc = dtht.find();
     if currency_cd = "" then
        rc = fcht.find();
     drop rc;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if &include_confidence_intervals %then %do;
     data work.forecast_lower;
        length date_value 8;
        length time_cd $ %eval(32*&FM_DBCSMULT);
        length intorg_cd $ %eval(32*&FM_DBCSMULT);
        length currency_cd $ %eval(32*&FM_DBCSMULT);
        set work.forecast_lower;
        if _n_ = 1 then do;
           declare hash fcht(dataset:"work.org_to_currency");
           fcht.defineKey("intorg_cd");
           fcht.defineData("currency_cd");
           fcht.defineDone();
           declare hash dtht(dataset:"work.date_value_to_time");
           dtht.defineKey("date_value");
           dtht.defineData("time_cd");
           dtht.defineDone();
        end;
        if &slicer_dim_codes_count > 0 then do;
           %do i = 1 %to &slicer_dim_codes_count;
              length "&&_slicer_dtype&i..&cd"n $32;
              "&&_slicer_dtype&i..&cd"n = "&&tmp_slicer_write_mem&i";
           %end;
        end;
        rc = dtht.find();
        if currency_cd = "" then
           rc = fcht.find();
        drop rc;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     data work.forecast_upper;
        length date_value 8;
        length time_cd $ %eval(32*&FM_DBCSMULT);
        length intorg_cd $ %eval(32*&FM_DBCSMULT);
        length currency_cd $ %eval(32*&FM_DBCSMULT);
        set work.forecast_upper;
        if _n_ = 1 then do;
           declare hash fcht(dataset:"work.org_to_currency");
           fcht.defineKey("intorg_cd");
           fcht.defineData("currency_cd");
           fcht.defineDone();
           declare hash dtht(dataset:"work.date_value_to_time");
           dtht.defineKey("date_value");
           dtht.defineData("time_cd");
           dtht.defineDone();
        end;
        if &slicer_dim_codes_count > 0 then do;
           %do i = 1 %to &slicer_dim_codes_count;
              length "&&_slicer_dtype&i..&cd"n $ %eval(32*&FM_DBCSMULT);
              "&&_slicer_dtype&i..&cd"n = "&&tmp_slicer_write_mem&i";
           %end;
        end;
        rc = dtht.find();
        if currency_cd = "" then
           rc = fcht.find();
        drop rc;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     /* Combine the 3 forecast tables. */
     data work.forecast_all;
        set work.forecast
            work.forecast_lower
            work.forecast_upper;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;
  %else %do;
     data work.forecast_all;
        set work.forecast;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  proc sort data=work.forecast_all;
     by date_value;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data work.forecast_all;
     set work.forecast_all;
     drop date_value;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
     delete from work.forecast_all where fact_value is missing ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
     select count(*) into :factCnt from work.forecast_all;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /* Output error to log if insufficient data and exit. */
  %if &factCnt = 0 %then %do;
     /* A statistical model that is generated when there is insufficient data provides */
     /* no value and should be deleted when not reusing a prexisting model or if there */
     /* isn't a prexisting model. This code is only valid for PA as a model generated  */
     /* by a Form Author is never saved.                                               */
     %if &forecast_initiator = 1 and
         (&model_exists = 0 or &use_existing_forecast_model = 0) %then %do;
        proc sql noprint;
           drop table &cSASData..&model_tablename;
        quit;
        %checkErr;
        %if &_tmpRC %then %goto RETURNZERO;
     %end;

     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5,
              msgPutlog=&putlog);
     %let fmsetl_errorId=5;
     %goto RETURNZERO;
  %end;

  %mbrConcat(&lcycledimtypes) ;

  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  %DimDimTypeFormats;

  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  %createHashTable;

  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  %createUpdateTable(input_table_name=work.forecast_all,
                     bfdtypes=&cBfDtypes,
                     dataLoadObjTypeId=&fmsetl_dataLoadObjectTypeId,
                     dataLoadObjId=&fmsetl_dataLoadObjectIds,
                     update_tablename=&cTmpFactWork_Nm);

  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  /********************************************************************************
   *
   *  Gather some stats on the outcome of the job, to send back to calling code
   *
   ********************************************************************************/

  /*  Create a table of all Successful imports */
  %fcst_buildStatusDS(&cStatusDS, 0);
  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  /*  Count successful Imports  */
  %fcst_updateValidStatusCnt( &cStatusDS, &cTmpFactWork_Nm, IMPORTED_TRANS_CNT );
  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  /*  Count Zero values  */
  %fcst_updateZeroStatusCnt( &cStatusDS, &cTmpFactWork_Nm, ZERO_TRANS_CNT);
  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  %if &nActionType NE 4 and &nActionType NE 5 %then %do;
     %fcst_uploadTempFactToSDM(&lcycledimtypes, &cTmpFactSDM_Nm, &cTmpFactWork_Nm, &cStatusDS );
     %if &_FMErrorCode = 0 %then
        %goto RETURNZERO;
  %end;
  %else %do;
     %fcst_replaceMatching(&cTmpRemoveROFactSDM_Nm, &cTmpRemoveRWFactSDM_Nm,
                             &cStatusDS, &nActionType, &lcycledimtypes,
                             &cSdmLib, &nSdmROFactSetId, &nSdmRWFactSetId, &cDLObjTypeId,
                             &cTmpFactWork_Nm, &cTmpFactSDM_Nm, &cETL_instanceID);
     %if &_FMErrorCode = 0 %then
        %goto RETURNZERO;
  %end;

  /*  Upload Status Report to SASSDM  */
  %fcst_uploadStatusTableToSDM(&cStatusDS, &cSDMStatusDS);
  %if &_FMErrorCode = 0 %then
     %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:

  %let _FMErrorCode=0;

  %THEEND:

%mend fcstload;

/***************************************************************************************/
%macro mbrConcat(lDTypes);

  /************************/
  /*  FMT_DTYPE_ID_TO_CD  */
  /************************/

  %let _tmpDSN = &cSDMLib..sas_dim_type;
  %let _tmpFormat = fmt_dtype_id_to_cd;
  %let _tmpStart = dim_type_id;
  %let _tmpLabel = dim_type_cd;

  proc sort data = &_tmpDSN(where=(valid_from_dttm <= &today_dttm_val and
                            valid_to_dttm >= &today_dttm_val))
             out = work._tmpsort(drop = valid_from_dttm valid_to_dttm)
             nodupkey;
     by &_tmpStart;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data work.&_tmpFormat;
     retain fmtname "&_tmpFormat";
     retain type "N";
     set work._tmpsort;
     start = &_tmpStart;
     label = &_tmpLabel;
     keep start label fmtname type;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc format cntlin=work.&_tmpFormat library=work;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %let fmsetl_errorId = 3;
     %goto RETURNZERO;
  %end;

/***************************************************************************************/
  proc sql noprint;
      create table work.factSet_dtype_ids as select dim_type_id, ordinal
         from &cSDMLib..sas_fact_set_dim_type
         where fact_set_id = &fmsetl_fact_set_id ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data &lDTypes;
     length code $ %eval(32*&FM_DBCSMULT);
     length ordinal 8;
     set work.factSet_dtype_ids;
     code = put(dim_type_id, fmt_dtype_id_to_cd.);
     keep ordinal code dim_type_id;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sort data = &lDTypes;
    by ordinal;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %global dimcnt;
  %global dtype_ids;
  %global cBfDtypes;
  %global tmp_keep_dtypes;

  %let dimcnt= ;
  %let dtype_id= ;
  %let cBfDtypes= ;
  %let tmp_keep_dtypes= ;

  proc sql noprint;
     select count(*) into :dimcnt from &lDTypes;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data _null_ ;
     set &lDTypes end=nEnd ;
     if _n_ = 1 then do ;
        call symput("tmp_keep_dtypes",'"'||strip(code)||'"n');
        call symput("cBfDtypes",strip(code));
        call symput("dtype_ids",dim_type_id);
     end ;
     else do ;
        call symput("tmp_keep_dtypes",strip(symget("tmp_keep_dtypes"))||' "'||strip(code)||'"n');
        call symput("cBfDtypes",strip(symget("cBfDtypes"))||"|"||strip(code));
        call symput("dtype_ids",strip(symget("dtype_ids"))||","||dim_type_id);
     end ;
  run ;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:

  %let _FMErrorCode=0;

  %THEEND:

%mend mbrConcat;

/***************************************************************************************/
/* Create formats needed for deriving the internal dimension id for each dimension     */
/* type of the temporary fact table.                                                   */
/***************************************************************************************/
%macro DimDimTypeFormats;

  proc sql noprint;
    create table work._tmpalldims as
    select dimension_cd, internal_dimension_id, dim_type_id from &cSDMLib..sas_dimension
      where dimension_id in (select dimension_id from &cSDMLib..sas_vcube_hierarchy where vcube_id in
           (select vcube_id from &cSDMLib..sas_vcube where vcube_cd = "&tmp_model_code"));

  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %let _tmpSortVar = dim_type_id;
  proc sort data = work._tmpalldims;
     by &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %let _tmpWhere = where=(valid_from_dttm <= &today_dttm_val and
                          valid_to_dttm >= &today_dttm_val);

  data work._tmpsdmdimtype;
     set &cSDMLib..sas_dim_type (&_tmpWhere);
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sort data = work._tmpsdmdimtype;
     by &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data work._tmpalldims;
     merge work._tmpalldims (in = inA)
           work._tmpsdmdimtype;
        by &_tmpSortVar;
     keep dimension_cd internal_dimension_id dim_type_id dim_type_cd;
     if (inA) then
        output;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /* FMT_TO_DIM */

  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_TO_DIM;
  %let _tmpStart=DIM_TYPE_CD;
  %let _tmpLabel=DIMENSION_CD;
  data WORK.&_tmpFormat;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "C";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %let fmsetl_errorId = 3;
     %goto RETURNZERO;
  %end;

  /* FMT_DIMS */

  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DIMS;
  %let _tmpStart=DIM_TYPE_ID;
  %let _tmpLabel=INTERNAL_DIMENSION_ID;

  data work.&_tmpFormat;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %let fmsetl_errorId = 3;
     %goto RETURNZERO;
  %end;

  /* FMT_DTYPE_ID */

  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DTYPE_ID;
  %let _tmpStart=DIMENSION_CD;
  %let _tmpLabel=DIM_TYPE_ID;

  data work.&_tmpFormat;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %let fmsetl_errorId = 3;
     %goto RETURNZERO;
  %end;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:

  %let _FMErrorCode=0;

  %THEEND:

%mend DimDimTypeFormats;

/******************************************************************************************/
/* Creation of hash table to replace the less efficient formatting of each individual     */
/* dimension type.                                                                        */
/******************************************************************************************/
%macro createHashTable;

  %let _tmpDSN=&cSDMLib..sas_member;

  proc sql noprint;
    create table work.memberHashTable as select internal_dimension_id, member_cd, member_id
       from &_tmpDSN
       where internal_dimension_id in (select internal_dimension_id from work._tmpalldims) and
             valid_from_dttm <= &today_dttm_val and valid_to_dttm >= &today_dttm_val;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:

  %let _FMErrorCode=0;

  %THEEND:

%mend createHashTable;

/***************************************************************************************/
%macro createUpdateTable(input_table_name=, bfdtypes=, dataLoadObjTypeId=,
                         dataLoadObjId=, update_tablename= );

  %macro datawork/parmbuff;

    %let fmt=fmt_dtype;
    %let x=_x.;
    %let cd=_CD;

    %let _tmpMissingCdErr = 0;

    data &update_tablename;
       length OBJECT_TYPE_ID 8 ;
       length OBJECT_ID 8 ;
       length &tmp_keep_dtypes vc_found numchar startpos 8;
       length strcheck $ 3;
       length dimension_cd $ %eval(32*&FM_DBCSMULT);
       set &input_table_name;

       if _n_ = 1 then do;
          declare hash mht(dataset:"work.memberHashTable");
          mht.defineKey("internal_dimension_id", "member_cd");
          mht.defineData("member_id");
          mht.defineDone();
       end;

       OBJECT_TYPE_ID = &dataLoadObjTypeId ;
       OBJECT_ID = &dataLoadObjId;
       %let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));
       %do i = 1 %to &dimcnt;
           %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
           %let dimname=%bquote(%sysfunc(putc(&dimtype, $FMT_TO_DIM.)));
           %let dimtype_id=%bquote(%sysfunc(inputn(&dimname, FMT_DTYPE_ID.)));
           %let int_dim_id=%sysfunc(inputn(&dimtype_id, FMT_DIMS.));
           %let dimtypecd=%sysfunc(strip(&dimtype))&cd;
           dimension_cd = "&dimname";
           internal_dimension_id = &int_dim_id;
           member_cd = "&dimtypecd"n;
           vc_found = 0;
           if kindex(member_cd,'.vc') > 0 then do;
          numchar = klength(member_cd);
        startpos = numchar - 2;
        strcheck = ksubstr(member_cd, startpos, 3);
              if strcheck = '.vc' then do;
                 numchar = numchar - 3;
                 member_cd = ksubstr(member_cd, 1, numchar);
                 vc_found = 1;
              end;
           end;
           rc = mht.find();
           if rc = 0 then do;
              if vc_found = 1 then
                 member_id = member_id * (-1);
              "&dimtype"n = member_id;
           end;
           else do;
              call symput("_tmpMissingCdErr", 1);
              call symput("_tmpMem", strip(member_cd));
              call symput("_tmpDim", strip(dimension_cd));
              stop;
           end;
       %end;
       length FACT_VALUE 8;
       keep OBJECT_TYPE_ID OBJECT_ID &tmp_keep_dtypes FACT_VALUE;
    run;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;

    %if &_tmpMissingCdErr %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6,
                 msgTxt1=<&_tmpMem>, msgTxt2=<&_tmpDim>, msgPutlog=&putlog);
        %let fmsetl_errorId = 6;
        %goto RETURNZERO;
    %end;

    %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

    %RETURNZERO:

    %let _FMErrorCode=0;

    %THEEND:

  %mend datawork;

  %datawork(&bfdtypes);

  %if &_FMErrorCode = 0 %then
      %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend createUpdateTable;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_buildStatusDS.sas                                                    */
/*  Purpose: Build a temporary data set to hold status of this import.                 */
/*                                                                                     */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/
%macro fcst_buildStatusDS(cStatusDS, statusOverride);

  %let cMethodNm = fcst_buildStatusDS;
  %let cTempStatusDS = WORK._FMSETLTEMP01;
  %let nowDTTM = %qsysfunc(datetime()) ;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if %sysfunc(exist(&cStatusDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cStatusDS, 2, '.');
    quit;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;
  %end;

  %if %sysfunc(exist(&cTempStatusDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempStatusDS, 2, '.');
    quit;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;
  %end;

  data &cTempStatusDS(keep=TIME_ID ANALYSIS_ID PERIOD_ID STATUS_ID
                           IMPORTED_TRANS_CNT ZERO_TRANS_CNT INVALID_ERROR_CNT
                           REMOVE_CNT) ;
     length TIME_ID 8;
     length ANALYSIS_ID 8;
     length PERIOD_ID 8;
     TIME_ID = -999999;
     ANALYSIS_ID = -999999;
     PERIOD_ID = -999999;
     STATUS_ID = &statusOverride;
     IMPORTED_TRANS_CNT = 0 ;
     ZERO_TRANS_CNT = 0 ;
     INVALID_ERROR_CNT = 0 ;
     REMOVE_CNT = 0 ;
  run ;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc append base=&cStatusDS data=&cTempStatusDS &FM_PROCAPPENDFORCE;
  run ;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend fcst_buildStatusDS;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_updateValidStatusCnt.sas                                             */
/*  Purpose: Update Valid Status Count.                                                */
/*                                                                                     */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/

%macro fcst_updateValidStatusCnt(cStatusDS, cValidDS, cImpCntColNm);

  %let cMethodNm = fcst_updateValidStatusCnt;
  %let cSummaryDS = work._tempFact1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cValidDS
    ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm =
       case when status_id ne 0
         then 0
         else &_nobs
       end
     , status_id =
       case when status_id eq 0 and &_nobs eq 0
         then 3
         else status_id
       end
    ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend fcst_updateValidStatusCnt;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_updateZeroStatusCnt.sas                                              */
/*  Purpose: Update Zero Status Count.                                                 */
/*                                                                                     */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/

%macro fcst_updateZeroStatusCnt(cStatusDS, cValidDS, cImpCntColNm);

  %let cMethodNm = fcst_updateZeroStatusCnt;
  %let cSummaryDS = WORK._TEMPFACT1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cValidDS
    where FACT_VALUE eq 0
    ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm =
       case when status_id eq 2
         then 0
         else &_nobs
       end
     , status_id =
       case when status_id ne 2 and &_nobs gt 0 and IMPORTED_TRANS_CNT = 0
         then 0
         else status_id
       end
    ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend fcst_updateZeroStatusCnt;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_uploadTempFactToSDM.sas                                              */
/*  Purpose: Upload Temporary Fact Table to SDM.                                       */
/*                                                                                     */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/
%macro fcst_uploadTempFactToSDM (lDTypes, cTmpFactSDM_Nm, cTmpFactWork_Nm, cStatusDS);

  %let cMethodNm = fcst_uploadTempFactToSDM;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Get the list of Time and Analysis Ids that have okay import status   */

  proc sql noprint;
    select count(*)
      into :num_data
      from &cStatusDS
      where STATUS_ID = 0;
  quit;

  %let num_data = %eval(&num_data);

  %if &num_data > 0 %then %do;
      /* Find the location of SOURCE and CURRENCY dimension types */
      %let ord_source=;
      %let ord_currency=;

      data _null_;
          set &lDTypes end=IsLast;
          if code='SOURCE' or code='source' then do;
              call symput('ord_source', _N_-1);
          end;
          if code='CURRENCY' or code = 'currency' then do;
              call symput('ord_currency', _N_-1);
          end;
          if IsLast = 1 then
              call symput('num_dim_type', _N_);
      run;

      %let dim_col_def=;  /* List of dimension type column definition in DDL */
      %let dim_col=;      /* List of dimension type column names             */

      %do i=0 %to &num_dim_type-1;
        %if &i = 0 %then %do;
            %let dim_col_def=DIM_&i._MEM_ID integer;
            %if &i ^= &ord_source & &i ^= &ord_currency %then %do;
                 %let dim_col=DIM_&i._MEM_ID;
            %end;
        %end;
        %else %do;
            %let dim_col_def=&dim_col_def,DIM_&i._MEM_ID integer;
             %if &i ^= &ord_source & &i ^= &ord_currency  %then %do;
                %if "&dim_col" = "" %then %do;
                    %let dim_col=DIM_&i._MEM_ID;
                %end;
                %else %do;
                    %let dim_col=&dim_col, DIM_&i._MEM_ID;
                %end;
             %end;
        %end;
      %end;

      data &cTmpFactWork_Nm (/*drop=TIME ANALYSIS INTORG IDXCNT*/);
          length OBJECT_TYPE_ID 8 ;
          length OBJECT_ID 8 ;
          set &cTmpFactWork_Nm;
          length FACT_VALUE 8 ;
      run ;
      %checkErr;
      %if &_tmpRC %then %goto RETURNZERO;

      /*------------------------------------------------------------------------*/
      /* Decide the storage type of a temp fact table to be created in SASSDM */
      /*------------------------------------------------------------------------*/
      %let stg_type_tmp=myisam;

      /*-----------------------------------------*/
      /* Move ETL base fact from SAS into SASSDM */
      /*-----------------------------------------*/
      %let tmpUploadNm = PUBLIC.%scan(&cTmpFactSDM_Nm, 2);
      proc sql noprint;
           connect to &sesetl_EngineType as PASSTHRU ( &sesetl_BestLogin &sesetl_ConnectionOptions );

           execute (DROP TABLE IF EXISTS &tmpUploadNm
           ) by PASSTHRU;

           execute(CREATE TABLE &tmpUploadNm (
                     OBJECT_TYPE_ID INTEGER NOT NULL default 0
                   , OBJECT_ID INTEGER NOT NULL default 0
                   , &dim_col_def
                   , FACT_VALUE DOUBLE PRECISION  NOT NULL default 0
                   )
             ) by PASSTHRU ;

           disconnect from PASSTHRU ;
      quit;
      %checkErr;
      %if &_tmpRC %then %goto RETURNZERO;

      %fcst_uploadDataToSDM(&cTmpFactWork_Nm, &tmpUploadNm);
      %if &_FMErrorCode = 0 %then %do;
        %goto RETURNZERO;
      %end ;

  %end;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;
  %THEEND:

%mend fcst_uploadTempFactToSDM;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_uploadDataToSDM.sas                                                  */
/*  Purpose: Upload Table to SDM.                                                      */
/*                                                                                     */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/
%macro fcst_uploadDataToSDM(cSrcNm, cTrgNm);

  %let cMethodNm = fcst_uploadDataToSDM;
  %let nTrackTime = %qsysfunc(datetime());

  %let fmsetl_tempCheck1 = ;
  proc sql noprint ;
       select count(*)
         into :fmsetl_tempCheck1
         from &cSrcNm ;
  quit ;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %let nCnt = &fmsetl_tempCheck1;
  %if &nCnt eq . %then %let nCnt = 0 ;
  %if &nCnt EQ 0 %then %do;
      /*  No records found, skip the rest  */
    %goto THEEND;
  %end ;

  /*  Create output file name  */
  %let cExportCSV = %qsysfunc(strip(%qsysfunc(tranwrd(%qsysfunc(pathname(SASDATA)), \, /))))/upload%sysfunc(kcompress(%sysfunc( datetime() ), .)).txt;

  /* %let cSrcNm=%qsysfunc(&cSrcNm); */
  filename _outfile "&cExportCSV" encoding='utf-8';
  proc export data=&cSrcNm 
       outfile = _outfile
       dbms = DLM REPLACE;
       delimiter='~';
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
       connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );

       execute( copy &cTrgNm
                from %str(%')&cExportCSV.%str(%')
                with (format csv,
                      delimiter '~',
                      header)
              ) by &sesetl_EngineType ;

       disconnect from &sesetl_EngineType ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /*  Delete text file  */
  data _null_;
    length fref $8;
    fref = _blank_;
    rc = filename(fref, "&cExportCSV" );

    if rc=0 and fexist(fref) then
      rc=fdelete(fref) ;

    rc=filename(fref, '') ;
  run;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend fcst_uploadDataToSDM;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_replaceMatching.sas                                                  */
/*  Purpose: Replace matching process.                                                 */
/*                                                                                     */
/*  History: 29 April 2009 by Tae Sik Han : Converted from SAS to SAS macro program.   */
/***************************************************************************************/
%macro fcst_replaceMatching(cTmpROFactSDM_Nm,
                                cTmpRWFactSDM_Nm,
                                cStatusDS,
                                nActionType,
                                lDTypes,
                                cSdmLib,
                                nSdmROFactSetId,
                                nSdmRWFactSetId,
                                cDLObjTypeId,
                                cSASFactsDS,
                                cSDMFactsDB,
                                cETL_instanceID);

%let cMethodNm = fcst_replaceMatching;

%let cSDMFactsDB_NEW=/*PUBLIC.*/ %scan(&cSDMFactsDB, 2);
%let cTmpROFactSDM_Nm=/*PUBLIC.*/ %scan(&cTmpROFactSDM_Nm, 2);
%let cTmpRWFactSDM_Nm=/*PUBLIC.*/ %scan(&cTmpRWFactSDM_Nm, 2);

%let cSASFactsDS_NEW=&cSASFactsDS;

/* Initial assignment. It is changed depending on nActionType */
%let cTmp_ETL_BASE    =  _etl_sas_fact_base_&cETL_instanceID;
%let cTmp_ETL_NO_BASE =  _etl_sas_fact_no_base_&cETL_instanceID;

%let cTmp_RO_BASE    = _sas_fact_ro_base_&cETL_instanceID;
%let cTmp_RW_BASE    = _sas_fact_rw_base_&cETL_instanceID;
%let cTmp_RO_NO_BASE = _sas_fact_ro_no_base_&cETL_instanceID;
%let cTmp_RW_NO_BASE = _sas_fact_rw_no_base_&cETL_instanceID;

%let nowDTTM = %qsysfunc(datetime());
%let nTrackTime = %qsysfunc(datetime());

%let cTmpROFactSDM_Nm2=&cTmpROFactSDM_Nm._&cETL_instanceID;
%let cTmpRWFactSDM_Nm2=&cTmpRWFactSDM_Nm._&cETL_instanceID;

/*  If this is not a Replace Matching action, return , nActionType should be 4 or 5 */
%if &nActionType NE 4 and &nActionType NE 5 %then %do;
  %goto THEEND;
%end;

/* Find the location of SOURCE, and CURRENCY dimension types */
%let ord_source=;
%let ord_currency=;

data _null_;
    set &lDTypes end=IsLast;
    if upcase(code)='SOURCE' then do;
        call symput('ord_source', _N_-1);
    end;
    if upcase(code)='CURRENCY' then do;
        call symput('ord_currency', _N_-1);
    end;
    if IsLast = 1 then
        call symput('num_dim_type', _N_);
run;
%checkErr;
%if &_tmpRC %then %goto RETURNZERO;

%let dim_col_def=;  /* List of dimension type column definition in DDL */
%let dim_col=;      /* List of dimension type column names             */
%let idx_cols=0;    /* Number of columns to be included as part of the index.  Limit of 16 */

%do i=0 %to &num_dim_type-1;
    %if &i = 0 %then %do;
        %let dim_col_def=DIM_&i._MEM_ID integer;
        %if &i ^= &ord_source & &i ^= &ord_currency %then %do;
              %let dim_col=DIM_&i._MEM_ID;
              %let idx_cols = %eval(&idx_cols + 1);
        %end;
    %end;
    %else %do;
        %let dim_col_def=&dim_col_def,DIM_&i._MEM_ID integer;
        %if &idx_cols lt 16 & &i ^= &ord_source & &i ^= &ord_currency  %then %do;
            %if "&dim_col" = "" %then %do;
                %let dim_col=DIM_&i._MEM_ID;
            %end;
            %else %do;
                %let dim_col=&dim_col, DIM_&i._MEM_ID;
            %end;
            %let idx_cols = %eval(&idx_cols + 1);
         %end;
    %end;
%end;


/*------------------------------------------------------------------------*/
/* Decide the storage type of a temp fact table to be created in SASSDM */
/*------------------------------------------------------------------------*/
%let stg_type_tmp=myisam;

/*-----------------------------------------*/
/* Move ETL base fact from SAS into SASSDM */
/*-----------------------------------------*/
proc sql noprint;
     connect to &sesetl_EngineType as PASSTHRU ( &sesetl_BestLogin &sesetl_ConnectionOptions );

     execute (DROP TABLE IF EXISTS &cSDMFactsDB_NEW
     ) by PASSTHRU;

     execute(CREATE TABLE &cSDMFactsDB_NEW  (
               OBJECT_TYPE_ID INTEGER NOT NULL default 0,
               OBJECT_ID INTEGER NOT NULL default 0,
               &dim_col_def,
               FACT_VALUE DOUBLE PRECISION  NOT NULL default 0
             )
       ) by PASSTHRU ;

     disconnect from PASSTHRU ;

quit;
%checkErr;
%if &_tmpRC %then %goto RETURNZERO;


/* Remove zero value records from sas fact table  */
data &cSASFactsDS_NEW;
   set &cSASFactsDS;
run;
%checkErr;
%if &_tmpRC %then %goto RETURNZERO;

/* Transfer new sas fact table into database area */

%fcst_uploadDataToSDM(&cSASFactsDS_NEW, &cSDMFactsDB_NEW);

%if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
%end ;

%if &fcstdbug = 1 %then %do;
   data &dbuglib..result_&table_suffix;
      set &cSASFactsDS_NEW;
   run;
   %checkErr;
   %if &_tmpRC %then %goto RETURNZERO;
%end;

/*------------------------------------------*/
/* Decide storage type of two result tables */
/*------------------------------------------*/
%let stg_type_res=myisam;

%fcst_updateRemoveStatusCnt( &cStatusDS, &cSASFactsDS, REMOVE_CNT );

%if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
%end ;

/*  Join : extracting commonly found data idx                                                         */
/*  If nActionType =              4 : Join (etl_base x sas_base  U  etl_no_base x sas_no_base)        */
/*                                5 : Join (etl_no_base x sas_no_base)  : all splitted and joined     */
/*                   other than 4,5 : Join (etl_base x etl_base)        : Nothing splitted and joined */

/* Prepare conditional statement in where clause */

/* In case comparing all dimensions : no_split */
%do i=0 %to &num_dim_type-1;
    %if (&i = 0) %then %do;
        %let where_no_split=S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
    %end;
    %else %do;
        %let where_no_split=&where_no_split and S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
    %end;
%end;

/* In case comparing all except source and currency */
%let where_split=;
%do i=0 %to &num_dim_type-1;
    %if ("&where_split" = "") %then %do;
        %if (&i = &ord_currency or &i = &ord_source) %then %do;
            %let where_split=;
        %end;
        %else %do;
            %let where_split=S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
        %end;
    %end;
    %else %do;
        %if (&i ^= &ord_currency and &i ^= &ord_source) %then %do;
            %let where_split=&where_split and S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
        %end;
    %end;
%end;

%let _tmpindex = idx_dims_%sysfunc(kcompress(%sysfunc( datetime() ), .));

proc sql noprint;
     connect to &sesetl_EngineType as PASSTHRU ( &sesetl_BestLogin &sesetl_ConnectionOptions );

     /* Create a composite index on sas etl fact table */
     execute(CREATE INDEX &_tmpindex on &cSDMFactsDB_NEW (&dim_col)
      ) by PASSTHRU ;

    /*-----------------------------------*/
    /* Prepare tables containing results */
    /*-----------------------------------*/

    /* Create result tables and temporary tables */
    execute (DROP TABLE IF EXISTS &cTmpROFactSDM_Nm
         ) by PASSTHRU;

    execute (CREATE TABLE &cTmpROFactSDM_Nm (
               FACT_IDX INTEGER NOT NULL,
               PRIMARY KEY (FACT_IDX)
             )
       ) by PASSTHRU;

    execute (CREATE RULE "ROFact_on_duplicate_ignore" AS ON INSERT TO &cTmpROFactSDM_Nm
               WHERE EXISTS (SELECT 1 FROM &cTmpROFactSDM_Nm
                             WHERE (FACT_IDX)=(NEW.FACT_IDX))
               DO INSTEAD NOTHING
       ) by PASSTHRU;

    execute (DROP TABLE IF EXISTS &cTmpRWFactSDM_Nm
         ) by PASSTHRU;

    execute (CREATE TABLE &cTmpRWFactSDM_Nm (
               FACT_IDX INTEGER NOT NULL,
               PRIMARY KEY (FACT_IDX)
             )
       ) by PASSTHRU;

    execute (CREATE RULE "RWFact_on_duplicate_ignore" AS ON INSERT TO &cTmpRWFactSDM_Nm
               WHERE EXISTS (SELECT 1 FROM &cTmpRWFactSDM_Nm
                             WHERE (FACT_IDX)=(NEW.FACT_IDX))
               DO INSTEAD NOTHING
       ) by PASSTHRU;


    /*---------------------------*/
    /* Join by concat_member_ids */
    /*---------------------------*/

    %if &nActionType = 4 %then %do;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where M.object_type_id=&cDLObjTypeId
                     and &where_no_split
        ) by PASSTHRU;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where M.object_type_id<>&cDLObjTypeId
                     and &where_split
        ) by PASSTHRU;

        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where M.object_type_id=&cDLObjTypeId
                     and &where_no_split
        ) by PASSTHRU;

        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where M.object_type_id<>&cDLObjTypeId
                     and &where_split
        ) by PASSTHRU;
    %end;
    %else %if &nActionType = 5 %then %do;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where &where_split
        ) by PASSTHRU;
        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where &where_split
        ) by PASSTHRU;
    %end;
    %else %do;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where &where_no_split
        ) by PASSTHRU;

        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where &where_no_split
        ) by PASSTHRU;

    %end;

    execute (DROP RULE "ROFact_on_duplicate_ignore" ON &cTmpROFactSDM_Nm
    ) by PASSTHRU;

    execute (DROP RULE "RWFact_on_duplicate_ignore" ON &cTmpRWFactSDM_Nm
    ) by PASSTHRU;


    disconnect from PASSTHRU ;

quit;
%checkErr;
%if &_tmpRC %then %goto RETURNZERO;


%goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

%ERRORRETURN:
  * %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

%RETURNZERO:

%let _FMErrorCode=0;

%THEEND:

%mend fcst_replaceMatching;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_updateRemoveStatusCnt.sas                                            */
/*  Purpose: Update Valid Status Count.                                                */
/*                                                                                     */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/
%macro fcst_updateRemoveStatusCnt(cStatusDS, cDS, cImpCntColNm);

  %let cMethodNm = fcst_updateRemoveStatusCnt;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %let _nobs = 0;
  data _null_;
    set &cDS nobs=_nobs;
    call symput('_nobs', _nobs);
    stop;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm = &cImpCntColNm +
       case when status_id ne 0
         then 0
         else &_nobs
       end
    ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend fcst_updateRemoveStatusCnt;

/***************************************************************************************/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.       */
/*  Name   : fcst_uploadStatusTableToSDM.sas                                           */
/*  Putpose: Upload Status Table to SDM.                                               */
/*                                                                                     */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.    */
/***************************************************************************************/

%macro fcst_uploadStatusTableToSDM( cSrcTable, cTrgTable);

  %let cMethodNm = fcst_uploadStatusTable;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %let tmpUploadNm = PUBLIC.%scan(&cTrgTable, 2);

  proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(CREATE TABLE &tmpUploadNm (
             TIME_ID INTEGER,
             ANALYSIS_ID INTEGER,
             PERIOD_ID INTEGER,
             STATUS_ID INTEGER,
             IMPORTED_TRANS_CNT INTEGER,
             ZERO_TRANS_CNT INTEGER,
             INVALID_ERROR_CNT INTEGER,
             REMOVE_CNT INTEGER
           )
     ) by &sesetl_EngineType ;
   disconnect from &sesetl_EngineType ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %fcst_uploadDataToSDM(&cSrcTable, &tmpUploadNm);
  %if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
  %end ;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:

  %let _FMErrorCode=0;

  %THEEND:

%mend fcst_uploadStatusTableToSDM;

/***************************************************************************************/
