%macro fcstxtrt;

  /******************************************************************************************/
  /* Create a table consisting of the model's time hierarchy members that are at the same   */
  /* level as the time members to be forecast upon...week, month, year, etc.                */
  /******************************************************************************************/
  proc sql noprint;
     create table work.tmp_time_members as
     select distinct member_cd from &cSDMLib..sas_member
     where member_id in (select member_id from &cSDMLib..sas_hierarchy_member
                         where hierarchy_id = &_tmp_time_hierarchy and
                               valid_from_dttm <= &_tmp_asof_dttm and
                               valid_to_dttm >= &_tmp_asof_dttm) and
           member_id in (select member_id from work.leafDatesOnly);
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  data work.tmp_time_members;
     set work.tmp_time_members;
     date_value = input(member_cd, fmt_mp_date.);
     if &predictor_variables_count > 0 then
        if date_value >= &tmp_hist_start_time;
     else
        if date_value >= &tmp_hist_start_time and date_value < &fcst_start_date;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc sql noprint;
    select count(*) into :time_cnt
    from work.tmp_time_members;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /******************************************************************************************/
  /* Create temporary table of TIME members to be included in query                         */
  /******************************************************************************************/
  data work.query_time_mbrs;
     length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
     length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
     set work.tmp_time_members;
     INCLUDE_MEMBER=1;
     INCLUDE_LEAVES=0;
     INCLUDE_ROLLUPS=0;
     DIMENSION_TYPE_CD = "TIME";
     keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /******************************************************************************************/
  /* Combine the ANALYSIS members and TIME members in query table                           */
  /******************************************************************************************/
  data work.&tempds;
     length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
     length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
     length tmp_pred_source $ %eval(32*&FM_DBCSMULT);
     INCLUDE_MEMBER=1;
     INCLUDE_LEAVES=0;
     INCLUDE_ROLLUPS=0;
     DIMENSION_TYPE_CD = "ANALYSIS";
     MEMBER_CD = "&tmp_analysis_historical_mbr";
     output;
     /* Only include forecast analysis member if predictor variables are */
     /* used and predictor source = "Choice Use Supplied Values".        */
     if &predictor_variables_count > 0 then do;
        tmp_pred_source = "&tmp_predictor_src";
        if tmp_pred_source = "Choice Use Supplied Values" then do;
           MEMBER_CD = "&tmp_analysis_forecast_mbr";
           output;
        end;
     end;
     keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc append base=work.&tempds data=work.query_time_mbrs &FM_PROCAPPENDFORCE;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /******************************************************************************************/
  /* Add Frequency member "PTD" to query table                                              */
  /******************************************************************************************/
  data work.query_frequency_mbr;
     length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
     length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
     INCLUDE_MEMBER=1;
     INCLUDE_LEAVES=0;
     INCLUDE_ROLLUPS=0;
     DIMENSION_TYPE_CD = "FREQUENCY";
     MEMBER_CD = "PTD";
     output;
     keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  proc append base=work.&tempds data=work.query_frequency_mbr &FM_PROCAPPENDFORCE;
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /***************************************************************************************/
  /*            Add By Variable dimension type members into the query table.             */
  /***************************************************************************************/
  %if &by_vars_dimension_codes_count > 0 %then
     %do i = 1 %to &by_vars_dimension_codes_count;

        /* Determine dimension type */
        data _null_;
           length tmp_dim_code $ %eval(32*&FM_DBCSMULT);
           length dimension_type_cd $ %eval(32*&FM_DBCSMULT);
           tmp_dim_code = "&&tmp_by_var_dim&i";
           dimension_type_cd = put(tmp_dim_code, $fmt_dim_to_dtype.);
           call symput("mbrDimType", dimension_type_cd);
        run;
        %checkErr;
        %if &_tmpRC %then %goto RETURNZERO;

        %let _tmpWhere = where=(etl_instance_id = &fmsetl_instanceId and dimension_cd = "&&tmp_by_var_dim&i");

        %if &mbrDimType = INTORG %then %do;
           data org_codes (keep = member_cd)
                currency_codes (keep = member_cd);
              length member_cd $ %eval(32*&FM_DBCSMULT);
              length intorg_cd $ %eval(32*&FM_DBCSMULT);
              length currency_cd $ %eval(32*&FM_DBCSMULT);
              set &cSDMLib..sas_etl_fact_dim_mbr (&_tmpWhere);
              if _n_ = 1 then do;
                 declare hash fcht(dataset:"work.org_to_currency");
                 fcht.defineKey("intorg_cd");
                 fcht.defineData("currency_cd");
                 fcht.defineDone();
              end;
              output org_codes;
              intorg_cd = member_cd;
              rc = fcht.find();
              if rc = 0 then
                 member_cd = currency_cd;
              output currency_codes;
           run;
           %checkErr;
           %if &_tmpRC %then %goto RETURNZERO;
        %end;
        %else %do;
           data work.member_codes;
              length dimension_type_cd $ %eval(32*&FM_DBCSMULT);
              length include_member 8;
              length include_leaves 8;
              length include_rollups 8;
              include_member=1;
              include_leaves=0;
              include_rollups=0;
              set &cSDMLib..sas_etl_fact_dim_mbr (&_tmpWhere);
              dimension_type_cd = put("&&tmp_by_var_dim&i", $fmt_dim_to_dtype.);
              keep dimension_type_cd member_cd include_member include_leaves include_rollups;
           run;
           %checkErr;
           %if &_tmpRC %then %goto RETURNZERO;

           proc append base=work.&tempds data=work.member_codes &FM_PROCAPPENDFORCE;
           run;
           %checkErr;
           %if &_tmpRC %then %goto RETURNZERO;
        %end;
     %end;

  /******************************************************************************************/
  /* Takes the dimension code (code_value) and finds it's dimension type and then extracts  */
  /* each member in the mbr_string adding each dimension type and member combination to the */
  /* query table.                                                                           */
  /******************************************************************************************/
  %macro hpf_decode_mbrs(code_value=, mbr_string=);

  /* Determine dimension type */
  data _null_;
     length tmp_dim_code $ %eval(32*&FM_DBCSMULT);
     length dimension_type_cd $ %eval(32*&FM_DBCSMULT);
     tmp_dim_code = "&code_value";
     dimension_type_cd = put(tmp_dim_code, $fmt_dim_to_dtype.);
     call symput("mbrDimType", dimension_type_cd);
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /* Begin process of extracting members from the string */
  %let code_string_length = %klength(&mbr_string);

  /* Special handling for INTORG as a separate query needs to be performed later for each */
  /* org / currency combination. */
  %if %bquote(&mbrDimType) = INTORG %then %do;
     data tmp_org_codes;
        length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
        length str_length_pos 8;
        length num_char 8;
        length start_pos 8;
        length period_check 8;
        tmp_string = "&mbr_string";
        str_length_pos = 1;
        do until (str_length_pos > &code_string_length);
           period_check = str_length_pos + 1;
           if ksubstr(tmp_string, period_check, 1) = '.' then do;
              num_char = ksubstr(tmp_string, str_length_pos, 1);
              start_pos = sum(str_length_pos, 2);
           end;
           else do;
              num_char = ksubstr(tmp_string, str_length_pos, 2);
              start_pos = sum(str_length_pos, 3);
           end;
           MEMBER_CD = ksubstr(tmp_string, start_pos, num_char);
           output;
           str_length_pos = sum(start_pos, num_char, 1);
        end;
        keep MEMBER_CD;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     proc append base=work.org_codes data=work.tmp_org_codes &FM_PROCAPPENDFORCE;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     data tmp_currency_codes (keep = member_cd);
        length member_cd $ %eval(32*&FM_DBCSMULT);
        length intorg_cd $ %eval(32*&FM_DBCSMULT);
        length currency_cd $ %eval(32*&FM_DBCSMULT);
        set tmp_org_codes;
        if _n_ = 1 then do;
           declare hash fcht(dataset:"work.org_to_currency");
           fcht.defineKey("intorg_cd");
           fcht.defineData("currency_cd");
           fcht.defineDone();
        end;
        intorg_cd = member_cd;
        rc = fcht.find();
        if rc = 0 then
           member_cd = currency_cd;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     proc append base=work.currency_codes data=work.tmp_currency_codes &FM_PROCAPPENDFORCE;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

  %end;
  %else %do;
     /* If not the INTORG dimension type then start loading members to the query table. */
     /* Begin process of extracting members from the string */
     data member_codes;
        length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
        length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
        length INCLUDE_MEMBER 8;
        length INCLUDE_LEAVES 8;
        length INCLUDE_ROLLUPS 8;
        length code_check $ %eval(32*&FM_DBCSMULT);
        length str_length_pos 8;
        length num_char 8;
        length start_pos 8;
        length period_check 8;
        DIMENSION_TYPE_CD = "&mbrDimType";
        INCLUDE_MEMBER=1;
        INCLUDE_LEAVES=0;
        INCLUDE_ROLLUPS=0;
        tmp_string = "&mbr_string";
        str_length_pos = 1;
        do until (str_length_pos > &code_string_length);
           period_check = str_length_pos + 1;
           if ksubstr(tmp_string, period_check, 1) = '.' then do;
              num_char = ksubstr(tmp_string, str_length_pos, 1);
              start_pos = sum(str_length_pos, 2);
           end;
           else do;
              num_char = ksubstr(tmp_string, str_length_pos, 2);
              start_pos = sum(str_length_pos, 3);
           end;
           MEMBER_CD = ksubstr(tmp_string, start_pos, num_char);
           output;
           str_length_pos = sum(start_pos, num_char, 1);
        end;
        keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     proc append base=work.&tempds data=work.member_codes &FM_PROCAPPENDFORCE;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

  %mend hpf_decode_mbrs;

  /***************************************************************************************/
  /* Add Business Driver dimension type members into the query table */
  %if &forecast_vars_member_codes_count = 1 %then %do;
     %hpf_decode_mbrs(code_value=%nrbquote(&tmp_forecast_vars_dim),
                      mbr_string=%nrbquote(&forecast_vars_member_codes));
     %if &_FMErrorCode = 0 %then
         %goto RETURNZERO;
  %end;
  %else
     %do i = 1 %to &forecast_vars_member_codes_count;
        %hpf_decode_mbrs(code_value=%nrbquote(&tmp_forecast_vars_dim),
                         mbr_string=%nrbquote(&&forecast_vars_member_codes&i));
        %if &_FMErrorCode = 0 %then
            %goto RETURNZERO;
     %end;

  /* Add Predictor Variable dimension type members into the query table */
  %if &predictor_variables_count > 0 %then %do;
     %if &predictor_variables_count = 1 %then %do;
        %hpf_decode_mbrs(code_value=%nrbquote(&tmp_forecast_vars_dim),
                         mbr_string=%nrbquote(&predictor_variables));
        %if &_FMErrorCode = 0 %then
           %goto RETURNZERO;
     %end;
     %else %do;
        %do i = 1 %to &predictor_variables_count;
           %hpf_decode_mbrs(code_value=%nrbquote(&tmp_forecast_vars_dim),
                            mbr_string=%nrbquote(&&predictor_variables&i));
           %if &_FMErrorCode = 0 %then
              %goto RETURNZERO;
        %end;
     %end;
  %end;

  /* Add members for all of the Slicer dimension types to the query table */
  %if &slicer_dim_codes_count > 0 %then %do;
     %do i = 1 %to &slicer_dim_codes_count;
        %hpf_decode_mbrs(code_value=%nrbquote(&&tmp_slicer_dim&i),
                         mbr_string=%nrbquote(&&tmp_slicer_read_mem&i));
        %if &_FMErrorCode = 0 %then
           %goto RETURNZERO;
     %end;
  %end;

  /***************************************************************************************/
  /*         Create macro variables for each org and it's functional currency.           */
  /***************************************************************************************/
  proc sql noprint ;
    select count(*) into :org_cnt
    from work.org_codes ;
  quit ;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %do i = 1 %to &org_cnt;
     data _null_;
        set work.org_codes (firstobs=&i obs=&i);
        call symput("org&i", member_cd);
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     data _null_;
        set work.currency_codes (firstobs=&i obs=&i);
        call symput("currency&i", member_cd);
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  /***************************************************************************************/

  %do i = 1 %to &time_cnt;
     data _null_;
        set work.tmp_time_members (firstobs=&i obs=&i);
        call symput("time&i", member_cd);
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  /***************************************************************************************/
  /* For each crossing instance perform a separate query for facts.                      */
  /***************************************************************************************/

  %let member_cnt1 = 0;
  %let member_cnt2 = 0;
  %let member_cnt3 = 0;

  %do i = 1 %to &query_crossing_count;
     %if &&tmp_query_dtype&i eq TIME %then %do;
         %let member_cnt&i = &time_cnt;
         %let query_dtype&i = TIME;
         %do a = 1 %to &time_cnt;
            %let query&i._mem&a = %bquote(&&time&a);
         %end;
     %end;
     %else %if &&tmp_query_dtype&i eq INTORG or (&&tmp_query_dtype&i eq TARGET and
               &tmp_target_dim_type eq INTORG) %then %do;
         %let member_cnt&i = &org_cnt;
         %let query_dtype&i = INTORG;
         %do b = 1 %to &org_cnt;
            %let query&i._mem&b = %bquote(&&org&b);
         %end;
         %let currency_idx = &i;
     %end;
     %else %do;
         %let member_cnt&i = &target_member_codes_count;
         %let query_dtype&i = %bquote(&tmp_target_dim_type);
         %do c = 1 %to &target_member_codes_count;
            %let query&i._mem&c = %bquote(&&tmp_target_mem&c);
         %end;
     %end;
  %end;

  %do i = 1 %to &member_cnt1;
     %if &query_crossing_count = 1 %then %do;
        data work.query_crossing&i;
           length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
           length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
           length INCLUDE_MEMBER 8;
           length INCLUDE_LEAVES 8;
           length INCLUDE_ROLLUPS 8;
           INCLUDE_MEMBER=1;
           INCLUDE_LEAVES=0;
           INCLUDE_ROLLUPS=0;
           DIMENSION_TYPE_CD = "&query_dtype1";
           MEMBER_CD = "&&query1_mem&i";
           output;
           DIMENSION_TYPE_CD = "CURRENCY";
           MEMBER_CD = "&&currency&i";
           output;
           keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
        run;
     %end;
     %else %do j = 1 %to &member_cnt2;
        %if &query_crossing_count = 2 %then %do;
           %let l = %eval( ((&i-1) * &member_cnt2) + &j);
           data work.query_crossing&l;
              length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
              length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
              length INCLUDE_MEMBER 8;
              length INCLUDE_LEAVES 8;
              length INCLUDE_ROLLUPS 8;
              INCLUDE_MEMBER=1;
              INCLUDE_LEAVES=0;
              INCLUDE_ROLLUPS=0;
              DIMENSION_TYPE_CD = "&query_dtype1";
              MEMBER_CD = "&&query1_mem&i";
              output;
              DIMENSION_TYPE_CD = "&query_dtype2";
              MEMBER_CD = "&&query2_mem&j";
              output;
              DIMENSION_TYPE_CD = "CURRENCY";
              if &currency_idx = 1 then
                 MEMBER_CD = "&&currency&i";
              else
                 MEMBER_CD = "&&currency&j";
              output;
              keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
           run;
        %end;
        %else %do k = 1 %to &member_cnt3;
           %let l = %eval( ((&i-1) * &member_cnt2 * &member_cnt3) + ((&j-1) * &member_cnt3) + &k);
           data work.query_crossing&l;
              length DIMENSION_TYPE_CD $ %eval(32*&FM_DBCSMULT);
              length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
              length INCLUDE_MEMBER 8;
              length INCLUDE_LEAVES 8;
              length INCLUDE_ROLLUPS 8;
              INCLUDE_MEMBER=1;
              INCLUDE_LEAVES=0;
              INCLUDE_ROLLUPS=0;
              DIMENSION_TYPE_CD = "&query_dtype1";
              MEMBER_CD = "&&query1_mem&i";
              output;
              DIMENSION_TYPE_CD = "&query_dtype2";
              MEMBER_CD = "&&query2_mem&j";
              output;
              DIMENSION_TYPE_CD = "&query_dtype3";
              MEMBER_CD = "&&query3_mem&k";
              output;
              DIMENSION_TYPE_CD = "CURRENCY";
              if &currency_idx = 1 then
                 MEMBER_CD = "&&currency&i";
              else if &currency_idx = 2 then
                 MEMBER_CD = "&&currency&j";
              else
                 MEMBER_CD = "&&currency&k";
              output;
              keep DIMENSION_TYPE_CD MEMBER_CD INCLUDE_MEMBER INCLUDE_LEAVES INCLUDE_ROLLUPS;
           run;
        %end;
     %end;
  %end;

  proc sql noprint;
    delete from work.&tempds
    where dimension_type_cd in
          (select distinct dimension_type_cd from work.query_crossing1);
  quit;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if &member_cnt3 > 0 %then
     %let queryCnt = %eval(&member_cnt1 * &member_cnt2 * &member_cnt3);
  %else %if &member_cnt2 > 0 %then
     %let queryCnt = %eval(&member_cnt1 * &member_cnt2);
  %else
     %let queryCnt = %eval(&member_cnt1);

  %do i = 1 %to &queryCnt;
     %let j = %eval(&i - 1);
     data &cSASData..&tempds&j;
        set work.&tempds
            work.query_crossing&i;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
     proc sort data=&cSASData..&tempds&j nodupkey;
        by dimension_type_cd member_cd;
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  %let y = 0;

  %macro query_count;

    proc sql noprint;
       create table query_dtypes as select distinct dimension_type_cd
       from &cSASData..&tempds&y;
    quit;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;

    proc sql noprint;
      select count(*) into :dtype_cnt
      from work.query_dtypes;
    quit;
    %checkErr;
    %if &_tmpRC %then %goto RETURNZERO;

    %global total_crossings;
    %let total_crossings = 1;

    %do x = 1 %to &dtype_cnt;
       data _null_;
          set query_dtypes(firstobs=&x obs=&x);
          call symput("query_dtype&x", dimension_type_cd);
       run;
       %checkErr;
       %if &_tmpRC %then %goto RETURNZERO;
       proc sql noprint;
          select count(*) into :tmp_dtype_cnt from &cSASData..&tempds&y
          where upcase(dimension_type_cd) = upcase("&&query_dtype&x");
       quit;
       %checkErr;
       %if &_tmpRC %then %goto RETURNZERO;
       %let total_crossings = %eval(&total_crossings * &tmp_dtype_cnt);
    %end;

    %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

    %RETURNZERO:
      %let _FMErrorCode=0;

    %THEEND:

  %mend;

  %if &fcstdbug = 1 %then %do;
     %query_count;
     %if &_FMErrorCode = 0 %then
        %goto RETURNZERO;
     %let all_crossings = %eval (&queryCnt * &total_crossings);
     %put NUMBER OF QUERIES: &queryCnt;
     %put CROSSINGS PER QUERY: &total_crossings;
     %put TOTAL CROSSINGS: &all_crossings;
  %end;

  %let nativeSign=1;

  %fcstqry(sasLibraryName="&cSASData", modelCode="&tmp_model_code",
           queryDataSetRootName="&tempds", resultDataSetRootName="&outds", queryCount="&queryCnt",
           concurrentQueryCount="&concurrent_query_count", nativeSign="&nativeSign");

  %global forecast_dtype;

  /* Determine the business driver dimension type */
  data _null_;
     length tmp_fcst_dtype $ %eval(32*&FM_DBCSMULT);
     tmp_fcst_dtype = put("&tmp_forecast_vars_dim", $fmt_dim_to_dtype.);
     call symput("forecast_dtype", strip(tmp_fcst_dtype));
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  %if &predictor_variables_count > 0 %then %do;
     /* initialize each of the predictor variable value columns as numeric */
     filename _predVal temp;
     data _null_;
        file _predVal;
        put 'length ';
        %do k = 1 %to &predictor_variables_count;
           put '"';
           put "&&tmp_predictor_mem&k";
           put '"n ' ;
        %end;
        put '8';
     run;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;
  %end;

  %let code = _CODE;
  %let maxIdx = %eval(&queryCnt - 1);

  %do i = 0 %to &maxIdx;
     %if &predictor_variables_count > 0 %then %do;
        data work.query_output;
           %let fcstcode=%sysfunc(strip(&forecast_dtype))&code;
           length "&fcstcode"n $ %eval(32*&FM_DBCSMULT);
           length date_value 8;
           %inc _predVal;
           ;
           set &cSASData..&outds&i;
           date_value = input(TIME_CODE, fmt_mp_date.);
           %do j = 1 %to &predictor_variables_count;
              if "&fcstcode"n = "&&tmp_predictor_mem&j" then do;
                 "&&tmp_predictor_mem&j"n = value;
                 value = .;
              end;
           %end;
        run;
        %checkErr;
        %if &_tmpRC %then %goto RETURNZERO;

        proc append base=&cSASData..&OUTDS data=work.query_output &FM_PROCAPPENDFORCE;
        run;
        %checkErr;
        %if &_tmpRC %then %goto RETURNZERO;
     %end;
     %else %do;
        data work.query_output;
           length date_value 8;
           set &cSASData..&outds&i;
           date_value = input(TIME_CODE, fmt_mp_date.);
        run;
        %checkErr;
        %if &_tmpRC %then %goto RETURNZERO;

        proc append base=&cSASData..&OUTDS data=work.query_output &FM_PROCAPPENDFORCE;
        run;
        %checkErr;
        %if &_tmpRC %then %goto RETURNZERO;
     %end;

    /* Clean Up */

     proc sql noprint;
        drop table &cSASData..&tempds&i;
     quit;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

     proc sql noprint;
        drop table &cSASData..&outds&i;
     quit;
     %checkErr;
     %if &_tmpRC %then %goto RETURNZERO;

  %end;

  /* Remove all facts that preceeded the first historical period */
  data &cSASData..&OUTDS;
     set &cSASData..&OUTDS;
     if date_value ge &tmp_hist_start_time and
        not (ANALYSIS_CODE = "&tmp_analysis_historical_mbr" and date_value ge &fcst_start_date);
  run;
  %checkErr;
  %if &_tmpRC %then %goto RETURNZERO;

  /***************************************************************************************/

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:

  %let _FMErrorCode=0;

  %THEEND:

%mend fcstxtrt;
