/* Copyright 2006 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro impsourc;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE,
          sesetl_validateLib=Y
         ) ;

%let dflt_from_dttm = '01JAN1970:00:00'DT;
%let curr_rec_dttm = '01JAN5999:00:00'DT;

%if &SYSSCP = WIN %then %do;
    %let propspth=%SYSGET(SASROOT)\finance\sasmisc;
%end;
%else %do;
    %let propspth=%SYSGET(SASROOT)/misc/finance;
%end;

%localze(ses_property_file_location=&propspth, ses_locale=en);

%if (&sas_install_default_locale ne en) %then
   %localze(ses_property_file_location=&propspth, ses_locale=&sas_install_default_locale);

/* Create the dimension type record for the SOURCE dimension type */

proc sql noprint;
   select count(*) into :dtype_rec_cnt from _SDMLIB.SAS_DIM_TYPE
   where DIM_TYPE_CD = 'SOURCE';
quit;

%if (&dtype_rec_cnt < 1) %then %do;

   proc sql noprint;
     select KEY_CNT into :dtype_id from _SDMLIB.SAS_KEYGEN
     where IDENTIFIER_CD = 'odcs/dimension_type';
   quit;

   /* Set 'odcs/dimension_type' in SAS_KEYGEN */
   %sesskey(keyCode=%str(odcs/dimension_type)) ;

   data work.dim_type;
      length DIM_TYPE_ID 8;
      length REVISION 8;
      length DIM_TYPE_ROLE 8;
      length INTERNAL_DIM_TYPE_ID 8;
      length DIM_TYPE_CD $ 32;
      length HIDDEN_FLG $5;
      length DFLT_DIMENSION_ID 8;
      length LOCALE_ID 8;
      length DIM_TYPE_NM $ 50;
      length DIM_TYPE_DESC $ 255;
      DIM_TYPE_ID = &dtype_id;
      REVISION = 1;
      DIM_TYPE_ROLE = 14;
      INTERNAL_DIM_TYPE_ID = DIM_TYPE_ID * (-1);
      DIM_TYPE_CD = 'SOURCE';
      HIDDEN_FLG = "false";
      DFLT_DIMENSION_ID = 0;
      LOCALE_ID = 1;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_SOURCE_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_SOURCE_DESC_VAR)";
      FORMAT VALID_FROM_DTTM DATETIME25.;
      VALID_FROM_DTTM = &dflt_from_dttm;
      FORMAT VALID_TO_DTTM DATETIME25.;
      VALID_TO_DTTM = &curr_rec_dttm;
      keep DIM_TYPE_ID REVISION DIM_TYPE_ROLE INTERNAL_DIM_TYPE_ID DIM_TYPE_CD HIDDEN_FLG
           DFLT_DIMENSION_ID LOCALE_ID DIM_TYPE_NM DIM_TYPE_DESC VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;
   data work.dim_type_nls;
      set work.dim_type;
      keep DIM_TYPE_ID LOCALE_ID REVISION DIM_TYPE_NM DIM_TYPE_DESC VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;
   data work.dim_type;
      set work.dim_type;
      keep DIM_TYPE_ID REVISION DIM_TYPE_ROLE INTERNAL_DIM_TYPE_ID DIM_TYPE_CD HIDDEN_FLG
           DFLT_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM;
   run;
   proc append base=_SDMLIB.SAS_DIM_TYPE data=work.dim_type &FM_PROCAPPENDFORCE;
   run;
   %if &ses_locale_found eq 1 %then %do;
      proc append base=_SDMLIB.SAS_DIM_TYPE_NLS data=work.dim_type_nls &FM_PROCAPPENDFORCE;
      run;
   %end;
   %addhstry(action_type_id=18, solution_id=46, object_type_id=2002,
             object_id_table=work.dim_type, object_id_column_nm=DIM_TYPE_ID,
             bldlibnm_sassdm=N);
%end;
%else %do;
   proc sql noprint;
      select DIM_TYPE_ID into :dtype_id from _SDMLIB.SAS_DIM_TYPE
      where DIM_TYPE_CD = 'SOURCE';
   quit;
%end;

/* Verify program hasn't already been run and if so, abort */

proc sql noprint;
   select count(*) into :dim_rec_cnt from _SDMLIB.SAS_DIMENSION
   where DIMENSION_CD = 'SOURCE';
quit;

%if (&dim_rec_cnt > 0) %then
    %return;

proc sql noprint;
  select KEY_CNT into :dim_id from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/dimension';
quit;

/* Set 'odcs/dimension' in SAS_KEYGEN */
%sesskey(keyCode=%str(odcs/dimension)) ;

/* Create dimension record for the SOURCE dimension */

data work.dimension;
   length DIMENSION_ID 8;
   length REVISION 8;
   length INTERNAL_DIMENSION_ID 8;
   length DIM_TYPE_ID 8;
   length DIMENSION_CD $ 32;
   length DFLT_HIERARCHY_ID 8;
   length EDITABLE_FLG $5;
   length EDITABLE_PAST_FLG $5;
   length HIDDEN_FLG $5;
   length LOCALE_ID 8;
   length DIMENSION_NM $ 50;
   length DIMENSION_DESC $ 255;
   DIMENSION_ID = &dim_id;
   REVISION = 1;
   INTERNAL_DIMENSION_ID = &dim_id * (-1);
   DIM_TYPE_ID = &dtype_id;
   DIMENSION_CD = 'SOURCE';
   EDITABLE_FLG = "false";
   EDITABLE_PAST_FLG = "false";
   HIDDEN_FLG = "false";
   LOCALE_ID = 1;
   DIMENSION_NM = "%nrbquote(&SES_DIMEN_SOURCE_NM_VAR)";
   DIMENSION_DESC = "%nrbquote(&SES_DIMEN_SOURCE_DESC_VAR)";
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   keep DIMENSION_ID REVISION INTERNAL_DIMENSION_ID DIM_TYPE_ID DIMENSION_CD DFLT_HIERARCHY_ID
        EDITABLE_FLG EDITABLE_PAST_FLG HIDDEN_FLG LOCALE_ID DIMENSION_NM DIMENSION_DESC
        VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.dimension_nls;
   set work.dimension;
   keep DIMENSION_ID LOCALE_ID REVISION DIMENSION_NM DIMENSION_DESC VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

data work.dimension;
   set work.dimension;
   keep DIMENSION_ID REVISION INTERNAL_DIMENSION_ID DIM_TYPE_ID DIMENSION_CD DFLT_HIERARCHY_ID
        EDITABLE_FLG EDITABLE_PAST_FLG HIDDEN_FLG VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_DIMENSION data=work.dimension &FM_PROCAPPENDFORCE;
run;

%if &ses_locale_found eq 1 %then %do;
   proc append base=_SDMLIB.SAS_DIMENSION_NLS data=work.dimension_nls &FM_PROCAPPENDFORCE;
   run;
%end;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2001,
          object_id_table=work.dimension, object_id_column_nm=DIMENSION_ID,
          bldlibnm_sassdm=N);

/* Set default dimension for the dimension type */

%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.,
          sesetl_useType=CON
          ) ;

proc sql noprint;
  connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
  execute(update SAS_DIM_TYPE
          set DFLT_DIMENSION_ID = &dim_id
          where DIM_TYPE_CD = 'SOURCE') by &sesetl_EngineType;
  disconnect from &sesetl_EngineType;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

proc sql noprint;
  select KEY_CNT into :mem_id from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/member';
quit;

/* Set 'odcs/member' in SAS_KEYGEN */
proc sql noprint;
  select KEY_CNT + 21 into :mem_key from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/member';
quit;

%sesskey(keyCode=%str(odcs/member), keyNext=&mem_key) ;

/* Create member records for the SOURCE dimension */

data work.member;
   length INTERNAL_DIMENSION_ID 8;
   length MEMBER_ID 8;
   length REVISION 8;
   length INTERNAL_DIM_TYPE_ID 8;
   length MEMBER_TYPE_ENUM 8;
   length MEMBER_CD $ 32;
   length HIDDEN_FLG $5;
   length CUSTOM_FLG $5;
   length LOCALE_ID 8;
   length MEMBER_NM $ 50;
   length MEMBER_DESC $ 255;
   INTERNAL_DIMENSION_ID = &dim_id * (-1);
   REVISION = 1;
   INTERNAL_DIM_TYPE_ID = &dtype_id * (-1);
   MEMBER_TYPE_ENUM = 1;
   HIDDEN_FLG = "false";
   CUSTOM_FLG = "false";
   LOCALE_ID = 1;
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   MEMBER_ID = &mem_id;
   MEMBER_CD = 'Total';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_TOTAL_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_TOTAL_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 1;
   MEMBER_CD = 'CTA';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_CTA_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_CTA_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 2;
   MEMBER_CD = 'RE';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_RE_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_RE_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 3;
   MEMBER_CD = 'REElim';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_REELIM_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_REELIM_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 4;
   MEMBER_CD = 'REAdj';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_READJ_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_READJ_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 5;
   MEMBER_CD = 'REData';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_REDATA_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_REDATA_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 6;
   MEMBER_CD = 'REForm';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_REFORM_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_REFORM_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 7;
   MEMBER_CD = 'REImport';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_REIMPORT_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_REIMPORT_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 8;
   MEMBER_CD = 'Elim';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_ELIM_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_ELIM_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 9;
   MEMBER_CD = 'TotalBeforeElim';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_TOTALB4ELM_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_TOTALB4ELM_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 10;
   MEMBER_CD = 'Adj';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_ADJ_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_ADJ_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 11;
   MEMBER_CD = 'CPO';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_CPO_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_CPO_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 12;
   MEMBER_CD = 'Alloc';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_ALLOC_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_ALLOC_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 13;
   MEMBER_CD = 'Reclass';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_RECLASS_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_RECLASS_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 14;
   MEMBER_CD = 'Bal';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_BAL_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_BAL_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 15;
   MEMBER_CD = 'Manual';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_MANUAL_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_MANUAL_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 16;
   MEMBER_CD = 'TotalBeforeAdj';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_TOTALB4ADJ_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_TOTALB4ADJ_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 17;
   MEMBER_CD = 'BaseForm';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_BASEFORM_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_BASEFORM_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 18;
   MEMBER_CD = 'TotalAfterImport';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_TOTALAFTIM_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_TOTALAFTIM_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 19;
   MEMBER_CD = 'BaseJourn';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_BASEJOURN_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_BASEJOURN_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 20;
   MEMBER_CD = 'Base';
   MEMBER_NM = "%nrbquote(&SES_MBR_SRC_BASE_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_SRC_BASE_DESC_VAR)";
   output;
   keep INTERNAL_DIMENSION_ID MEMBER_ID REVISION INTERNAL_DIM_TYPE_ID MEMBER_TYPE_ENUM
        MEMBER_CD HIDDEN_FLG CUSTOM_FLG LOCALE_ID MEMBER_NM MEMBER_DESC VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

data work.member_nls;
   set work.member;
   keep INTERNAL_DIMENSION_ID MEMBER_ID LOCALE_ID REVISION MEMBER_NM MEMBER_DESC VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

data work.member;
   set work.member;
   keep INTERNAL_DIMENSION_ID MEMBER_ID REVISION INTERNAL_DIM_TYPE_ID MEMBER_TYPE_ENUM
        MEMBER_CD HIDDEN_FLG CUSTOM_FLG VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_MEMBER data=work.member &FM_PROCAPPENDFORCE;
run;

%if &ses_locale_found eq 1 %then %do;
   proc append base=_SDMLIB.SAS_MEMBER_NLS data=work.member_nls &FM_PROCAPPENDFORCE;
   run;
%end;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2009,
          object_id_table=work.member, object_id_column_nm=MEMBER_ID,
          bldlibnm_sassdm=N);

/* Create format FMT_SOURCE */

data work.fmt_source;
   length START $ 32;
   length LABEL 8;
   length FMTNAME $ 255;
   length TYPE $ 255;
   set work.member;
   START = MEMBER_CD;
   LABEL = MEMBER_ID;
   FMTNAME = 'FMT_SOURCE';
   TYPE = 'I';
   keep START LABEL FMTNAME TYPE;
run;

proc format cntlin=fmt_source library=work;
run;

proc sql noprint;
  select KEY_CNT into :hier_id from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/hierarchy';
quit;

/* Set 'odcs/hierarchy' in SAS_KEYGEN */
%sesskey(keyCode=%str(odcs/hierarchy)) ;

/* Create hierarchy record(s) for the SOURCE dimension */

data work.hier;
   length HIERARCHY_ID 8;
   length REVISION 8;
   length HIERARCHY_CD $ 32;
   length INTERNAL_DIMENSION_ID 8;
   length HIDDEN_FLG $5;
   length LOCALE_ID 8;
   length HIERARCHY_NM $ 50;
   length HIERARCHY_DESC $ 255;
   HIERARCHY_ID = &hier_id;
   REVISION = 1;
   HIERARCHY_CD = 'SOURCE';
   INTERNAL_DIMENSION_ID = &dim_id * (-1);
   HIDDEN_FLG = "false";
   LOCALE_ID = 1;
   HIERARCHY_NM = "%nrbquote(&SES_HIER_SOURCE_NM_VAR)";
   HIERARCHY_DESC = "%nrbquote(&SES_HIER_SOURCE_DESC_VAR)";
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   keep HIERARCHY_ID REVISION HIERARCHY_CD INTERNAL_DIMENSION_ID HIDDEN_FLG LOCALE_ID
        HIERARCHY_NM HIERARCHY_DESC VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.hier_nls;
   set work.hier;
   keep HIERARCHY_ID LOCALE_ID REVISION HIERARCHY_NM HIERARCHY_DESC VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.hier;
   set work.hier;
   keep HIERARCHY_ID REVISION HIERARCHY_CD INTERNAL_DIMENSION_ID HIDDEN_FLG VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_HIERARCHY data=work.hier &FM_PROCAPPENDFORCE;
run;

%if &ses_locale_found eq 1 %then %do;
   proc append base=_SDMLIB.SAS_HIERARCHY_NLS data=work.hier_nls &FM_PROCAPPENDFORCE;
   run;
%end;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2007,
          object_id_table=work.hier, object_id_column_nm=HIERARCHY_ID,
          bldlibnm_sassdm=N);

proc sql noprint;
  connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
  execute(update SAS_DIMENSION
          set DFLT_HIERARCHY_ID = &hier_id
          where DIMENSION_CD = 'SOURCE') by &sesetl_EngineType;
  disconnect from &sesetl_EngineType;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

/* Create hierarchy_member records for the imported dimension */

data work.hierarchy_member;
   length HIERARCHY_ID 8;
   length MEMBER_ID 8;
   length REVISION 8;
   length PARENT_MEMBER_ID 8;
   length MEMBER_RANK 8;
   HIERARCHY_ID = &hier_id;
   REVISION = 1;
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   MEMBER_ID = input('Total', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('CTA', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Total', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('RE', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Total', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = input('REElim', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('RE', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('REAdj', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('RE', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = input('REData', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('RE', FMT_SOURCE.);
   MEMBER_RANK = 2;
   output;
   MEMBER_ID = input('REForm', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('REData', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('REImport', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('REData', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = input('Elim', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Total', FMT_SOURCE.);
   MEMBER_RANK = 2;
   output;
   MEMBER_ID = input('TotalBeforeElim', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Total', FMT_SOURCE.);
   MEMBER_RANK = 3;
   output;
   MEMBER_ID = input('Adj', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('TotalBeforeElim', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('CPO', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Adj', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('Alloc', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Adj', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = input('Reclass', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Adj', FMT_SOURCE.);
   MEMBER_RANK = 2;
   output;
   MEMBER_ID = input('Bal', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Adj', FMT_SOURCE.);
   MEMBER_RANK = 3;
   output;
   MEMBER_ID = input('Manual', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('Adj', FMT_SOURCE.);
   MEMBER_RANK = 4;
   output;
   MEMBER_ID = input('TotalBeforeAdj', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('TotalBeforeElim', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = input('BaseForm', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('TotalBeforeAdj', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('TotalAfterImport', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('TotalBeforeAdj', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = input('BaseJourn', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('TotalAfterImport', FMT_SOURCE.);
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = input('Base', FMT_SOURCE.);
   PARENT_MEMBER_ID = input('TotalAfterImport', FMT_SOURCE.);
   MEMBER_RANK = 1;
   output;
   keep HIERARCHY_ID MEMBER_ID REVISION PARENT_MEMBER_ID MEMBER_RANK
        VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_HIERARCHY_MEMBER data=work.hierarchy_member &FM_PROCAPPENDFORCE;
run;

/* Get default hierarchy for the dimension */

proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(update SAS_DIMENSION
           set DFLT_HIERARCHY_ID = &hier_id
           where DIMENSION_CD = 'SOURCE'
          ) by &sesetl_EngineType;
   disconnect from &sesetl_EngineType;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

/* Create hierarchy_dflt_member record(s) for the imported dimension */

data work.hierarchy_dflt_member;
   length HIERARCHY_ID 8;
   length DIMENSION_ID 8;
   length REVISION 8;
   length DFLT_MEMBER_ID 8;
   HIERARCHY_ID = &hier_id;
   DIMENSION_ID = &dim_id;
   REVISION = 1;
   DFLT_MEMBER_ID = input('Total', FMT_SOURCE.);
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   keep HIERARCHY_ID DIMENSION_ID REVISION DFLT_MEMBER_ID VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc sort data = work.hierarchy_dflt_member NODUPKEY;
   by HIERARCHY_ID DIMENSION_ID;
run;

proc append base=_SDMLIB.SAS_HIERARCHY_DFLT_MEMBER data=work.hierarchy_dflt_member &FM_PROCAPPENDFORCE;
run;

%mend impsourc;
