/* Copyright 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro impss (default_locale=, fmsetl_processedDTTM_From=, fmsetl_processedDTTM_To=);

%let default_locale = %unquote(&default_locale);
%let fmsetl_processedDTTM_From = %unquote(&fmsetl_processedDTTM_From);
%let fmsetl_processedDTTM_To = %unquote(&fmsetl_processedDTTM_To);

%bldLibNm(sesetl_sasLibRef=_Confrm
        , sesetl_OMRSASLibraryObjName=&FMdatatier_folder./&CONFLIBOBJ./&CONFLIBOBJ.
        , sesetl_useType=BASE,
          sesetl_validateLib=Y
         ) ;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
        ) ;

%let curr_rec_dttm = '01JAN5999:00:00'DT;
%let current_dttm = datetime();

%let _where1=((VALID_FROM_DTTM <= &current_dttm)
          and (VALID_TO_DTTM >= &current_dttm or VALID_TO_DTTM = .));

%let _tmpWhere=where=(&_where1);

proc sort data = _Confrm.CODE_LANGUAGE(&_tmpWhere)
           out = WORK._CODE_LANG_SORT(keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD
                       LOCALE_VARIANT_CD LANGUAGE_CD DEFAULT_LANGUAGE_FLG)
           NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;

data WORK._SDM_LOCALES;
   set _SDMLIB.SAS_DATA_LOCALE;
   LOCALE_LANGUAGE_CD = ISO639_LANGUAGE_CD;
   LOCALE_COUNTRY_CD = ISO3166_COUNTRY_CD;
   keep LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD LOCALE_ID;
run;

proc sort data = WORK._SDM_LOCALES NODUPKEY;
   by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;
run;

/***********************************************************************************/
/* Create temporary table containing mapping between the locales in the dds and    */
/* sdm using language, country and variant as sorting variables.                   */
/***********************************************************************************/
data WORK._STAGE_SDM_LOCALE_MAP
    (keep=LANGUAGE_CD LOCALE_ID LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD
        DEFAULT_LANGUAGE_FLG);

   merge WORK._CODE_LANG_SORT (in = inA)
         WORK._SDM_LOCALES (in = inB);
      by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD LOCALE_VARIANT_CD;

   if (inA and inB) then
      output;
run;

/***********************************************************************************/
/* Create locale format, FMT_LOCALE, using the WORK._DDS_SDM_LOCALE_MAP table.     */
/***********************************************************************************/
%let _tmpFormat=FMT_LOCALE;
%let _tmpStart=LANGUAGE_CD;
%let _tmpLabel=LOCALE_ID;
%let _tmpWhere=;

proc sort data = WORK._STAGE_SDM_LOCALE_MAP
           out = WORK._TMPSORT (keep=&_tmpStart &_tmpLabel)
           NODUPKEY;
   by &_tmpStart;
run;

data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE);
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   set WORK._TMPSORT(rename=(&_tmpStart=START &_tmpLabel=LABEL));
run;

proc format cntlin=work.&_tmpFormat library=work;
run;

/* Import the source_system records */

data WORK.SOURCE_SYSTEM
     (keep= SOURCE_SYSTEM_CD SOURCE_SYSTEM_NM SOURCE_SYSTEM_DESC LOCALE_ID
        VALID_FROM_DTTM VALID_TO_DTTM);

   set _SDMLIB.SAS_SOURCE_SYSTEM(obs=0)
       _SDMLIB.SAS_SOURCE_SYSTEM_NLS(obs=0)
       _Confrm.SOURCE_SYSTEM;

   if (SOURCE_SYSTEM_CD = 'UKN') then
      SOURCE_SYSTEM_CD = 'UNKNOWN';

   SOURCE_SYSTEM_NM = SOURCE_SYSTEM_CD;
   LOCALE_ID = input(LANGUAGE_CD, FMT_LOCALE.);

   if (VALID_TO_DTTM > &current_dttm or VALID_TO_DTTM = .) then
      VALID_TO_DTTM = &curr_rec_dttm;

   if (LANGUAGE_CD = "&default_locale") and (VALID_FROM_DTTM < &current_dttm) then
      output;
run;

proc sort data = _SDMLIB.SAS_SOURCE_SYSTEM OUT = WORK.SDM_SOURCE_SYSTEM;
   by SOURCE_SYSTEM_CD;
run;

proc sort data = WORK.SOURCE_SYSTEM;
   by SOURCE_SYSTEM_CD;
run;

data WORK.SOURCE_SYSTEM_TEMP
     WORK.SOURCE_SYSTEM_NEW1;

   merge WORK.SDM_SOURCE_SYSTEM (in = inA)
         WORK.SOURCE_SYSTEM (in = inB);
        by SOURCE_SYSTEM_CD;

   if (inA) and (inB) then
      output WORK.SOURCE_SYSTEM_TEMP;

   if (^inA) and (inB) then
      output WORK.SOURCE_SYSTEM_NEW1;
run;


%let last_ID = 0;
proc sql noprint;
select max(SOURCE_SYSTEM_ID) into :last_ID from _SDMLIB.SAS_SOURCE_SYSTEM;
quit;

data WORK.SOURCE_SYSTEM_NEW1
    (keep=SOURCE_SYSTEM_ID REVISION SOURCE_SYSTEM_CD SOURCE_SYSTEM_NM SOURCE_SYSTEM_DESC
        LOCALE_ID);

   retain REVISION 0;

   set WORK.SOURCE_SYSTEM_NEW1;
      by SOURCE_SYSTEM_CD;

   if first.SOURCE_SYSTEM_CD then do;
      SOURCE_SYSTEM_ID + symgetn('last_ID') + 1;
      REVISION = 1;
   end;
   else
      REVISION + 1;
run;

proc sort data = _SDMLIB.SAS_SOURCE_SYSTEM_NLS OUT=WORK.SDM_SOURCE_SYSTEM_NLS;
   by SOURCE_SYSTEM_ID LOCALE_ID DESCENDING REVISION;
run;

proc sort data = WORK.SDM_SOURCE_SYSTEM_NLS NODUPKEY;
   by SOURCE_SYSTEM_ID LOCALE_ID;
run;

proc sort data = WORK.SOURCE_SYSTEM_TEMP;
   by SOURCE_SYSTEM_ID LOCALE_ID VALID_FROM_DTTM;
run;

data WORK.SOURCE_SYSTEM_NEW2
  (keep=SOURCE_SYSTEM_ID REVISION SOURCE_SYSTEM_CD SOURCE_SYSTEM_NM SOURCE_SYSTEM_DESC
        LOCALE_ID);

   merge WORK.SDM_SOURCE_SYSTEM_NLS
         WORK.SOURCE_SYSTEM_TEMP (in = inA);
      by SOURCE_SYSTEM_ID LOCALE_ID;

   REVISION + 1;

   if (inA) then
      output;
run;

data WORK.SOURCE_SYSTEM_NEW
     WORK.SOURCE_SYSTEM_NLS_NEW
    (keep=SOURCE_SYSTEM_ID REVISION SOURCE_SYSTEM_NM SOURCE_SYSTEM_DESC LOCALE_ID);

   set WORK.SOURCE_SYSTEM_NEW1
       WORK.SOURCE_SYSTEM_NEW2;
run;

proc sort data = WORK.SOURCE_SYSTEM_NEW1(keep=SOURCE_SYSTEM_ID SOURCE_SYSTEM_CD)
          out  = WORK.SOURCE_SYSTEM_NEW
          NODUPKEY;
   by SOURCE_SYSTEM_ID;
run;

proc append base=_SDMLIB.SAS_SOURCE_SYSTEM DATA=WORK.SOURCE_SYSTEM_NEW &FM_PROCAPPENDFORCE;
run;
proc append base=_SDMLIB.SAS_SOURCE_SYSTEM_NLS DATA=WORK.SOURCE_SYSTEM_NLS_NEW &FM_PROCAPPENDFORCE;
run;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2016,
          object_id_table=WORK.SOURCE_SYSTEM_NEW, object_id_column_nm=SOURCE_SYSTEM_ID,
          bldlibnm_sassdm=Y);

/* Set 'SOURCE_SYSTEM' in SAS_KEYGEN */
proc sql noprint;
   select max(SOURCE_SYSTEM_ID)+1 into :max_rec
   from _SDMLIB.SAS_SOURCE_SYSTEM;
quit;
%sesskey( keyCode=%str(ses/source_system), keyNext=&max_rec );

%mend impss;
