/* Copyright 2006 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro impxtype;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE,
          sesetl_validateLib=Y
         ) ;

%let dflt_from_dttm = '01JAN1970:00:00'DT;
%let curr_rec_dttm = '01JAN5999:00:00'DT;

%if &SYSSCP = WIN %then %do;
    %let propspth=%SYSGET(SASROOT)\finance\sasmisc;
%end;
%else %do;
    %let propspth=%SYSGET(SASROOT)/misc/finance;
%end;

%localze(ses_property_file_location=&propspth, ses_locale=en);

%if (&sas_install_default_locale ne en) %then
   %localze(ses_property_file_location=&propspth, ses_locale=&sas_install_default_locale);

/* Create the dimension type record for the XRATE_TYPE dimension type */

proc sql noprint;
   select count(*) into :dtype_rec_cnt from _SDMLIB.SAS_DIM_TYPE
   where DIM_TYPE_CD = 'XRATE_TYPE';
quit;

%if (&dtype_rec_cnt < 1) %then %do;

   proc sql noprint;
     select KEY_CNT into :dtype_id from _SDMLIB.SAS_KEYGEN
     where IDENTIFIER_CD = 'odcs/dimension_type';
   quit;

   /* Set 'odcs/dimension_type' in SAS_KEYGEN */
   %sesskey(keyCode=%str(odcs/dimension_type)) ;

   data work.dim_type;
      length DIM_TYPE_ID 8;
      length REVISION 8;
      length DIM_TYPE_ROLE 8;
      length INTERNAL_DIM_TYPE_ID 8;
      length DIM_TYPE_CD $ 32;
      length HIDDEN_FLG $5;
      length DFLT_DIMENSION_ID 8;
      length DIM_TYPE_NM $ 50;
      length DIM_TYPE_DESC $ 255;
      length LOCALE_ID 8;
      DIM_TYPE_ID = &dtype_id;
      REVISION = 1;
      DIM_TYPE_ROLE = 17;
      INTERNAL_DIM_TYPE_ID = -&dtype_id;
      DIM_TYPE_CD = 'XRATE_TYPE';
      HIDDEN_FLG = "true";
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_XRATETYPE_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_XRATETYPE_DESC_VAR)";
      LOCALE_ID = 1;
      FORMAT VALID_FROM_DTTM DATETIME25.;
      VALID_FROM_DTTM = &dflt_from_dttm;
      FORMAT VALID_TO_DTTM DATETIME25.;
      VALID_TO_DTTM = &curr_rec_dttm;
      keep DIM_TYPE_ID REVISION DIM_TYPE_ROLE INTERNAL_DIM_TYPE_ID DIM_TYPE_CD HIDDEN_FLG
           DFLT_DIMENSION_ID DIM_TYPE_NM DIM_TYPE_DESC LOCALE_ID VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;
   data work.dim_type_nls;
      set work.dim_type;
      keep DIM_TYPE_ID LOCALE_ID REVISION DIM_TYPE_NM DIM_TYPE_DESC VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;
   data work.dim_type;
      set work.dim_type;
      keep DIM_TYPE_ID REVISION DIM_TYPE_ROLE INTERNAL_DIM_TYPE_ID DIM_TYPE_CD HIDDEN_FLG
           DFLT_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM;
   run;
   proc append base=_SDMLIB.SAS_DIM_TYPE data=work.dim_type &FM_PROCAPPENDFORCE;
   run;
   %if &ses_locale_found eq 1 %then %do;
      proc append base=_SDMLIB.SAS_DIM_TYPE_NLS data=work.dim_type_nls &FM_PROCAPPENDFORCE;
      run;
   %end;
   %addhstry(action_type_id=18, solution_id=46, object_type_id=2002,
             object_id_table=work.dim_type, object_id_column_nm=DIM_TYPE_ID,
             bldlibnm_sassdm=N);
%end;
%else %do;
   proc sql noprint;
      select DIM_TYPE_ID into :dtype_id from _SDMLIB.SAS_DIM_TYPE
      where DIM_TYPE_CD = 'XRATE_TYPE';
   quit;
%end;

/* Verify program hasn't already been run and if so, abort */

proc sql noprint;
   select count(*) into :dim_rec_cnt from _SDMLIB.SAS_DIMENSION
   where DIMENSION_CD = 'XRATE_TYPE';
quit;

%if (&dim_rec_cnt > 0) %then
    %return;

/* Add XRATE_TYPE records to dimension and dimension_nls tables */

proc sql noprint;
  select KEY_CNT into :dim_id from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/dimension';
quit;

/* Set 'odcs/dimension' in SAS_KEYGEN */
%sesskey(keyCode=%str(odcs/dimension)) ;

/* Create dimension record for the SOURCE dimension */

data work.dimension;
   length DIMENSION_ID 8;
   length REVISION 8;
   length INTERNAL_DIMENSION_ID 8;
   length DIM_TYPE_ID 8;
   length DIMENSION_CD $ 32;
   length DFLT_HIERARCHY_ID 8;
   length EDITABLE_FLG $5;
   length EDITABLE_PAST_FLG $5;
   length HIDDEN_FLG $5;
   length LOCALE_ID 8;
   length DIMENSION_NM $ 50;
   length DIMENSION_DESC $ 255;
   DIMENSION_ID = &dim_id;
   REVISION = 1;
   INTERNAL_DIMENSION_ID = -&dim_id;
   DIM_TYPE_ID = &dtype_id;
   DIMENSION_CD = 'XRATE_TYPE';
   EDITABLE_FLG = "true";
   EDITABLE_PAST_FLG = "false";
   HIDDEN_FLG = "true";
   LOCALE_ID = 1;
   DIMENSION_NM = "%nrbquote(&SES_DIMEN_XRATETYPE_NM_VAR)";
   DIMENSION_DESC = "%nrbquote(&SES_DIMEN_XRATETYPE_DESC_VAR)";
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   keep DIMENSION_ID REVISION INTERNAL_DIMENSION_ID DIM_TYPE_ID DIMENSION_CD DFLT_HIERARCHY_ID
        EDITABLE_FLG EDITABLE_PAST_FLG HIDDEN_FLG LOCALE_ID DIMENSION_NM DIMENSION_DESC
        VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.dimension_nls;
   set work.dimension;
   keep DIMENSION_ID LOCALE_ID REVISION DIMENSION_NM DIMENSION_DESC VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

data work.dimension;
   set work.dimension;
   keep DIMENSION_ID REVISION INTERNAL_DIMENSION_ID DIM_TYPE_ID DIMENSION_CD DFLT_HIERARCHY_ID
        EDITABLE_FLG EDITABLE_PAST_FLG HIDDEN_FLG VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_DIMENSION data=work.dimension &FM_PROCAPPENDFORCE;
run;

%if &ses_locale_found eq 1 %then %do;
   proc append base=_SDMLIB.SAS_DIMENSION_NLS data=work.dimension_nls &FM_PROCAPPENDFORCE;
   run;
%end;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2001,
          object_id_table=work.dimension, object_id_column_nm=DIMENSION_ID,
          bldlibnm_sassdm=N);

/* Set default dimension for the dimension type */

%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.,
          sesetl_useType=CON
          ) ;

proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(update SAS_DIM_TYPE
           set DFLT_DIMENSION_ID = &dim_id
           where DIM_TYPE_CD = 'XRATE_TYPE'
          ) by &sesetl_EngineType;
   disconnect from &sesetl_EngineType;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

proc sql noprint;
  select KEY_CNT into :mem_id from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/member';
quit;

/* Set 'odcs/member' in SAS_KEYGEN */
proc sql noprint;
  select KEY_CNT + 8 into :mem_key from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/member';
quit;

%sesskey(keyCode=%str(odcs/member), keyNext=&mem_key) ;

/* Create member records for the XRATE_TYPE dimension */

data work.member;
   length INTERNAL_DIMENSION_ID 8;
   length MEMBER_ID 8;
   length REVISION 8;
   length INTERNAL_DIM_TYPE_ID 8;
   length MEMBER_TYPE_ENUM 8;
   length MEMBER_CD $ 32;
   length HIDDEN_FLG $5;
   length CUSTOM_FLG $5;
   length LOCALE_ID 8;
   length MEMBER_NM $ 50;
   length MEMBER_DESC $ 255;
   INTERNAL_DIMENSION_ID = -&dim_id;
   REVISION = 1;
   INTERNAL_DIM_TYPE_ID = -&dtype_id;
   MEMBER_TYPE_ENUM = 1;
   HIDDEN_FLG = "false";
   CUSTOM_FLG = "false";
   LOCALE_ID = 1;
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   MEMBER_ID = &mem_id;
   MEMBER_CD = 'None';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_NONE_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_NONE_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 1;
   MEMBER_CD = 'Historic';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_HIST_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_HIST_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 2;
   MEMBER_CD = 'Derived';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_DERIVED_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_DERIVED_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 3;
   MEMBER_CD = 'PeriodAverage';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_PRDAVG_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_PRDAVG_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 4;
   MEMBER_CD = 'PeriodClose';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_PRDCLS_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_PRDCLS_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 5;
   MEMBER_CD = 'PeriodOpen';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_PRDOPN_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_PRDOPN_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 6;
   MEMBER_CD = 'Custom1';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_CUSTOM1_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_CUSTOM1_DESC_VAR)";
   output;
   MEMBER_ID = &mem_id + 7;
   MEMBER_CD = 'Custom2';
   MEMBER_NM = "%nrbquote(&SES_MBR_XRATE_CUSTOM2_NM_VAR)";
   MEMBER_DESC = "%nrbquote(&SES_MBR_XRATE_CUSTOM2_DESC_VAR)";
   output;
   keep INTERNAL_DIMENSION_ID MEMBER_ID REVISION INTERNAL_DIM_TYPE_ID MEMBER_TYPE_ENUM MEMBER_CD
        HIDDEN_FLG CUSTOM_FLG LOCALE_ID MEMBER_NM MEMBER_DESC VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.member_nls;
   set work.member;
   keep INTERNAL_DIMENSION_ID MEMBER_ID LOCALE_ID REVISION MEMBER_NM MEMBER_DESC VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

data work.member;
   set work.member;
   keep INTERNAL_DIMENSION_ID MEMBER_ID REVISION INTERNAL_DIM_TYPE_ID MEMBER_TYPE_ENUM MEMBER_CD
        HIDDEN_FLG CUSTOM_FLG VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_MEMBER data=work.member &FM_PROCAPPENDFORCE;
run;

%if &ses_locale_found eq 1 %then %do;
   proc append base=_SDMLIB.SAS_MEMBER_NLS data=work.member_nls &FM_PROCAPPENDFORCE;
   run;
%end;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2009,
          object_id_table=work.member, object_id_column_nm=MEMBER_ID,
          bldlibnm_sassdm=N);

/* Add XRATE_TYPE records to hierarchy and hierarchy_nls tables */

proc sql noprint;
  select KEY_CNT into :hier_id from _SDMLIB.SAS_KEYGEN
  where IDENTIFIER_CD = 'odcs/hierarchy';
quit;

/* Set 'odcs/hierarchy' in SAS_KEYGEN */
%sesskey(keyCode=%str(odcs/hierarchy)) ;

data work.hier;
   length HIERARCHY_ID 8;
   length REVISION 8;
   length HIERARCHY_CD $ 32;
   length INTERNAL_DIMENSION_ID 8;
   length HIDDEN_FLG $5;
   length LOCALE_ID 8;
   length HIERARCHY_NM $ 50;
   length HIERARCHY_DESC $ 255;
   HIERARCHY_ID = &hier_id;
   REVISION = 1;
   HIERARCHY_CD = 'XRATE_TYPE';
   INTERNAL_DIMENSION_ID = -&dim_id;
   HIDDEN_FLG = "false";
   LOCALE_ID = 1;
   HIERARCHY_NM = "%nrbquote(&SES_HIER_XRATETYPE_NM_VAR)";
   HIERARCHY_DESC = "%nrbquote(&SES_HIER_XRATETYPE_DESC_VAR)";
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   keep HIERARCHY_ID REVISION HIERARCHY_CD INTERNAL_DIMENSION_ID HIDDEN_FLG LOCALE_ID
        HIERARCHY_NM HIERARCHY_DESC VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.hier_nls;
   set work.hier;
   keep HIERARCHY_ID LOCALE_ID REVISION HIERARCHY_NM HIERARCHY_DESC VALID_FROM_DTTM VALID_TO_DTTM;
run;

data work.hier;
   set work.hier;
   keep HIERARCHY_ID REVISION HIERARCHY_CD INTERNAL_DIMENSION_ID HIDDEN_FLG VALID_FROM_DTTM
        VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_HIERARCHY data=work.hier &FM_PROCAPPENDFORCE;
run;

%if &ses_locale_found eq 1 %then %do;
   proc append base=_SDMLIB.SAS_HIERARCHY_NLS data=work.hier_nls &FM_PROCAPPENDFORCE;
   run;
%end;

%addhstry(action_type_id=18, solution_id=46, object_type_id=2007,
          object_id_table=work.hier, object_id_column_nm=HIERARCHY_ID,
          bldlibnm_sassdm=N);

/* Set default hierarchy for the dimension */

proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(update SAS_DIMENSION
           set DFLT_HIERARCHY_ID = &hier_id
           where DIMENSION_CD = 'XRATE_TYPE'
          ) by &sesetl_EngineType;
   disconnect from &sesetl_EngineType;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

/* Create child_parent records for the XRATE_TYPE hierarchy */

data work.hierarchy_member;
   length HIERARCHY_ID 8;
   length MEMBER_ID 8;
   length REVISION 8;
   length PARENT_MEMBER_ID 8;
   length MEMBER_RANK 8;
   HIERARCHY_ID = &hier_id;
   REVISION = 1;
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   MEMBER_ID = &mem_id;
   MEMBER_RANK = 0;
   output;
   MEMBER_ID = &mem_id + 1;
   MEMBER_RANK = 1;
   output;
   MEMBER_ID = &mem_id + 2;
   MEMBER_RANK = 2;
   output;
   MEMBER_ID = &mem_id + 3;
   MEMBER_RANK = 3;
   output;
   MEMBER_ID = &mem_id + 4;
   MEMBER_RANK = 4;
   output;
   MEMBER_ID = &mem_id + 5;
   MEMBER_RANK = 5;
   output;
   MEMBER_ID = &mem_id + 6;
   MEMBER_RANK = 6;
   output;
   MEMBER_ID = &mem_id + 7;
   MEMBER_RANK = 7;
   output;
   keep HIERARCHY_ID MEMBER_ID REVISION PARENT_MEMBER_ID MEMBER_RANK
        VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc append base=_SDMLIB.SAS_HIERARCHY_MEMBER data=work.hierarchy_member &FM_PROCAPPENDFORCE;
run;

/* Add record to the hierarchy_dflt_member table making PeriodOpen the default member for the */
/* XRATE_TYPE hierarchy.                                                                      */

data work.hierarchy_dflt_member;
   length HIERARCHY_ID 8;
   length DIMENSION_ID 8;
   length REVISION 8;
   length DFLT_MEMBER_ID 8;
   HIERARCHY_ID = &hier_id;
   DIMENSION_ID = &dim_id;
   REVISION = 1;
   DFLT_MEMBER_ID = &mem_id + 5;
   FORMAT VALID_FROM_DTTM DATETIME25.;
   VALID_FROM_DTTM = &dflt_from_dttm;
   FORMAT VALID_TO_DTTM DATETIME25.;
   VALID_TO_DTTM = &curr_rec_dttm;
   keep HIERARCHY_ID DIMENSION_ID REVISION DFLT_MEMBER_ID VALID_FROM_DTTM VALID_TO_DTTM;
run;

proc sort data = work.hierarchy_dflt_member NODUPKEY;
   by HIERARCHY_ID DIMENSION_ID;
run;

proc append base=_SDMLIB.SAS_HIERARCHY_DFLT_MEMBER data=work.hierarchy_dflt_member &FM_PROCAPPENDFORCE;
run;

%mend impxtype;
