/*********************************************************************************
 *  Copyright (c) 2007 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *  NAME:      migr8odc.sas
 *
 *  PURPOSE:   Migrates ODCS data to the current release
 *
 *********************************************************************************/
%macro migrod53;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
           , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
           , sesetl_useType=BASE,
             sesetl_validateLib=Y
            ) ;

%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.,
             sesetl_useType=CON
             ) ;


/******************************************************************************/
/*                  Set status of migration to Failed                         */
/******************************************************************************/
%sesssver(solName=ODCS, solVer=&sesetl_version, solStatus=Failed, solMethod=Migration);


/******************************************************************************/
/*                  Prepare table and column information                      */
/******************************************************************************/
proc contents data=_SDMLIB._all_
              out=contents(keep=memname name) noprint;
run;

/******************************************************************************/
/*             Add a column  FORMULA_MDX_TEXT to   SAS_FORMULA                */
/******************************************************************************/

%let _colCount = 0 ;

proc sql noprint;
   select count(*) into :_colCount
     from work.contents
    where upcase(memname) = "SAS_FORMULA"
      and upcase(name) = "FORMULA_MDX_TEXT";
quit;

%if &_colCount EQ 0 %then %do ;
    proc sql ;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

        execute (
                 CREATE TABLE if not exists public.tmp_formula (
                   formula_id integer NOT NULL DEFAULT 0,
                   namespace character(1) NOT NULL DEFAULT ''::bpchar,
                   formula_type_enum character(1) NOT NULL DEFAULT ''::bpchar,
                   target character varying(255) DEFAULT NULL::character varying,
                   result_type character(1) NOT NULL DEFAULT ''::bpchar,
                   formula_text text NOT NULL,
                   byte_code bytea NOT NULL,
                   formula_mdx_text text,
                   CONSTRAINT tmp_formula_pkey PRIMARY KEY (formula_id )
                )
           ) by PASSTHRU;

        execute (
                 INSERT INTO public.tmp_formula
                 SELECT formula_id, namespace, formula_type_enum, target, result_type, formula_text, byte_code, ''
                 FROM public.sas_formula
           ) by PASSTHRU;

        execute (
                 DROP TABLE public.sas_formula
           ) by PASSTHRU;

        execute (
                 CREATE TABLE if not exists public.sas_formula (
                   formula_id integer NOT NULL DEFAULT 0,
                   namespace character(1) NOT NULL DEFAULT ''::bpchar,
                   formula_type_enum character(1) NOT NULL DEFAULT ''::bpchar,
                   target character varying(255) DEFAULT NULL::character varying,
                   result_type character(1) NOT NULL DEFAULT ''::bpchar,
                   formula_text text NOT NULL,
                   byte_code bytea NOT NULL,
                   formula_mdx_text text,
                   CONSTRAINT sas_formula_pkey PRIMARY KEY (formula_id )
                )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_formula__namespace ON public.sas_formula (namespace COLLATE pg_catalog."default" )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_formula__target ON public.sas_formula (target COLLATE pg_catalog."default" )
           ) by PASSTHRU;

        execute (
                 INSERT INTO public.sas_formula
                 SELECT * FROM public.tmp_formula
           ) by PASSTHRU;

        execute (
                 DROP TABLE public.tmp_formula
           ) by PASSTHRU;

        disconnect from PASSTHRU ;
    quit ;
    %checkErr;
    %if &_tmpRC %then
       %return ;
%end;

/******************************************************************************/
/*             Add a table  SAS_HIERARCHY_MEMBER_DEPTH                        */
/******************************************************************************/

%let _colCount = 0 ;

proc sql noprint;
   select count(*) into :_colCount
     from work.contents
    where upcase(memname) = "SAS_HIERARCHY_MEMBER_DEPTH";
quit;

%if &_colCount EQ 0 %then %do ;
    proc sql ;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;
        execute (
                 CREATE TABLE if not exists public.sas_hierarchy_member_depth (
                   hierarchy_id integer NOT NULL DEFAULT 0,
                   member_id integer NOT NULL DEFAULT 0,
                   revision integer NOT NULL DEFAULT 0,
                   depth integer NOT NULL DEFAULT (-1),
                   valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
                   valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
                   CONSTRAINT sas_hierarchy_member_depth_pkey PRIMARY KEY (hierarchy_id , member_id , revision )
                )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_hierarchy_member_depth__hierarchy_rk ON public.sas_hierarchy_member_depth (hierarchy_id )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_hierarchy_member_depth__hierarchy_rk_2 ON public.sas_hierarchy_member_depth (hierarchy_id , valid_from_dttm , valid_to_dttm )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_hierarchy_member_depth__member_rk ON public.sas_hierarchy_member_depth (member_id )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_hierarchy_member_depth__valid_from_dttm ON public.sas_hierarchy_member_depth (valid_from_dttm )
           ) by PASSTHRU;

        execute (
                 CREATE INDEX sas_hierarchy_member_depth__valid_to_dttm ON public.sas_hierarchy_member_depth (valid_to_dttm )
           ) by PASSTHRU;

        disconnect from PASSTHRU ;
    quit ;
    %checkErr;
    %if &_tmpRC %then
       %return ;
%end;

/******************************************************************************/
/*              Add columns USER_ID and TIMESTAMP_TS to SAS_FACT              */
/******************************************************************************/
%let _tblCount = 0;
%let _col1Count = 0;
%let _col2Count = 0;

proc sql noprint;
   select count(*) into :_tblCount
     from work.contents
    where upcase(memname) = "SAS_FACT";
quit;

%if (&_tblCount NE 0) %then %do;    /* if table sas_fact exists */
    proc sql noprint;
       select count(*) into :_col1_Count
         from work.contents
        where upcase(memname) = "SAS_FACT"
          and upcase(name) = "USER_ID";
    quit;

    proc sql noprint;
       select count(*) into :_col2Count
         from work.contents
        where upcase(memname) = "SAS_FACT"
          and upcase(name) = "TIMESTAMP_TS";
    quit;

%end;

%if (&_tblCount NE 0) and (&_col1Count EQ 0 or &_col2Count EQ 0) %then %do;
    proc sql ;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

        execute (
                 CREATE TABLE if not exists public.tmp_fact (
                       fact_set_id integer NOT NULL default '0',
                       fact_idx integer NOT NULL default '0',
                       concat_member_ids varchar(255) NOT NULL default '',
                       fact_value double precision NOT NULL default '0',
                       object_type_id integer NOT NULL default '0',
                       object_id integer NOT NULL default '0',
                       user_id varchar(255),
                       timestamp_ts timestamp without time zone,
                       CONSTRAINT tmp_fact_pkey PRIMARY KEY (FACT_SET_ID, FACT_IDX)
                )
           ) by PASSTHRU;

        disconnect from PASSTHRU ;

    quit;
    %checkErr;
    %if &_tmpRC %then
       %return ;

    %if &_col1Count EQ 0 and &_col2Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_fact
                    SELECT fact_set_id, fact_idx, concat_member_ids, fact_value, object_type_id, object_id, '', now()
                    FROM public.sas_fact
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col1Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_fact
                    SELECT fact_set_id, fact_idx, concat_member_ids, fact_value, object_type_id, object_id, '', timestamp_ts
                    FROM public.sas_fact
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_fact
                    SELECT fact_set_id, fact_idx, concat_member_ids, fact_value, object_type_id, object_id, user_id, now()
                    FROM public.sas_fact
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
    %end;

    proc sql;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

        execute (
                 DROP TABLE public.sas_fact
           ) by PASSTHRU;

        execute (
                 CREATE TABLE if not exists public.sas_fact (
                       fact_set_id integer NOT NULL default '0',
                       fact_idx integer NOT NULL default '0',
                       concat_member_ids varchar(255) NOT NULL default '',
                       fact_value double precision NOT NULL default '0',
                       object_type_id integer NOT NULL default '0',
                       object_id integer NOT NULL default '0',
                       user_id varchar(255),
                       timestamp_ts timestamp without time zone,
                       CONSTRAINT sas_fact_pkey PRIMARY KEY (FACT_SET_ID, FACT_IDX)
                )
           ) by PASSTHRU;

        execute (
                 INSERT INTO public.sas_fact
                 SELECT * FROM public.tmp_fact
           ) by PASSTHRU;

        execute (
                 DROP TABLE public.tmp_fact
           ) by PASSTHRU;

        disconnect from PASSTHRU ;
    quit ;
    %checkErr;
    %if &_tmpRC %then
       %return ;

%end;

/******************************************************************************/
/*   Add columns MIN_VALUE, MAX_VALUE and PROPERTY_OPTIONS to SAS_PROPERTY    */
/******************************************************************************/
%let _tblCount = 0;
%let _col1Count = 0;
%let _col2Count = 0;
%let _col3Count = 0;

proc sql noprint;
   select count(*) into :_tblCount
     from work.contents
    where upcase(memname) = "SAS_PROPERTY";
quit;

%if (&_tblCount NE 0) %then %do;    /* if table sas_fact exists */
    proc sql noprint;
       select count(*) into :_col1_Count
         from work.contents
        where upcase(memname) = "SAS_PROPERTY"
          and upcase(name) = "MIN_VALUE";
    quit;

    proc sql noprint;
       select count(*) into :_col2Count
         from work.contents
        where upcase(memname) = "SAS_PROPERTY"
          and upcase(name) = "MAX_VALUE";
    quit;

    proc sql noprint;
       select count(*) into :_col3Count
         from work.contents
        where upcase(memname) = "SAS_PROPERTY"
          and upcase(name) = "PROPERTY_OPTIONS";
    quit;
%end;

%if (&_tblCount NE 0) and (&_col1Count EQ 0 or &_col2Count EQ 0 or &_col3Count EQ 0) %then %do;
    proc sql ;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

        execute (
                 CREATE TABLE if not exists public.tmp_property (
                   property_id integer NOT NULL DEFAULT 0,
                   revision integer NOT NULL DEFAULT 0,
                   property_cd character varying(32) NOT NULL DEFAULT ''::character varying,
                   property_type_enum smallint NOT NULL DEFAULT 0::smallint,
                   custom_flg smallint NOT NULL DEFAULT 1,
                   dflt_value character varying(255) DEFAULT NULL::character varying,
                   enum_values text,
                   min_value double precision,
                   max_value double precision,
                   property_options integer NOT NULL DEFAULT 1,
                   CONSTRAINT tmp_property_pkey PRIMARY KEY (property_id )
                )
           ) by PASSTHRU;

        disconnect from PASSTHRU ;

    quit;
    %checkErr;
    %if &_tmpRC %then
       %return ;

    %if &_col1Count EQ 0 and &_col2Count EQ 0 and &_col3Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col1Count EQ 0 and &_col2Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values, NULL, NULL, property_options
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col1Count EQ 0 and &_col3Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values, NULL, max_value, 1
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col2Count EQ 0 and &_col3Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values, min_value, NULL, 1
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col1Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values, NULL, max_value, property_options
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col2Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values, min_value, NULL, property_options
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;
    %else %if &_col3Count EQ 0 %then %do;
       proc sql;
           connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

           execute (
                    INSERT INTO public.tmp_property
                    SELECT property_id, revision, property_cd, property_type_enum, custom_flg, dflt_value, enum_values, min_value, max_value, 1
                    FROM public.sas_property
                   ) by PASSTHRU;

           disconnect from PASSTHRU;

       quit;
       %checkErr;
       %if &_tmpRC %then
          %return ;
    %end;

    proc sql;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;

        execute (
                 DROP TABLE public.sas_property
           ) by PASSTHRU;

        execute (
                 CREATE TABLE if not exists public.sas_property (
                   property_id integer NOT NULL DEFAULT 0,
                   revision integer NOT NULL DEFAULT 0,
                   property_cd character varying(32) NOT NULL DEFAULT ''::character varying,
                   property_type_enum smallint NOT NULL DEFAULT 0::smallint,
                   custom_flg smallint NOT NULL DEFAULT 1,
                   dflt_value character varying(255) DEFAULT NULL::character varying,
                   enum_values text,
                   min_value double precision,
                   max_value double precision,
                   property_options integer NOT NULL DEFAULT 1,
                   CONSTRAINT sas_property_pkey PRIMARY KEY (property_id )
                )
           ) by PASSTHRU;

        execute (
                 CREATE UNIQUE INDEX sas_property__code ON public.sas_property (property_cd COLLATE pg_catalog."default" )
           ) by PASSTHRU;

        execute (
                 INSERT INTO public.sas_property
                 SELECT * FROM public.tmp_property
           ) by PASSTHRU;

        execute (
                 DROP TABLE public.tmp_property
           ) by PASSTHRU;

        disconnect from PASSTHRU ;
    quit ;
    %checkErr;
    %if &_tmpRC %then
       %return ;

%end;

/******************************************************************************/
/* SAS_FACT table is split into multiple number of fact set tables named      */
/* as SAS_FACT_XXX.  XXX is a fact set ID                                     */
/******************************************************************************/
%let _tblCount = 0 ;

proc sql noprint;
   select count(*) into :_tblCount
     from work.contents
    where upcase(memname) = "SAS_FACT";
quit;

%if &_tblCount NE 0 %then %do ;


    proc sql;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );

        execute (
           create or replace function separate_col(a_fact_set_id integer) returns void as $$

            DECLARE

               sql_str              varchar(10000);
               v_fact_set_id        smallint;
               v_fact_idx           bigint;
               v_concat_member_ids  varchar(255);
               v_fact_value         double precision;
               v_object_type_id     integer;
               v_object_id          integer;
               v_user_id            varchar(255);
               v_timestamp_ts       timestamp without time zone;

               sql_prefix           varchar(50);
               i                    bigint;
               _record              record;

            BEGIN

               sql_prefix=concat('insert into public.SAS_FACT_', a_fact_set_id);
               sql_prefix=concat(sql_prefix, ' values(');

               i = 1;
               sql_str = sql_prefix;

               for _record in select fact_idx, object_type_id, object_id, concat_member_ids, fact_value, user_id, timestamp_ts from public.sas_fact
                        where fact_set_id=a_fact_set_id
               loop
                   v_fact_idx           = _record.fact_idx;
                   v_concat_member_ids  = _record.concat_member_ids;
                   v_fact_value         = _record.fact_value;
                   v_object_type_id     = _record.object_type_id;
                   v_object_id          = _record.object_id;
                   v_user_id            = _record.user_id;
                   v_timestamp_ts       = _record.timestamp_ts;

                   if mod(i, 50) != 1 then
                      sql_str=concat(sql_str, ',', '(');
                   end if;

                   sql_str=concat(sql_str, v_fact_idx, ',', v_object_type_id, ',', v_object_id, ',', v_concat_member_ids, ',', v_fact_value);

                   if v_user_id is null then
                       sql_str=concat(sql_str, ', NULL, ',  E'\'', v_timestamp_ts,  E'\'', ')');
                   else
                       sql_str=concat(sql_str, ',', E'\'', v_user_id, E'\'', ',',  E'\'', v_timestamp_ts, E'\'', ')');
                   end if;

                   if mod (i , 50) = 0 then
                      execute sql_str;
                      sql_str = sql_prefix;
                   end if;

                   i = i + 1;

               end loop;

               i = i - 1;

               if mod(i, 50) > 0 then
                   execute sql_str;
               end if;

           END;
           $$ LANGUAGE 'plpgsql';
        ) by PASSTHRU;

        execute (
           create or replace function create_idx(a_fact_set_id INT) returns void as $$

           DECLARE

               sql_str            varchar(10000);
               v_dim_type_id      integer;
               _record            record;

           BEGIN

               /* create indexes */
               /* Primary Index on (FACT_IDX) and a compisite key on ( object_type_id, objectid, ACCOUNT, TIME ) */
               /* Dimension type ID for ACCOUNT is 1, and for TIME is 9 */

               sql_str=concat('alter table public.SAS_FACT_',a_fact_set_id, ' add primary key (FACT_IDX)');

               execute sql_str;

               sql_str=concat('create index SAS_FACT_',a_fact_set_id,'__object_type_id on public.SAS_FACT_',a_fact_set_id,' (OBJECT_TYPE_ID, OBJECT_ID' );

               for _record in select ordinal from public.sas_fact_set_dim_type
                              where fact_set_id=a_fact_set_id and ( dim_type_id =1 or dim_type_id=9 )
                              order by dim_type_id
               loop
                  v_dim_type_id = _record.ordinal;
                  sql_str=concat(sql_str, ', DIM_', v_dim_type_id, '_MEM_ID');
               end loop;

               sql_str=concat(sql_str, ')');

               execute sql_str;

           END;
           $$ LANGUAGE 'plpgsql';
        ) by PASSTHRU;

        execute (
           create or replace function create_fact_set() returns void as $$

           DECLARE

               sql_prefix      varchar(50);
               sql_str         varchar(10000);
               v_fact_set_id   integer;
               v_num_dim       integer;
               i               integer;
               j               integer;
               fact_cnt        integer;
               _record         record;

           BEGIN

               sql_prefix='create table if not exists public.SAS_FACT_';

               i = 1;
               sql_str = sql_prefix;
               for _record in select fact_set_id, count(*) as num_dim from public.sas_fact_set_dim_type s
                              group by fact_set_id
               loop
                   v_fact_set_id = _record.fact_set_id;
                   v_num_dim = _record.num_dim;
                   sql_str=concat(sql_str, v_fact_set_id, '(');
                   sql_str=concat(sql_str, 'FACT_IDX int,');
                   sql_str=concat(sql_str, 'OBJECT_TYPE_ID int,');
                   sql_str=concat(sql_str, 'OBJECT_ID int,');

                   j = 1;
                   while (j <= v_num_dim)
           loop
                       sql_str=concat(sql_str, 'DIM_', j-1, '_MEM_ID int' );
                       if (j < v_num_dim ) then
                           sql_str=concat(sql_str, ',');
                       else
                           sql_str=concat(sql_str, ',FACT_VALUE double precision,' );
                           sql_str=concat(sql_str, 'USER_ID varchar(255),' );
                           sql_str=concat(sql_str, 'TIMESTAMP_TS timestamp without time zone' );
                           sql_str=concat(sql_str, ')' );
                       end if;
                       j = j + 1;
                   end loop;

                   execute sql_str;  /* Each sas_factxx is created */

                   select count(*) from public.sas_fact
                   where fact_set_id = v_fact_set_id
           into fact_cnt;

                   if fact_cnt > 0 then
                      /* Moving data here to separate fact set table */
                      perform separate_col (v_fact_set_id) ;
                   end if;

                   /* Creating index on each fact set table */
                   perform create_idx(v_fact_set_id);

                   /* reset sql_str varialbe for the next sas_fact_set table */
                   sql_str = sql_prefix;

                   i = i + 1;
               end loop;
           END;
           $$ LANGUAGE 'plpgsql';
        ) by PASSTHRU;

        execute (
                 select create_fact_set ()
        ) by PASSTHRU;

        /* remove old sas_fact table */

        execute (
            drop table public.sas_fact;
        ) by PASSTHRU;

        /* remove all three stored functions */

        disconnect from PASSTHRU ;

    quit ;

    %checkErr;
    %if &_tmpRC %then
      %return ;
%end;

/******************************************************************************/
/*             Update values in SAS_MEMBER (INTERNAL_DIM_TYPE_ID)             */
/******************************************************************************/

%let _dataCount = 0 ;

proc sql noprint;
    select count(*)
      into :_dataCount
      from _SDMLIB.SAS_MEMBER
     where INTERNAL_DIM_TYPE_ID=0;
quit;

%if &_dataCount > 0 %then %do ;

    proc sql ;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions ) ;
        execute (
                 create table _tmp_sas_member as
                 select a.MEMBER_ID, a.REVISION, c.INTERNAL_DIM_TYPE_ID
                 from public.SAS_MEMBER a
                 join public.SAS_DIMENSION b on a.INTERNAL_DIMENSION_ID=b.INTERNAL_DIMENSION_ID
                 join public.SAS_DIM_TYPE c on b.DIM_TYPE_ID=c.DIM_TYPE_ID
                      where a.INTERNAL_DIM_TYPE_ID=0

        ) by PASSTHRU;

        execute (
                 update public.SAS_MEMBER as a
                 set INTERNAL_DIM_TYPE_ID = b.INTERNAL_DIM_TYPE_ID
                 from _tmp_sas_member b
                 where a.MEMBER_ID = b.MEMBER_ID
                   and a.REVISION = b.REVISION
                   and a.INTERNAL_DIM_TYPE_ID=0
        ) by PASSTHRU;

        execute (
                 drop table if exists _tmp_sas_member
        ) by PASSTHRU;

        disconnect from PASSTHRU ;
    quit ;
    %checkErr;
    %if &_tmpRC %then
       %return ;
%end;

/******************************************************************************/
/*                 Hierarchy Member Depth Computation                         */
/******************************************************************************/
/*---------------------------------------------------------------*/
/*                       %insert_split_row                       */
/*---------------------------------------------------------------*/

proc sql noprint;
        connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );

        execute (
            drop table if exists public._sas_hierarchy_new
        ) by PASSTHRU;

        execute (
                 create table if not exists public._sas_hierarchy_new (like public.sas_hierarchy_member)
           ) by PASSTHRU ;

        execute (
                 insert into public._sas_hierarchy_new
                 select * from public.sas_hierarchy_member
                 where -1 > 1
           ) by PASSTHRU ;

        execute (
            alter table public._sas_hierarchy_new
            add column depth integer
        ) by PASSTHRU;

        execute (
            drop table if exists public.sas_hierarchy_error
        ) by PASSTHRU;

        execute (
                 create table if not exists public.sas_hierarchy_error (like public._sas_hierarchy_new)
           ) by PASSTHRU ;

        execute (
                 insert into public.sas_hierarchy_error
                 select * from public._sas_hierarchy_new
           ) by PASSTHRU ;

        execute (
            drop table if exists public._h
        ) by PASSTHRU;

        execute (
                 create table if not exists public._h (like public.sas_hierarchy_member)
           ) by PASSTHRU ;

        execute (
                 insert into public._h
                 select * from public.sas_hierarchy_member
                 where -1 > 1
           ) by PASSTHRU ;

        execute (
            create index _h_dttm on public._h (valid_from_dttm, valid_to_dttm)
        ) by PASSTHRU;

        execute (
            create table if not exists public._split (
                dttm timestamp without time zone,
                primary key (dttm)
            )
        ) by PASSTHRU;

        execute (
            create table if not exists public._split_tmp (
                dttm timestamp without time zone
            )
        ) by PASSTHRU;

        execute (
           create or replace function insert_split_row(IN h_id int,
                                                       IN m_id int,
                                                       IN r int,
                                                       IN p_id int,
                                                       IN rank int,
                                                       IN tmp_table_src varchar(50),
                                                       IN tmp_table_tgt varchar(50) ) returns void as $$

           DECLARE

               t1              timestamp without time zone;
               t2              timestamp without time zone;
               i               integer default 1;
               _record         record;

           BEGIN

               for _record in select dttm from public._split_tmp
               loop
                   if i = 1 then
                      t1 = _record.dttm;
                      i = i+1;
                   else
                      t1 = t2;
                   end if;

                   t2 = _record.dttm;

                   insert into public._h values (h_id, m_id, r, p_id, rank, t1, t2);

               end loop;

           END;
           $$ LANGUAGE 'plpgsql';
        ) by PASSTHRU;

        execute (
           create or replace function time_split(IN h_id int ) returns void as $$

           DECLARE

               m_id            integer;
               r               integer;
               p_id            integer;
               rank            integer;
               f_dttm          timestamp without time zone;
               t_dttm          timestamp without time zone;
               str_sql         varchar(200);
               num_rows        integer default 0;
               _record         record;

           BEGIN

               /* store unique time points in a table _split */
               truncate table public._split;
               truncate table public._split_tmp;

               insert into public._split_tmp
               select distinct valid_from_dttm from public.sas_hierarchy_member
               where hierarchy_id = h_id;

               insert into public._split_tmp
               select distinct valid_to_dttm from public.sas_hierarchy_member
               where hierarchy_id = h_id;

               insert into public._split
               select distinct dttm from public._split_tmp;

               /* empty temporary table _h
                  It stores time-splitted rows of one hierarchy ID
                  and it is just a copy of the structure of sas_hierarchy_member not including data. */
               truncate table public._h;

               /* Visiting each row via cursor for sas_hierarchy_member,
                  split the row if time interval include any inclusive time point in _split */

               for _record in select hierarchy_id, member_id, revision, parent_member_id, member_rank,
                                     valid_from_dttm, valid_to_dttm from public.sas_hierarchy_member
                              where hierarchy_id = h_id
               loop
                   h_id =   _record.hierarchy_id;
                   m_id =   _record.member_id;
                   r =      _record.revision;
                   p_id =   _record.parent_member_id;
                   rank =   _record.member_rank;
                   f_dttm = _record.valid_from_dttm;
                   t_dttm = _record.valid_to_dttm;

                   truncate table public._split_tmp;

                   insert into public._split_tmp
                   select dttm from public._split s
                   where s.dttm >= f_dttm and s.dttm <= t_dttm
                   order by dttm;

                   select count(*) into num_rows
                   from public._split_tmp;

                   if num_rows > 2 then
                       /* run another stored function */
                       perform insert_split_row(h_id, m_id, r, p_id, rank, '_split_tmp', '_h');
                   else
                       /* copy current row */
                       insert into public._h values (h_id, m_id, r, p_id, rank, f_dttm, t_dttm);
                   end if;

                   num_rows = 0;

               end loop;

           END;
           $$ LANGUAGE 'plpgsql';
        ) by PASSTHRU;

        execute (
           create or replace function merge_interval() returns void as $$

           DECLARE

               flag_last_c         integer default 0;
               h_id1               integer;
               h_id2               integer;
               m_id1               integer;
               m_id2               integer;
               rev1                integer;
               rev2                integer;
               d1                  integer;
               d2                  integer;
               p_id1               integer;
               p_id2               integer;
               rank1               integer;
               rank2               integer;
               f_dttm1             timestamp without time zone;
               f_dttm2             timestamp without time zone;
               t_dttm1             timestamp without time zone;
               t_dttm2             timestamp without time zone;
               new_rev             integer;
               _record             record;

           BEGIN

               new_rev = 1;
               h_id1 = 0;

               for _record in select hierarchy_id, member_id, revision, depth, parent_member_id, member_rank,
                                     valid_from_dttm, valid_to_dttm
                              from public._sas_hierarchy_new
                              order by hierarchy_id, member_id, valid_from_dttm, revision, depth, member_rank,
                                       parent_member_id, valid_to_dttm

               loop
                    if h_id1 = 0 then
                       h_id1   = _record.hierarchy_id;
                       m_id1   = _record.member_id;
                       rev1    = _record.revision;
                       d1      = _record.depth;
                       p_id1   = _record.parent_member_id;
                       rank1   = _record.member_rank;
                       f_dttm1 = _record.valid_from_dttm;
                       t_dttm1 = _record.valid_to_dttm;
                    else
                       h_id2   = _record.hierarchy_id;
                       m_id2   = _record.member_id;
                       rev2    = _record.revision;
                       d2      = _record.depth;
                       p_id2   = _record.parent_member_id;
                       rank2   = _record.member_rank;
                       f_dttm2 = _record.valid_from_dttm;
                       t_dttm2 = _record.valid_to_dttm;

                       if (h_id1=h_id2) and (m_id1=m_id2) and (d1=d2) and (t_dttm1=f_dttm2) then
                           t_dttm1=t_dttm2;
                       else
                           insert into public.sas_hierarchy_member_depth
                                values (h_id1, m_id1, new_rev, d1, f_dttm1, t_dttm1);
                           h_id1=h_id2;
                           if (m_id1 = m_id2) then
                               new_rev = new_rev + 1;
                           else
                               new_rev = 1;
                               m_id1=m_id2;
                           end if;
                           rev1=rev2;
                           d1=d2;
                           p_id1=p_id2;
                           rank1=rank2;
                           f_dttm1=f_dttm2;
                           t_dttm1=t_dttm2;
                       end if;

                    end if;

               end loop;

               insert into public.sas_hierarchy_member_depth
                    values (h_id1, m_id1, new_rev, d1, f_dttm1, t_dttm1);

           END;
           $$ LANGUAGE 'plpgsql';
        ) by PASSTHRU;


        disconnect from PASSTHRU;

quit;

%checkErr;
%if &_tmpRC %then
    %return ;


/*---------------------------------------------------------------*/
/*                       %depth_finder                           */
/*---------------------------------------------------------------*/
%macro depth_finder();

    %let cur_depth=0;

    sasfile _a open;

    data _a_root (keep=member_id);
        set _a (where = (parent_member_id is NULL));
    run;

    proc sql noprint;
        update _a
           set depth=&cur_depth
         where _a.member_id in (select member_id from _a_root);
    quit;

    %let cur_depth=1;
    %let cnt_a_root = 0;


    data _null_;
        if 0 then set _a_root nobs=nobs;
        call symputx('cnt_a_root',nobs);
        stop;
    run;

    %do %while (&cnt_a_root > 0);

       /* save _a_root into _a_root_old */
       data _a_root_old;
           set _a_root;
       run;

       sasfile _a_root_old open;

       /* empty _a_root */
       data _a_root;
           set _a_root;
           stop;
       run;

       proc sql noprint;
          insert into _a_root
          select member_id
            from _a
           where parent_member_id in (select member_id from _a_root_old);

           update _a
             set depth=&cur_depth
            where member_id in (select member_id from _a_root);
       quit;

       sasfile _a_root_old close;

       %let cur_depth=%eval(&cur_depth+1);

       data _null_;
           if 0 then set _a_root nobs=nobs;
           call symputx('cnt_a_root',nobs);
           stop;
       run;

    %end;


    proc append base=_sas_hierarchy_new data=_a &FM_PROCAPPENDFORCE;
    quit;

    sasfile _a close;

%mend depth_finder;


/*------------------------------------------------------------------*/
/*                       %extract_n_find_depth                      */
/*                                                                  */
/* For each [t1, t2] in _h                                          */
/*       _h_temp <----- all rows in _h in [t1, t2]                  */
/*       depth_finder() on _h_temp : using _a, _a_root, _a_root_old */
/*       _a <------ BFS to compute depth                            */
/*      _sas_hierarchy_new <----- _sas_hierarchy_new + _a           */
/*------------------------------------------------------------------*/

%macro extract_n_find_depth();
%let ds_splitID = %sysfunc(open( _split ));
%if &ds_splitID %then %do;
    %let a_dttm  = %sysfunc(varnum(&ds_splitID, dttm)) ;
    %let t1 = 0;
    %let t2 = 0;
    %let nRC = %sysfunc(rewind(&ds_splitID));

    %let flag_fetch= %sysfunc(fetch(&ds_splitID));
    %if (&flag_fetch ^= 0) %then %goto RETURNZERO;
    %let t1 = %sysfunc(getvarn(&ds_splitID, &a_dttm));

    %do %while ( &flag_fetch EQ 0);

        /* In case valid_from_dttm = valid_to_dttm */

        data _a (index = (parent_member_id member_id ));
            set _h (where= (valid_from_dttm=&t1 and valid_to_dttm=&t1) );
            length depth 8;
        run;

        %depth_finder();


        /* In case valid_from_dttm ^= valid_to_dttm */
        %let flag_fetch= %sysfunc(fetch(&ds_splitID));
        %if (&flag_fetch EQ 0) %then %do;
            %let t2 = %sysfunc(getvarn(&ds_splitID, &a_dttm));

            data _a (index = (parent_member_id member_id ));
                set _h (where= (valid_from_dttm=&t1 and valid_to_dttm=&t2) );
                length depth 8;
            run;

            %depth_finder();


            %let t1 = &t2;
        %end;
    %end;

    %let dsClose = %sysfunc(close(&ds_splitID));
%end;
%else %do;
    %goto RETURNZERO;
%end;

%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
%end;
%goto THEEND;

%RETURNZERO:
    %let _FMErrorCode=0;
%THEEND:

%mend extract_n_find_depth;


/*---------------------------------------------------------------*/
/*                       %all_depth_finder                       */
/*---------------------------------------------------------------*/
%macro all_depth_finder();

/* block log */

filename junk dummy;
proc printto  log=junk; run;


/* copy sas_hierarchy_member to SAS work area */
proc sql noprint;
     create table work.sas_hierarchy_member as
     select *
       from _SDMLIB.sas_hierarchy_member;
quit;

/* result table (without merged intervals) */
data _sas_hierarchy_new;
     set sas_hierarchy_member;
     length depth 8;
     stop;
run;

/* result table (with merged intervals) */
proc sql noprint;

run;

%let num_h = 0;
proc sql noprint;
    select max(hierarchy_id)
      into :num_h
      from sas_hierarchy_member;
quit;

data _h;
     set sas_hierarchy_member;
     stop;
run;

data _a;
     set _h;
     length depth 8;
     stop;
run;

data _a_root;
     length member_id 8;
     stop;
run;

data _a_root_old;
     length member_id 8;
     stop;
run;



/* sasfile sas_hierarchy_member load; */


%let _time_1=0;  /* sum of all time spent on %time_split() */
%let _time_2=0;  /* sum of all time spent on %extract_n_find_depth() */
%let _time_3=0;  /* sum of all time spend on %merge_interval() */

%let _time_x=0;

%let i = 1;

%do %while (&i <= &num_h );

    %let num_h_id = 0;
    proc sql noprint;
        select count(*)
      into :num_h_id
          from sas_hierarchy_member
         where hierarchy_id = &i;
    quit;

    %if &num_h_id > 0 %then %do;
        /* stop blocking to print out running status */

        proc printto; run;

        %put ------------------------------------------------;
        %put Hierarchy Id :&i;
        %put Number of rows (before conversion) : &num_h_id;

        /*---------call %time_split------------*/
        /* Distinct time points are stored in _split*/
        /* Newly splitted rows are stored in _h */


        /* Start blocking log */

        filename junk dummy;
        proc printto  log=junk; run;


        %let _start1=%sysfunc( time() );
        proc sql noprint;

            %let _startx=%sysfunc( time() );
            connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
            %let _endx=%sysfunc( time() );
            %let _time_x= %sysevalf  (&_time_x + %sysfunc( intck(second, &_startx, &_endx )));

            execute (
                select time_split(&i)
            ) by PASSTHRU;

            disconnect from PASSTHRU;

            insert into _h
            select *
              from _SDMLIB._h;

            create table _split as
            select *
              from _SDMLIB._split
             order by dttm;

            /* sasfile _h open; */
            sasfile _split open;

        quit;

        %checkErr;
        %if &_tmpRC %then
            %return ;

        %let _end1=%sysfunc( time() );
        %let _time_1= %sysevalf  (&_time_1 + %sysfunc( intck(second, &_start1, &_end1 )));


        /*---------call %extract_n_find_depth------------*/
        /* compute depth in each time interval */
        /* In each time interval from _h */
        /* _h_temp stores all rows from _h for a time interval */
        /* It calls %depth_finder on _h_temp */
        /* _a stores BFS result */
        /* _sas_hierarchy_new stores result in _a */

        %let _start2=%sysfunc( time() );
            %extract_n_find_depth ();
        %let _end2=%sysfunc( time() );

        /* sasfile _h close; */
        sasfile _split close;

        %let _time_2= %sysevalf  (&_time_2 + %sysfunc( intck(second, &_start2, &_end2 )));

        /* Count new rows for hierarchy_id = i */
        %let num_h_id = 0;

        %let _start3=%sysfunc( time() );
        proc sql noprint;
            insert into _SDMLIB._sas_hierarchy_new
            select *
              from _sas_hierarchy_new
             where depth is not null;

            insert into _SDMLIB.sas_hierarchy_error
            select *
              from _sas_hierarchy_new
             where depth is null;


            connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
            execute (
                     select merge_interval()
            ) by PASSTHRU;

            execute (
                truncate table public._sas_hierarchy_new
            ) by PASSTHRU;

            disconnect from PASSTHRU ;

        quit;
        %let _end3=%sysfunc( time() );
        %let _time_3= %sysevalf  (&_time_3 + %sysfunc( intck(second, &_start3, &_end3 )));

        /* stop blocking to print out running status */

        proc printto; run;


        %put Number of rows (after conversion) : &num_h_id;

        /* Start blocking log */

        filename junk dummy;
        proc printto  log=junk; run;


        /* empty temporary tables */

        data _split;
            set _split;
            stop;
        run;

        data _h;
            set _h;
            stop;
        run;

        data _sas_hierarchy_new;
            set _sas_hierarchy_new;
            stop;
        run;

    %end; /* of if */

    %let i = %eval(&i + 1);
%end; /* of while */


/* create error table */

proc sql noprint;
    connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );

    execute (
      drop table if exists public._h
      ) by PASSTHRU;

    execute (
      drop table if exists public._split
      ) by PASSTHRU;

    execute (
        drop table if exists public._split_tmp
    ) by PASSTHRU;

    execute (
      drop table if exists public._sas_hierarchy_new
      ) by PASSTHRU;

    execute (
      drop function separate_col(integer)
      ) by PASSTHRU;

    execute (
      drop function create_idx(integer)
      ) by PASSTHRU;

    execute (
      drop function create_fact_set()
      ) by PASSTHRU;

    execute (
      drop function time_split(integer)
      ) by PASSTHRU;

    execute (
      drop function insert_split_row(integer, integer, integer, integer, integer, character varying, character varying)
      ) by PASSTHRU;

    execute (
           drop function merge_interval()
      ) by PASSTHRU;

    disconnect from PASSTHRU ;

quit ;

%checkErr;
%if &_tmpRC %then
  %return ;


/* block log */

proc printto; run;

%put ********** Time spent on Depth Finding ***********;
%put ********** Elapsed time for time_split: &_time_1 seconds;
%put ********** Elapsed time for extract_n_find_depth : &_time_2 seconds;
%put ********** Elapsed time for merging data : &_time_3 seconds;
%put ********** Elapsed time for DB connection: &_time_x seconds;

%mend all_depth_finder;

%let _dataCount = 0;
proc sql noprint;
    select count(*)
     into :_dataCount
      from _SDMLIB.SAS_HIERARCHY_MEMBER_DEPTH;
quit;

%if &_dataCount EQ 0 %then %do ;
    %all_depth_finder();
%end;

/******************************************************************************/
/*                 Set status of migration to Successful                      */
/******************************************************************************/

%sesssver(solName=ODCS, solVer=&sesetl_version, solStatus=Successful, solMethod=Migration);

%mend migrod53;
