/*-- Copyright (c) 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. --*/
/* This program is for synchronzing the users in metadata with the SASSDM USERS_X_GROUPS table*/

%macro sdmuxg;


%bldLibNm(sesetl_sasLibRef=_SDMLIB
          , sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SDMLIBOBJ.
          , sesetl_useType=BASE
          , sesetl_validateLib=Y
           ) ;

/* extract the users/groups from OMR into the WORK libref */
%mduextr(libref=WORK);

proc sql noprint;
  create table SOURCE_SYSTEM AS
   SELECT * FROM _SDMLIB.SAS_SOURCE_SYSTEM
   WHERE upcase(SOURCE_SYSTEM_CD)= 'ETL';
quit;
DATA SOURCE_SYSTEM; FORMAT SOURCE_SYSTEM_ID 8.; SET SOURCE_SYSTEM; RUN;

proc sql noprint;
   SELECT SOURCE_SYSTEM_ID INTO :ETLrk FROM SOURCE_SYSTEM
   WHERE upcase(SOURCE_SYSTEM_CD)= 'ETL';
quit;


DATA GRPMEMS_NEW(rename=(GRPKEYID=GROUP_ID MEMKEYID=keyid ));
SET WORK.GRPMEMS;
RUN;

/* merge to pickup name of the person */
proc sort data=WORK.PERSON out=PERSON force; by KEYID ; run;
proc sort data=GRPMEMS_NEW out=GRPMEMS; by KEYID ; run;

data GRPMEMS1(keep=USER_GROUP_ID USER_GROUP_TYPE_FLG GROUP_ID);
merge GRPMEMS(IN=a) PERSON(IN=b) ;
by KEYID;
IF a AND b;
USER_GROUP_ID=NAME;
USER_GROUP_TYPE_FLG=1; /* USER_GROUP_ID is a USER */
RUN;

/* merge to pickup name of the group */
proc sort data=GRPMEMS1(rename=(GROUP_ID=KEYID)) out=GRPMEMS; by KEYID ; run;

proc sort data=WORK.IDGRPS(where=(upcase(GRPTYPE) ne 'ROLE')) out=IDGRPS force; by KEYID ; run;

data GRPMEMS1(keep=USER_GROUP_ID GROUP_ID USER_GROUP_TYPE_FLG  );
merge GRPMEMS(IN=a) IDGRPS(IN=b) ;
by KEYID;
IF a AND b;
GROUP_ID=NAME;
RUN;


/* merge to pickup name of the group */
proc sort data=GRPMEMS_NEW(rename=(KEYID=MEMKEYID GROUP_ID=KEYID)) out=GRPMEMS; by KEYID ; run;

data GRPMEMS2(keep=GROUP_ID  MEMKEYID);
merge GRPMEMS(IN=a) IDGRPS(IN=b) ;
by keyid;
IF a AND b;
GROUP_ID=name;
RUN;

proc sort data=GRPMEMS2(RENAME=MEMKEYID=KEYID) out=GRPMEMS; by KEYID ; run;

/* merge to pickup name of the member group */
data GRPMEMS2(keep=USER_GROUP_ID GROUP_ID USER_GROUP_TYPE_FLG);
merge GRPMEMS(IN=a) IDGRPS(IN=b ) ;
by KEYID;
IF a AND b;
USER_GROUP_ID=NAME;
USER_GROUP_TYPE_FLG=0; /* USER_GROUP_ID is a GROUP */
RUN;

DATA GRPMEMS3;
SET GRPMEMS1 GRPMEMS2;
SOURCE_SYSTEM_ID=.;
%if %symexist(etlrk)=1 %then SOURCE_SYSTEM_ID=&ETLRK;;
run;


%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SDMLIBOBJ.
        , sesetl_useType=CON)

run;

%if %symexist(etlrk)=1 %then %do;
 proc sql;
   connect to &sesetl_EngineType as PASSTHRU
                   (&sesetl_BestLogin &sesetl_ConnectionOptions );
           execute(DELETE FROM SAS_USER_X_GROUP
                   ) by PASSTHRU;
      disconnect from PASSTHRU;
 quit;
%end;
PROC append base=_SDMLIB.SAS_USER_X_GROUP data=GRPMEMS3 force ;run;


/********************************************************************************/
/****                                                                        ****/
/**** This section populates a mysql table that contains a users relation to ****/
/**** groups as well as how many "steps" the user is from that group (direct ****/
/**** or indirect member).  The grouplvl macro does this.  The               ****/
/**** getgroupforgroup macro is a utility macro that is called recursively.  ****/
/****                                                                        ****/
/********************************************************************************/

data GRPMEMS1_trans;
  length user_group_id group_id $180;
  set GRPMEMS1;
  if indexc(user_group_id, '()') or indexc(group_id, '()') then do;
    if indexc(user_group_id, '()') then do;
      user_group_id = tranwrd(user_group_id, '(', '^~~');
      user_group_id = tranwrd(user_group_id, ')', '~~^');
    end;
    if indexc(group_id, '()') then do;
      group_id = tranwrd(group_id, '(', '^~~');
      group_id = tranwrd(group_id, ')', '~~^');
    end;
  end;
run;

data GRPMEMS3_trans;
  length user_group_id group_id $180;
  set GRPMEMS3;
  if indexc(user_group_id, '()') or indexc(group_id, '()') then do;
    if indexc(user_group_id, '()') then do;
      user_group_id = tranwrd(user_group_id, '(', '^~~');
      user_group_id = tranwrd(user_group_id, ')', '~~^');
    end;
    if indexc(group_id, '()') then do;
      group_id = tranwrd(group_id, '(', '^~~');
      group_id = tranwrd(group_id, ')', '~~^');
    end;
  end;
run;

%macro _creds(_usrgrpid=, _usrgrpflg=, _grpid=, _levelds=);
  data _tempds;
    set grpmems3_trans(where=(user_group_type_flg=0 and user_group_id="&_grpid"));
  length group_id1 $180;
  group_id1 = group_id;
  group_id  = user_group_id;
  user_group_id = "&_usrgrpid";
  user_group_type_flg = &_usrgrpflg;
  run;
  proc append base=&_levelds data=_tempds &FM_PROCAPPENDFORCE;
  run;
%mend;

proc datasets lib=work;
  delete _level: sort_level:;
run;
filename _temp temp lrecl=1024;

%let _nobs=-1;

%macro _jumps(_dsin=, _dsout=, _i=);
%let _nobs=-1;
data _null_;
  file _temp;
  set &_dsin;
  _levds = "&_dsout";
  _jumps = symget('_i');
  put '%_creds(_usrgrpid=%bquote(' user_group_id '), _grpid=%bquote(' group_id
             '), _usrgrpflg=' _jumps  ', _levelds=' _levds ');';
run;
%inc _temp;

proc sql noprint;
  select nobs into :_nobs
  from dictionary.tables
  where libname = 'WORK'
    and memname = upcase("&_dsout");
  ;
quit;

%if &_nobs gt 0 %then %do;
  proc sort data = &_dsout
             out = sort&_dsout.(drop=group_id rename=(group_id1 = group_id))
            nodupkey;
    by user_group_id group_id1;
  run;
  proc append base=_X_JUMPS
              data=sort&_dsout.(rename=(user_group_type_flg=JUMPS
                             user_group_id=USER_ID)
                                drop=source_system_id) &FM_PROCAPPENDFORCE;
  run;
%end;

%mend;

%let _i=2;
data _X_JUMPS;
  set grpmems1_trans(rename=(user_group_type_flg=JUMPS
                       user_group_id=USER_ID));
run;
%_jumps(_dsin=grpmems1_trans, _dsout=_level2, _i=&_i);

%macro _loop;
  %do %while(&_nobs ne 0);
    %_jumps(_dsin=sort_level&_i, _dsout=_level%eval(&_i + 1), _i=%eval(&_i + 1));
    %if &_nobs ne 0 %then %let _i=%eval(&_i + 1);
  %end;
%mend;
%_loop;


/* report problems */
data _X_JUMPS_trans(drop=userid groupid);
  length user_id group_id $60;
  set _X_JUMPS(rename=(user_id=userid group_id=groupid));
  if userid ne tranwrd(userid, '^~~', '(') or userid ne tranwrd(userid, '~~^', ')') then do;
    userid = tranwrd(userid, '^~~', '(');
    userid = tranwrd(userid, '~~^', ')');
  end;
  if groupid ne tranwrd(groupid, '^~~', '(') or groupid ne tranwrd(groupid, '~~^', ')') then do;
    groupid = tranwrd(groupid, '^~~', '(');
    groupid = tranwrd(groupid, '~~^', ')');
  end;
  user_id = userid;
  group_id = groupid;
run;
proc sort data=_X_JUMPS_trans out=_X_JUMPS;
        by USER_ID GROUP_ID JUMPS;
run;

data problems newgroups;
        set _X_JUMPS;
        by USER_ID GROUP_ID;
        /* user should not be in the same group more than one way (directly and indirectly). */
        if not(first.GROUP_ID and last.GROUP_ID) then do;
                output problems;
                if first.GROUP_ID then output newgroups;
        end;
        else output newgroups;
run;

data _null_;
        set problems end=eof;
        by USER_ID GROUP_ID;
        if _N_ = 1 then do;
                put '***********************************************************************';
                put '****                                                               ****';
                put '**** Note: The following records show that user(s) are in a group  ****';
                put '**** both directly and indirectly.  If a user is in a group        ****';
                put '**** indirectly (i.e. they are in a group that is in that group),  ****';
                put '**** and in the same group directly, the direct association        ****';
                put '**** will be kept.                                                 ****';
                put '****                                                               ****';
                put '***********************************************************************';
        end;
       _keep = 'Discard: ';
        if first.GROUP_ID then do;
          _keep = 'Keep   : ';
          put ;
        end;

        put @3 _keep USER_ID= GROUP_ID= JUMPS=;
        if eof then do;
                put ;
                put '***********************************************************************';
                put '****                                                               ****';
                put '**** end of Note.                                                  ****';
                put '****                                                               ****';
                put '***********************************************************************';
        end;
run;

%if %symexist(etlrk)=1 %then %do;
        proc sql;
        connect to &sesetl_EngineType as PASSTHRU
                   (&sesetl_BestLogin &sesetl_ConnectionOptions );
        execute(DELETE FROM SAS_USER_X_GROUP_X_JUMPS) by PASSTHRU;
        disconnect from PASSTHRU;
        quit;
%end;

PROC append base=_SDMLIB.SAS_USER_X_GROUP_X_JUMPS data=newgroups force ;
run;

%mend sdmuxg;
