
%MACRO sesloc;


/**********************************************************
 * sesloc_checkSASLocaleDup.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_checkSASLocaleDup.sas                                                    */
/*  Purpose: Verify there are no dupes in the SAS CODE_LANGUAGE table on locale.             */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_checkSASLocaleDup(cSASLib, cSASTable, nDTTM);

  %let cMethodNm = sesloc_checkSASLocaleDup;
  %let cTableNm = %upcase(%qsysfunc(strip(&cSASLib))).%upcase(%qsysfunc(strip(&cSASTable)));
  %let cTmpTable1 = WORK._TMP_SAS_LOCALE;
  %let rk = 1 ;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if not %sysfunc(exist(&cTableNm, DATA)) %then %do;
    %let error_number=1;
    %let errObj=&cTableNm;
    %goto ERRORRETURN;
  %end ;

  %if %sysfunc(exist(&cTmpTable1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTmpTable1, 2, '.');
    quit;
  %end;

  proc sort data=&cTableNm(where=(&_where1))
             out=&cTmpTable1
          dupout=dups
          nodupkeys;
    by LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number = 2;
    %let errObj = &cTmpTable1;
    %goto ERRORRETURN;
  %end;

  %let soletl_tempValue1 = ;
  data _null_;
      set dups(obs=1);
      call symput('soletl_tempValue1', strip(LOCALE_LANGUAGE_CD)
                               !! '_' !! strip(LOCALE_COUNTRY_CD));
      stop;
  run;
  %if ((&SYSERR ne 0 and &SYSERR ne 4) or %nrbquote(&soletl_tempValue1) ne %str())  %then %do;
      %sesrpt_logError( &cLogEntryUtil, L, 3, &soletl_tempValue1, &cTableNm, &cMethodNm);
      %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_checkSASLocaleDup;


/**********************************************************
 * sesloc_importLocale.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : LoadCycleBaseFact.sas                                                           */
/*  PURPOSE:   Import SAS CODE_LANGUAGE locales to the SASSDM.SAS_ETL_DATA_LOCALE            */
/*                                                                                           */
/*             Also, upload (%if available) locale information for pre-defined               */
/*             dimensional metadata.                                                         */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesloc_importLocale;

  %global _SOLErrorCode;
  %let _SOLErrorCode=1; /* 0: error, 1:normal execution */

  %let nTrackTime = %sysfunc(datetime());

  %sesloc_init;
  %sesrpt_init;

  /*  Seed initial return values  */
  %let soletl_statusId=1;  /* success=0,error=1, warning=3  */

  /*  Turn logging information on/off based on soletl_debug macro value  */
  %let cSaveOptNote = %sysfunc(getoption(NOTES));
  %let cSaveOptSrc  = %sysfunc(getoption(SOURCE));

  %let _LocaleUtil_getDebug = %sesrpt_getDebug;
  %if &_LocaleUtil_getDebug eq 1 %then %do;
    options NOTES SOURCE;
  %end;

  %let cOMRServer     = %sesrpt_removeQuotes(&soletl_omrServer);
  %let nOMRPort       = &soletl_omrPort;
  %let cConformLib    = &soletl_conformLibref;
  %let nNowDTTM       = &soletl_nowDTTM;
  %let nETLInstanceId = &soletl_instanceId;
  %let cSDMLib        = &soletl_sdmLibref;

  /*  Assign constants  */
  %let cSCLEntryNm          = sesloc_importLocale;
  %let cLogEntry            = LocaleImport;
  %let cLogEntryUtil        = LocaleImportUtil;
  %let cCodeLangTableNm     = CODE_LANGUAGE;
  %let cETLLocaleTableNm    = SAS_ETL_DATA_LOCALE;
  %let cLocaleStringTableNm = WORK._LOCALE_TO_STRING;

  %let _where1=((VALID_FROM_DTTM <= &soletl_nowDTTM)
          and (VALID_TO_DTTM >= &soletl_nowDTTM or VALID_TO_DTTM = .));

  /*  Make sure required values have been supplied  */
  %if &cOMRServer eq  %then %do;
      %sesrpt_logError( &cLogEntry, L, 1, &soletl_omrServer, &cSCLEntryNm);
      %goto Error;
  %end;

  %if &nOMRPort EQ . or &nOMRPort EQ 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 1, &soletl_omrPort, &cSCLEntryNm);
    %goto Error;
  %end;

  %if &cConformLib eq  %then %do;
    %sesrpt_logError( &cLogEntry, L, 1, &soletl_conformLibref, &cSCLEntryNm);
    %goto Error;
  %end;

  %if &nNowDTTM EQ . or &nNowDTTM EQ 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 1, &soletl_nowDTTM, &cSCLEntryNm);
    %goto Error;
  %end;

  %if &nETLInstanceId EQ . or &nETLInstanceId EQ 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 1, &soletl_InstanceId, &cSCLEntryNm);
    %goto Error;
  %end;

  %if &cSDMLib eq  %then %do;
    %sesrpt_logError( &cLogEntry, L, 1, &soletl_sdmLibref, &cSCLEntryNm);
    %goto Error;
  %end;


  /*
  *  Conform Libref
  */
  %bldLibNm(sesetl_sasLibRef=&cConformLib
          , sesetl_OMRSASLibraryObjName=&FMdatatier_folder./&CONFLIBOBJ./&CONFLIBOBJ.
          , sesetl_useType=META
          , sesetl_validateLib=Y
          );

  %if %qsysfunc(libref(&cConformLib)) NE 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 2, strip(&cConformLib), &cSCLEntryNm);
    %goto Error;
  %end;

  /*
  *  SDM Libref
  */
  %bldLibNm(sesetl_sasLibRef=&cSDMLib
             , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
             , sesetl_useType=BASE,
               sesetl_validateLib=Y
              ) ;

  %if %qsysfunc(libref(&cSDMLib)) NE 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 2, strip(cSDMLib), &cSCLEntryNm);
    %goto Error;
  %end;


  /*
   *  Check for Locale dupes in the SAS CODE_LANGUAGE table.
   */
  %sesloc_checkSASLocaleDup(&cConformLib, &cCodeLangTableNm, &nNowDTTM);

  %if &_SOLErrorCode EQ 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 3, &cConformLib, &cCodeLangTableNm, &&cSCLEntryNm);
    %goto Error;
  %end;

    /*
     *  Upload SAS CODE_LANGUAGE table to SASSDM.SAS_ETL_DATA_LOCALE
     */
  %sesloc_uploadLocaleUpdate(&cConformLib, &cCodeLangTableNm, &nNowDTTM, &cSDMLib,
                            &cETLLocaleTableNm, &nETLInstanceId, &cLocaleStringTableNm);


  %if &_SOLErrorCode EQ 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 4, &cCodeLangTableNm, &cETLLocaleTableNm, &cSCLEntryNm);
    %goto Error;
  %end;

  /*
  *  Upload Predefined Static Dimensional Metadata Labels
  */
  %sesloc_uploadPredefLabel(&cLocaleStringTableNm, &nETLInstanceId, &cSDMlib);
  %if &_SOLErrorCode EQ 0 %then %do;
    %sesrpt_logError( &cLogEntry, L, 5, &cSCLEntryNm);
    %goto Error;
  %end;

  /*
  *  Set import status to successful
  */
  %let soletl_statusId=0;  /* success=0,error=1, warning=3  */

  /*  Dump the Time  */
  %let nTrackTime = %qsysfunc(datetime()) - nTrackTime;

  %let _LocaleUtil_getTimer = %sesrpt_getTimer;
  %if &_LocaleUtil_getTimer NE . %then %do;
    %put TIMER: total time to Import Locales: &nTrackTime;
  %end;

  %goto THEEND;

  %Error:
    %let soletl_statusId=1;  /* success=0, critical error=1, validation error=2, warning=3  */

  %THEEND:

%mend sesloc_importLocale;


/**********************************************************
 * sesloc_init.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_init.sas                                                               */
/*  Purpose: Initialize global variables for (class) methods                                 */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_init;

  %global nDebug     /* If equal Y (default=N), turns on debugging. */
          nTimer     /* If equal Y (default=N), will dump timing information to log. */
          ;

  %let nDebug = 0 ;
  %if %symexist(soletl_debug) %then %do;
    %if %upcase(&soletl_debug) eq Y %then %let nDebug = 1 ;
  %end;

  %let nTimer = 0 ;
  %if %symexist(soletl_timer) %then %do;
    %if %upcase(&soletl_timer) eq Y %then %let nTimer = 1 ;
  %end;

%mend sesloc_init;


/**********************************************************
 * sesloc_uploadDimLabel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadDimLabel.sas                                                */
/*  Purpose: Upload predefined metadata labels for Dimensions.                               */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_uploadDimLabel(nETLInstanceId, cLocale, cSDMLib);

  %let cMethodNm = sesloc_uploadDimLabels;
  %let cWorkTableNm = WORK.SAS_ETL_DIMENSION;
  %let cSDMTableNm = %upcase(%qsysfunc(strip(&cSDMLib))).SAS_ETL_DIMENSION;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if not %sysfunc(exist(&cSDMTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSDMTableNm;
    %goto ERRORRETURN;
  %end ;

  %if %sysfunc(exist(&cWorkTableNm)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cWorkTableNm, 2, '.');
    quit;
  %end;

  data &cWorkTableNm;
      length ETL_INSTANCE_ID 8;
      length DIM_TYPE_CD $ %eval(32*&FM_DBCSMULT);
      length DIMENSION_CD $ %eval(32*&FM_DBCSMULT);
      length DIMENSION_NM $ %eval(50*&FM_DBCSMULT);
      length DIMENSION_DESC $ %eval(255*&FM_DBCSMULT);
      length LOCALE_CD $ %eval(38*&FM_DBCSMULT);
      length LOCALE_ID 8;
      length VALID_FROM_DTTM 8;
      length VALID_TO_DTTM 8;

      ETL_INSTANCE_ID = &nETLInstanceId;
      LOCALE_CD = "&cLocale";
      LOCALE_ID = 0;

      DIM_TYPE_CD = "TO_CURRENCY";
      DIMENSION_CD = "TO_CURRENCY";
      DIMENSION_NM = symget("SES_DIMEN_TOCRNCY_NM_VAR");
      DIMENSION_DESC = symget("SES_DIMEN_TOCRNCY_DESC_VAR");
      output;

      DIM_TYPE_CD = "RATE_TYPE";
      DIMENSION_CD = "RATE_TYPE";
      DIMENSION_NM = symget("SES_DIMEN_RATETYPE_NM_VAR");
      DIMENSION_DESC = symget("SES_DIMEN_RATETYPE_DESC_VAR");
      output;

      DIM_TYPE_CD = "SOURCE";
      DIMENSION_CD = "SOURCE";
      DIMENSION_NM = symget("SES_DIMEN_SOURCE_NM_VAR");
      DIMENSION_DESC = symget("SES_DIMEN_SOURCE_DESC_VAR");
      output;

      DIM_TYPE_CD = "XRATE_TYPE";
      DIMENSION_CD = "XRATE_TYPE";
      DIMENSION_NM = symget("SES_DIMEN_XRATETYPE_NM_VAR");
      DIMENSION_DESC = symget("SES_DIMEN_XRATETYPE_DESC_VAR");
      output;

      DIM_TYPE_CD = "FREQUENCY";
      DIMENSION_CD = "FREQUENCY";
      DIMENSION_NM = symget("SES_DIMEN_FREQ_NM_VAR");
      DIMENSION_DESC = symget("SES_DIMEN_FREQ_DESC_VAR");
      output;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 12, &cMethodNm);
    %goto RETURNZERO;
  %end;


  /*  Upload to SDM  */
  proc append base=&cSDMTableNm data=&cWorkTableNm &FM_PROCAPPENDFORCE;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 8, &cWorkTableNm, &cSDMTableNm, &cMethodNm);
    %goto RETURNZERO;
  %end;


  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadDimLabel;


/**********************************************************
 * sesloc_uploadDimTypeLabel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadDimTypeLabel.sas                                            */
/*  Purpose: Upload predefined metadata labels for Dimension Types                           */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_uploadDimTypeLabel(nETLInstanceId, cLocale, cSDMLib);

  %let cMethodNm = sesloc_uploadDimTypeLabels;
  %let cWorkTableNm = WORK.SAS_ETL_DIM_TYPE;
  %let cSDMTableNm =  %upcase(%qsysfunc(strip(&cSDMLib))).SAS_ETL_DIM_TYPE;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if not %sysfunc(exist(&cSDMTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSDMTableNm;
    %goto ERRORRETURN;
  %end ;

  %if %sysfunc(exist(&cWorkTableNm)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cWorkTableNm, 2, '.');
    quit;
  %end;


  data &cWorkTableNm;
      length ETL_INSTANCE_ID 8;
      length DIM_TYPE_CD $ %eval(32*&FM_DBCSMULT);
      length DIM_TYPE_NM $ %eval(50*&FM_DBCSMULT);
      length DIM_TYPE_DESC $ %eval(255*&FM_DBCSMULT);
      length LOCALE_CD $ %eval(38*&FM_DBCSMULT);
      length LOCALE_ID 8;
      length VALID_FROM_DTTM 8;
      length VALID_TO_DTTM 8;
      length DIM_TYPE_ROLE 8;

      ETL_INSTANCE_ID = &nETLInstanceId;
      LOCALE_CD = "&cLocale";
      LOCALE_ID = 0;

      DIM_TYPE_CD = "ACCOUNT";
      DIM_TYPE_ROLE = 1;
      DIM_TYPE_NM = symget("SES_DIMTYPE_ACCOUNT_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_ACCOUNT_DESC_VAR");
      output;

      DIM_TYPE_CD = "ANALYSIS";
      DIM_TYPE_ROLE = 3;
      DIM_TYPE_NM = symget("SES_DIMTYPE_ANALYSIS_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_ANALYSIS_DESC_VAR");
      output;

      DIM_TYPE_CD = "COSTCTR";
      DIM_TYPE_ROLE = 4;
      DIM_TYPE_NM = symget("SES_DIMTYPE_COSTCTR_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_COSTCTR_DESC_VAR");
      output;

      DIM_TYPE_CD = "CURRENCY";
      DIM_TYPE_ROLE = 5;
      DIM_TYPE_NM = symget("SES_DIMTYPE_CURRENCY_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_CURRENCY_DESC_VAR");
      output;

      DIM_TYPE_CD = "EXTORG";
      DIM_TYPE_ROLE = 6;
      DIM_TYPE_NM = symget("SES_DIMTYPE_EXTORG_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_EXTORG_DESC_VAR");
      output;

      DIM_TYPE_CD = "INTORG";
      DIM_TYPE_ROLE = 7;
      DIM_TYPE_NM = symget("SES_DIMTYPE_INTORG_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_INTORG_DESC_VAR");
      output;

      DIM_TYPE_CD = "ITEMCAT";
      DIM_TYPE_ROLE = 8;
      DIM_TYPE_NM = symget("SES_DIMTYPE_ITEMCAT_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_ITEMCAT_DESC_VAR");
      output;

      DIM_TYPE_CD = "PROFITCTR";
      DIM_TYPE_ROLE = 10;
      DIM_TYPE_NM = symget("SES_DIMTYPE_PROFITCTR_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_PROFITCTR_DESC_VAR");
      output;

      DIM_TYPE_CD = "TIME";
      DIM_TYPE_ROLE = 11;
      DIM_TYPE_NM = symget("SES_DIMTYPE_TIME_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_TIME_DESC_VAR");
      output;

      DIM_TYPE_CD = "TRADER";
      DIM_TYPE_ROLE = 12;
      DIM_TYPE_NM = symget("SES_DIMTYPE_TRADER_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_TRADER_DESC_VAR");
      output;

      DIM_TYPE_CD = "TO_CURRENCY";
      DIM_TYPE_ROLE = 16;
      DIM_TYPE_NM = symget("SES_DIMTYPE_TOCRNCY_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_TOCRNCY_DESC_VAR");
      output;

      DIM_TYPE_CD = "RATE_TYPE";
      DIM_TYPE_ROLE = 15;
      DIM_TYPE_NM = symget("SES_DIMTYPE_RATETYPE_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_RATETYPE_DESC_VAR");
      output;

      DIM_TYPE_CD = "SOURCE";
      DIM_TYPE_ROLE = 14;
      DIM_TYPE_NM = symget("SES_DIMTYPE_SOURCE_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_SOURCE_DESC_VAR");
      output;

      DIM_TYPE_CD = "XRATE_TYPE";
      DIM_TYPE_ROLE = 17;
      DIM_TYPE_NM = symget("SES_DIMTYPE_XRATETYPE_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_XRATETYPE_DESC_VAR");
      output;

      DIM_TYPE_CD = "FREQUENCY";
      DIM_TYPE_ROLE = 18;
      DIM_TYPE_NM = symget("SES_DIMTYPE_FREQ_NM_VAR");
      DIM_TYPE_DESC = symget("SES_DIMTYPE_FREQ_DESC_VAR");
      output;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 13, &cMethodNm);
    %goto RETURNZERO;
  %end;


  /*  Upload to SDM  */
  proc append base=&cSDMTableNm data=&cWorkTableNm &FM_PROCAPPENDFORCE;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 8, &cWorkTableNm, &cSDMTableNm, &cMethodNm);
    %goto RETURNZERO;
  %end;


  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadDimTypeLabel;


/**********************************************************
 * sesloc_uploadHierLabel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadHierLabel.sas                                               */
/*  Purpose: Upload predefined metadata labels for Hierarchies.                              */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_uploadHierLabel(nETLInstanceId, cLocale, cSDMLib);

  %let cMethodNm = sesloc_uploadHierLabels;
  %let cWorkTableNm = WORK.SAS_ETL_HIERARCHY;
  %let cSDMTableNm = %upcase(%qsysfunc(strip(&cSDMLib))).SAS_ETL_HIERARCHY;
  %let nTrackTime = %qsysfunc(datetime());

  %if not %sysfunc(exist(&cSDMTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSDMTableNm;
    %goto ERRORRETURN;
  %end ;

  %if %sysfunc(exist(&cWorkTableNm)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cWorkTableNm, 2, '.');
    quit;
  %end;

  data &cWorkTableNm;
      length HIERARCHY_CD $ %eval(32*&FM_DBCSMULT);
      length HIERARCHY_NM $ %eval(50*&FM_DBCSMULT);
      length HIERARCHY_DESC $ %eval(255*&FM_DBCSMULT);
      length HIDDEN_FLG 8;
      length LOCALE_CD $ %eval(38*&FM_DBCSMULT);
      length LOCALE_ID 8;
      length VALID_FROM_DTTM 8;
      length VALID_TO_DTTM 8;
      length ETL_INSTANCE_ID 8;
      length DIMENSION_CD $ %eval(32*&FM_DBCSMULT);

      ETL_INSTANCE_ID = &nETLInstanceId;
      LOCALE_CD = "&cLocale";
      LOCALE_ID = 0;
      HIDDEN_FLG = 0;

      HIERARCHY_CD = "RATE_TYPE";
      DIMENSION_CD = "RATE_TYPE";
      HIERARCHY_NM = symget("SES_HIER_RATETYPE_NM_VAR");
      HIERARCHY_DESC = symget("SES_HIER_RATETYPE_DESC_VAR");
      output;

      HIERARCHY_CD = "SOURCE_STD";
      DIMENSION_CD = "SOURCE";
      HIERARCHY_NM = symget("SES_HIER_SOURCE_NM_VAR");
      HIERARCHY_DESC = symget("SES_HIER_SOURCE_DESC_VAR");
      output;

      HIERARCHY_CD = "XRATE_TYPE";
      DIMENSION_CD = "XRATE_TYPE";
      HIERARCHY_NM = symget("SES_HIER_XRATETYPE_NM_VAR");
      HIERARCHY_DESC = symget("SES_HIER_XRATETYPE_DESC_VAR");
      output;

      HIERARCHY_CD = "FREQUENCY";
      DIMENSION_CD = "FREQUENCY";
      HIERARCHY_NM = symget("SES_HIER_FREQ_NM_VAR");
      HIERARCHY_DESC = symget("SES_HIER_FREQ_DESC_VAR");
      output;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %sesrpt_logError( &cLogEntryUtil, L, 11, &cMethodNm);
      %goto RETURNZERO;
  %end;

  /*  Upload to SDM  */
  proc append base=&cSDMTableNm data=&cWorkTableNm &FM_PROCAPPENDFORCE;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %sesrpt_logError( &cLogEntryUtil, L, 8, &cWorkTableNm, &cSDMTableNm, &cMethodNm);
      %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadHierLabel;


/**********************************************************
 * sesloc_uploadLocaleUpdate.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadLocaleUpdate.sas                                                   */
/*  Purpose: Upload SAS Locales to SDM.                                                      */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_uploadLocaleUpdate(cSASLib, cSASTable, nDTTM, cSDMLib, cETLTable, nETLInstanceId, cLocaleStringTableNm);

  %let cMethodNm = sesloc_uploadLocaleUpdates;
  %let cSrcTableNm = %upcase(%qsysfunc(strip(&cSASLib))).%upcase(%qsysfunc(strip(&cSASTable)));
  %let cTrgTableNm = %upcase(%qsysfunc(strip(&cSDMLib))).%upcase(%qsysfunc(strip(&cETLTable)));
  %let cTmpTable1 = WORK._TMP_SAS_LOCALE1;
  %let cTmpTable2 = WORK._TMP_SAS_LOCALE2;
  %let cTmpTable3 = WORK._TMP_SAS_LOCALE3;
  %let cTmpSDMTable1 = WORK._TMP_SDM_LOCALE1;
  %let cSDMDataLocaleTableNm = %upcase(%qsysfunc(strip(&cSDMLib))).SAS_DATA_LOCALE;
  %let rk = 1 ;
  %let nowDTTM = %qsysfunc(datetime()) ;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if not %sysfunc(exist(&cSrcTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSrcTableNm;
    %goto ERRORRETURN;
  %end ;

  %if not %sysfunc(exist(&cTrgTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cTrgTableNm;
    %goto ERRORRETURN;
  %end ;

  %if not %sysfunc(exist(&cSDMDataLocaleTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSDMDataLocaleTableNm;
    %goto ERRORRETURN;
  %end ;


  /* Clean up */
  %if %sysfunc(exist(&cTmpTable1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTmpTable1, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cTmpTable2)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTmpTable2, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cTmpTable3)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTmpTable3, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cTmpSDMTable1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTmpSDMTable1, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cLocaleStringTableNm)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cLocaleStringTableNm, 2, '.');
    quit;
  %end;

  data &cTmpTable1
       (keep=ETL_INSTANCE_ID ISO639_LANGUAGE_CD ISO3166_COUNTRY_CD
            LOCALE_VARIANT_CD DEFAULT_FLG LOCALE);

       set &cTrgTableNm(obs=0 drop=DEFAULT_FLG)
           &cSrcTableNm(where=(&_where1));
       
       LENGTH DEFAULT_FLG $5;
       
       ETL_INSTANCE_ID = &nETLInstanceId;
       ISO639_LANGUAGE_CD = LOCALE_LANGUAGE_CD;
       ISO3166_COUNTRY_CD = LOCALE_COUNTRY_CD;
       LOCALE_VARIANT_CD = ''; /* LOCALE_VARIANT_CD not used for 5.1 */
       DEFAULT_FLG = '0';
       
       LOCALE = strip(LOCALE_LANGUAGE_CD);
       if strip(LOCALE_COUNTRY_CD) ne "" or
          (strip(LOCALE_LANGUAGE_CD) ne "" and
           strip(LOCALE_VARIANT_CD) ne "") then do;
            LOCALE = strip(LOCALE) || "_" || strip(LOCALE_COUNTRY_CD);
       end;
       if strip(LOCALE_VARIANT_CD) ne "" and
          (strip(LOCALE_LANGUAGE_CD) ne "" or
           strip(LOCALE_COUNTRY_CD) ne "") then do;
            LOCALE = strip(LOCALE) || "_" || strip(LOCALE_VARIANT_CD);
       end;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number = 5;
    %let errObj = &cSrcTableNm;
    %goto ERRORRETURN;
  %end;

  /*  Sort SAS Data  */
  proc sort in=&cTmpTable1 out=&cTmpTable2;
       by ISO639_LANGUAGE_CD ISO3166_COUNTRY_CD LOCALE_VARIANT_CD;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number = 6;
    %let errObj = &cTmpTable1;
    %goto ERRORRETURN;
  %end;

  /* Sort SDM Data  */
  proc sort in=&cSDMDataLocaleTableNm out=&cTmpSDMTable1;
       by ISO639_LANGUAGE_CD ISO3166_COUNTRY_CD LOCALE_VARIANT_CD;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number = 6;
    %let errObj = &cSDMDataLocaleTableNm;
    %goto ERRORRETURN;
  %end;

  /* Look for new locales from the SAS */
  data &cTmpTable3(drop=LOCALE_ID LOCALE DEFAULT_FLG rename=(TMP_DEFAULT_FLG=DEFAULT_FLG))
         &cLocaleStringTableNm(keep=LOCALE);
      
      LENGTH TMP_DEFAULT_FLG 8;
      
      merge &cTmpSDMTable1(in=SDM) &cTmpTable2(in=SAS);
      by ISO639_LANGUAGE_CD ISO3166_COUNTRY_CD LOCALE_VARIANT_CD;
      
      TMP_DEFAULT_FLG = input(DEFAULT_FLG, best5.);
      if SAS and not SDM then do ;
        output &cTmpTable3;
        output &cLocaleStringTableNm;
      end;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number = 7;
    %let errObj = &cSDMDataLocaleTableNm;
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &cMethodNm);
    %goto RETURNZERO;
  %end;

  /*  Upload to SDM  */
  proc append base=&cTrgTableNm data=&cTmpTable3 &FM_PROCAPPENDFORCE;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 8, &cTmpTable3, &cTrgTableNm, &cMethodNm);
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadLocaleUpdate;



/**********************************************************
 * sesloc_uploadMemberLabel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadMemberLabel.sas                                                    */
/*  Purpose: Upload predefined metadata labels for Members.                                  */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_uploadMemberLabel(nETLInstanceId, cLocale, cSDMLib);

  %let cMethodNm = sesloc_uploadMemberLabels;
  %let cWorkTableNm = WORK.SAS_ETL_MEMBER;
  %let cSDMTableNm = %upcase(%qsysfunc(strip(&cSDMLib))).SAS_ETL_MEMBER;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if not %sysfunc(exist(&cSDMTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSDMTableNm;
    %goto ERRORRETURN;
  %end ;

  %if %sysfunc(exist(&cWorkTableNm)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cWorkTableNm, 2, '.');
    quit;
  %end;

  data &cWorkTableNm;
       length MEMBER_CD $ %eval(32*&FM_DBCSMULT);
       length MEMBER_NM $ %eval(50*&FM_DBCSMULT);
       length MEMBER_DESC $ %eval(255*&FM_DBCSMULT);
       length LOCALE_CD $ %eval(38*&FM_DBCSMULT);
       length LOCALE_ID 8;
       length VALID_FROM_DTTM 8;
       length VALID_TO_DTTM 8;
       length ETL_INSTANCE_ID 8;
       length MEMBER_TYPE_ENUM 8;
       length HIDDEN_FLG 8;
       length DIMENSION_CD $ %eval(32*&FM_DBCSMULT);

       ETL_INSTANCE_ID = &nETLInstanceId;
       LOCALE_CD = "&cLocale";
       LOCALE_ID = 0;
       VALID_FROM_DTTM = 0;
       VALID_TO_DTTM = 0;

       MEMBER_CD = "Total";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_TOTAL_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_TOTAL_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "CTA";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_CTA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_CTA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "RE";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_RE_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_RE_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "REElim";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_REELIM_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_REELIM_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "REAdj";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_READJ_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_READJ_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "REData";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_REDATA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_REDATA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "REForm";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_REFORM_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_REFORM_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "REImport";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_REIMPORT_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_REIMPORT_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Elim";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_ELIM_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_ELIM_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "TotalBeforeElim";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_TOTALB4ELM_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_TOTALB4ELM_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Adj";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_ADJ_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_ADJ_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "CPO";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_CPO_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_CPO_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Alloc";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_ALLOC_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_ALLOC_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Reclass";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_RECLASS_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_RECLASS_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Bal";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_BAL_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_BAL_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Manual";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_MANUAL_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_MANUAL_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "TotalBeforeAdj";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_TOTALB4ADJ_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_TOTALB4ADJ_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "BaseForm";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_BASEFORM_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_BASEFORM_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "TotalAfterImport";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_TOTALAFTIM_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_TOTALAFTIM_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "BaseJourn";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_BASEJOURN_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_BASEJOURN_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Base";
       DIMENSION_CD = "SOURCE";
       MEMBER_NM = symget("SES_MBR_SRC_BASE_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_SRC_BASE_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "None";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_NONE_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_NONE_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Historic";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_HIST_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_HIST_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Derived";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_DERIVED_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_DERIVED_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "PeriodAverage";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_PRDAVG_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_PRDAVG_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "PeriodClose";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_PRDCLS_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_PRDCLS_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "PeriodOpen";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_PRDOPN_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_PRDOPN_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Custom1";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_CUSTOM1_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_CUSTOM1_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "Custom2";
       DIMENSION_CD = "XRATE_TYPE";
       MEMBER_NM = symget("SES_MBR_XRATE_CUSTOM2_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_XRATE_CUSTOM2_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "LA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_LA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_LA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "YA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_YA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_YA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "HA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_HA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_HA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "QA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_QA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_QA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "MA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_MA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_MA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "WA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_WA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_WA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "DA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_DA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_DA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "PA";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_PA_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_PA_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "LTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_LTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_LTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "YTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_YTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_YTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "HTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_HTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_HTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "QTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_QTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_QTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "MTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_MTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_MTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "WTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_WTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_WTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "DTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_DTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_DTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;

       MEMBER_CD = "PTD";
       DIMENSION_CD = "FREQUENCY";
       MEMBER_NM = symget("SES_MBR_FREQ_PTD_NM_VAR");
       MEMBER_DESC = symget("SES_MBR_FREQ_PTD_DESC_VAR");
       MEMBER_TYPE_ENUM = 1;
       HIDDEN_FLG = 0;
       output;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 10, &cMethodNm);
    %goto RETURNZERO;
  %end;

  /*  Upload to SDM  */
  proc append base=&cSDMTableNm data=&cWorkTableNm &FM_PROCAPPENDFORCE;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesrpt_logError( &cLogEntryUtil, L, 8, &cWorkTableNm, &cSDMTableNm, &cMethodNm);
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadMemberLabel;


/**********************************************************
 * sesloc_uploadPredefLabel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadPredefLabel.sas                                                    */
/*  Purpose: upload predefined metadata labels to SAS_ETL_ tables.                           */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesloc_uploadPredefLabel(cLocaleCodeTable, nETLInstanceId, cSDMLib);

  %let cMethodNm = sesloc_uploadPredefLables;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %let cOS = %upcase(%qsysfunc(strip(&SYSSCP)));

  /*  Check if any errors where found  */
  %if %qsysfunc(exist(&cLocaleCodeTable)) %then %do;
    proc sql noprint;
      select count(*)
        into :numrow
        from &cLocaleCodeTable;
        quit;
    %let numrow=%eval(&numrow);
    %if &numrow > 0 %then %do;
      proc sql noprint;
        select  LOCALE
            into :cLocale1 - :cLocale&numrow
            from &cLocaleCodeTable;
      quit;


      %do ii=1 %to &numrow;

          /*
           * If locale is found, will populate macro variables with locale
           * specific information for dimtype, dim, mbr, hiers, properties
           */
          %if %upcase(&cOS) eq WIN %then %do;
              %localze(ses_property_file_location=!sasroot/finance/sasmisc, ses_locale=&&cLocale&ii);
          %end;
          %else %do;
              %localze(ses_property_file_location=!sasroot/misc/finance, ses_locale=&&cLocale&ii);
          %end;

          %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;

          %let error_number=14;
        %let errObj=&&cLocale&i;
               %goto ERRORRETURN;
          %end;

          %let nLocaleFound = &ses_locale_found;

          %if &nLocaleFound = 1 %then %do;
          /*  Properties  */
              %sesloc_uploadPropLabel(&nETLInstanceId, &&cLocale&ii, &cSDMLib);
              %if &_SOLErrorCode = 0 %then %do;
                  %goto RETURNZERO;
              %end;

              /*  Dimen Types  */
              %sesloc_uploadDimTypeLabel(&nETLInstanceId, &&cLocale&ii, &cSDMLib);
              %if &_SOLErrorCode = 0 %then %do;
                  %goto RETURNZERO;
              %end;

              /*  Dimensions  */
              %sesloc_uploadDimLabel(&nETLInstanceId, &&cLocale&ii, &cSDMLib);
              %if &_SOLErrorCode = 0 %then %do;
                  %goto RETURNZERO;
              %end;

              /*  Members  */
              %sesloc_uploadMemberLabel(&nETLInstanceId, &&cLocale&ii, &cSDMLib);
              %if &_SOLErrorCode = 0 %then %do;
                  %goto RETURNZERO;
              %end;

              /*  Hierarchies  */
              %sesloc_uploadHierLabel(&nETLInstanceId, &&cLocale&ii, &cSDMLib);
              %if &_SOLErrorCode = 0 %then %do;
                  %goto RETURNZERO;
              %end;
          %end ;
      %end;
    %end;
  %end;


  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadPredefLabel;


/**********************************************************
 * sesloc_uploadPropLabel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesloc_uploadPropLabel.sas                                                      */
/*  Purpose: Upload predefined metadata labels for Properties.                               */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesloc_uploadPropLabel(nETLInstanceId, cLocale, cSDMLib);

  %let cMethodNm = sesloc_uploadPropLabels;
  %let cWorkTableNm = WORK.SAS_ETL_PROPERTY;
  %let cSDMTableNm = %upcase(%qsysfunc(strip(&cSDMLib))).SAS_ETL_PROPERTY;
  %let nTrackTime = %qsysfunc(datetime());

  %if not %sysfunc(exist(&cSDMTableNm, DATA)) %then %do;
    %let error_number=4;
    %let errObj=&cSDMTableNm;
    %goto ERRORRETURN;
  %end;

  %if %sysfunc(exist(&cWorkTableNm)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cWorkTableNm, 2, '.');
    quit;
  %end;

  data &cWorkTableNm;
      length PROPERTY_CD $ %eval(32*&FM_DBCSMULT);
      length PROPERTY_NM $ %eval(50*&FM_DBCSMULT);
      length PROPERTY_DESC $ %eval(255*&FM_DBCSMULT);
      length PROPERTY_TYPE_CD $ %eval(32*&FM_DBCSMULT);
      length ETL_INSTANCE_ID 8;
      length LOCALE_CD $ %eval(38*&FM_DBCSMULT);
      length LOCALE_ID 8;
      length ENUM_VALUES $ 32767;
      length MIN_VALUE 8;
      length MAX_VALUE 8;
      length PROPERTY_OPTIONS 8;
      PROPERTY_OPTIONS=1;

      ETL_INSTANCE_ID = &nETLInstanceId;
      LOCALE_CD = "&cLocale";
      LOCALE_ID = 0;
      PROPERTY_CD = "AccountType";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_ACCTTYPE_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_ACCTTYPE_DESC_VAR");
      output;

      PROPERTY_CD = "AccountBehavior";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_ACCTBEHAVIOR_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_ACCTBEHAVIOR_DESC_VAR");
      output;

      PROPERTY_CD = "EndDate";
      PROPERTY_TYPE_CD = "6";
      PROPERTY_NM = symget("SES_PROP_ENDDATE_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_ENDDATE_DESC_VAR");
      output;

      PROPERTY_CD = "BasisData";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_BASISDATA_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_BASISDATA_DESC_VAR");
      output;

      PROPERTY_CD = "RollForwardMethod";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_ROLLFWDMTH_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_ROLLFWDMTH_DESC_VAR");
      output;

      PROPERTY_CD = "TotalAfterImport";
      PROPERTY_TYPE_CD = "3";
      PROPERTY_NM = symget("SES_PROP_TOTALAFTERIMP_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_TOTALAFTERIMP_DESC_VAR");
      output;

      PROPERTY_CD = "Intercompany";
      PROPERTY_TYPE_CD = "2";
      PROPERTY_NM = symget("SES_PROP_INTERCOMPANY_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_INTERCOMPANY_DESC_VAR");
      output;

      PROPERTY_CD = "ReportingEntity";
      PROPERTY_TYPE_CD = "2";
      PROPERTY_NM = symget("SES_PROP_RPTENTITY_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_RPTENTITY_DESC_VAR");
      output;

      PROPERTY_CD = "Level";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_LEVEL_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_LEVEL_DESC_VAR");
      output;

      PROPERTY_CD = "SourceAccounts";
      PROPERTY_TYPE_CD = "7";
      PROPERTY_NM = symget("SES_PROP_SRCACCTS_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_SRCACCTS_DESC_VAR");
      output;

      PROPERTY_CD = "BalanceType";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_BALTYPE_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_BALTYPE_DESC_VAR");
      output;

      PROPERTY_CD = "FunctionalCurrency";
      PROPERTY_TYPE_CD = "3";
      PROPERTY_NM = symget("SES_PROP_FUNCCRNCY_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_FUNCCRNCY_DESC_VAR");
      output;

      PROPERTY_CD = "StartDate";
      PROPERTY_TYPE_CD = "6";
      PROPERTY_NM = symget("SES_PROP_STARTDATE_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_STARTDATE_DESC_VAR");
      output;

      PROPERTY_CD = "ExchangeRateType";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_EXCHGRATETYPE_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_EXCHGRATETYPE_DESC_VAR");
      output;

      PROPERTY_CD = "FormulaPrecedence";
      PROPERTY_TYPE_CD = "7";
      PROPERTY_NM = symget("SES_PROP_FORMPRCDNC_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_FORMPRCDNC_DESC_VAR");
      output;

      PROPERTY_CD = "CTAElimBehavior";
      PROPERTY_TYPE_CD = "4";
      PROPERTY_NM = symget("SES_PROP_CTAELIMBEHVR_NM_VAR");
      PROPERTY_DESC = symget("SES_PROP_CTAELIMBEHVR_DESC_VAR");
      output;

  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %sesrpt_logError( &cLogEntryUtil, L, 9, &cMethodNm);
      %goto RETURNZERO;
  %end;

  /*  Upload to SDM  */
  proc append base=&cSDMTableNm(dbmax_text=32767) data=&cWorkTableNm &FM_PROCAPPENDFORCE;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %sesrpt_logError( &cLogEntryUtil, L, 8, &cWorkTableNm, &cSDMTableNm, &cMethodNm);
      %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesrpt_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _SOLErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesrpt_logError( &cLogEntryUtil, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _SOLErrorCode=0;

  %THEEND:

%mend sesloc_uploadPropLabel;

%MEND sesloc;
