/*-- Copyright (c) 2007 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. --
 * startsln.sas
 *
 * Macro used to provide SAS Financial Management with global
 * variable values for metadata folders, libraries etc.
 *
 */

%macro startsln(_product, _reassign=Y);

   %global FM_DBCSMULT FM_PROCAPPENDFORCE;
   %let FM_DBCSMULT=1;
   %let FM_PROCAPPENDFORCE=;
   %if %sysfunc(getoption(DBCS)) = %str(DBCS) %then %do;
     %let FM_DBCSMULT=3;
     %let FM_PROCAPPENDFORCE=FORCE;
   %end;  

   %if %str("&_product") eq %str("") %then %do;
     %let _product=FMS;
   %end;

   %macro _putLogMsg(_msg);
     %put ****************************************************;
     %put &_msg;
     %put ****************************************************;
   %mend;
   %_putLogMsg(NOTE: Running sas setup for product "&_product".);

   /* set product specific macros */
   %macro _setMacros(_prodName=, _prodLegalName=);
     %let _frefmac=_cfgm&_product;
     %let _freffmt=_cfgf&_product;
     %let _productFileref=_cfg&_product;
     %let _productName=&_prodName;
     %let _productLegalName=&_prodLegalName;
   %mend;

   /* Macro to check for existence of macro and value */
   %macro _chkMacro(_macro);
     %if %symexist(&_macro) %then %do;
       %if "&&&_macro" eq "" %then %do;
         %_putLogMsg(ERROR: Macro "&_macro" need value.);
         %return;
       %end;
     %end;
     %else %do;
       %_putLogMsg(ERROR: Macro "&_macro" need to be defined for product "&_product".);
       %return;
     %end;
   %mend;

   %if %str("&_product") eq %str("") %then %do;
     %_putLogMsg(ERROR: No property file found. Product not defined.);
     %return;
   %end;

   %if %symexist(_cfgStartsln_&_product) %then %do;
     %if &&_cfgStartsln_&_product eq 1 %then %do;
       %_putLogMsg(NOTE: Session already setup for product "&_product".);
       %return;
     %end;
   %end;
   %else %do;
     %global _cfgStartsln_&_product;
     %let _cfgStartsln_&_product=0;
   %end;

   %if %sysfunc(fileref(_cfg&_product)) gt 0 %then %do;
     %_putLogMsg(ERROR: No property file found. Fileref "_cfg&_product" has not been assigned.);
     %return;
   %end;
   %else %if %sysfunc(fileref(_cfg&_product)) lt 0 %then %do;
     %let _file=%sysfunc(pathname(_cfg&_product));
     %_putLogMsg(ERROR: File "&_file" assigned to fileref "_cfg&_product" does not exist.);
     %return;
   %end;

   %if &SYSSCP = WIN %then %do;
     filename _cfgmisc "!SASROOT/finance/sasmisc";
   %end;
   %else %do;
     filename _cfgmisc "!SASROOT/misc/finance";
   %end;

   %let _frefmac=~~NONE~~;
   %let _freffmt=~~NONE~~;
   %let _productFileref=~~NONE~~;
   %let _productName=~~NONE~~;
   %let _productLegalName=~~NONE~~;

   /*** DO NOT REMOVE "SOL" CODE FOR FUTURE MIGRATIONS FROM 5.1 AND 5.2 ***/
   %if %str(%kupcase("&_product")) eq %str("SOL") %then %do;
     %_setMacros(_prodName=SolutionsServices
               , _prodLegalName=SAS Solutions Services);
   %end;
   %else %if %str(%kupcase("&_product")) eq %str("FMS") %then %do;
     %_setMacros(_prodName=FinancialManagement
               , _prodLegalName=SAS Financial Management);
   %end;

   %if %str("&_product") eq %str("") or "&_productFileref" eq "~~NONE~~" %then %do;
     %_putLogMsg(WARNING: No property file found. Product "&_product" not defined.);
     %return;
   %end;
   %else %do;

     data _null_;
       length text command $1024;
       infile &_productFileref MISSOVER EXPANDTABS LENGTH=linelen lrecl=1024;
       input text varying1024. linelen;
       _text=strip(text);
       if ksubstr(_text, 1, 1) ne '#';

       _macro=kscan(_text, 1, '=');
       if linelen gt 0 then do;
         if nvalid(_macro) and kindexc(_text, '=') then do;
           command = '%global ' !! _macro !! ';';
           call execute(command);
           command='%let ' !! _text !! ';';
           call execute(command);
         end;
         else do;
           if kindexc(ksubstr(_macro, 1, 1), '0123456789') then
             _wrongc=1;
           else
             _wrongc=notname(_macro, 2);

           if _wrongc gt length(_macro) then
             _putline='WARNING: No assigned value for SAS macro (' !! strip(_macro) !! ').';
           else
             _putline='WARNING: Not a valid SAS macro name (' !! strip(_macro) !! '). Wrong character in position: '
                     !! strip(_wrongc) !! ' (' !! quote(ksubstr(_macro, _wrongc, 1)) !! ')';

           put _putline;
         end;
       end;
     run;

     %if "&_reassign" eq "Y" %then %do;
       %if %symexist(&_frefmac) %then %do;
         filename &_frefmac "&&&_frefmac";
       %end;
       %else %do;
         filename &_frefmac "&SASAppDir/SASEnvironment/&_productName./SASMacro";
       %end;

       %if %symexist(&_freffmt) %then %do;
         libname &_freffmt "&&&_freffmt";
       %end;
       %else %do;
         libname &_freffmt "&SASAppDir/SASEnvironment/&_productName./SASFormats";
       %end;
     %end;

     /* check for mandatory macros in property files */
     %_chkMacro(SASAppDir);
     %_chkMacro(&_product.datatier_topfolder);

     %global &_product.datatier_folder &_product.system_folder;
     %let &_product.datatier_folder=&&&_product.datatier_topfolder./&_productLegalName;
     %let &_product.system_folder=/System/Applications/&_productLegalName./Common;

     /*** DO NOT REMOVE "SOL" CODE FOR FUTURE MIGRATIONS FROM 5.1 AND 5.2 ***/
     %if %str(%upcase("&_product")) eq %str("SOL") %then %do;
       %global CrossIndustryDDStier_folder;
       %global StgLibObj CrossIndustryDDSLibObj;
       %let CrossIndustryDDStier_folder=&&&_product.datatier_topfolder./Cross Industry Detail Data Store;
       %let StgLibObj=StageDDS;
       %let CrossIndustryDDSLibObj=CrossIndustryDDS;
     %end;
     %else %if %str(%upcase("&_product")) eq %str("FMS") %then %do;
       %global SOLsystem_folder StgLibObj SASSUPLibObj  SDMLibObj CONFLibObj FMdatatier_folder;
       %let StgLibObj=StageFM;
       %let SASSUPLibObj=SAS Supplied FM;
       %let SDMLibObj=SASSDM;
       %let CONFLibObj=Conform;

       %let FMdatatier_folder=&&&_product.datatier_folder;
       %let SOLsystem_folder=&&&_product.system_folder;
     %end;

   %end;

   %_OPTIONS:
   option validvarname=ANY;

   %let _sasauto=;
   %let _fmtsrch=;

   proc sql noprint;
     select setting into :_sasauto
     from dictionary.options
     where optname in('SASAUTOS')
     ;
     select setting into :_fmtsrch
     from dictionary.options
     where optname in('FMTSEARCH')
     ;
   quit;

   %_MACROS:
   %if "&_frefmac" ne "~~NONE~~" %then %do;
     %if %sysfunc(fileref(&_frefmac)) gt 0 %then %do;
       %_putLogMsg(WARNING: Fileref "&_frefmac" has not been assigned.);
       %goto _FORMATS;
     %end;
     %else %if %sysfunc(fileref(&_frefmac)) lt 0 %then %do;
       %let _file=%sysfunc(pathname(&_frefmac));
       %_putLogMsg(WARNING: Directory "&_file" assigned to fileref "&_frefmac" does not exist.);
       %goto _FORMATS;
     %end;
     %else %do;
       data _null_;
         call symput('_sasauto', "&_frefmac "  !! kcompress(strip(symget('_sasauto')), '()') );
       run;
       options sasautos=(&_sasauto);
     %end;
   %end;

   %_FORMATS:
   %if "&_freffmt" ne "~~NONE~~" %then %do;
     %if %sysfunc(libref(&_freffmt)) ne 0 %then %do;
       %_putLogMsg(WARNING: Libref "&_freffmt" has not been assigned.);
       %goto THEEND;
     %end;
     %else %do;
       data _null_;
         call symput('_fmtsrch', "&_freffmt " !! kcompress(strip(symget('_fmtsrch')), '()') );
       run;
       options fmtsearch=(&_fmtsrch);
     %end;
   %end;

   %THEEND:;
   %let _cfgStartsln_&_product=1;
   %_putLogMsg(NOTE: Sas setup completed for product "&_product".);

 %mend;
