 /*--------------------------------------------------------------
 *  Copyright (c) 2006 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *  NAME:      regddlo.sas
 *
 *  PURPOSE:   Creates physical relational tables and/or data sets for the SDM (Solutions Data Mart)
 *             for ODCS.
 *
 *  NOTES:
 *
 *-------------------------------------------------------------*/
%macro regddlo;

%local _recordFound ;

/* Create relational tables */

%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.,
          sesetl_useType=CON
          ) ;

%sesssver(solName=ODCS, solVer=&sesetl_version, solStatus=Failed, solMethod=Installation);

%if &SYSSCP = WIN %then %do;
    %let propspth=%SYSGET(SASROOT)\finance\sasmisc;
%end;
%else %do;
    %let propspth=%SYSGET(SASROOT)/misc/finance;
%end;

%localze(ses_property_file_location=&propspth, ses_locale=en);

%if (&sas_install_default_locale ne en) %then
   %localze(ses_property_file_location=&propspth, ses_locale=&sas_install_default_locale);

data _null_;
   length LOCALE_CD $ 40 ;
   length LOCALE_LANGUAGE_CD $ 2 ;
   length LOCALE_COUNTRY_CD $ 2 ;
   length LOCALE_VARIANT_CD $ 32 ;
   LOCALE_CD = "&sas_install_default_locale" ;
   LOCALE_LANGUAGE_CD = strip( kscan(LOCALE_CD, 1, '_') ) ;
   NLS_COUNT1 = count(LOCALE_CD, '_') ;
   if NLS_COUNT1 eq 0 then do ;
      LOCALE_COUNTRY_CD = '' ;
      LOCALE_VARIANT_CD = '' ;
   end ;
   else do ;
      NLS_COUNT2 = count(LOCALE_CD, '__') ;
      if NLS_COUNT2 eq 1 then do ;
         LOCALE_COUNTRY_CD = '' ;
         LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 2, '_') ) ;
      end ;
      else do ;
         LOCALE_COUNTRY_CD = strip( kscan(LOCALE_CD, 2, '_') ) ;
         LOCALE_VARIANT_CD = strip( kscan(LOCALE_CD, 3, '_') ) ;
      end ;
   end ;
   call symput('ses_language', %unquote(strip(LOCALE_LANGUAGE_CD)));
   call symput('ses_country', %unquote(strip(LOCALE_COUNTRY_CD)));
   call symput('ses_variant', %unquote(strip(LOCALE_VARIANT_CD)));
run ;

proc sql ;
  connect to &sesetl_EngineType
          as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions);

  execute (
           CREATE TABLE if not exists sas_config (
             config_key character varying(255) NOT NULL DEFAULT ''::character varying,
             config_value text NOT NULL,
             CONSTRAINT sas_config_pkey PRIMARY KEY (config_key )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_data_locale (
             locale_id integer NOT NULL DEFAULT 0,
             iso639_language_cd character varying(2) NOT NULL DEFAULT ''::character varying,
             iso3166_country_cd character varying(2) NOT NULL DEFAULT ''::character varying,
             locale_variant_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             default_flg boolean NOT NULL DEFAULT true,
             CONSTRAINT sas_data_locale_pkey PRIMARY KEY (locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_data_locale__locale_ux ON sas_data_locale (iso639_language_cd COLLATE pg_catalog."default" , iso3166_country_cd COLLATE pg_catalog."default" , locale_variant_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_dimension (
             dimension_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             internal_dimension_id integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             dimension_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             dflt_hierarchy_id integer,
             editable_flg boolean NOT NULL DEFAULT true,
             editable_past_flg boolean NOT NULL DEFAULT false,
             hidden_flg boolean NOT NULL DEFAULT false,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_dimension_pkey PRIMARY KEY (dimension_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension__deflt_hierarchy_rk ON sas_dimension (dflt_hierarchy_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension__dim_internal_rk ON sas_dimension (internal_dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension__dim_internal_rk_2 ON sas_dimension (internal_dimension_id , dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension__dim_type_rk ON sas_dimension (dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension__valid_from_dttm ON sas_dimension (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension__valid_to_dttm ON sas_dimension (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_dimension_nls (
             dimension_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             dimension_nm character varying(50) DEFAULT NULL::character varying,
             dimension_desc character varying(255) DEFAULT NULL::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_dimension_nls_pkey PRIMARY KEY (dimension_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_nls__dimension_rk ON sas_dimension_nls (dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_nls__locale_rk ON sas_dimension_nls (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_nls__valid_from_dttm ON sas_dimension_nls (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_nls__valid_to_dttm ON sas_dimension_nls (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_dimension_property (
             dimension_id integer NOT NULL DEFAULT 0,
             property_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_value character varying(255) NOT NULL DEFAULT ''::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_dimension_property_pkey PRIMARY KEY (property_id , dimension_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_property__dimension_rk ON sas_dimension_property (dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_property__locale_rk ON sas_dimension_property (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_property__property_rk ON sas_dimension_property (property_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_property__valid_from_dttm ON sas_dimension_property (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dimension_property__valid_to_dttm ON sas_dimension_property (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_dim_type (
             dim_type_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             dim_type_role integer NOT NULL DEFAULT 0,
             internal_dim_type_id integer NOT NULL DEFAULT 0,
             dim_type_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             hidden_flg boolean NOT NULL DEFAULT false,
             dflt_dimension_id integer,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_dim_type_pkey PRIMARY KEY (dim_type_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type__valid_from_dttm ON sas_dim_type (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type__valid_to_dttm ON sas_dim_type (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_dim_type_nls (
             dim_type_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             dim_type_nm character varying(50) DEFAULT NULL::character varying,
             dim_type_desc character varying(255) DEFAULT NULL::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_dim_type_nls_pkey PRIMARY KEY (dim_type_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_nls__dim_type_rk ON sas_dim_type_nls (dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_nls__locale_rk ON sas_dim_type_nls (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_nls__valid_from_dttm ON sas_dim_type_nls (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_nls__valid_to_dttm ON sas_dim_type_nls (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_dim_type_property (
             dim_type_id integer NOT NULL DEFAULT 0,
             property_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_value character varying(255) NOT NULL DEFAULT ''::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_dim_type_property_pkey PRIMARY KEY (dim_type_id , property_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_property__dim_type_rk ON sas_dim_type_property (dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_property__locale_rk ON sas_dim_type_property (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_property__property_rk ON sas_dim_type_property (property_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_property__valid_from_dttm ON sas_dim_type_property (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_dim_type_property__valid_to_dttm ON sas_dim_type_property (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_etl_fact_dim_mbr (
             etl_instance_id integer NOT NULL DEFAULT 0,
             dimension_id integer NOT NULL DEFAULT 0,
             dimension_cd character varying(32) DEFAULT NULL::character varying,
             member_id integer NOT NULL DEFAULT 0,
             member_cd character varying(32) DEFAULT NULL::character varying
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_etl_fact_dim_mbr__etl_instance_id ON sas_etl_fact_dim_mbr (etl_instance_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_fact_set (
             fact_set_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             fact_set_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             fact_set_type_enum smallint NOT NULL DEFAULT 0::smallint,
             fact_cnt integer NOT NULL DEFAULT 0,
             unique_fact_cnt integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_fact_set_pkey PRIMARY KEY (fact_set_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_fact_set__code ON sas_fact_set (fact_set_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_fact_set_dim_type (
             fact_set_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_fact_set_dim_type_pkey PRIMARY KEY (fact_set_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_fact_set_dim_type__fact_set_dim_type ON sas_fact_set_dim_type (fact_set_id , dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_fact_set_dim_type__dim_type_rk ON sas_fact_set_dim_type (dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_fact_set_nls (
             fact_set_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             fact_set_nm character varying(50) DEFAULT NULL::character varying,
             fact_set_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_fact_set_nls_pkey PRIMARY KEY (fact_set_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_fact_set_nls__fact_set_rk ON sas_fact_set_nls (fact_set_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_fact_set_nls__locale_rk ON sas_fact_set_nls (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_formula (
             formula_id integer NOT NULL DEFAULT 0,
             namespace character(1) NOT NULL DEFAULT ''::bpchar,
             formula_type_enum character(1) NOT NULL DEFAULT ''::bpchar,
             target character varying(255) DEFAULT NULL::character varying,
             result_type character(1) NOT NULL DEFAULT ''::bpchar,
             formula_text text NOT NULL,
             byte_code bytea NOT NULL,
             formula_mdx_text text,
             CONSTRAINT sas_formula_pkey PRIMARY KEY (formula_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula__namespace ON sas_formula (namespace COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula__target ON sas_formula (target COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_formula_operand (
             formula_id integer NOT NULL DEFAULT 0,
             namespace character(1) NOT NULL DEFAULT ''::bpchar,
             ordinal smallint NOT NULL DEFAULT 0::smallint,
             operand_group character varying(48) DEFAULT NULL::character varying,
             operand_element character varying(48) DEFAULT NULL::character varying,
             table_nm character varying(32) DEFAULT NULL::character varying
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_operand__dimension_cd ON sas_formula_operand (operand_group COLLATE pg_catalog."default" , operand_element COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_operand__formula_rk ON sas_formula_operand (formula_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_operand__namespace ON sas_formula_operand (namespace COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_operand__table_name ON sas_formula_operand (table_nm COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_formula_target (
             formula_target_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             formula_id integer NOT NULL DEFAULT 0,
             target_internal_dimension_id integer NOT NULL DEFAULT 0,
             target_member_id integer NOT NULL DEFAULT 0,
             formula_type integer NOT NULL DEFAULT 0,
             formula_phase integer NOT NULL DEFAULT 0,
             precedence_class integer NOT NULL DEFAULT 0,
             formula_lhs_scope text,
             formula_rhs_scope text,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             ordinal integer NOT NULL DEFAULT 0,
             target_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_formula_target_pkey PRIMARY KEY (formula_target_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_target__dim_dttms ON sas_formula_target (target_internal_dimension_id , valid_from_dttm , valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_target__formula_index ON sas_formula_target (formula_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_target__valid_from_dttm ON sas_formula_target (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_formula_target__valid_to_dttm ON sas_formula_target (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_hierarchy (
             hierarchy_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             hierarchy_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             internal_dimension_id integer NOT NULL DEFAULT 0,
             hidden_flg boolean NOT NULL DEFAULT false,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_hierarchy_pkey PRIMARY KEY (hierarchy_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy__dim_internal_rk ON sas_hierarchy (internal_dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy__valid_from_dttm ON sas_hierarchy (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy__valid_to_dttm ON sas_hierarchy (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_hierarchy_dflt_member (
             hierarchy_id integer NOT NULL DEFAULT 0,
             dimension_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             dflt_member_id integer NOT NULL DEFAULT 0,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_hierarchy_dflt_member_pkey PRIMARY KEY (hierarchy_id , dimension_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_dflt_member__deflt_member_rk ON sas_hierarchy_dflt_member (dflt_member_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_dflt_member__dimension_rk ON sas_hierarchy_dflt_member (dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_dflt_member__hierarchy_rk ON sas_hierarchy_dflt_member (hierarchy_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_dflt_member__valid_from_dttm ON sas_hierarchy_dflt_member (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_dflt_member__valid_to_dttm ON sas_hierarchy_dflt_member (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_hierarchy_member (
             hierarchy_id integer NOT NULL DEFAULT 0,
             member_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             parent_member_id integer,
             member_rank integer NOT NULL DEFAULT 0,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_hierarchy_member_pkey PRIMARY KEY (hierarchy_id , member_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_member__child_member_rk ON sas_hierarchy_member (member_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_member__hierarchy_rk ON sas_hierarchy_member (hierarchy_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_member__hierarchy_rk_2 ON sas_hierarchy_member (hierarchy_id , valid_from_dttm , valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_member__parent_member_rk ON sas_hierarchy_member (parent_member_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_member__valid_from_dttm ON sas_hierarchy_member (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_member__valid_to_dttm ON sas_hierarchy_member (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_hierarchy_nls (
             hierarchy_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             hierarchy_nm character varying(50) DEFAULT NULL::character varying,
             hierarchy_desc character varying(255) DEFAULT NULL::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_hierarchy_nls_pkey PRIMARY KEY (hierarchy_id , locale_id , revision )
)
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_nls__hierarchy_rk ON sas_hierarchy_nls (hierarchy_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_nls__locale_rk ON sas_hierarchy_nls (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_nls__valid_from_dttm ON sas_hierarchy_nls (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_nls__valid_to_dttm ON sas_hierarchy_nls (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_hierarchy_property (
             hierarchy_id integer NOT NULL DEFAULT 0,
             property_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_value character varying(255) NOT NULL DEFAULT ''::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_hierarchy_property_pkey PRIMARY KEY (property_id , hierarchy_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_property__hierarchy_rk ON sas_hierarchy_property (hierarchy_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_property__locale_rk ON sas_hierarchy_property (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_property__property_rk ON sas_hierarchy_property (property_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_property__valid_from_dttm ON sas_hierarchy_property (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_hierarchy_property__valid_to_dttm ON sas_hierarchy_property (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_member (
             internal_dimension_id integer NOT NULL DEFAULT 0,
             member_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             internal_dim_type_id integer NOT NULL DEFAULT 0,
             member_type_enum smallint NOT NULL DEFAULT 0::smallint,
             member_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             hidden_flg boolean NOT NULL DEFAULT false,
             custom_flg boolean NOT NULL DEFAULT true,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_member_pkey PRIMARY KEY (member_id , internal_dimension_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member__dim_internal_rk ON sas_member (internal_dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member__dim_internal_rk_2 ON sas_member (internal_dimension_id , valid_from_dttm , valid_to_dttm );
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member__valid_from_dttm ON sas_member (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member__valid_to_dttm ON sas_member (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_member_nls (
             internal_dimension_id integer NOT NULL DEFAULT 0,
             member_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             member_nm character varying(50) DEFAULT NULL::character varying,
             member_desc character varying(255) DEFAULT NULL::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_member_nls_pkey PRIMARY KEY (member_id , internal_dimension_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_nls__dim_internal_rk ON sas_member_nls (internal_dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_nls__dim_internal_rk_2 ON sas_member_nls (internal_dimension_id , valid_from_dttm , valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_nls__locale_rk ON sas_member_nls (locale_id )
     ) by PASSTHRU ;

  execute (
            CREATE INDEX sas_member_nls__member_rk ON sas_member_nls (member_id , internal_dimension_id )
     ) by PASSTHRU ;

  execute (
            CREATE INDEX sas_member_nls__valid_from_dttm ON sas_member_nls (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_nls__valid_to_dttm ON sas_member_nls (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_member_property (
             internal_dimension_id integer NOT NULL DEFAULT 0,
             member_id integer NOT NULL DEFAULT 0,
             property_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_value character varying(255) NOT NULL DEFAULT ''::character varying,
             valid_from_dttm timestamp without time zone NOT NULL DEFAULT '1970-01-01 00:00:00'::timestamp without time zone,
             valid_to_dttm timestamp without time zone NOT NULL DEFAULT '5999-01-01 00:00:00'::timestamp without time zone,
             CONSTRAINT sas_member_property_pkey PRIMARY KEY (property_id , member_id , internal_dimension_id , locale_id , revision )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__dim_internal_rk ON sas_member_property (internal_dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__dim_internal_rk_2 ON sas_member_property (internal_dimension_id , valid_from_dttm , valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__locale_rk ON sas_member_property (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__member_rk ON sas_member_property (member_id , internal_dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__property_rk ON sas_member_property (property_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__valid_from_dttm ON sas_member_property (valid_from_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_member_property__valid_to_dttm ON sas_member_property (valid_to_dttm )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate (
             prate_set_id integer NOT NULL DEFAULT 0,
             prate_idx integer NOT NULL DEFAULT 0,
             prate_type_member_id integer NOT NULL DEFAULT 0,
             concat_member_ids character varying(255) NOT NULL DEFAULT ''::character varying,
             prate double precision NOT NULL DEFAULT 0::double precision,
             CONSTRAINT sas_prate_pkey PRIMARY KEY (prate_set_id , prate_idx )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_prate__unique_crossing ON sas_prate (prate_set_id , prate_type_member_id , concat_member_ids COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_group (
             prate_group_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             prate_group_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             CONSTRAINT sas_prate_group_pkey PRIMARY KEY (prate_group_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_prate_group__unique_code ON sas_prate_group (prate_group_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_group_lookthru (
             prate_group_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             prate_set_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_prate_group_lookthru_pkey PRIMARY KEY (prate_group_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_prate_group_lookthru__unique_set ON sas_prate_group_lookthru (prate_group_id , prate_set_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_group_nls (
             prate_group_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             prate_group_nm character varying(50) DEFAULT NULL::character varying,
             prate_group_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_prate_group_nls_pkey PRIMARY KEY (prate_group_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_group_precedence (
             prate_group_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_prate_group_precedence_pkey PRIMARY KEY (prate_group_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_prate_group_precedence__unique_dim_type ON sas_prate_group_precedence (prate_group_id , dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_set (
             prate_set_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             prate_set_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             CONSTRAINT sas_prate_set_pkey PRIMARY KEY (prate_set_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_prate_set__unique_code ON sas_prate_set (prate_set_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_set_dim_type (
             prate_set_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_prate_set_dim_type_pkey PRIMARY KEY (prate_set_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_prate_set_dim_type__unique_dim_type ON sas_prate_set_dim_type (prate_set_id , dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_prate_set_nls (
             prate_set_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             prate_set_nm character varying(50) DEFAULT NULL::character varying,
             prate_set_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_prate_set_nls_pkey PRIMARY KEY (prate_set_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_property (
             property_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             property_type_enum smallint NOT NULL DEFAULT 0::smallint,
             custom_flg boolean NOT NULL DEFAULT true,
             dflt_value character varying(255) DEFAULT NULL::character varying,
             enum_values text,
             min_value double precision,
             max_value double precision,
             property_options integer NOT NULL DEFAULT 1,
             CONSTRAINT sas_property_pkey PRIMARY KEY (property_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_property__code ON sas_property (property_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_property_nls (
             property_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_nm character varying(50) DEFAULT NULL::character varying,
             property_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_property_nls_pkey PRIMARY KEY (property_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_property_nls__locale_rk ON sas_property_nls (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_property_nls__property_rk ON sas_property_nls (property_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube (
             vcube_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             vcube_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             poc_fact_set_id integer,
             driver_formula_only smallint NOT NULL DEFAULT 0::smallint,
             CONSTRAINT sas_vcube_pkey PRIMARY KEY (vcube_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_vcube__code ON sas_vcube (vcube_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube_fact_set (
             vcube_id integer NOT NULL DEFAULT 0,
             fact_set_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_vcube_fact_set_pkey PRIMARY KEY (vcube_id , fact_set_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_fact_set__fact_set_rk ON sas_vcube_fact_set (fact_set_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_fact_set__vcube_rk ON sas_vcube_fact_set (vcube_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube_hierarchy (
             vcube_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             dimension_id integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             hierarchy_id integer NOT NULL DEFAULT 0,
             asof_dttm timestamp without time zone,
             CONSTRAINT sas_vcube_hierarchy_pkey PRIMARY KEY (vcube_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_hierarchy__dimension_rk ON sas_vcube_hierarchy (dimension_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_hierarchy__hierarchy_rk ON sas_vcube_hierarchy (hierarchy_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_hierarchy__vcube_rk ON sas_vcube_hierarchy (vcube_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube_nls (
             vcube_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             vcube_nm character varying(50) DEFAULT NULL::character varying,
             vcube_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_vcube_nls_pkey PRIMARY KEY (vcube_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_nls__locale_rk ON sas_vcube_nls (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_nls__vcube_rk ON sas_vcube_nls (vcube_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube_prate_group (
             vcube_id integer NOT NULL DEFAULT 0,
             analysis_member_id integer NOT NULL DEFAULT 0,
             prate_group_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_vcube_prate_group_pkey PRIMARY KEY (vcube_id , analysis_member_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube_property (
             vcube_id integer NOT NULL DEFAULT 0,
             property_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             property_value character varying(255) NOT NULL DEFAULT ''::character varying,
             CONSTRAINT sas_vcube_property_pkey PRIMARY KEY (vcube_id , property_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_property__locale_rk ON sas_vcube_property (locale_id )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_vcube_property__property_rk ON sas_vcube_property (property_id )
     ) by PASSTHRU ;

   execute (
           CREATE INDEX sas_vcube_property__vcube_rk ON sas_vcube_property (vcube_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_vcube_xrate_group (
             vcube_id integer NOT NULL DEFAULT 0,
             analysis_member_id integer NOT NULL DEFAULT 0,
             xrate_group_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_vcube_xrate_group_pkey PRIMARY KEY (vcube_id , analysis_member_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate (
             xrate_set_id integer NOT NULL DEFAULT 0,
             xrate_idx integer NOT NULL DEFAULT 0,
             xrate_type_member_id integer NOT NULL DEFAULT 0,
             from_currency_member_id integer NOT NULL DEFAULT 0,
             to_currency_member_id integer NOT NULL DEFAULT 0,
             time_member_id integer NOT NULL DEFAULT 0,
             concat_member_ids character varying(255) NOT NULL DEFAULT ''::character varying,
             xrate double precision NOT NULL DEFAULT 0::double precision,
             CONSTRAINT sas_xrate_pkey PRIMARY KEY (xrate_set_id , xrate_idx )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_xrate__unique_crossing ON sas_xrate (xrate_set_id , xrate_type_member_id , from_currency_member_id , to_currency_member_id , time_member_id , concat_member_ids COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_group (
             xrate_group_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             xrate_group_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             base_currency_member_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_xrate_group_pkey PRIMARY KEY (xrate_group_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE INDEX sas_xrate_group__curr_ndx ON sas_xrate_group (base_currency_member_id )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_xrate_group__unique_code ON sas_xrate_group (xrate_group_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_group_lookthru (
             xrate_group_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             xrate_set_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_xrate_group_lookthru_pkey PRIMARY KEY (xrate_group_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_xrate_group_lookthru__unique_set ON sas_xrate_group_lookthru (xrate_group_id , xrate_set_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_group_nls (
             xrate_group_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             xrate_group_nm character varying(50) DEFAULT NULL::character varying,
             xrate_group_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_xrate_group_nls_pkey PRIMARY KEY (xrate_group_id , locale_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_group_precedence (
             xrate_group_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_xrate_group_precedence_pkey PRIMARY KEY (xrate_group_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_xrate_group_precedence__unique_dim_type ON sas_xrate_group_precedence (xrate_group_id , dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_group_tgt_curnc (
             xrate_group_id integer NOT NULL DEFAULT 0,
             target_currency_member_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_xrate_group_tgt_curnc_pkey PRIMARY KEY (xrate_group_id , target_currency_member_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_set (
             xrate_set_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             xrate_set_cd character varying(32) NOT NULL DEFAULT ''::character varying,
             base_currency_member_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_xrate_set_pkey PRIMARY KEY (xrate_set_id )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_xrate_set__unique_code ON sas_xrate_set (xrate_set_cd COLLATE pg_catalog."default" )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_set_dim_type (
             xrate_set_id integer NOT NULL DEFAULT 0,
             ordinal integer NOT NULL DEFAULT 0,
             dim_type_id integer NOT NULL DEFAULT 0,
             CONSTRAINT sas_xrate_set_dim_type_pkey PRIMARY KEY (xrate_set_id , ordinal )
          )
     ) by PASSTHRU ;

  execute (
           CREATE UNIQUE INDEX sas_xrate_set_dim_type__unique_dim_type ON sas_xrate_set_dim_type (xrate_set_id , dim_type_id )
     ) by PASSTHRU ;

  execute (
           CREATE TABLE if not exists sas_xrate_set_nls (
             xrate_set_id integer NOT NULL DEFAULT 0,
             locale_id integer NOT NULL DEFAULT 0,
             revision integer NOT NULL DEFAULT 0,
             xrate_set_nm character varying(50) DEFAULT NULL::character varying,
             xrate_set_desc character varying(255) DEFAULT NULL::character varying,
             CONSTRAINT sas_xrate_set_nls_pkey PRIMARY KEY (xrate_set_id , locale_id )
          )
     ) by PASSTHRU ;

  disconnect from PASSTHRU ;

quit ;
%checkErr;
%if &_tmpRC %then
  %return ;

/* Table updates */

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder/&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE,
          sesetl_validateLib=Y
         ) ;

/* SAS_DATA_LOCALE */

%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_DATA_LOCALE
quit;

data _null_;
   call symput("ses_language", "'" || strip("&ses_language") || "'");
   call symput("ses_country", "'" || strip("&ses_country") || "'");
   call symput("ses_variant", "'" || strip("&ses_variant") || "'");
run;

%if &_recordFound EQ 0 %then %do ;
   proc sql ;
      connect to &sesetl_EngineType
              as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions) ;
      execute(insert into SAS_DATA_LOCALE
              values (1, &ses_language, &ses_country, &ses_variant, true)) by PASSTHRU ;
      disconnect from PASSTHRU ;
   quit ;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* VALID_FROM_DTTM & VALID_TO_DTTM */

%let dflt_from_dttm = '01JAN1970:00:00'DT;
%let curr_rec_dttm = '01JAN5999:00:00'DT;

/* SAS_DIM_TYPE & SAS_DIM_TYPE_NLS */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_DIM_TYPE
quit;

%if &_recordFound EQ 0 %then %do ;
   data work.dimension_type;
      length DIM_TYPE_ID 8;
      length REVISION 8;
      length DIM_TYPE_ROLE 8;
      length INTERNAL_DIM_TYPE_ID 8;
      length DIM_TYPE_CD $ 32;
      length HIDDEN_FLG $5;
      length DFLT_HIERARCHY_ID 8;
      length DIM_TYPE_NM $ 50;
      length DIM_TYPE_DESC $ 255;
      length LOCALE_ID 8;
      REVISION = 1;
      HIDDEN_FLG = "false";
      LOCALE_ID = 1;
      FORMAT VALID_FROM_DTTM DATETIME25.;
      VALID_FROM_DTTM = &dflt_from_dttm;
      FORMAT VALID_TO_DTTM DATETIME25.;
      VALID_TO_DTTM = &curr_rec_dttm;
      DIM_TYPE_ID = 1;
      DIM_TYPE_ROLE = 1;
      INTERNAL_DIM_TYPE_ID = -1;
      DIM_TYPE_CD = 'ACCOUNT';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_ACCOUNT_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_ACCOUNT_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 2;
      DIM_TYPE_ROLE = 3;
      INTERNAL_DIM_TYPE_ID = -2;
      DIM_TYPE_CD = 'ANALYSIS';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_ANALYSIS_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_ANALYSIS_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 3;
      DIM_TYPE_ROLE = 4;
      INTERNAL_DIM_TYPE_ID = -3;
      DIM_TYPE_CD = 'COSTCTR';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_COSTCTR_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_COSTCTR_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 4;
      DIM_TYPE_ROLE = 5;
      INTERNAL_DIM_TYPE_ID = -4;
      DIM_TYPE_CD = 'CURRENCY';
      DFLT_DIMENSION_ID = 2;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_CURRENCY_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_CURRENCY_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 5;
      DIM_TYPE_ROLE = 6;
      INTERNAL_DIM_TYPE_ID = -5;
      DIM_TYPE_CD = 'EXTORG';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_EXTORG_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_EXTORG_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 6;
      DIM_TYPE_ROLE = 7;
      INTERNAL_DIM_TYPE_ID = -6;
      DIM_TYPE_CD = 'INTORG';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_INTORG_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_INTORG_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 7;
      DIM_TYPE_ROLE = 8;
      INTERNAL_DIM_TYPE_ID = -7;
      DIM_TYPE_CD = 'ITEMCAT';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_ITEMCAT_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_ITEMCAT_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 8;
      DIM_TYPE_ROLE = 10;
      INTERNAL_DIM_TYPE_ID = -8;
      DIM_TYPE_CD = 'PROFITCTR';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_PROFITCTR_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_PROFITCTR_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 9;
      DIM_TYPE_ROLE = 11;
      INTERNAL_DIM_TYPE_ID = -9;
      DIM_TYPE_CD = 'TIME';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_TIME_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_TIME_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 10;
      DIM_TYPE_ROLE = 12;
      INTERNAL_DIM_TYPE_ID = -6;
      DIM_TYPE_CD = 'TRADER';
      DFLT_DIMENSION_ID = 0;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_TRADER_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_TRADER_DESC_VAR)";
      HIDDEN_FLG = "false";
      output;
      DIM_TYPE_ID = 11;
      DIM_TYPE_ROLE = 16;
      INTERNAL_DIM_TYPE_ID = -4;
      DIM_TYPE_CD = 'TO_CURRENCY';
      DFLT_DIMENSION_ID = 3;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_TOCRNCY_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_TOCRNCY_DESC_VAR)";
      HIDDEN_FLG = "true";
      output;
      DIM_TYPE_ID = 12;
      DIM_TYPE_ROLE = 15;
      INTERNAL_DIM_TYPE_ID = -12;
      DIM_TYPE_CD = 'RATE_TYPE';
      DFLT_DIMENSION_ID = 4;
      DIM_TYPE_NM = "%nrbquote(&SES_DIMTYPE_RATETYPE_NM_VAR)";
      DIM_TYPE_DESC = "%nrbquote(&SES_DIMTYPE_RATETYPE_DESC_VAR)";
      HIDDEN_FLG = "true";
      output;
      keep DIM_TYPE_ID REVISION DIM_TYPE_ROLE INTERNAL_DIM_TYPE_ID DIM_TYPE_CD HIDDEN_FLG
           DFLT_DIMENSION_ID DIM_TYPE_NM DIM_TYPE_DESC LOCALE_ID VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   data work.dimension_type_nls;
      set work.dimension_type;
      keep DIM_TYPE_ID LOCALE_ID REVISION DIM_TYPE_NM DIM_TYPE_DESC VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;

   data work.dimension_type;
      set work.dimension_type;
      keep DIM_TYPE_ID REVISION DIM_TYPE_ROLE INTERNAL_DIM_TYPE_ID DIM_TYPE_CD HIDDEN_FLG
           DFLT_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   proc append base=_SDMLIB.SAS_DIM_TYPE data=work.dimension_type &FM_PROCAPPENDFORCE;
   run;

   %if &ses_locale_found eq 1 %then %do;
      proc append base=_SDMLIB.SAS_DIM_TYPE_NLS data=work.dimension_type_nls &FM_PROCAPPENDFORCE;
      run;
   %end;

   %addhstry(action_type_id=18, solution_id=46, object_type_id=2002,
             object_id_table=work.dimension_type, object_id_column_nm=DIM_TYPE_ID,
             bldlibnm_sassdm=N);
%end;

/* SAS_DIMENSION & SAS_DIMENSION_NLS */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_DIMENSION
quit;

%if &_recordFound EQ 0 %then %do ;
   data work.dimension;
      length DIMENSION_ID 8;
      length REVISION 8;
      length INTERNAL_DIMENSION_ID 8;
      length DIM_TYPE_ID 8;
      length DIMENSION_CD $ 32;
      length DFLT_HIERARCHY_ID 8;
      length EDITABLE_FLG $5;
      length EDITABLE_PAST_FLG $5;
      length HIDDEN_FLG $5;
      length LOCALE_ID 8;
      length DIMENSION_NM $ 50;
      length DIMENSION_DESC $ 255;
      REVISION = 1;
      EDITABLE_FLG = "true";
      EDITABLE_PAST_FLG = "false";
      LOCALE_ID = 1;
      FORMAT VALID_FROM_DTTM DATETIME25.;
      VALID_FROM_DTTM = &dflt_from_dttm;
      FORMAT VALID_TO_DTTM DATETIME25.;
      VALID_TO_DTTM = &curr_rec_dttm;
      DIMENSION_ID = 1;
      INTERNAL_DIMENSION_ID = -1;
      DIM_TYPE_ID = 9;
      DIMENSION_CD = 'TIME_Default';
      HIDDEN_FLG = "false";
      DIMENSION_NM = "%nrbquote(&SES_DIMEN_TIMEDFLT_NM_VAR)";
      DIMENSION_DESC = "%nrbquote(&SES_DIMEN_TIMEDFLT_DESC_VAR)";
      output;
      DIMENSION_ID = 2;
      INTERNAL_DIMENSION_ID = -2;
      DIM_TYPE_ID = 4;
      DIMENSION_CD = 'CURRENCY';
      HIDDEN_FLG = "false";
      DIMENSION_NM = "%nrbquote(&SES_DIMEN_CURRENCY_NM_VAR)";
      DIMENSION_DESC = "%nrbquote(&SES_DIMEN_CURRENCY_DESC_VAR)";
      output;
      DIMENSION_ID = 3;
      INTERNAL_DIMENSION_ID = -2;
      DIM_TYPE_ID = 11;
      DIMENSION_CD = 'TO_CURRENCY';
      HIDDEN_FLG = "true";
      DIMENSION_NM = "%nrbquote(&SES_DIMEN_TOCRNCY_NM_VAR)";
      DIMENSION_DESC = "%nrbquote(&SES_DIMEN_TOCRNCY_DESC_VAR)";
      output;
      DIMENSION_ID = 4;
      INTERNAL_DIMENSION_ID = -4;
      DIM_TYPE_ID = 12;
      DIMENSION_CD = 'RATE_TYPE';
      DFLT_HIERARCHY_ID = 3;
      HIDDEN_FLG = "true";
      DIMENSION_NM = "%nrbquote(&SES_DIMEN_RATETYPE_NM_VAR)";
      DIMENSION_DESC = "%nrbquote(&SES_DIMEN_RATETYPE_DESC_VAR)";
      output;
      keep DIMENSION_ID REVISION INTERNAL_DIMENSION_ID DIM_TYPE_ID DIMENSION_CD DFLT_HIERARCHY_ID
           EDITABLE_FLG EDITABLE_PAST_FLG HIDDEN_FLG LOCALE_ID DIMENSION_NM DIMENSION_DESC
           VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   data work.dimension_nls;
      set work.dimension;
      keep DIMENSION_ID LOCALE_ID REVISION DIMENSION_NM DIMENSION_DESC VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;

   data work.dimension;
      set work.dimension;
      keep DIMENSION_ID REVISION INTERNAL_DIMENSION_ID DIM_TYPE_ID DIMENSION_CD DFLT_HIERARCHY_ID
           EDITABLE_FLG EDITABLE_PAST_FLG HIDDEN_FLG VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   proc append base=_SDMLIB.SAS_DIMENSION data=work.dimension &FM_PROCAPPENDFORCE;
   run;

   %if &ses_locale_found eq 1 %then %do;
      proc append base=_SDMLIB.SAS_DIMENSION_NLS data=work.dimension_nls &FM_PROCAPPENDFORCE;
      run;
   %end;

   %addhstry(action_type_id=18, solution_id=46, object_type_id=2001,
             object_id_table=work.dimension, object_id_column_nm=DIMENSION_ID,
             bldlibnm_sassdm=N);
%end;

/* SAS_HIERARCHY & SAS_HIERARCHY_NLS */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_HIERARCHY
quit;

%if &_recordFound EQ 0 %then %do ;
   data work.hier;
      length HIERARCHY_ID 8;
      length REVISION 8;
      length HIERARCHY_CD $ 32;
      length INTERNAL_DIMENSION_ID 8;
      length HIDDEN_FLG $5;
      length LOCALE_ID 8;
      length HIERARCHY_NM $ 50;
      length HIERARCHY_DESC $ 255;
      REVISION = 1;
      LOCALE_ID = 1;
      FORMAT VALID_FROM_DTTM DATETIME25.;
      VALID_FROM_DTTM = &dflt_from_dttm;
      FORMAT VALID_TO_DTTM DATETIME25.;
      VALID_TO_DTTM = &curr_rec_dttm;
      HIERARCHY_ID = 1;
      HIERARCHY_CD = 'TIME_Default';
      INTERNAL_DIMENSION_ID = -1;
      HIDDEN_FLG = "false";
      HIERARCHY_NM = "%nrbquote(&SES_HIER_TIMEDFLT_NM_VAR)";
      HIERARCHY_DESC = "%nrbquote(&SES_HIER_TIMEDFLT_DESC_VAR)";
      output;
      HIERARCHY_ID = 2;
      HIERARCHY_CD = 'CURRENCY';
      INTERNAL_DIMENSION_ID = -2;
      HIDDEN_FLG = "false";
      HIERARCHY_NM = "%nrbquote(&SES_HIER_CURRENCY_NM_VAR)";
      HIERARCHY_DESC = "%nrbquote(&SES_HIER_CURRENCY_DESC_VAR)";
      output;
      HIERARCHY_ID = 3;
      HIERARCHY_CD = 'RATE_TYPE';
      INTERNAL_DIMENSION_ID = -4;
      HIDDEN_FLG = "false";
      HIERARCHY_NM = "%nrbquote(&SES_HIER_RATETYPE_NM_VAR)";
      HIERARCHY_DESC = "%nrbquote(&SES_HIER_RATETYPE_DESC_VAR)";
      output;
      keep HIERARCHY_ID REVISION HIERARCHY_CD INTERNAL_DIMENSION_ID HIDDEN_FLG LOCALE_ID
           HIERARCHY_NM HIERARCHY_DESC VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   data work.hier_nls;
      set work.hier;
      keep HIERARCHY_ID LOCALE_ID REVISION HIERARCHY_NM HIERARCHY_DESC VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   data work.hier;
      set work.hier;
      keep HIERARCHY_ID REVISION HIERARCHY_CD INTERNAL_DIMENSION_ID HIDDEN_FLG VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   proc append base=_SDMLIB.SAS_HIERARCHY data=work.hier &FM_PROCAPPENDFORCE;
   run;

   %if &ses_locale_found eq 1 %then %do;
      proc append base=_SDMLIB.SAS_HIERARCHY_NLS data=work.hier_nls &FM_PROCAPPENDFORCE;
      run;
   %end;

   %addhstry(action_type_id=18, solution_id=46, object_type_id=2007,
             object_id_table=work.hier, object_id_column_nm=HIERARCHY_ID,
             bldlibnm_sassdm=N);
%end;

/* SAS_MEMBER & SAS_MEMBER_NLS */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_MEMBER
quit;

%if &_recordFound EQ 0 %then %do ;
   data work.member;
      length INTERNAL_DIMENSION_ID 8;
      length MEMBER_ID 8;
      length REVISION 8;
      length INTERNAL_DIM_TYPE_ID 8;
      length MEMBER_TYPE_ENUM 8;
      length MEMBER_CD $ 32;
      length HIDDEN_FLG $5;
      length CUSTOM_FLG $5;
      length LOCALE_ID 8;
      length MEMBER_NM $ 50;
      length MEMBER_DESC $ 255;
      INTERNAL_DIMENSION_ID = -1;
      REVISION = 1;
      INTERNAL_DIM_TYPE_ID = -9;
      MEMBER_TYPE_ENUM = 1;
      HIDDEN_FLG = "false";
      CUSTOM_FLG = "true";
      LOCALE_ID = 1;
      FORMAT VALID_FROM_DTTM DATETIME25.;
      VALID_FROM_DTTM = &dflt_from_dttm;
      FORMAT VALID_TO_DTTM DATETIME25.;
      VALID_TO_DTTM = &curr_rec_dttm;
      MEMBER_ID = 1;
      MEMBER_CD = 'MULTYR_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MULTYR_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MULTYR_D_DESC_VAR)";
      output;
      MEMBER_ID = 2;
      MEMBER_CD = '2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 3;
      MEMBER_CD = '2000Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2000Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2000Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 4;
      MEMBER_CD = '2000Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2000Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2000Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 5;
      MEMBER_CD = '2000Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2000Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2000Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 6;
      MEMBER_CD = '2000Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2000Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2000Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 7;
      MEMBER_CD = 'JAN2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 8;
      MEMBER_CD = 'FEB2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 9;
      MEMBER_CD = 'MAR2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 10;
      MEMBER_CD = 'APR2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 11;
      MEMBER_CD = 'MAY2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 12;
      MEMBER_CD = 'JUN2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 13;
      MEMBER_CD = 'JUL2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 14;
      MEMBER_CD = 'AUG2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 15;
      MEMBER_CD = 'SEP2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 16;
      MEMBER_CD = 'OCT2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 17;
      MEMBER_CD = 'NOV2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 18;
      MEMBER_CD = 'DEC2000_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2000_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2000_D_DESC_VAR)";
      output;
      MEMBER_ID = 19;
      MEMBER_CD = '2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 20;
      MEMBER_CD = '2001Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2001Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2001Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 21;
      MEMBER_CD = '2001Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2001Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2001Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 22;
      MEMBER_CD = '2001Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2001Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2001Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 23;
      MEMBER_CD = '2001Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2001Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2001Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 24;
      MEMBER_CD = 'JAN2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 25;
      MEMBER_CD = 'FEB2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 26;
      MEMBER_CD = 'MAR2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 27;
      MEMBER_CD = 'APR2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 28;
      MEMBER_CD = 'MAY2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 29;
      MEMBER_CD = 'JUN2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 30;
      MEMBER_CD = 'JUL2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 31;
      MEMBER_CD = 'AUG2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 32;
      MEMBER_CD = 'SEP2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 33;
      MEMBER_CD = 'OCT2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 34;
      MEMBER_CD = 'NOV2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 35;
      MEMBER_CD = 'DEC2001_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2001_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2001_D_DESC_VAR)";
      output;
      MEMBER_ID = 36;
      MEMBER_CD = '2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 37;
      MEMBER_CD = '2002Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2002Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2002Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 38;
      MEMBER_CD = '2002Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2002Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2002Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 39;
      MEMBER_CD = '2002Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2002Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2002Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 40;
      MEMBER_CD = '2002Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2002Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2002Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 41;
      MEMBER_CD = 'JAN2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 42;
      MEMBER_CD = 'FEB2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 43;
      MEMBER_CD = 'MAR2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 44;
      MEMBER_CD = 'APR2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 45;
      MEMBER_CD = 'MAY2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 46;
      MEMBER_CD = 'JUN2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 47;
      MEMBER_CD = 'JUL2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 48;
      MEMBER_CD = 'AUG2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 49;
      MEMBER_CD = 'SEP2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 50;
      MEMBER_CD = 'OCT2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 51;
      MEMBER_CD = 'NOV2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 52;
      MEMBER_CD = 'DEC2002_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2002_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2002_D_DESC_VAR)";
      output;
      MEMBER_ID = 53;
      MEMBER_CD = '2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 54;
      MEMBER_CD = '2003Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2003Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2003Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 55;
      MEMBER_CD = '2003Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2003Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2003Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 56;
      MEMBER_CD = '2003Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2003Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2003Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 57;
      MEMBER_CD = '2003Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2003Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2003Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 58;
      MEMBER_CD = 'JAN2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 59;
      MEMBER_CD = 'FEB2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 60;
      MEMBER_CD = 'MAR2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 61;
      MEMBER_CD = 'APR2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 62;
      MEMBER_CD = 'MAY2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 63;
      MEMBER_CD = 'JUN2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 64;
      MEMBER_CD = 'JUL2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 65;
      MEMBER_CD = 'AUG2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 66;
      MEMBER_CD = 'SEP2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 67;
      MEMBER_CD = 'OCT2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 68;
      MEMBER_CD = 'NOV2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 69;
      MEMBER_CD = 'DEC2003_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2003_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2003_D_DESC_VAR)";
      output;
      MEMBER_ID = 70;
      MEMBER_CD = '2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 71;
      MEMBER_CD = '2004Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2004Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2004Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 72;
      MEMBER_CD = '2004Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2004Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2004Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 73;
      MEMBER_CD = '2004Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2004Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2004Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 74;
      MEMBER_CD = '2004Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2004Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2004Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 75;
      MEMBER_CD = 'JAN2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 76;
      MEMBER_CD = 'FEB2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 77;
      MEMBER_CD = 'MAR2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 78;
      MEMBER_CD = 'APR2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 79;
      MEMBER_CD = 'MAY2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 80;
      MEMBER_CD = 'JUN2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 81;
      MEMBER_CD = 'JUL2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 82;
      MEMBER_CD = 'AUG2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 83;
      MEMBER_CD = 'SEP2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 84;
      MEMBER_CD = 'OCT2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 85;
      MEMBER_CD = 'NOV2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 86;
      MEMBER_CD = 'DEC2004_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2004_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2004_D_DESC_VAR)";
      output;
      MEMBER_ID = 87;
      MEMBER_CD = '2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 88;
      MEMBER_CD = '2005Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2005Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2005Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 89;
      MEMBER_CD = '2005Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2005Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2005Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 90;
      MEMBER_CD = '2005Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2005Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2005Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 91;
      MEMBER_CD = '2005Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2005Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2005Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 92;
      MEMBER_CD = 'JAN2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 93;
      MEMBER_CD = 'FEB2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 94;
      MEMBER_CD = 'MAR2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 95;
      MEMBER_CD = 'APR2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 96;
      MEMBER_CD = 'MAY2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 97;
      MEMBER_CD = 'JUN2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 98;
      MEMBER_CD = 'JUL2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 99;
      MEMBER_CD = 'AUG2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 100;
      MEMBER_CD = 'SEP2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 101;
      MEMBER_CD = 'OCT2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 102;
      MEMBER_CD = 'NOV2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 103;
      MEMBER_CD = 'DEC2005_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2005_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2005_D_DESC_VAR)";
      output;
      MEMBER_ID = 104;
      MEMBER_CD = '2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 105;
      MEMBER_CD = '2006Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2006Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2006Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 106;
      MEMBER_CD = '2006Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2006Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2006Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 107;
      MEMBER_CD = '2006Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2006Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2006Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 108;
      MEMBER_CD = '2006Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2006Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2006Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 109;
      MEMBER_CD = 'JAN2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 110;
      MEMBER_CD = 'FEB2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 111;
      MEMBER_CD = 'MAR2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 112;
      MEMBER_CD = 'APR2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 113;
      MEMBER_CD = 'MAY2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 114;
      MEMBER_CD = 'JUN2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 115;
      MEMBER_CD = 'JUL2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 116;
      MEMBER_CD = 'AUG2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 117;
      MEMBER_CD = 'SEP2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 118;
      MEMBER_CD = 'OCT2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 119;
      MEMBER_CD = 'NOV2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 120;
      MEMBER_CD = 'DEC2006_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2006_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2006_D_DESC_VAR)";
      output;
      MEMBER_ID = 121;
      MEMBER_CD = '2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 122;
      MEMBER_CD = '2007Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2007Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2007Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 123;
      MEMBER_CD = '2007Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2007Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2007Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 124;
      MEMBER_CD = '2007Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2007Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2007Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 125;
      MEMBER_CD = '2007Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2007Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2007Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 126;
      MEMBER_CD = 'JAN2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 127;
      MEMBER_CD = 'FEB2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 128;
      MEMBER_CD = 'MAR2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 129;
      MEMBER_CD = 'APR2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 130;
      MEMBER_CD = 'MAY2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 131;
      MEMBER_CD = 'JUN2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 132;
      MEMBER_CD = 'JUL2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 133;
      MEMBER_CD = 'AUG2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 134;
      MEMBER_CD = 'SEP2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 135;
      MEMBER_CD = 'OCT2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 136;
      MEMBER_CD = 'NOV2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 137;
      MEMBER_CD = 'DEC2007_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2007_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2007_D_DESC_VAR)";
      output;
      MEMBER_ID = 138;
      MEMBER_CD = '2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 139;
      MEMBER_CD = '2008Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2008Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2008Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 140;
      MEMBER_CD = '2008Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2008Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2008Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 141;
      MEMBER_CD = '2008Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2008Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2008Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 142;
      MEMBER_CD = '2008Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2008Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2008Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 143;
      MEMBER_CD = 'JAN2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 144;
      MEMBER_CD = 'FEB2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 145;
      MEMBER_CD = 'MAR2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 146;
      MEMBER_CD = 'APR2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 147;
      MEMBER_CD = 'MAY2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 148;
      MEMBER_CD = 'JUN2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 149;
      MEMBER_CD = 'JUL2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 150;
      MEMBER_CD = 'AUG2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 151;
      MEMBER_CD = 'SEP2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 152;
      MEMBER_CD = 'OCT2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 153;
      MEMBER_CD = 'NOV2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 154;
      MEMBER_CD = 'DEC2008_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2008_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2008_D_DESC_VAR)";
      output;
      MEMBER_ID = 155;
      MEMBER_CD = '2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 156;
      MEMBER_CD = '2009Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2009Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2009Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 157;
      MEMBER_CD = '2009Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2009Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2009Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 158;
      MEMBER_CD = '2009Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2009Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2009Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 159;
      MEMBER_CD = '2009Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2009Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2009Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 160;
      MEMBER_CD = 'JAN2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 161;
      MEMBER_CD = 'FEB2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 162;
      MEMBER_CD = 'MAR2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 163;
      MEMBER_CD = 'APR2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 164;
      MEMBER_CD = 'MAY2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 165;
      MEMBER_CD = 'JUN2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 166;
      MEMBER_CD = 'JUL2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 167;
      MEMBER_CD = 'AUG2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 168;
      MEMBER_CD = 'SEP2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 169;
      MEMBER_CD = 'OCT2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 170;
      MEMBER_CD = 'NOV2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 171;
      MEMBER_CD = 'DEC2009_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2009_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2009_D_DESC_VAR)";
      output;
      MEMBER_ID = 172;
      MEMBER_CD = '2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 173;
      MEMBER_CD = '2010Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2010Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2010Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 174;
      MEMBER_CD = '2010Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2010Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2010Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 175;
      MEMBER_CD = '2010Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2010Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2010Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 176;
      MEMBER_CD = '2010Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2010Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2010Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 177;
      MEMBER_CD = 'JAN2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 178;
      MEMBER_CD = 'FEB2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 179;
      MEMBER_CD = 'MAR2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 180;
      MEMBER_CD = 'APR2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 181;
      MEMBER_CD = 'MAY2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 182;
      MEMBER_CD = 'JUN2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 183;
      MEMBER_CD = 'JUL2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 184;
      MEMBER_CD = 'AUG2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 185;
      MEMBER_CD = 'SEP2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 186;
      MEMBER_CD = 'OCT2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 187;
      MEMBER_CD = 'NOV2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 188;
      MEMBER_CD = 'DEC2010_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2010_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2010_D_DESC_VAR)";
      output;
      MEMBER_ID = 189;
      MEMBER_CD = '2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 190;
      MEMBER_CD = '2011Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2011Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2011Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 191;
      MEMBER_CD = '2011Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2011Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2011Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 192;
      MEMBER_CD = '2011Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2011Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2011Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 193;
      MEMBER_CD = '2011Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2011Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2011Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 194;
      MEMBER_CD = 'JAN2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 195;
      MEMBER_CD = 'FEB2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 196;
      MEMBER_CD = 'MAR2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 197;
      MEMBER_CD = 'APR2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 198;
      MEMBER_CD = 'MAY2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 199;
      MEMBER_CD = 'JUN2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 200;
      MEMBER_CD = 'JUL2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 201;
      MEMBER_CD = 'AUG2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 202;
      MEMBER_CD = 'SEP2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 203;
      MEMBER_CD = 'OCT2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 204;
      MEMBER_CD = 'NOV2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 205;
      MEMBER_CD = 'DEC2011_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2011_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2011_D_DESC_VAR)";
      output;
      MEMBER_ID = 206;
      MEMBER_CD = '2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 207;
      MEMBER_CD = '2012Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2012Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2012Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 208;
      MEMBER_CD = '2012Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2012Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2012Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 209;
      MEMBER_CD = '2012Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2012Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2012Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 210;
      MEMBER_CD = '2012Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2012Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2012Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 211;
      MEMBER_CD = 'JAN2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 212;
      MEMBER_CD = 'FEB2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 213;
      MEMBER_CD = 'MAR2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 214;
      MEMBER_CD = 'APR2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 215;
      MEMBER_CD = 'MAY2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 216;
      MEMBER_CD = 'JUN2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 217;
      MEMBER_CD = 'JUL2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 218;
      MEMBER_CD = 'AUG2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 219;
      MEMBER_CD = 'SEP2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 220;
      MEMBER_CD = 'OCT2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 221;
      MEMBER_CD = 'NOV2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 222;
      MEMBER_CD = 'DEC2012_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2012_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2012_D_DESC_VAR)";
      output;
      MEMBER_ID = 223;
      MEMBER_CD = '2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 224;
      MEMBER_CD = '2013Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2013Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2013Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 225;
      MEMBER_CD = '2013Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2013Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2013Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 226;
      MEMBER_CD = '2013Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2013Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2013Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 227;
      MEMBER_CD = '2013Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2013Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2013Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 228;
      MEMBER_CD = 'JAN2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 229;
      MEMBER_CD = 'FEB2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 230;
      MEMBER_CD = 'MAR2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 231;
      MEMBER_CD = 'APR2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 232;
      MEMBER_CD = 'MAY2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 233;
      MEMBER_CD = 'JUN2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 234;
      MEMBER_CD = 'JUL2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 235;
      MEMBER_CD = 'AUG2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 236;
      MEMBER_CD = 'SEP2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 237;
      MEMBER_CD = 'OCT2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 238;
      MEMBER_CD = 'NOV2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 239;
      MEMBER_CD = 'DEC2013_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2013_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2013_D_DESC_VAR)";
      output;
      MEMBER_ID = 240;
      MEMBER_CD = '2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 241;
      MEMBER_CD = '2014Q1_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2014Q1_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2014Q1_D_DESC_VAR)";
      output;
      MEMBER_ID = 242;
      MEMBER_CD = '2014Q2_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2014Q2_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2014Q2_D_DESC_VAR)";
      output;
      MEMBER_ID = 243;
      MEMBER_CD = '2014Q3_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2014Q3_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2014Q3_D_DESC_VAR)";
      output;
      MEMBER_ID = 244;
      MEMBER_CD = '2014Q4_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_2014Q4_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_2014Q4_D_DESC_VAR)";
      output;
      MEMBER_ID = 245;
      MEMBER_CD = 'JAN2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JAN2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JAN2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 246;
      MEMBER_CD = 'FEB2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_FEB2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_FEB2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 247;
      MEMBER_CD = 'MAR2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAR2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAR2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 248;
      MEMBER_CD = 'APR2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_APR2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_APR2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 249;
      MEMBER_CD = 'MAY2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_MAY2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_MAY2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 250;
      MEMBER_CD = 'JUN2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUN2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUN2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 251;
      MEMBER_CD = 'JUL2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_JUL2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_JUL2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 252;
      MEMBER_CD = 'AUG2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_AUG2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_AUG2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 253;
      MEMBER_CD = 'SEP2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_SEP2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_SEP2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 254;
      MEMBER_CD = 'OCT2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_OCT2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_OCT2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 255;
      MEMBER_CD = 'NOV2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_NOV2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_NOV2014_D_DESC_VAR)";
      output;
      MEMBER_ID = 256;
      MEMBER_CD = 'DEC2014_Default';
      MEMBER_NM = "%nrbquote(&SES_MBR_TIME_DEC2014_D_NM_VAR)";
      MEMBER_DESC = "%nrbquote(&SES_MBR_TIME_DEC2014_D_DESC_VAR)";
      output;
   run;

   data work.member_nls;
      set work.member;
      keep INTERNAL_DIMENSION_ID MEMBER_ID LOCALE_ID REVISION MEMBER_NM MEMBER_DESC VALID_FROM_DTTM
           VALID_TO_DTTM;
   run;

   data work.member;
      set work.member;
      keep INTERNAL_DIMENSION_ID MEMBER_ID REVISION INTERNAL_DIM_TYPE_ID MEMBER_TYPE_ENUM
           MEMBER_CD HIDDEN_FLG CUSTOM_FLG VALID_FROM_DTTM VALID_TO_DTTM;
   run;

   proc append base=_SDMLIB.SAS_MEMBER data=work.member &FM_PROCAPPENDFORCE;
   run;

   %if &ses_locale_found eq 1 %then %do;
      proc append base=_SDMLIB.SAS_MEMBER_NLS data=work.member_nls &FM_PROCAPPENDFORCE;
      run;
   %end;

   %addhstry(action_type_id=18, solution_id=46, object_type_id=2009,
       object_id_table=work.member, object_id_column_nm=MEMBER_ID,
       bldlibnm_sassdm=N);
%end;

/* SAS_HIERARCHY_DFLT_MEMBER */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_HIERARCHY_DFLT_MEMBER
quit;

%if &_recordFound EQ 0 %then %do ;
   proc sql ;
     connect to &sesetl_EngineType
             as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions);
     execute(insert into SAS_HIERARCHY_DFLT_MEMBER
             values (1,1,1,1,'1970-01-01 00:00:00','5999-01-01 00:00:00')
            ) by PASSTHRU ;
     disconnect from PASSTHRU ;
   quit ;
%end;
%checkErr;
%if &_tmpRC %then
  %return ;


/* SAS_HIERARCHY_MEMBER */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_HIERARCHY_MEMBER
quit;

%if &_recordFound EQ 0 %then %do ;
   proc sql ;
     connect to &sesetl_EngineType
             as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions);
     execute(insert into SAS_HIERARCHY_MEMBER
             values (1,1,1,NULL,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,2,1,1,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,3,1,2,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,4,1,2,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,5,1,2,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,6,1,2,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,7,1,3,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,8,1,3,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,9,1,3,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,10,1,4,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,11,1,4,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,12,1,4,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,13,1,5,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,14,1,5,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,15,1,5,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,16,1,6,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,17,1,6,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,18,1,6,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,19,1,1,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,20,1,19,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,21,1,19,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,22,1,19,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,23,1,19,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,24,1,20,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,25,1,20,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,26,1,20,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,27,1,21,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,28,1,21,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,29,1,21,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,30,1,22,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,31,1,22,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,32,1,22,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,33,1,23,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,34,1,23,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,35,1,23,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,36,1,1,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,37,1,36,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,38,1,36,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,39,1,36,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,40,1,36,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,41,1,37,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,42,1,37,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,43,1,37,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,44,1,38,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,45,1,38,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,46,1,38,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,47,1,39,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,48,1,39,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,49,1,39,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,50,1,40,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,51,1,40,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,52,1,40,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,53,1,1,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,54,1,53,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,55,1,53,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,56,1,53,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,57,1,53,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,58,1,54,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,59,1,54,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,60,1,54,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,61,1,55,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,62,1,55,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,63,1,55,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,64,1,56,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,65,1,56,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,66,1,56,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,67,1,57,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,68,1,57,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,69,1,57,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,70,1,1,4,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,71,1,70,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,72,1,70,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,73,1,70,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,74,1,70,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,75,1,71,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,76,1,71,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,77,1,71,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,78,1,72,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,79,1,72,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,80,1,72,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,81,1,73,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,82,1,73,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,83,1,73,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,84,1,74,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,85,1,74,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,86,1,74,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,87,1,1,5,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,88,1,87,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,89,1,87,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,90,1,87,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,91,1,87,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,92,1,88,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,93,1,88,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,94,1,88,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,95,1,89,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,96,1,89,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,97,1,89,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,98,1,90,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,99,1,90,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,100,1,90,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,101,1,91,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,102,1,91,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,103,1,91,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,104,1,1,6,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,105,1,104,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,106,1,104,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,107,1,104,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,108,1,104,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,109,1,105,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,110,1,105,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,111,1,105,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,112,1,106,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,113,1,106,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,114,1,106,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,115,1,107,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,116,1,107,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,117,1,107,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,118,1,108,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,119,1,108,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,120,1,108,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,121,1,1,7,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,122,1,121,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,123,1,121,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,124,1,121,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,125,1,121,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,126,1,122,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,127,1,122,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,128,1,122,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,129,1,123,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,130,1,123,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,131,1,123,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,132,1,124,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,133,1,124,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,134,1,124,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,135,1,125,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,136,1,125,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,137,1,125,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,138,1,1,8,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,139,1,138,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,140,1,138,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,141,1,138,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,142,1,138,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,143,1,139,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,144,1,139,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,145,1,139,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,146,1,140,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,147,1,140,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,148,1,140,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,149,1,141,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,150,1,141,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,151,1,141,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,152,1,142,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,153,1,142,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,154,1,142,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,155,1,1,9,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,156,1,155,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,157,1,155,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,158,1,155,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,159,1,155,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,160,1,156,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,161,1,156,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,162,1,156,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,163,1,157,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,164,1,157,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,165,1,157,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,166,1,158,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,167,1,158,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,168,1,158,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,169,1,159,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,170,1,159,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,171,1,159,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,172,1,1,10,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,173,1,172,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,174,1,172,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,175,1,172,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,176,1,172,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,177,1,173,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,178,1,173,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,179,1,173,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,180,1,174,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,181,1,174,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,182,1,174,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,183,1,175,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,184,1,175,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,185,1,175,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,186,1,176,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,187,1,176,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,188,1,176,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,189,1,1,11,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,190,1,189,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,191,1,189,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,192,1,189,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,193,1,189,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,194,1,190,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,195,1,190,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,196,1,190,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,197,1,191,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,198,1,191,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,199,1,191,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,200,1,192,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,201,1,192,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,202,1,192,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,203,1,193,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,204,1,193,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,205,1,193,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,206,1,1,12,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,207,1,206,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,208,1,206,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,209,1,206,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,210,1,206,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,211,1,207,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,212,1,207,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,213,1,207,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,214,1,208,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,215,1,208,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,216,1,208,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,217,1,209,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,218,1,209,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,219,1,209,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,220,1,210,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,221,1,210,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,222,1,210,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,223,1,1,13,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,224,1,223,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,225,1,223,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,226,1,223,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,227,1,223,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,228,1,224,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,229,1,224,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,230,1,224,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,231,1,225,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,232,1,225,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,233,1,225,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,234,1,226,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,235,1,226,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,236,1,226,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,237,1,227,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,238,1,227,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,239,1,227,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,240,1,1,14,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,241,1,240,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,242,1,240,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,243,1,240,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,244,1,240,3,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,245,1,241,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,246,1,241,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,247,1,241,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,248,1,242,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,249,1,242,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,250,1,242,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,251,1,243,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,252,1,243,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,253,1,243,2,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,254,1,244,0,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,255,1,244,1,'1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (1,256,1,244,2,'1970-01-01 00:00:00','5999-01-01 00:00:00')
            ) by PASSTHRU ;
     disconnect from PASSTHRU ;
   quit ;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* SAS_PROPERTY */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_PROPERTY
quit;

%if &_recordFound EQ 0 %then %do ;
   proc sql ;
     connect to &sesetl_EngineType
             as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions);
     execute(insert into SAS_PROPERTY
             values (1,1,'AccountType',4,false,'','',NULL, NULL,1),
                    (2,1,'AccountBehavior',4,false,'','',NULL,NULL,1),
                    (3,1,'EndDate',6,false,'','',NULL,NULL,1),
                    (4,1,'BasisData',4,false,'','',NULL,NULL,1),
                    (5,1,'RollForwardMethod',4,false,'','',NULL,NULL,1),
                    (6,1,'Intercompany',2,false,'','',NULL,NULL,1),
                    (7,1,'ReportingEntity',2,false,'','',NULL,NULL,1),
                    (8,1,'Level',4,false,'','',NULL,NULL,1),
                    (9,1,'SourceAccounts',4,false,'','',NULL,NULL,1),
                    (10,1,'BalanceType',4,false,'','',NULL,NULL,1),
                    (11,1,'FunctionalCurrency',3,false,'','',NULL,NULL,1),
                    (12,1,'StartDate',6,false,'','',NULL,NULL,1),
                    (13,1,'ExchangeRateType',4,false,'','',NULL,NULL,1),
                    (14,1,'FormulaPrecedence',4,false,'','',NULL,NULL,1),
                    (15,1,'CTAElimBehavior',4,false,'','',NULL,NULL,1)
            ) by PASSTHRU ;
     disconnect from PASSTHRU ;
   quit ;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* SAS_PROPERTY_NLS */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_PROPERTY_NLS
quit;

%global tmp_accttype_nm_var;
%global tmp_accttype_desc_var;
%global tmp_acctbehavior_nm_var;
%global tmp_acctbehavior_desc_var;
%global tmp_enddate_nm_var;
%global tmp_enddate_desc_var;
%global tmp_basisdata_nm_var;
%global tmp_basisdata_desc_var;
%global tmp_rollfwdmth_nm_var;
%global tmp_rollfwdmth_desc_var;
%global tmp_intercompany_nm_var;
%global tmp_intercompany_desc_var;
%global tmp_rptentity_nm_var;
%global tmp_rptentity_desc_var;
%global tmp_level_nm_var;
%global tmp_level_desc_var;
%global tmp_srcaccts_nm_var;
%global tmp_srcaccts_desc_var;
%global tmp_baltype_nm_var;
%global tmp_baltype_desc_var;
%global tmp_funccrncy_nm_var;
%global tmp_funccrncy_desc_var;
%global tmp_startdate_nm_var;
%global tmp_startdate_desc_var;
%global tmp_exchgratetype_nm_var;
%global tmp_exchgratetype_desc_var;
%global tmp_formprcdnc_nm_var;
%global tmp_formprcdnc_desc_var;
%global tmp_ctaelimbehvr_nm_var;
%global tmp_ctaelimbehvr_desc_var;

data _null_;
   call symput("tmp_accttype_nm_var ", "'" || strip("%nrbquote(&SES_PROP_ACCTTYPE_NM_VAR)") || "'");
   call symput("tmp_accttype_desc_var", "'" || strip("%nrbquote(&SES_PROP_ACCTTYPE_DESC_VAR)") || "'");
   call symput("tmp_acctbehavior_nm_var", "'" || strip("%nrbquote(&SES_PROP_ACCTBEHAVIOR_NM_VAR)") || "'");
   call symput("tmp_acctbehavior_desc_var", "'" || strip("%nrbquote(&SES_PROP_ACCTBEHAVIOR_DESC_VAR)") || "'");
   call symput("tmp_enddate_nm_var", "'" || strip("%nrbquote(&SES_PROP_ENDDATE_NM_VAR)") || "'");
   call symput("tmp_enddate_desc_var", "'" || strip("%nrbquote(&SES_PROP_ENDDATE_DESC_VAR)") || "'");
   call symput("tmp_basisdata_nm_var", "'" || strip("%nrbquote(&SES_PROP_BASISDATA_NM_VAR)") || "'");
   call symput("tmp_basisdata_desc_var", "'" || strip("%nrbquote(&SES_PROP_BASISDATA_DESC_VAR)") || "'");
   call symput("tmp_rollfwdmth_nm_var", "'" || strip("%nrbquote(&SES_PROP_ROLLFWDMTH_NM_VAR)") || "'");
   call symput("tmp_rollfwdmth_desc_var", "'" || strip("%nrbquote(&SES_PROP_ROLLFWDMTH_DESC_VAR)") || "'");
   call symput("tmp_intercompany_nm_var", "'" || strip("%nrbquote(&SES_PROP_INTERCOMPANY_NM_VAR)") || "'");
   call symput("tmp_intercompany_desc_var", "'" || strip("%nrbquote(&SES_PROP_INTERCOMPANY_DESC_VAR)") || "'");
   call symput("tmp_rptentity_nm_var", "'" || strip("%nrbquote(&SES_PROP_RPTENTITY_NM_VAR)") || "'");
   call symput("tmp_rptentity_desc_var", "'" || strip("%nrbquote(&SES_PROP_RPTENTITY_DESC_VAR)") || "'");
   call symput("tmp_level_nm_var", "'" || strip("%nrbquote(&SES_PROP_LEVEL_NM_VAR)") || "'");
   call symput("tmp_level_desc_var", "'" || strip("%nrbquote(&SES_PROP_LEVEL_DESC_VAR)") || "'");
   call symput("tmp_srcaccts_nm_var", "'" || strip("%nrbquote(&SES_PROP_SRCACCTS_NM_VAR)") || "'");
   call symput("tmp_srcaccts_desc_var", "'" || strip("%nrbquote(&SES_PROP_SRCACCTS_DESC_VAR)") || "'");
   call symput("tmp_baltype_nm_var", "'" || strip("%nrbquote(&SES_PROP_BALTYPE_NM_VAR)") || "'");
   call symput("tmp_baltype_desc_var", "'" || strip("%nrbquote(&SES_PROP_BALTYPE_DESC_VAR)") || "'");
   call symput("tmp_funccrncy_nm_var", "'" || strip("%nrbquote(&SES_PROP_FUNCCRNCY_NM_VAR)") || "'");
   call symput("tmp_funccrncy_desc_var", "'" || strip("%nrbquote(&SES_PROP_FUNCCRNCY_DESC_VAR)") || "'");
   call symput("tmp_startdate_nm_var", "'" || strip("%nrbquote(&SES_PROP_STARTDATE_NM_VAR)") || "'");
   call symput("tmp_startdate_desc_var", "'" || strip("%nrbquote(&SES_PROP_STARTDATE_DESC_VAR)") || "'");
   call symput("tmp_exchgratetype_nm_var", "'" || strip("%nrbquote(&SES_PROP_EXCHGRATETYPE_NM_VAR)") || "'");
   call symput("tmp_exchgratetype_desc_var", "'" || strip("%nrbquote(&SES_PROP_EXCHGRATETYPE_DESC_VAR)") || "'");
   call symput("tmp_formprcdnc_nm_var", "'" || strip("%nrbquote(&SES_PROP_FORMPRCDNC_NM_VAR)") || "'");
   call symput("tmp_formprcdnc_desc_var", "'" || strip("%nrbquote(&SES_PROP_FORMPRCDNC_DESC_VAR)") || "'");
   call symput("tmp_ctaelimbehvr_nm_var", "'" || strip("%nrbquote(&SES_PROP_CTAELIMBEHVR_NM_VAR)") || "'");
   call symput("tmp_ctaelimbehvr_desc_var", "'" || strip("%nrbquote(&SES_PROP_CTAELIMBEHVR_DESC_VAR)") || "'");
run;

%if &_recordFound eq 0 and &ses_locale_found eq 1 %then %do ;
   proc sql ;
     connect to &sesetl_EngineType
             as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions);
     execute(insert into SAS_PROPERTY_NLS
             values (1,1,1,&tmp_accttype_nm_var, &tmp_accttype_desc_var),
                    (2,1,1,&tmp_acctbehavior_nm_var, &tmp_acctbehavior_desc_var),
                    (3,1,1,&tmp_enddate_nm_var, &tmp_enddate_desc_var),
                    (4,1,1,&tmp_basisdata_nm_var, &tmp_basisdata_desc_var),
                    (5,1,1,&tmp_rollfwdmth_nm_var, &tmp_rollfwdmth_desc_var),
                    (6,1,1,&tmp_intercompany_nm_var, &tmp_intercompany_desc_var),
                    (7,1,1,&tmp_rptentity_nm_var, &tmp_rptentity_desc_var),
                    (8,1,1,&tmp_level_nm_var, &tmp_level_desc_var),
                    (9,1,1,&tmp_srcaccts_nm_var, &tmp_srcaccts_desc_var),
                    (10,1,1,&tmp_baltype_nm_var, &tmp_baltype_desc_var),
                    (11,1,1,&tmp_funccrncy_nm_var, &tmp_funccrncy_desc_var),
                    (12,1,1,&tmp_startdate_nm_var, &tmp_startdate_desc_var),
                    (13,1,1,&tmp_exchgratetype_nm_var, &tmp_exchgratetype_desc_var),
                    (14,1,1,&tmp_formprcdnc_nm_var, &tmp_formprcdnc_desc_var),
                    (15,1,1,&tmp_ctaelimbehvr_nm_var, &tmp_ctaelimbehvr_desc_var)
            ) by PASSTHRU ;
     disconnect from PASSTHRU ;
   quit ;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* SAS_MEMBER_PROPERTY */
%let _recordFound=0;
proc sql noprint;
  select count(*) into :_recordFound
        from _SDMLIB.SAS_MEMBER_PROPERTY
quit;

%if &_recordFound EQ 0 %then %do ;
   proc sql ;
     connect to &sesetl_EngineType
             as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions);
     execute(insert into SAS_MEMBER_PROPERTY
             values (-1,1,8,0,1,'AllYears','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,1,3,0,1,'59991231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,1,12,0,1,'19700101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,2,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,2,3,0,1,'20001231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,2,12,0,1,'20000101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,3,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,3,3,0,1,'20000331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,3,12,0,1,'20000101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,4,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,4,3,0,1,'20000630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,4,12,0,1,'20000401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,5,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,5,3,0,1,'20000930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,5,12,0,1,'20000701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,6,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,6,3,0,1,'20001231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,6,12,0,1,'20001001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,7,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,7,3,0,1,'20000131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,7,12,0,1,'20000101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,8,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,8,3,0,1,'20000229','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,8,12,0,1,'20000201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,9,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,9,3,0,1,'20000331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,9,12,0,1,'20000301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,10,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,10,3,0,1,'20000430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,10,12,0,1,'20000401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,11,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,11,3,0,1,'20000531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,11,12,0,1,'20000501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,12,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,12,3,0,1,'20000630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,12,12,0,1,'20000601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,13,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,13,3,0,1,'20000731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,13,12,0,1,'20000701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,14,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,14,3,0,1,'20000831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,14,12,0,1,'20000801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,15,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,15,3,0,1,'20000930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,15,12,0,1,'20000901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,16,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,16,3,0,1,'20001031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,16,12,0,1,'20001001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,17,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,17,3,0,1,'20001130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,17,12,0,1,'20001101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,18,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,18,3,0,1,'20001231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,18,12,0,1,'20001201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,19,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,19,3,0,1,'20011231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,19,12,0,1,'20010101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,20,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,20,3,0,1,'20010331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,20,12,0,1,'20010101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,21,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,21,3,0,1,'20010630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,21,12,0,1,'20010401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,22,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,22,3,0,1,'20010930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,22,12,0,1,'20010701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,23,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,23,3,0,1,'20011231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,23,12,0,1,'20011001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,24,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,24,3,0,1,'20010131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,24,12,0,1,'20010101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,25,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,25,3,0,1,'20010228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,25,12,0,1,'20010201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,26,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,26,3,0,1,'20010331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,26,12,0,1,'20010301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,27,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,27,3,0,1,'20010430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,27,12,0,1,'20010401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,28,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,28,3,0,1,'20010531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,28,12,0,1,'20010501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,29,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,29,3,0,1,'20010630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,29,12,0,1,'20010601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,30,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,30,3,0,1,'20010731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,30,12,0,1,'20010701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,31,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,31,3,0,1,'20010831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,31,12,0,1,'20010801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,32,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,32,3,0,1,'20010930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,32,12,0,1,'20010901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,33,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,33,3,0,1,'20011031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,33,12,0,1,'20011001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,34,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,34,3,0,1,'20011130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,34,12,0,1,'20011101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,35,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,35,3,0,1,'20011231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,35,12,0,1,'20011201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,36,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,36,3,0,1,'20021231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,36,12,0,1,'20020101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,37,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,37,3,0,1,'20020331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,37,12,0,1,'20020101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,38,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,38,3,0,1,'20020630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,38,12,0,1,'20020401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,39,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,39,3,0,1,'20020930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,39,12,0,1,'20020701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,40,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,40,3,0,1,'20021231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,40,12,0,1,'20021001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,41,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,41,3,0,1,'20020131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,41,12,0,1,'20020101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,42,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,42,3,0,1,'20020228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,42,12,0,1,'20020201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,43,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,43,3,0,1,'20020331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,43,12,0,1,'20020301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,44,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,44,3,0,1,'20020430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,44,12,0,1,'20020401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,45,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,45,3,0,1,'20020531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,45,12,0,1,'20020501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,46,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,46,3,0,1,'20020630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,46,12,0,1,'20020601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,47,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,47,3,0,1,'20020731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,47,12,0,1,'20020701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,48,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,48,3,0,1,'20020831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,48,12,0,1,'20020801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,49,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,49,3,0,1,'20020930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,49,12,0,1,'20020901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,50,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,50,3,0,1,'20021031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,50,12,0,1,'20021001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,51,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,51,3,0,1,'20021130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,51,12,0,1,'20021101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,52,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,52,3,0,1,'20021231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,52,12,0,1,'20021201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,53,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,53,3,0,1,'20031231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,53,12,0,1,'20030101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,54,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,54,3,0,1,'20030331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,54,12,0,1,'20030101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,55,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,55,3,0,1,'20030630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,55,12,0,1,'20030401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,56,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,56,3,0,1,'20030930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,56,12,0,1,'20030701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,57,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,57,3,0,1,'20031231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,57,12,0,1,'20031001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,58,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,58,3,0,1,'20030131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,58,12,0,1,'20030101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,59,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,59,3,0,1,'20030228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,59,12,0,1,'20030201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,60,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,60,3,0,1,'20030331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,60,12,0,1,'20030301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,61,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,61,3,0,1,'20030430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,61,12,0,1,'20030401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,62,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,62,3,0,1,'20030531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,62,12,0,1,'20030501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,63,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,63,3,0,1,'20030630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,63,12,0,1,'20030601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,64,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,64,3,0,1,'20030731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,64,12,0,1,'20030701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,65,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,65,3,0,1,'20030831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,65,12,0,1,'20030801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,66,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,66,3,0,1,'20030930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,66,12,0,1,'20030901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,67,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,67,3,0,1,'20031031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,67,12,0,1,'20031001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,68,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,68,3,0,1,'20031130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,68,12,0,1,'20031101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,69,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,69,3,0,1,'20031231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,69,12,0,1,'20031201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,70,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,70,3,0,1,'20041231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,70,12,0,1,'20040101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,71,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,71,3,0,1,'20040331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,71,12,0,1,'20040101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,72,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,72,3,0,1,'20040630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,72,12,0,1,'20040401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,73,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,73,3,0,1,'20040930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,73,12,0,1,'20040701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,74,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,74,3,0,1,'20041231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,74,12,0,1,'20041001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,75,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,75,3,0,1,'20040131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,75,12,0,1,'20040101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,76,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,76,3,0,1,'20040229','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,76,12,0,1,'20040201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,77,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,77,3,0,1,'20040331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,77,12,0,1,'20040301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,78,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,78,3,0,1,'20040430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,78,12,0,1,'20040401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,79,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,79,3,0,1,'20040531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,79,12,0,1,'20040501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,80,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,80,3,0,1,'20040630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,80,12,0,1,'20040601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,81,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,81,3,0,1,'20040731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,81,12,0,1,'20040701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,82,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,82,3,0,1,'20040831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,82,12,0,1,'20040801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,83,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,83,3,0,1,'20040930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,83,12,0,1,'20040901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,84,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,84,3,0,1,'20041031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,84,12,0,1,'20041001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,85,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,85,3,0,1,'20041130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,85,12,0,1,'20041101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,86,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,86,3,0,1,'20041231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,86,12,0,1,'20041201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,87,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,87,3,0,1,'20051231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,87,12,0,1,'20050101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,88,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,88,3,0,1,'20050331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,88,12,0,1,'20050101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,89,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,89,3,0,1,'20050630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,89,12,0,1,'20050401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,90,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,90,3,0,1,'20050930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,90,12,0,1,'20050701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,91,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,91,3,0,1,'20051231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,91,12,0,1,'20051001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,92,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,92,3,0,1,'20050131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,92,12,0,1,'20050101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,93,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,93,3,0,1,'20050228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,93,12,0,1,'20050201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,94,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,94,3,0,1,'20050331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,94,12,0,1,'20050301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,95,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,95,3,0,1,'20050430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,95,12,0,1,'20050401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,96,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,96,3,0,1,'20050531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,96,12,0,1,'20050501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,97,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,97,3,0,1,'20050630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,97,12,0,1,'20050601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,98,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,98,3,0,1,'20050731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,98,12,0,1,'20050701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,99,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,99,3,0,1,'20050831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,99,12,0,1,'20050801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,100,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,100,3,0,1,'20050930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,100,12,0,1,'20050901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,101,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,101,3,0,1,'20051031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,101,12,0,1,'20051001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,102,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,102,3,0,1,'20051130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,102,12,0,1,'20051101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,103,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,103,3,0,1,'20051231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,103,12,0,1,'20051201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,104,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,104,3,0,1,'20061231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,104,12,0,1,'20060101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,105,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,105,3,0,1,'20060331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,105,12,0,1,'20060101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,106,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,106,3,0,1,'20060630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,106,12,0,1,'20060401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,107,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,107,3,0,1,'20060930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,107,12,0,1,'20060701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,108,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,108,3,0,1,'20061231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,108,12,0,1,'20061001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,109,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,109,3,0,1,'20060131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,109,12,0,1,'20060101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,110,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,110,3,0,1,'20060228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,110,12,0,1,'20060201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,111,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,111,3,0,1,'20060331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,111,12,0,1,'20060301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,112,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,112,3,0,1,'20060430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,112,12,0,1,'20060401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,113,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,113,3,0,1,'20060531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,113,12,0,1,'20060501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,114,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,114,3,0,1,'20060630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,114,12,0,1,'20060601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,115,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,115,3,0,1,'20060731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,115,12,0,1,'20060701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,116,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,116,3,0,1,'20060831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,116,12,0,1,'20060801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,117,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,117,3,0,1,'20060930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,117,12,0,1,'20060901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,118,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,118,3,0,1,'20061031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,118,12,0,1,'20061001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,119,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,119,3,0,1,'20061130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,119,12,0,1,'20061101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,120,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,120,3,0,1,'20061231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,120,12,0,1,'20061201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,121,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,121,3,0,1,'20071231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,121,12,0,1,'20070101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,122,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,122,3,0,1,'20070331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,122,12,0,1,'20070101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,123,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,123,3,0,1,'20070630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,123,12,0,1,'20070401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,124,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,124,3,0,1,'20070930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,124,12,0,1,'20070701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,125,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,125,3,0,1,'20071231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,125,12,0,1,'20071001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,126,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,126,3,0,1,'20070131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,126,12,0,1,'20070101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,127,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,127,3,0,1,'20070228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,127,12,0,1,'20070201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,128,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,128,3,0,1,'20070331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,128,12,0,1,'20070301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,129,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,129,3,0,1,'20070430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,129,12,0,1,'20070401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,130,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,130,3,0,1,'20070531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,130,12,0,1,'20070501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,131,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,131,3,0,1,'20070630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,131,12,0,1,'20070601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,132,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,132,3,0,1,'20070731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,132,12,0,1,'20070701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,133,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,133,3,0,1,'20070831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,133,12,0,1,'20070801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,134,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,134,3,0,1,'20070930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,134,12,0,1,'20070901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,135,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,135,3,0,1,'20071031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,135,12,0,1,'20071001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,136,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,136,3,0,1,'20071130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,136,12,0,1,'20071101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,137,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,137,3,0,1,'20071231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,137,12,0,1,'20071201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,138,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,138,3,0,1,'20081231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,138,12,0,1,'20080101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,139,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,139,3,0,1,'20080331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,139,12,0,1,'20080101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,140,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,140,3,0,1,'20080630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,140,12,0,1,'20080401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,141,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,141,3,0,1,'20080930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,141,12,0,1,'20080701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,142,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,142,3,0,1,'20081231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,142,12,0,1,'20081001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,143,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,143,3,0,1,'20080131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,143,12,0,1,'20080101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,144,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,144,3,0,1,'20080228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,144,12,0,1,'20080201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,145,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,145,3,0,1,'20080331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,145,12,0,1,'20080301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,146,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,146,3,0,1,'20080430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,146,12,0,1,'20080401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,147,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,147,3,0,1,'20080531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,147,12,0,1,'20080501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,148,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,148,3,0,1,'20080630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,148,12,0,1,'20080601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,149,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,149,3,0,1,'20080731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,149,12,0,1,'20080701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,150,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,150,3,0,1,'20080831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,150,12,0,1,'20080801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,151,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,151,3,0,1,'20080930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,151,12,0,1,'20080901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,152,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,152,3,0,1,'20081031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,152,12,0,1,'20081001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,153,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,153,3,0,1,'20081130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,153,12,0,1,'20081101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,154,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,154,3,0,1,'20081231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,154,12,0,1,'20081201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,155,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,155,3,0,1,'20091231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,155,12,0,1,'20090101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,156,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,156,3,0,1,'20090331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,156,12,0,1,'20090101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,157,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,157,3,0,1,'20090630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,157,12,0,1,'20090401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,158,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,158,3,0,1,'20090930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,158,12,0,1,'20090701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,159,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,159,3,0,1,'20091231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,159,12,0,1,'20091001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,160,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,160,3,0,1,'20090131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,160,12,0,1,'20090101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,161,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,161,3,0,1,'20090228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,161,12,0,1,'20090201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,162,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,162,3,0,1,'20090331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,162,12,0,1,'20090301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,163,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,163,3,0,1,'20090430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,163,12,0,1,'20090401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,164,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,164,3,0,1,'20090531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,164,12,0,1,'20090501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,165,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,165,3,0,1,'20090630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,165,12,0,1,'20090601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,166,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,166,3,0,1,'20090731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,166,12,0,1,'20090701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,167,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,167,3,0,1,'20090831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,167,12,0,1,'20090801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,168,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,168,3,0,1,'20090930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,168,12,0,1,'20090901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,169,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,169,3,0,1,'20091031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,169,12,0,1,'20091001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,170,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,170,3,0,1,'20091130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,170,12,0,1,'20091101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,171,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,171,3,0,1,'20091231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,171,12,0,1,'20091201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,172,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,172,3,0,1,'20101231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,172,12,0,1,'20100101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,173,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,173,3,0,1,'20100331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,173,12,0,1,'20100101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,174,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,174,3,0,1,'20100630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,174,12,0,1,'20100401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,175,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,175,3,0,1,'20100930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,175,12,0,1,'20100701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,176,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,176,3,0,1,'20101231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,176,12,0,1,'20101001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,177,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,177,3,0,1,'20100131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,177,12,0,1,'20100101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,178,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,178,3,0,1,'20100228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,178,12,0,1,'20100201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,179,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,179,3,0,1,'20100331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,179,12,0,1,'20100301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,180,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,180,3,0,1,'20100430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,180,12,0,1,'20100401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,181,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,181,3,0,1,'20100531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,181,12,0,1,'20100501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,182,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,182,3,0,1,'20100630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,182,12,0,1,'20100601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,183,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,183,3,0,1,'20100731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,183,12,0,1,'20100701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,184,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,184,3,0,1,'20100831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,184,12,0,1,'20100801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,185,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,185,3,0,1,'20100930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,185,12,0,1,'20100901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,186,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,186,3,0,1,'20101031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,186,12,0,1,'20101001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,187,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,187,3,0,1,'20101130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,187,12,0,1,'20101101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,188,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,188,3,0,1,'20101231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,188,12,0,1,'20101201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,189,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,189,3,0,1,'20111231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,189,12,0,1,'20110101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,190,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,190,3,0,1,'20110331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,190,12,0,1,'20110101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,191,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,191,3,0,1,'20110630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,191,12,0,1,'20110401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,192,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,192,3,0,1,'20110930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,192,12,0,1,'20110701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,193,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,193,3,0,1,'20111231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,193,12,0,1,'20111001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,194,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,194,3,0,1,'20110131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,194,12,0,1,'20110101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,195,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,195,3,0,1,'20110228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,195,12,0,1,'20110201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,196,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,196,3,0,1,'20110331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,196,12,0,1,'20110301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,197,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,197,3,0,1,'20110430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,197,12,0,1,'20110401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,198,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,198,3,0,1,'20110531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,198,12,0,1,'20110501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,199,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,199,3,0,1,'20110630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,199,12,0,1,'20110601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,200,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,200,3,0,1,'20110731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,200,12,0,1,'20110701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,201,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,201,3,0,1,'20110831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,201,12,0,1,'20110801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,202,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,202,3,0,1,'20110930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,202,12,0,1,'20110901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,203,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,203,3,0,1,'20111031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,203,12,0,1,'20111001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,204,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,204,3,0,1,'20111130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,204,12,0,1,'20111101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,205,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,205,3,0,1,'20111231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,205,12,0,1,'20111201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,206,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,206,3,0,1,'20121231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,206,12,0,1,'20120101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,207,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,207,3,0,1,'20120331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,207,12,0,1,'20120101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,208,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,208,3,0,1,'20120630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,208,12,0,1,'20120401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,209,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,209,3,0,1,'20120930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,209,12,0,1,'20120701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,210,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,210,3,0,1,'20121231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,210,12,0,1,'20121001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,211,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,211,3,0,1,'20120131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,211,12,0,1,'20120101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,212,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,212,3,0,1,'20120229','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,212,12,0,1,'20120201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,213,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,213,3,0,1,'20120331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,213,12,0,1,'20120301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,214,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,214,3,0,1,'20120430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,214,12,0,1,'20120401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,215,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,215,3,0,1,'20120531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,215,12,0,1,'20120501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,216,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,216,3,0,1,'20120630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,216,12,0,1,'20120601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,217,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,217,3,0,1,'20120731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,217,12,0,1,'20120701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,218,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,218,3,0,1,'20120831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,218,12,0,1,'20120801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,219,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,219,3,0,1,'20120930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,219,12,0,1,'20120901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,220,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,220,3,0,1,'20121031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,220,12,0,1,'20121001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,221,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,221,3,0,1,'20121130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,221,12,0,1,'20121101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,222,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,222,3,0,1,'20121231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,222,12,0,1,'20121201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,223,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,223,3,0,1,'20131231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,223,12,0,1,'20130101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,224,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,224,3,0,1,'20130331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,224,12,0,1,'20130101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,225,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,225,3,0,1,'20130630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,225,12,0,1,'20130401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,226,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,226,3,0,1,'20130930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,226,12,0,1,'20130701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,227,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,227,3,0,1,'20131231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,227,12,0,1,'20131001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,228,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,228,3,0,1,'20130131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,228,12,0,1,'20130101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,229,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,229,3,0,1,'20130228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,229,12,0,1,'20130201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,230,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,230,3,0,1,'20130331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,230,12,0,1,'20130301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,231,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,231,3,0,1,'20130430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,231,12,0,1,'20130401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,232,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,232,3,0,1,'20130531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,232,12,0,1,'20130501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,233,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,233,3,0,1,'20130630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,233,12,0,1,'20130601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,234,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,234,3,0,1,'20130731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,234,12,0,1,'20130701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,235,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,235,3,0,1,'20130831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,235,12,0,1,'20130801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,236,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,236,3,0,1,'20130930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,236,12,0,1,'20130901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,237,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,237,3,0,1,'20131031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,237,12,0,1,'20131001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,238,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,238,3,0,1,'20131130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,238,12,0,1,'20131101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,239,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,239,3,0,1,'20131231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,239,12,0,1,'20131201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,240,8,0,1,'Year','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,240,3,0,1,'20141231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,240,12,0,1,'20140101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,241,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,241,3,0,1,'20140331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,241,12,0,1,'20140101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,242,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,242,3,0,1,'20140630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,242,12,0,1,'20140401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,243,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,243,3,0,1,'20140930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,243,12,0,1,'20140701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,244,8,0,1,'QuarterYear','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,244,3,0,1,'20141231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,244,12,0,1,'20141001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,245,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,245,3,0,1,'20140131','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,245,12,0,1,'20140101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,246,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,246,3,0,1,'20140228','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,246,12,0,1,'20140201','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,247,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,247,3,0,1,'20140331','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,247,12,0,1,'20140301','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,248,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,248,3,0,1,'20140430','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,248,12,0,1,'20140401','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,249,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,249,3,0,1,'20140531','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,249,12,0,1,'20140501','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,250,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,250,3,0,1,'20140630','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,250,12,0,1,'20140601','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,251,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,251,3,0,1,'20140731','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,251,12,0,1,'20140701','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,252,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,252,3,0,1,'20140831','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,252,12,0,1,'20140801','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,253,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,253,3,0,1,'20140930','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,253,12,0,1,'20140901','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,254,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,254,3,0,1,'20141031','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,254,12,0,1,'20141001','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,255,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,255,3,0,1,'20141130','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,255,12,0,1,'20141101','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,256,8,0,1,'Month','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,256,3,0,1,'20141231','1970-01-01 00:00:00','5999-01-01 00:00:00'),
                    (-1,256,12,0,1,'20141201','1970-01-01 00:00:00','5999-01-01 00:00:00')
            ) by PASSTHRU ;
     disconnect from PASSTHRU ;
   quit ;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
/*                       Update SAS_KEYGEN table                              */
/******************************************************************************/
%sesskey(keyCode=%str(odcs/dimension), keyNext=5) ;
%sesskey(keyCode=%str(odcs/dimension_type), keyNext=13) ;
%sesskey(keyCode=%str(odcs/hierarchy), keyNext=4) ;
%sesskey(keyCode=%str(odcs/locale), keyNext=2) ;
%sesskey(keyCode=%str(odcs/member), keyNext=500) ;
%sesskey(keyCode=%str(odcs/property), keyNext=18) ;
%sesskey(keyCode=%str(odcs/server), keyNext=1) ;

%impsourc;
%checkErr;
%if &_tmpRC %then
  %return ;

%impxtype;
%checkErr;
%if &_tmpRC %then
  %return ;

%impfreq;
%checkErr;
%if &_tmpRC %then
  %return ;

/******************************************************************************/
/*   Remove indication of failure from installation record in the             */
/*   SAS_VERSION_DATA table in SASSDM and DDS_SOLUTIONS_VERSION table in DDS  */
/******************************************************************************/

 %sesssver(solName=ODCS, solVer=&sesetl_version, solStatus=Successful, solMethod=Installation);

/******************************************************************************/

%mend regddlo;
