%MACRO sesfms;

/**********************************************************
 * sesFMS_addSourceCode.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_addSourceCode.sas                                                       */
/*  Purpose: Adds a column of type String with a specific Code value.                        */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_addSourceCode(cDS, cColNm, cSourceCode);

  %let cMethodNm = sesFMS_addSourceCode;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Make sure we have a source data set  */
  %if not %sysfunc(exist(&cDS)) %then %do;
    %let error_number=27;
    %let errObj=&cDS;
    %goto ERRORRETURN;
  %end ;

  /*  Add the column to the data set   */
  data &cDS ;
       length &cColNm $ %eval(32*&FM_DBCSMULT) ;
       set &cDS ;
       &cColNm = "&cSourceCode" ;
       if TRANSACTION_AMT not in(.) ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_addSourceCode;


/**********************************************************
 * sesFMS_appendDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_appendDS.sas                                                            */
/*  Purpose: Appends non-derived Period activity trans. to newly derived Period activity     */
/*           trans.                                                                          */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_appendDS(cBaseDS, cDataDS);
  %let cMethodNm = sesFMS_appendDS;
  %let  nTrackTime = %qsysfunc(datetime()) ;

  /*  Make sure we have both data sets  */
  %if not %sysfunc(exist(&cBaseDS, DATA)) %then %do;
    %let error_number=22;
    %let errObj=&cBaseDS;
    %goto ERRORRETURN;
  %end ;

  %if not %sysfunc(exist(&cDataDS, DATA)) %then %do;
    %let error_number=23;
    %let errObj=&cDataDS;
    %goto ERRORRETURN;
  %end ;

  /*  Run the Append  */
  proc append base=&cBaseDS data=&cDataDS FORCE ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_appendDS;


/**********************************************************
 * sesFMS_buildStatusDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_buildStatusDS.sas                                                       */
/*  Purpose: Build a temporary data set to hold status of this import.                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_buildStatusDS( cTimeDS, cAnalysisDS, cStatusDS, statusOverride);

  %let cMethodNm = sesFMS_buildStatusDS;
  %let cTempStatusDS = WORK._FMSETLTEMP01;
  %let nowDTTM = %qsysfunc(datetime()) ;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if %sysfunc(exist(&cStatusDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cStatusDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cTempStatusDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempStatusDS, 2, '.');
    quit;
  %end;

  data &cTempStatusDS(keep=TIME_ID ANALYSIS_ID PERIOD_ID STATUS_ID
                           IMPORTED_TRANS_CNT ZERO_TRANS_CNT INVALID_ERROR_CNT
                           ORG_EQ_TRADER_CNT IC_ERROR_CNT NON_IC_ERROR_CNT REMOVE_CNT) ;
     length TIME_ID 8;
     length ANALYSIS_ID 8;
     length PERIOD_ID 8;
     TIME_ID = -999999;
     ANALYSIS_ID = -999999;
     PERIOD_ID = -999999;
     STATUS_ID = &statusOverride;
     IMPORTED_TRANS_CNT = 0 ;
     ZERO_TRANS_CNT = 0 ;
     INVALID_ERROR_CNT = 0 ;
     ORG_EQ_TRADER_CNT = 0 ;
     IC_ERROR_CNT = 0 ;
     NON_IC_ERROR_CNT = 0 ;
     REMOVE_CNT = 0 ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc append base=&cStatusDS data=&cTempStatusDS &FM_PROCAPPENDFORCE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_buildStatusDS;


/**********************************************************
 * sesFMS_calcPeriodActivity.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_calcPeriodActivity.sas                                                  */
/*  Purpose: Calculate period activity transaction values.                                   */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_calcPeriodActivity(cCurrentPrdDS, cPriorPrdDS, lTranCols, cCurrentAmtColNm, cPriorAmtColNm, cOutputDS, cPrdIdColNm);
  %let cMethodNm = sesFMS_calcPeriodActivity;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction tables in Stage  */
  %if not %sysfunc(exist(&cCurrentPrdDS, DATA))  %then %do;
    %let error_number=19;
    %let errObj=&cCurrentPrdDS;
    %goto ERRORRETURN;
  %end ;
  %if not %sysfunc(exist(&cPriorPrdDS, DATA))  %then %do;
    %let error_number=20;
    %let errObj=&cPriorPrdDS;
    %goto ERRORRETURN;
  %end ;

  /*  Build the column merge by list  */
  filename _col temp;
  data _null_;
      file _col;
    set &lTranCols end=eof;
    put '"' value + (-1) '"n ';
      if eof then put ';';
  run;

  /*  Do the math and build a resulting converted data set  */
  proc sort in=&cCurrentPrdDS out=&cCurrentPrdDS;
    by %inc _col;
  run;

  %if "&cProduct" ne "OP" %then %do;
    %let _col=AFFECTED_TIME_PERIOD_ID;
  %end;
  %else %do;
    %let _col=TIME_PERIOD_ID;
  %end;

  data &cOutputDS (drop=&cCurrentAmtColNm &cPriorAmtColNm &cPrdIdColNm) ;
     length TRANSACTION_DT 8 ;
     merge &cCurrentPrdDS (in=nCurrentObs) &cPriorPrdDS(drop=&_col) ;
       by %inc _col ;
     if nCurrentObs then do;
        if &cPriorAmtColNm = . then &cPriorAmtColNm = 0 ;
        TRANSACTION_AMT = &cCurrentAmtColNm - &cPriorAmtColNm ;
        TRANSACTION_DT  = . ;
        if TRANSACTION_AMT not in(.) then output ;
     end ;
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 21, &cMethodNm) ;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_calcPeriodActivity;

/**********************************************************
 * sesFMS_createDimTypeFactSetOrd.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_createDimTypeFactSetOrd.sas                                             */
/* Create Dim Type List in Fact Set Order.                                                   */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_createDimTypeFactSetOrd(cWorkAreaDS, lDim);
  %let cMethodNm = sesFMS_createDimTypeFactSetOrd;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Turn(copy) the table into a named table  */
  %if (%sysfunc(exist(&cWorkAreaDS))) %then %do;
    data &lDim;
      set &cWorkAreaDS;
      run;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_createDimTypeFactSetOrd;


/**********************************************************
 * sesFMS_createErrorReport.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : YTDConv_createErrorReport.sas                                                   */
/*  Purpose: Print error report to Dav.                                                      */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_createErrorReport(lCycleDimTypes,
                                  cInvalidDS,
                                  nInvalidCnt,
                                  cOrgEqTraderDS,
                                  nOrgEqTraderCnt,
                                  cIcErrDS,
                                  nIcErrCnt,
                                  cNonIcErrDS,
                                  nNonIcErrCnt,
                                  cDataLoadId);

  %let cMethodNm = sesFMS.createErrorReport;
  %let cInvalidTitle = %sesSOL_getMessage( YTDConv, R, 1, "", "", "", "", "") ;
  %let cInvalidTitle2 = %sesSOL_getMessage( YTDConv, R, 9, "", "", "", "", "") ;

  %if "&cProduct" ne "OP" %then
    %let cInvalidTitle3 = %sesSOL_getMessage( YTDConv, R, 10, "", "", "", "", "") ;
  %else
    %let cInvalidTitle3 = %sesSOL_getMessage( YTDConv, R, 12, "", "", "", "", "") ;

  %let cInvalidTitle4 = %sesSOL_getMessage( YTDConv, R, 11, "", "", "", "", "") ;
  %let cICErrorTitle = %sesSOL_getMessage( YTDConv, R, 2, "", "", "", "", "") ;
  %let cNonICErrorTitle = %sesSOL_getMessage( YTDConv, R, 3, "", "", "", "", "") ;
  %let cOrgEqTraderTitle = %sesSOL_getMessage( YTDConv, R, 4, "", "", "", "", "") ;
  %let cValueFormat = %sesSOL_getMessage( YTDConv, R, 5, "", "", "", "", "") ;
  %let cDateFormat = %sesSOL_getMessage( YTDConv, R, 6, "", "", "", "", "") ;
  %let cTimeFormat = %sesSOL_getMessage( YTDConv, R, 7, "", "", "", "", "") ;
  %let displayMaxObs = 1000 ; /* Max display rows in error report */
  %let nTrackTime = %sysfunc(datetime());

  /*  Set HTML files names  */
  /* %let cDataLoadId  =&fmsetl_tempCheck1; */

  %let cIndexHTML   = index_&cDataLoadId..html;
  %let cBodyHTML    = body_&cDataLoadId..html;
  %let cContentHTML = contents_&cDataLoadId..html;

  /*  Source dim type is not part of the list, but we need it for this operation   */

  /*  Clean up  first */
  %if %sysfunc(exist(lDimTypes)) %then  %do;
    proc datasets library=work nolist;
      delete lDimTypes;
    quit;
  %end;

  /* create lDimTypes */
  proc sql noprint;
    create table lDimTypes as
    select *
      from &lCycleDimTypes;
  quit;

  footnote %sysfunc(today(), &cDateFormat), %sysfunc(time(), &cTimeFormat) ;
  ods html file="&cBodyHTML" frame="&cIndexHTML" contents="&cContentHTML" ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*  Create Invalid Transactions Report  */
  /*--------------------------------------*/
  %let displayObs = &nInvalidCnt;
  %let cSubTitle = %sesSOL_getMessage(YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;

  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle - %sesSOL_getMessage( YTDConv, R, 9,
                                        %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cInvalidTitle";
  title2 "&cInvalidTitle2";
  title3 "&cInvalidTitle3";
  title4 "&cInvalidTitle4";
  ods proclabel="&cInvalidTitle";


  filename _col_st1 temp;
  data _null_;
      file _col_st1;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_ERROR";
    put '"' code +(-1) '"n ' ;
  run;

  data _null_;
    file _col_st1 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    put '"' code +(-1) '"n ' ;
    if eof then
      put ' MEASURE;';
  run;

  filename _col_st3 temp;
  data _null_;
    file _col_st3;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_ERROR";
      put 'define "' code +(-1) '"n / noprint; ' ;
  run;

  data _null_;
    file _col_st3 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    cDimTypeLabel = strip(Propcase(code)) ;
    put 'define "' code +(-1) '"n / display "' cDimTypeLabel +(-1) '";';
    if eof then
      put 'define MEASURE / display "Value" format=&cValueFormat;';
  run;

  filename _col_st5 temp;
  data _null_;
    file _col_st5;
    set lDimTypes end=eof;
    code1 = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    code2 = strip(putc(code, '$DTPCOL.'));
    put 'compute "' code1 +(-1) '"n;';
    put 'if _C' _N_ +(-1) '_=1 then do;';
    put 'call define(_COL_, "STYLE", "style(CALLDEF)={background=#f0f0f0 foreground=red font_weight=bold}" );';
    put 'end;';
    put 'endcomp;';
  run;

  proc report data=&cInvalidDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
    column
      %inc _col_st1;
      %inc _col_st3;
      %inc _col_st5;
      ;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*    Create Org=Trader Error Report    */
  /*--------------------------------------*/
  filename _col_st2 temp;
  data _null_;
    file _col_st2 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    put '"' code +(-1) '"n ' ;
    if eof then
      put ' MEASURE;';
  run;

  filename _col_st4 temp;
  data _null_;
    file _col_st4 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    cDimTypeLabel = strip(Propcase(code)) ;
    put 'define "' code +(-1) '"n / display "' cDimTypeLabel +(-1) '";';
    if eof then
      put 'define MEASURE / display "Value" format=&cValueFormat;';
  run;

  %let displayObs = &nOrgEqTraderCnt ;
  %let cSubTitle = %sesSOL_getMessage( YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;
  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle - %sesSOL_getMessage( YTDConv, R, 9,
                                          %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cOrgEqTraderTitle";
  ods proclabel="&cOrgEqTraderTitle";
  proc report data=&cOrgEqTraderDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
     column
       %inc _col_st2;
       %inc _col_st4;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*        Create IC Error Report        */
  /*--------------------------------------*/
  %let displayObs = &nIcErrCnt ;
  %let cSubTitle = %sesSOL_getMessage( YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;
  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle %sesSOL_getMessage( YTDConv, R, 9,
                                          %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cICErrorTitle";
  ods proclabel="&cICErrorTitle";

  proc report data=&cIcErrDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
     column
       %inc _col_st2;
       %inc _col_st4;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*      Create Non IC Error Report      */
  /*--------------------------------------*/
  %let displayObs = &nNonIcErrCnt ;
  %let cSubTitle = %sesSOL_getMessage( YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;
  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle  -  %sesSOL_getMessage( YTDConv, R, 9,
                                          %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cNonICErrorTitle";
  ods proclabel="&cNonICErrorTitle";
  proc report data=&cNonIcErrDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
     column
       %inc _col_st2;
       %inc _col_st4;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_createErrorReport;


/**********************************************************
 * sesFMS_createFormat.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_createFormat.sas                                                        */
/*  Purpose: Create a SAS Format from work area data set and return a named list in a table  */
/*           It sets 1 to global _FMErrorCode, if succeeds. Otherwise, 0 and jump to the end.*/
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_createFormat(cWorkAreaDS,cFormatNm,codeIsNum,lDim,nBuildList);

  %let _FMErrorCode=1;  /* it is a global, should be defined in the parent programs
                           0 : error, 1: successful for all processes in this macro */
  %let cMethodNm = sesFMS_createFormat;
  %let cFormatDS = _dsformatYTD;
  %let nZeroObs = 0;
  %let nCodeFld=0;
  %let nValueFld=0;
  %let nTrackTime=0;
  %let dsID=0;
  %let nRC=0;
  %let returnNum=0;
  %let nTrackTime = %sysfunc(datetime());
  %let zero=0;

  %if %sysfunc(exist(&cWorkAreaDS)) %then %do;
    %let dsID=%sysfunc(open(&cWorkAreaDS));
    %if %sysfunc(attrn(&dsID, NLOBS)) EQ 0 %then
      %let nZeroObs = 1;
    %if &dsID %then
      %let dsID=%sysfunc(close(&dsID));  /* 0 if successuful */
  %end;
  %else %do;
    %let error_number=4;
    %let errObj=&cWorkAreaDS;
    %goto ERRORRETURN;
  %end;

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cFormatDS)) %then  %do;
    proc datasets library=work nolist;
      delete &cFormatDS;
    run;
  %end;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;

  /* Build format */
  /*----------------------------------------*/
  /*----------------------------------------*/
  /* Check if cWorkAreaDS has FMTNAME field */
  /*----------------------------------------*/
  /*----------------------------------------*/

  proc contents data=&cWorkAreaDS out=_content_(keep=name) noprint;
  run;

  proc sql noprint;
      select count(*)
        into :moreThan2fmt
        from _content_
       where name="FMTNAME";;

       drop table _content_;
  quit;

  %let moreThan2fmt=%eval(&moreThan2fmt);
  %if &moreThan2fmt > 0 %then %do;
    %let fmtname_stmt = FMTNAME=FMTNAME;
    %let _dup_option =;
    %let _sort_key = FMTNAME CODE;
  %end;
  %else %do;
    %let fmtname_stmt = FMTNAME="&cFormatNm";
    %let _dup_option = nodupkey;
    %let _sort_key = CODE;
  %end;

  proc sort data=&cWorkAreaDS out=&cFormatDS &_dup_option;
          by &_sort_key;
  run;

  %if (&SYSERR NE 0) AND (&SYSERR NE 4) %then %do;
        %let error_number=5;
      %let errObj=&cFormatNm;
        %goto ERRORRETURN;
  %end;

  /*  Turn(copy) the table into a named table  */
  %if &nBuildList %then %do;

    %let _var=&lDim;
    %let _wrong=%sysfunc( kcompress( %sysfunc( ksubstr(&_var, 1, 1))
                      , '_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'));
    %let _wrongrest=%sysfunc( kcompress( %sysfunc( ksubstr(&_var, 2))
                      , '0123456789_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'));

    %put _wrong=&_wrong;
    %put _wrongrest=&_wrongrest;
    %if %nrbquote(&_wrong) = %str() and %nrbquote(&_wrongrest) = %str() %then %do;
      data &lDim;
      set &cFormatDS;
      run;
    %end;
  %end;

  %if &codeIsNum %then %do;
    %if &nZeroObs %then %do;
      data &cFormatDS(keep=START LABEL FMTNAME HLO);
              length START 8;
              length LABEL $1;
              length FMTNAME $32;
              length HLO $1;
              keep START LABEL FMTNAME HLO;
              &fmtname_stmt;
              START   = .;
              LABEL   = "*";
              HLO     = "O";
              output;
      run;
    %end;
    %else %do;
      data &cFormatDS(keep=START LABEL FMTNAME HLO);
        set &cFormatDS(rename=(CODE=START VALUE=LABEL)) end=nEnd;
        length FMTNAME $32;
        length HLO $1;
        &fmtname_stmt;
        HLO     = "";
        output;
        if nEnd then do;
          START   = .;
          LABEL   = "*";
          HLO     = "O";
          output;
        end;
      run;
    %end;
  %end;
  %else %do;
    %if &nZeroObs %then %do;
      data &cFormatDS(keep=START LABEL FMTNAME HLO);
              length START $1;
              length LABEL $1;
              length FMTNAME $32;
              length HLO $1;
              &fmtname_stmt;
              START   = "";
              LABEL   = "*";
              HLO     = "O";
              output;
      run;
    %end;
    %else %do;
      data &cFormatDS(keep=START LABEL FMTNAME HLO);
              set &cFormatDS(rename=(CODE=START VALUE=LABEL)) end=nEnd;
              length FMTNAME $32;
              length HLO $1;
              &fmtname_stmt;
              HLO     = "";
              output;
              if nEnd then do;
                START   = "";
                LABEL   = "*";
                HLO     = "O";
                output;
                end;
            run;
    %end;
  %end;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;

  proc format cntlin=&cFormatDS library=WORK;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %let _FMErrorCode=0;
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %THEEND:

%mend sesFMS_createFormat;

/**********************************************************
 * sesFMS_generateErrorReports.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_generateErrorReports.sas                                                */
/*  Purpose: Print error report to Dav.                                                      */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_generateErrorReports( cStatusDS,
                                  lCycleDimTypes,
                                  cInvalidDS,
                                  cOrgEqTraderDS,
                                  cIcErrDS,
                                  cNonIcErrDS,
                                  cDataLoadId);

  %let cMethodNm = sesFMS_generateErrorReports;
  /* %let nowDTTM = %sysfunc(datetime()); */
  %let nTrackTime = %sysfunc(datetime());

  /*  Prep and submit the STPBegin macro  */

  %global _odsstyle _ODSOPTIONS rptDate rptTime ;
  %let _odsstyle=SASWeb;


  %STPBEGIN;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %let dsID = %sysfunc(open( &cStatusDS ));
  %if &dsID %then %do;
    %let nStatusFld      = %sysfunc(varnum(&dsID, STATUS_ID)) ;
    %let nInvalidFld     = %sysfunc(varnum(&dsID, INVALID_ERROR_CNT));
    %let nOrgEqTraderFld = %sysfunc(varnum(&dsID, ORG_EQ_TRADER_CNT));
    %let nIcErrFld       = %sysfunc(varnum(&dsID, IC_ERROR_CNT));
    %let nNonIcErrFld    = %sysfunc(varnum(&dsID, NON_IC_ERROR_CNT));
    %let nInvalidCnt     = 0;
    %let nOrgEqTraderCnt = 0;
    %let nIcErrCnt       = 0;
    %let nNonIcErrCnt    = 0;

    %let nRC = %sysfunc(rewind(&dsID));
    %do %while(%sysfunc(fetch(&dsID)) EQ 0);  /*(0, %sysrc(_SWNOUPD), %sysrc(_SENOLCK)));  */
      %if %sysfunc(getvarn(&dsID, &nStatusFld)) ne . %then %do;
        %let nStatusId = %sysfunc(getvarn(&dsID, &nStatusFld));
      %end;

      %if %sysfunc(getvarn(&dsID, &nInvalidFld)) ne . %then %do;
        %let nInvalidCnt = %eval(&nInvalidCnt + %sysfunc(getvarn(&dsID, &nInvalidFld))) ;
      %end;
      %if %sysfunc(getvarn(&dsID, &nOrgEqTraderFld)) ne . %then %do;
        %let nOrgEqTraderCnt = %eval(&nOrgEqTraderCnt + %sysfunc(getvarn(&dsID, &nOrgEqTraderFld))) ;
      %end;
      %if %sysfunc(getvarn(&dsID, &nIcErrFld)) ne . %then %do;
        %let nIcErrCnt = %eval(&nIcErrCnt + %sysfunc(getvarn(&dsID, &nIcErrFld))) ;
      %end;
      %if %sysfunc(getvarn(&dsID, &nNonIcErrFld)) ne . %then %do;
        %let nNonIcErrCnt = %eval(&nNonIcErrCnt + %sysfunc(getvarn(&dsID, &nNonIcErrFld))) ;
      %end;

    %end;

    %if &nStatusID eq 2 %then %do ;
      %sesFMS_createErrorReport( lCycleDimTypes, &cInvalidDS, &nInvalidCnt,
                                      &cOrgEqTraderDS, &nOrgEqTraderCnt, &cIcErrDS, &nIcErrCnt,
                                      &cNonIcErrDS, &nNonIcErrCnt, &cDataLoadId );

      %if &_FMErrorCode EQ 0 %then %do;
        %if &dsID %then %do;
           %let dsID = %sysfunc(close(&dsID));
           %goto RETURNZERO;
        %end ;
      %end ;
    %end;

    %if &dsID %then
      %let dsID = %sysfunc(close(&dsID));
  %end ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;
  %THEEND:

  /*  End the report output  */
  ods html close ;
  %STPEND;

%mend sesFMS_generateErrorReports;


/**********************************************************
 * sesFMS_getAccountTypes.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getAccountTypes.sas                                                     */
/*  Purpose: Build a temporary data set of account types.                                    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getAccountTypes(cAccountDS, cSASLib, nAccountDimId, nAcctTypePropertyId, nNowDTTM);

  %let cMethodNm = sesFMS_getAccountTypes;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up  */
  %if %sysfunc(exist(&cAccountDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cAccountDS, 2, '.');
    quit;
  %end;

  /*  Get a list of accounts  */
  proc sql noprint ;
    create table &cAccountDS as
        select distinct SAS_MEMBER.MEMBER_CD as CODE, SAS_MEMBER_PROPERTY.PROPERTY_VALUE as VALUE , SAS_MEMBER_PROPERTY.PROPERTY_ID
            from &cSASlib..SAS_MEMBER, &cSASlib..SAS_DIMENSION, &cSASlib..SAS_MEMBER_PROPERTY
           where SAS_MEMBER.INTERNAL_DIMENSION_ID=SAS_DIMENSION.INTERNAL_DIMENSION_ID and
                 SAS_MEMBER.MEMBER_ID=SAS_MEMBER_PROPERTY.MEMBER_ID  and
           SAS_DIMENSION.DIMENSION_ID=&nAccountDimId and
                 SAS_MEMBER_PROPERTY.PROPERTY_ID= &nAcctTypePropertyId and
                 (SAS_MEMBER.VALID_FROM_DTTM < &nNowDTTM and
                 SAS_MEMBER.VALID_TO_DTTM > &nNowDTTM) and
                 (SAS_MEMBER_PROPERTY.VALID_FROM_DTTM < &nNowDTTM and
                 SAS_MEMBER_PROPERTY.VALID_TO_DTTM > &nNowDTTM) ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cAccountDS ;
       by code ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getAccountTypes;


/**********************************************************
 * sesFMS_validateAnalysisMembers.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Purpose: Validate that the analysis members exist in the stage area.                     */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/*             10NOV2009 by Bill Jones:                                                      */
/*              - Renamed from sesFMS_getAnalysisDDSRKs to sesFMS_validateAnalysisMembers.   */
/*              - Since facts are now getting loaded from the staging area, rather than the  */
/*                DDS, the retrieval of DDS RKs is now obsolete.                             */
/*              - Changed analysis member validation from DDS to staging area.               */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_validateAnalysisMembers(cAnalysisCodesDS, cStageLib);

  %let cMethodNm = sesFMS_validateAnalysisMembers ;
  %let cTempDS1 = work._fmsetlTemp01 ;
  %let cTempDS2 = work._fmsetlTemp02 ;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up  */

  %if %sysfunc(exist(&cTempDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS1, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cTempDS2)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS2, 2, '.');
    quit;
  %end;

  proc sort in=&cAnalysisCodesDS ;
           by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint ;
    create table &cTempDS1 as
    select distinct ANALYSIS_ID as CODE
        from &cStageLib..ANALYSIS
     order by CODE;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  data &cTempDS2(keep=CODE) ;
     merge &cAnalysisCodesDS(in=a) &cTempDS1(in=b) ;
     by CODE ;
     if a and not b then do;
     /*  Error, mbr code exists in SDM, but not the staging area  */
       output &cTempDS2;
     end;
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Check if any errors where found  */
  %if %sysfunc(exist(&cTempDS2)) %then %do;
    proc sql noprint;
      select count(*)
        into :numrow
        from &cTempDS2;
        quit;
    %let numrow=%eval(&numrow);
    %if &numrow > 0 %then %do;
      proc sql noprint;
        select  code
          into :cCode1 - :cCode&numrow
          from &cTempDS2;
      quit;
      %do ii=1 %to &numrow;
        %sesSOL_logError(YTDConv, L, 54, &&cCode&ii, &cMethodNm);
      %end;
      /* %goto RETURNZERO; Change to WARNING: S0542829 */
    %end;
  %end;
  %else %do;
    %let error_number=52;
    %let errObj=&cTempDS2;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_validateAnalysisMembers;

/**********************************************************
 * sesFMS_getStageDimensionTypes.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getStageDimensionTypes.sas                                                */
/*  Purpose: Get active records from DDS Dimension_type table and load into work area        */
/*           data set.                                                                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getStageDimensionTypes(cStageLibRef, cStageTableNm, cWorkAreaDS, cDimTypeTableNmDS,
                                     cDimTypeKeyColumnNmDS, cBFColNm);

  %let cMethodNm = sesFMS_getStageDimensionTypes;
  %let cStageLibRef = %qsysfunc(strip(&cStageLibRef));
  %let cStageTableNm = %qsysfunc(strip(&cStageTableNm));
  %let cStageDimTypeDS =&cStageLibRef..&cStageTableNm;
  %if &cBFColNm eq %str() %then %do;
    %let cBFColNm = BASE_FACT_COLUMN_NM;
  %end;

  %let cKeyColNm = KEY_COLUMN_NM ;
  %let nTrackTime = %sysfunc(datetime());

  /*  Make sure DDS Dimension Type table exists  */
  %if %sysfunc(exist(&cStageDimTypeDS)) EQ 0 %then %do;
      %let error_number=6;
      %let errObj=&cStageDimTypeDS;
        %goto ERRORRETURN;
  %end ;

  /*  Get a list of the active records and subset down to 2 columns of information (code, value)  */
  data &cWorkAreaDS (keep=CODE VALUE);
*     length CODE $64 ;
*     length VALUE $64 ;
     set &cStageDimTypeDS
            (where=((VALID_TO_DTTM > datetime() or VALID_TO_DTTM = .)
                and &cBFColNm NE "" and &cKeyColNm NE "" ));

     CODE = DIMENSION_TYPE_CD;
     VALUE = &cBFColNm;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %let error_number=7;
      %let errObj=&cStageDimTypeDS;
        %goto ERRORRETURN;
  %end ;

  /*
   *  Because the Stage model supports multiple languages for name/description, we could have
   *  duplicate dimension_type_cd values.  Run a proc sort nodupkey to get rid of the dupes.
   */
  proc sort in=&cWorkAreaDS out=&cWorkAreaDS nodupkey ;
          by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %sesSOL_logError( YTDConv, L, 8,  &cMethodNm);
      %goto RETURNZERO;
  %end ;

  /*
   *  Build a table of Dimension_type_cd to table_nm
   */

  proc sql noprint ;
     create table &cDimTypeTableNmDS as
     select distinct DIMENSION_TYPE_CD as CODE, TABLE_NM as VALUE
        from &cStageDimTypeDS
        where &cBFColNm NE "" and KEY_COLUMN_NM NE "" and
              (VALID_FROM_DTTM < &fmsetl_processedDTTM_To) and
              (VALID_TO_DTTM > &fmsetl_processedDTTM_To or VALID_TO_DTTM = .);
  quit ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %let error_number=9;
      %let errObj=&cDimTypeTableNmDS;
        %goto ERRORRETURN;
  %end;

  /*
   *  Build a table of Dimension_type_cd to Key_Column_Nm
   */

  proc sql noprint ;
    create table &cDimTypeKeyColumnNmDS as
              select distinct DIMENSION_TYPE_CD as CODE, KEY_COLUMN_NM as VALUE
                from &cStageDimTypeDS
                where &cBFColNm NE "" and KEY_COLUMN_NM NE ""
                      and (VALID_FROM_DTTM < &fmsetl_processedDTTM_To
                           and VALID_TO_DTTM > &fmsetl_processedDTTM_To);
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %let error_number=9;
      %let errObj=&cDimTypeKeyColumnNmDS;
        %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getStageDimensionTypes;

/**********************************************************
 * sesFMS_getDimMbrSubset.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getDimMbrSubset.sas                                                     */
/*  Purpose: subsets ETL temp fact dim member table by given ETL instance ID                 */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getDimMbrSubset(ETL_instanceID, TempFactDimMbr, subTempFactDimMbr);
  /* ETL_InstanceID : (Input) ETL instance ID from mid-tier parameter */
  /* TempFactDimMbr : (Input) table name includes all dimension type information */
  /* subTempFActDimMbr : (output) table name subsetting TempFactDimMbr by ETL_instanceID */

  %let cMethodNm = sesFMS_getDimMbrSubset;
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&TempFactDimMbr)) %then %do;
      /* Subsetting by ETL_instanceID */
      data &subTempFactDimMbr;
        set &TempFactDimMbr(where=(ETL_INSTANCE_ID = &ETL_instanceID));
      run;
  %end;
  %else %do;
      %let error_number = 52;
      %let errObj=&TempFactDimMbr;
      %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getDimMbrSubset;


/**********************************************************
 * sesFMS_getJournsFromStage.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getJournsFromStage.sas                                                     */
/*  Purpose: Get a subset of Journal trans. from Stage and dump into a work area data set.   */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getJournsFromStage(cStageLibRef,
                                 cCurrTimeMbrDS,
                                 cOutputJrnlDS,
                                 lKeepCols,
                                 cStageJrnlDS,
                                 cStageJrnlDetailDS,
                                 cSourceColNm,
                                 cSourceCode,
                                 csubTempFactDimMbr,
                                 cdimDimTypeDS);

  %let cMethodNm = sesFMS_getJournsFromStage;
  %let cStageLibRef = %qsysfunc(strip(&cStageLibRef));
  %let cStageJrnlDS = %qsysfunc(strip(&cStageJrnlDS));
  %let cInputDS =&cStageLibRef..&cStageJrnlDS;
  %let cStageJrnlDetailDS = %qsysfunc(strip(&cStageJrnlDetailDS));
  %let cInputDetailDS =&cStageLibRef..&cStageJrnlDetailDS;
  %let cStageDimTypeTableNm =  &cStageLibRef..dimension_type;
  %let nTrackTime = %qsysfunc(datetime());
  %let nJrnlCnt = 0;

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputJrnlDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputJrnlDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cInputDS, DATA))%then %do;
    %let error_number=37;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;
  %if not %sysfunc(exist(&cInputDetailDS, DATA))%then %do;
    %let error_number=37;
    %let errObj=&cInputDetailDS;
    %goto ERRORRETURN;
  %end ;

  /*------------------------------------------------*/
  /*  Build the subsetted transaction output table  */
  /*------------------------------------------------*/

  /* Write a dynamic SQL statement in a file */
  filename _sqlstmt temp;
  data _null_;
     file _sqlstmt;
     set _dim_type_id_col end=eof;

     if _N_ = 1 then do;
        put 'create table &cOutputJrnlDS as ';
        put 'select &cStageJrnlDS..AFFECTED_TIME_PERIOD_ID, &cStageJrnlDetailDS..*';
        put '  from &cInputDS, &cInputDetailDS';
        put ' where &cStageJrnlDS..GL_JRNL_ID = &cStageJrnlDetailDS..GL_JRNL_ID and';
        put '       &cStageJrnlDS..AFFECTED_TIME_PERIOD_ID in (select VALUE from &cCurrTimeMbrDS) and';
     end;

     /* dim_type_fmt = '$' !! strip(dim_type) !! "_FMT."; */

     /* put "       put(&cStageJrnlDetailDS.." dim_id_col_nm ", " dim_type_fmt ") NE ""*"" and" ; */
     put '        ' dim_id_col +(-1) ' in (select a.member_cd from &cSubTempFactDimMbr a,
                                                                       &cdimDimTypeDS b
                                            where b.dim_type_cd eq "' dim_type +(-1) '" and
                                                  b.dim_cd eq a.dimension_cd) and';

     if eof then
        put '       &cStageJrnlDetailDS..TRANSACTION_AMT not in(.) ;';

  run;

  /* run the dynamic SQL statement */
  proc sql noprint;
      %inc _sqlstmt;
  quit;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 38, &cInputDS, &cInputDetailDS, &cMethodNm);
    %goto ERRORRETURN;
  %end;

  /*  Did we find any Journals  */
  %let fmsetl_tempCheck1 = 0;
  proc sql noprint ;
    select count(*)
        into :fmsetl_tempCheck1
        from &cOutputJrnlDS ;
  quit ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
       %sesSOL_logError( YTDConv, L, 39, &cMethodNm) ;
    %goto ERRORRETURN;
  %end;

  %let nJrnlCnt = %eval(&fmsetl_tempCheck1);
  %if &nJrnlCnt eq 0 %then %let nJrnlCnt = 0 ;
  %if &nJrnlCnt EQ 0 %then %do ;
        /*  No journals found, skip the rest  */
        %goto THEEND;
  %end ;


  /*
   *  Does the resulting Journal table have the required Cycle Dimension Type columns
   */
  proc contents data=&cOutputJrnlDS out=_jrnlTranCols(keep=name rename=(name=code)) noprint;
  run;

  /*
   *  Keep just the columns we need
   */

  proc sort in=&lKeepCols out=&lKeepCols;
    by value;
  run;

  proc sql noprint;
    create table _temp_diff as
     /* lKeepCol-_jrnlTranCols */
     select upcase(value)
       from &lKeepCols
     except
     select upcase(code)
       from _jrnlTranCols;
  quit;

  %let num_data=0;
  proc sql noprint;
    select count(*)
      into :num_data
      from _temp_diff;
  quit;
  %let num_data=%eval(&num_data);


  %if &num_data > 0 %then %do;
    proc sql noprint;
  /*    select code */
      select value
        into :_col1 - :_col&num_data
        from _temp_diff;
    quit;

    %do ii=1 %to &num_data;
      %sesSOL_logError(YTDConv, L, 40, &cInputDS, &cInputDetailDS, &&_col&ii, &cMethodNm) ;
    %end;
    %goto RETURNZERO;
  %end;

  /* move all column names into variables, col1 to colxx */
  filename _col temp;
  data _null_;
      file _col;
    set &lKeepCols end=eof;
    put '"' value + (-1) '"n ';
  run;

  data &cOutputJrnlDS  ;
  keep %inc _col;
     TRANSACTION_AMT
     TRANSACTION_DT
     &cSourceColNm;
     length "SOURCE_CD"n $ %eval(32*&FM_DBCSMULT) ;
     set &cOutputJrnlDS ;
     &cSourceColNm = "&cSourceCode" ;
  run ;

  %goto THEEND;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=11;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getJournsFromStage;

/**********************************************************
 * sesFMS_getPriorTransFromStage.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getPriorTransFromStage.sas                                                 */
/*  Purpose:                                                                                 */
/*  Get a subset of the prior period GL transactions from Stage and dump into a work area    */
/*  data set. We only want those transactions where the YTD amount flag is true.             */
/*                                                                                           */
/*  Subset by:                                                                               */
/*     > prior time period                                                                   */
/*     > transaction_amt_ytd_flg eq "Y"                                                      */
/*     > specific analysis code                                                                */
/*     > currency hierarchy leaf nodes                                                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getPriorTransFromStage(cStageLibRef,
                                     cStageTransTableNm,
                                     cPriorTimeMbrDS,
                                     cAnalysisFormat,
                                     cCurrencyFormat,
                                     cOutputYtdDS,
                                     cNewAmtColNm,
                                     cPeriod_Fmt,
                                     cTempPrdColNm);

  %let cMethodNm = sesFMS_getPriorTransFromStage;
  %let cStageLibRef = %qsysfunc(strip(&cStageLibRef));
  %let cStageTransTableNm = %qsysfunc(strip(&cStageTransTableNm));
  %let cInputDS =&cStageLibRef..&cStageTransTableNm;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputYtdDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputYtdDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction table in Stage  */
  %if not %sysfunc(exist(&cInputDS, DATA)) and not %sysfunc(exist(&cInputDS, VIEW)) %then %do;
    %let error_number=10;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;

  data &cOutputYtdDS (rename=(TRANSACTION_AMT=&cNewAmtColNm)) ;
     length "&cTempPrdColNm"n 8;
     set &cInputDS ;
     if index("&fmsetl_priorTimeMbrCodes", strip(AFFECTED_TIME_PERIOD_ID)) > 0;
     if upcase(TRANSACTION_AMT_YTD_FLG) eq "Y" ;

     if TRANSACTION_AMT NE . ;
     &cTempPrdColNm = put(AFFECTED_TIME_PERIOD_ID, &cPeriod_Fmt..);
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=11;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getPriorTransFromStage;


/**********************************************************
 * sesFMS_createCodePrdIDTable.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_createCodePrdIDTable.sas                                                 */
/*  Purpose: Build a temporary data set of TIME_PERIOD IDs from Stage.                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_createCodePrdIDTable(cTimeMbrCodesDS, cOutputDS, cStageLib);

  %let cMethodNm = sesFMS_createCodePrdIDTable ;
  %let  cStgTimeCds = work._fmsetlTemp01 ;
  %let  cMemMissing = work._fmsetlTemp02 ;

  %let  nowDTTM = %qsysfunc(datetime()) ;
  %let  nTrackTime = %qsysfunc(datetime()) ;


  /*  Clean up  */
  %if %sysfunc(exist(&cOutputDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputDS, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cStgTimeCds)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cStgTimeCds, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cMemMissing)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cMemMissing, 2, '.');
    quit;
  %end;

  /*
   *  It's possible for Prior Time Mbrs to be blank.  We'll exclude them from the list
   */

  data &cOutputDS(keep=CODE VALUE);
      length PERIOD_ID 8;
      set &cTimeMbrCodesDS;
      if strip(VALUE) NE "" ;
      PERIOD_ID = input(strip(CODE), 32.);
      CODE = VALUE;
      VALUE = PERIOD_ID;
  run ;

  proc sort in=&cOutputDS;
       by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint ;
    create table &cStgTimeCds as
      select distinct TIME_PERIOD_ID as CODE
        from &cStageLib..TIME_PERIOD
     order by CODE;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cStgTimeCds;
       by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  data &cMemMissing(keep=CODE) ;
        merge &cOutputDS(in=a) &cStgTimeCds(in=b) ;
        by CODE ;
        if a and not b then
            /*  Error, mbr code exists in SDM, but not Stage  */
            output &cMemMissing;
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Check if any errors were found  */
  %if %sysfunc(exist(&cMemMissing)) %then %do;
    proc sql noprint;
      select count(*)
        into :numrow
        from &cMemMissing;
    quit;
    %let numrow=%eval(&numrow);
    %if &numrow %then %do;
      proc sql noprint;
        select  code
            into :cCode1 - :cCode&numrow
            from &cMemMissing;
      quit;

      %do ii=1 %to &numrow;
        %sesSOL_logError(YTDConv, L, 53, &&cCode&ii, cMethodNm);
      %end;
      /* %goto RETURNZERO; Change to WARNING: S0542829 */
    %end;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_createCodePrdIDTable;


/**********************************************************
 * sesFMS_getTimer.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getTimer.sas                                                            */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getTimer;
  %sysevalf(&nTimer);
%mend sesFMS_getTimer;


/**********************************************************
 * sesFMS_getTransFromStage.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getTransFromStage.sas                                                    */
/*  Purpose: Get a subset of transaction from Stage and dump into two work area data set.    */
/*           One data set with transactions flagged as YTD amounts, other flagged as either  */
/*           (Period activity amounts or Statistical accounts).                              */
/*  Subset by:                                                                               */
/*     > range of Process_dttm                                                               */
/*     > list of current time period(s)                                                      */
/*     > list of analysis(s)                                                                 */
/*     > currency hierarchy leaf nodes                                                       */
/*  Where the following is true, Transactions are dumped to a work area YTD data set:        */
/*     > transaction_amt_ytd_flg is 0 (false)                                                */
/*     > account type is statistical                                                         */
/*     > current period is the first period of the fiscal year and account type is I/S       */
/*  Otherwise transactions are dumped to a work area Period Activity data set                */
/*     (i.e. transactions are imported but not converted)                                    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getTransFromStage(cStageLibRef,
                                cStageTransTableNm,
                                cCurrTimeMbrDS,
                                cOutputYtdDS,
                                cOutputPaDS,
                                cOutputYtdColBlankDS,
                                cAccountFormat,
                                cNewAmtColNm,
                                cIsFirstPrd_Fmt,
                                lKeepCols,
                                cAcctTypeRevenue,
                                cAcctTypeExpense,
                                cAcctTypeStatFlow,
                                cAcctTypeNonFrequency,
                                cRunYtdPA_Fmt,
                                cPeriod_Fmt,
                                cTempPrdColNm,
                                csubTempFactDimMbr,
                                cdimDimTypeDS);

  %let cMethodNm = sesFMS_getTransFromStage;
  %let cStageLibRef = %qsysfunc(strip(&cStageLibRef));
  %let cStageTransTableNm = %qsysfunc(strip(&cStageTransTableNm));
  %let cInputDS =&cStageLibRef..&cStageTransTableNm;
  %let cStageDimTypeTableNm =  &cStageLibRef..dimension_type;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputYtdDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputYtdDS, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cOutputPaDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputPaDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction table in Stage  */
  %if not %sysfunc(exist(&cInputDS, DATA)) and not %sysfunc(exist(&cInputDS, VIEW)) %then %do;
    %let error_number=10;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;

  /*  Build the subsetted transaction output table  */
  /* extract all rows in the list, , into col_name */

  filename _keep temp;
  data _null_;
    file _keep;
    if _N_ = 1 then do;
       put '"&cTempPrdColNm"n "TRANSACTION_AMT"n  "TRANSACTION_DT"n ';
    end;
    set &lKeepCols end=eof;
    value = strip(value);

    put '"' value +(-1) '"n ' ;
    if eof then
      put ';';
  run;

  /* Statements for All dimensional operations should be written to a file */
  proc sql noprint;
    create table _dim_type_id_col as
    select distinct(a.dimension_type_cd) as dim_type, base_fact_column_nm as dim_id_col
    from &cStageDimTypeTableNm a
    where a.dimension_type_cd NE 'TIME'
      and a.valid_from_dttm le &fmsetl_processeddttm_to
      and (a.valid_to_dttm ge &fmsetl_processeddttm_to or
           a.valid_to_dttm eq .)
      and exists (select dimension_id
                  from &cDimDimTypeDS b, &csubTempFactDimMbr c
                  where b.dim_cd = c.dimension_cd
                    and a.dimension_type_cd = b.dim_type_cd
                  )
    ;
  quit;

  data work.tmp_dtypes;
     merge work.factDimMbr(in=inA) work.dimDimType end=eof;
        by dimension_cd;
     if inA and dim_type_cd ne 'TIME';
     keep dim_type_cd member_cd;
  run;

  filename _fstmt temp;
  data _null_;
     file _fstmt;
     set _dim_type_id_col;
     put 'rc = 0;';
     put 'dim_type_cd = "';
     put dim_type;
     put '";';
     put 'member_cd = ';
     put dim_id_col;
     put ';';
     put 'rc = ht.find();';
     put 'if rc = 0;';
  run;

  data &cOutputYtdDS (rename=(TRANSACTION_AMT=&cNewAmtColNm))
       &cOutputPaDS(drop=&cTempPrdColNm)
       &cOutputYtdColBlankDS(drop=&cTempPrdColNm);
     length dim_type_cd $ %eval(32*&FM_DBCSMULT);
     length member_cd $ %eval(32*&FM_DBCSMULT);
     keep
     %inc _keep;
     length "&cTempPrdColNm"n 8;
     set &cInputDS ;

     if _n_ = 1 then do;
        declare hash ht(dataset:"work.tmp_dtypes");
        ht.defineKey("dim_type_cd", "member_cd");
        ht.defineDone();
     end;

     if index("&fmsetl_currentTimeMbrCodes", strip(AFFECTED_TIME_PERIOD_ID)) > 0;

     %inc _fstmt;

     if TRANSACTION_AMT eq . then TRANSACTION_AMT = 0 ;

     &cTempPrdColNm = put(AFFECTED_TIME_PERIOD_ID, &cPeriod_Fmt..);

     /*  If RunYtdToPA = N, then dump all facts to cOutputPaDs  */
     if put(&cTempPrdColNm, &cRunYtdPA_Fmt..) eq "N" then do;
        output &cOutputPaDs ;
     end;
     else do;
        if put(&cTempPrdColNm, &cIsFirstPrd_Fmt..) eq "Y" then do;
           if (upcase(TRANSACTION_AMT_YTD_FLG) NE "Y") and (upcase(TRANSACTION_AMT_YTD_FLG) NE "N") then do;
              output &cOutputYtdColBlankDS;
           end;
           else if (upcase(TRANSACTION_AMT_YTD_FLG) eq "N" or
                    put(GL_ACCOUNT_ID, &cAccountFormat..) eq "&cAcctTypeStatFlow" or
                    put(GL_ACCOUNT_ID, &cAccountFormat..) eq "&cAcctTypeNonFrequency" or
                    put(GL_ACCOUNT_ID, &cAccountFormat..) eq "&cAcctTypeRevenue" or
                    put(GL_ACCOUNT_ID, &cAccountFormat..) eq "&cAcctTypeExpense" ) then do ;
                   output &cOutputPaDs ;
                end ;
                else do;
                   output &cOutputYtdDs ;
                end;
        end;
        else do;
           if (upcase(TRANSACTION_AMT_YTD_FLG) NE "Y") and (upcase(TRANSACTION_AMT_YTD_FLG) NE "N") then do;
              output &cOutputYtdColBlankDS;
           end;
           else if (upcase(TRANSACTION_AMT_YTD_FLG) eq "N" or
                    put(GL_ACCOUNT_ID, &cAccountFormat..) EQ "&cAcctTypeNonFrequency" ) then do ;
                   output &cOutputPaDs ;
                end ;
                else do;
                   output &cOutputYtdDs ;
                end;
        end;
     end;
  run ;

  %goto THEEND;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=11;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end;

  /*  Check for empty YTD Flag errors  */
  %let fmsetl_tempCheck1 = ;
  proc sql noprint ;
      select count(*)
        into :fmsetl_tempCheck1
        from &cOutputYtdColBlankDS ;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 35, &cMethodNm) ;
    %goto RETURNZERO;
  %end;

  %if &fmsetl_tempCheck1 GT 0 %then %do;
    %let error_number=36;
    %let errObj=&fmsetl_tempCheck1;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getTransFromStage;

/**********************************************************
 * sesFMS_imptdataWithoutTr.sas
 **********************************************************/

/* Copyright 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro sesFMS_imptdataWithoutTr (gl_trans_sum_table=, update_tablename=,
                 cycle_dim_table=, bfdtypes=, fmsetl_processedDTTM_To=,
                 dataLoadObjTypeId=, dataLoadObjId= );

  %put FMSETL_CONCATMBRSTMT = &fmsetl_concatMbrStmt;

  %let fmsetl_error = ;
  %let fmsetl_errormsg = ;
  %let fmsetl_statusId = 1 ; /* Will be changed to 0 if program runs successful */
  %let format_missing = 0;

  %let entry = imptdata;
  %let putlog = Y;

  %let curr_rec_dttm = '01JAN5999:00:00'DT;

  /* options metarepository="&SlnRepos"; */

  /* Check if datetime passed in is valid */
  %if %datatyp(&fmsetl_processedDTTM_To) ne NUMERIC %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=1,
              msgTxt1=<fmsetl_processedDTTM_To : &fmsetl_processedDTTM_To>, msgPutlog=&putlog);
    %goto exit;
  %end;

  /* Check if update_tablename macro variable is passed */

  %if (&update_tablename=.) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
              msgTxt1=<update_tablename : &update_tablename>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Check transaction records for validity */

  %macro datawork/parmbuff;

  %let err=_ERROR;
  %let fmt=fmt_dtype;
  %let x=_x.;
  %let cd=_CD;
  %let code=_CODE;

  /* Check if transaction table exists */
  %let _tmpDSN = &gl_trans_sum_table;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto datawork_exit;
  %end ;

  /* ANALYSIS TIME INTORG */
  /* ACCOUNT CURRENCY ANALYSIS TIME INTORG SOURCE */

    data &update_tablename.(keep=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList MEASURE rename=(MEASURE=FACT_VALUE))
       WORK.BADRECS(drop=OBJECT_TYPE_ID OBJECT_ID)
       WORK.ORG_EQ_TRADER(drop=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList  )
       WORK.ICRECS(drop=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList  )
       WORK.NONICRECS(drop= OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList )
     ;

     length BAD_RECORD 8;
     length MEM_DIMTYPE $ 8;
     length DIMTYPE_HOLD $ 8;
     length OBJECT_TYPE_ID 8 ;
     length OBJECT_ID 8 ;
     length &fmsetl_dimTypeList 8;
     set &_tmpDSN;

     BAD_RECORD = 0;

     if _n_ = 1 then do;
        declare hash mht(dataset:"work.memberHashTable");
        mht.defineKey("internal_dimension_id", "member_cd");
        mht.defineData("member_id");
        mht.defineDone();
     end;

     /* Validate the members of the dimension types existing in the basefact table */
     %let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));
     %do i = 1 %to &dimcnt;
         %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
         %let dimname=%sysfunc(putc(&dimtype, $FMT_TO_DIM.));
         %let dimtype_id=%sysfunc(inputn(&dimname, FMT_DTYPE_ID.));
         %let int_dim_id=%sysfunc(inputn(&dimtype_id, FMT_DIMS.));
         %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
         %let dimtypecd=%sysfunc(strip(&dimtype1))&cd;
         %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
         %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;

         DIMTYPE_HOLD = "&dimtype";

         internal_dimension_id = &int_dim_id;
         member_cd = "&dimtypecd"n;

         rc = mht.find();
         if rc = 0 then
            "&dimtype"n = member_id;

         checkId = "&dimtype"n ;

         MEM_DIMTYPE = strip(put(checkId, FMT_MEMTYPE.));
         if (MEM_DIMTYPE = DIMTYPE_HOLD) then do;
            "&dimtypeerr"n = 0;
         end;
         else do;
            BAD_RECORD = 1;
            "&dimtypeerr"n = 1;
         end;
         if "&dimtypeerr"n eq 1 then do;
           "&dimtypecode"n = "*stage ID* " || strip(member_cd) ;
         end;
         else
           "&dimtypecode"n = "&dimtypecd"n;
     %end;

     length MEASURE 8;
     format MEASURE best32.;
     MEASURE = TRANSACTION_AMT;

     if (BAD_RECORD = 1) then do;
        keep MEASURE OBJECT_TYPE_ID OBJECT_ID
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
                 %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;
                 "&dimtype"n "&dimtypecode"n "&dimtypeerr"n
             %end;
             ;
        output WORK.BADRECS;
     end;
     else do;
        keep MEASURE
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
/*                 "&dimtype"n "&dimtypecode"n */
             %end;
             ;

             /*-------------remove ------------------------
             CONCAT_MEMBER_IDS = &fmsetl_concatMbrStmt ;
             ----------------------------------------------*/
             OBJECT_TYPE_ID = &dataLoadObjTypeId ;

             OBJECT_ID = &dataLoadObjId;
             output &update_tablename;
     end;
  run;

  %checkErr;
  %if &_tmpRC %then %goto datawork_exit;

  %datawork_exit:

  %mend datawork;

  %datawork(&bfdtypes);

  %checkErr;
  %if &_tmpRC %then %goto exit;

  %let fmsetl_statusId = 0; /* Program ran successful */

  %exit:

  %if &fmsetl_statusID ne 0 %then %do;
     %let fmsetl_error = &sesgmsg_msgCodeReturn;
     %let fmsetl_errormsg = &sesgmsg_msgReturn;
  %end;

%mend sesFMS_imptdataWithoutTr;


/**********************************************************
 * sesFMS_imptdataWithTr.sas
 **********************************************************/

/* Copyright 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro sesFMS_imptdataWithTr (gl_trans_sum_table=, update_tablename=, ic_check_ind=, non_ic_check_ind=,
                 cycle_dim_table=, bfdtypes=, ext_mem=, fmsetl_processedDTTM_To=,
                 dataLoadObjTypeId=, dataLoadObjId= );

  /*
  %put FMSETL_CONCATMBRSTMT = &fmsetl_concatMbrStmt;
  */

  %let fmsetl_error = ;
  %let fmsetl_errormsg = ;
  %let fmsetl_statusId = 1 ; /* Will be changed to 0 if program runs successful */
  %let format_missing = 0;

  %let entry = imptdata;
  %let putlog = Y;

  %let curr_rec_dttm = '01JAN5999:00:00'DT;

  /* options metarepository="&SlnRepos"; */

  /* Check if datetime passed in is valid */
  %if %datatyp(&fmsetl_processedDTTM_To) ne NUMERIC %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=1,
              msgTxt1=<fmsetl_processedDTTM_To : &fmsetl_processedDTTM_To>, msgPutlog=&putlog);
    %goto exit;
  %end;

  /* Check if update_tablename macro variable is passed */

  %if (&update_tablename=.) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
              msgTxt1=<update_tablename : &update_tablename>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Check transaction records for validity */

  %macro datawork/parmbuff;

  %let err=_ERROR;
  %let fmt=fmt_dtype;
  %let x=_x.;
  %let cd=_CD;
  %let code=_CODE;

  /* Check if transaction table exists */
  %let _tmpDSN = &gl_trans_sum_table;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto datawork_exit;
  %end ;

  /* ANALYSIS TIME INTORG */
  /* ACCOUNT CURRENCY ANALYSIS TIME INTORG SOURCE */

    data &update_tablename.(keep=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList MEASURE rename=(MEASURE=FACT_VALUE))
       WORK.BADRECS(drop=OBJECT_TYPE_ID OBJECT_ID)
       WORK.ORG_EQ_TRADER(drop=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList )
       WORK.ICRECS(drop=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList )
       WORK.NONICRECS(drop=OBJECT_TYPE_ID OBJECT_ID &fmsetl_dimTypeList )
       ;
     length BAD_RECORD 8;
     length MEM_DIMTYPE $ 8;
     length DIMTYPE_HOLD $ 8;
     length INVALID_IC_RECORD 8;
     length INVALID_NON_IC_RECORD 8;
     length INTERCOMPANY_ACCOUNT $ 1;

     length OBJECT_TYPE_ID 8 ;
     length OBJECT_ID 8 ;
     length &fmsetl_dimTypeList 8;

     set &_tmpDSN;

     BAD_RECORD = 0;

     if _n_ = 1 then do;
        declare hash mht(dataset:"work.memberHashTable");
        mht.defineKey("internal_dimension_id", "member_cd");
        mht.defineData("member_id");
        mht.defineDone();
     end;

     /* Validate the members of the dimension types existing in the basefact table */
     %let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));

     %do i = 1 %to &dimcnt;
         %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
         %let dimname=%sysfunc(putc(&dimtype, $FMT_TO_DIM.));
         %let dimtype_id=%sysfunc(inputn(&dimname, FMT_DTYPE_ID.));
         %let int_dim_id=%sysfunc(inputn(&dimtype_id, FMT_DIMS.));
         %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
         %let dimtypecd=%sysfunc(strip(&dimtype1))&cd;
         %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
         %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;
         DIMTYPE_HOLD = "&dimtype";

         internal_dimension_id = &int_dim_id;
         member_cd = "&dimtypecd"n;

         rc = mht.find();
         if rc = 0 then
            "&dimtype"n = member_id;

         checkId = "&dimtype"n ;
         MEM_DIMTYPE = strip(put(checkId, FMT_MEMTYPE.));

         if (DIMTYPE_HOLD = 'TRADER') then do;
            if (MEM_DIMTYPE = 'INTORG') then do;
               "&dimtypeerr"n = 0;
            end;
            else do;
               bad_record = 1;
               "&dimtypeerr"n = 1;
            end;
         end;
         else do;
            if (DIMTYPE_HOLD = 'INTORG' and (INTORG = &ext_mem or INTORG = &all_mem)) then do;
               BAD_RECORD = 1;
               "&dimtypeerr"n = 1;
            end;
            else do;
               if (MEM_DIMTYPE = DIMTYPE_HOLD) then do;
                  "&dimtypeerr"n = 0;
               end;
               else do;
                  BAD_RECORD = 1;
                  "&dimtypeerr"n = 1;
               end;
            end;
         end;
         if "&dimtypeerr"n eq 1 then do;
           "&dimtypecode"n = "*stage ID* " || strip(member_cd) ;
         end;
         else
           "&dimtypecode"n = "&dimtypecd"n;
     %end;

     length MEASURE 8;
     format MEASURE best32.;
     MEASURE = TRANSACTION_AMT;

     /* Perform intercompany account check if indicator set to 'Y' */
     INVALID_IC_RECORD = 0;
     %if &ic_check_ind eq Y %then %do;
        INTERCOMPANY_ACCOUNT = put(ACCOUNT, FMT_INTERCOMPANY.);
        if ((INTERCOMPANY_ACCOUNT = 'Y') and (TRADER = INTORG or TRADER = &ext_mem)) then
           INVALID_IC_RECORD = 1;
     %end;

     /* Perform non-intercompany account check if indicator set to 'Y' */
     INVALID_NON_IC_RECORD = 0;
     %if &non_ic_check_ind eq Y %then %do;
        INTERCOMPANY_ACCOUNT = put(ACCOUNT, FMT_INTERCOMPANY.);
        if ((INTERCOMPANY_ACCOUNT = 'N') and (TRADER ^= INTORG) and (TRADER ^= &ext_mem)) then
           INVALID_NON_IC_RECORD = 1;
     %end ;

     if (BAD_RECORD = 1) then do;
        keep MEASURE OBJECT_TYPE_ID OBJECT_ID
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
                 %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;
                 "&dimtype"n "&dimtypecode"n "&dimtypeerr"n
             %end;
             ;
        output WORK.BADRECS;
     end;
     else do;
        keep MEASURE
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
/*                 "&dimtype"n "&dimtypecode"n */
             %end;
             ;
        if (INTORG = TRADER) then
           output WORK.ORG_EQ_TRADER;
        else
           if (INVALID_IC_RECORD = 1) then
              output WORK.ICRECS;
           else
              if (INVALID_NON_IC_RECORD = 1) then
                 output WORK.NONICRECS;
              else do ;

                 OBJECT_TYPE_ID = &dataLoadObjTypeId ;

                 OBJECT_ID = &dataLoadObjId;

                 output &update_tablename;
              end ;
      end;
  run;

  %checkErr;
  %if &_tmpRC %then %goto datawork_exit;

  %datawork_exit:

  %mend datawork;

  %datawork(&bfdtypes);



  %checkErr;
  %if &_tmpRC %then %goto exit;

  %let fmsetl_statusId = 0; /* Program ran successful */

  %exit:

  %if &fmsetl_statusID ne 0 %then %do;
     %let fmsetl_error = &sesgmsg_msgCodeReturn;
     %let fmsetl_errormsg = &sesgmsg_msgReturn;
  %end;

%mend sesFMS_imptdataWithTr;


/**********************************************************
 * sesFMS_init.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : YTDConv.sas                                                                     */
/*  Purpose: 1) All macros %sesSOL_* should be compiled before it                           */
/*              YTDConv is using macros of FMSUtil                                           */
/*           2) %FMSUtil should be run before YTDConv                                        */
/*              %FMSUtil includes globals used in YTDConv and YTDConv                        */
/*                                                                                           */
/*  History: 23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.            */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_init;

  /* Initialize list (dataset) */
  proc sql noprint;
    create table lStageTranCols (code varchar(50));
    insert into lStageTranCols
        values ('TRANSACTION_AMT')
        values ('TRANSACTION_AMT_YTD_FLG')
        values ('AFFECTED_TIME_PERIOD_ID')
        values ('INITIATING_INTERNAL_ORG_ID')
        values ('AFFECTED_INTERNAL_ORG_ID')
        values ('GL_ACCOUNT_ID')
        values ('ANALYSIS_ID')
        values ('CURRENCY_CD')
     ;
  quit;

  proc sql noprint;
    create table lStageTranOPCols (code varchar(50));
    insert into lStageTranOPCols
        values ('TRANSACTION_AMT')
        values ('TRANSACTION_AMT_YTD_FLG')
        values ('TIME_PERIOD_ID')
        values ('INTERNAL_ORG_ID')
        values ('GL_ACCOUNT_ID')
        values ('ANALYSIS_ID')
        values ('CURRENCY_CD')
     ;
  quit;

%mend sesFMS_init;


/**********************************************************
 * sesFMS_LoadCycleBaseFact.sas
 **********************************************************/

/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */
/* ----------------------------------------------------------------------------------------- */
/*  Name   : LoadCycleBaseFact.sas                                                           */
/*  PURPOSE:   Loads basefacts from the Staging area into a Cycles basefact table.  Process  */
/*             includes:                                                                     */
/*                                                                                           */
/*             Convert YTD transaction amount to Period Activity values.  This conversion    */
/*             occurs when transactions are imported into a Cycle from the Staging area      */
/*             transaction table.                                                            */
/*                                                                                           */
/*             If a libref to the data set does not exist, this process will attempt to      */
/*             submit one.  The libref is not removed when this code completes.              */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_LoadCycleBaseFact;

  /* Initialize global macros to run macros from FMSUtil and YTDConv */

  %global _FMErrorCode;
  %let _FMErrorCode=1; /* 0: error, 1:normal execution */

  %let nTrackTime = %qsysfunc(datetime()) ;

  /*  Seed initial return values  */
  %global fmsetl_statusId;
  %let fmsetl_statusId=1; /* success=0, critical error=1, validation error=2, warning=3  */

  %sesSOL_init ;
  %sesFMS_init ;

  /*  Turn logging information on/off based on fmsetl_YtdDebug macro value  */

  /*
   *  We need to create a empty or blank report and dump to DAV, otherwise we get a package exception
   *  when the StoredProcess completes
   */

  %sesSOL_blankReport ;

  /*
   *  Pick up parameters to this job.  This job is intended to be launched via a store process
   *  from a Java client.  Parameters are passed either by macro variable or by streamed input.
   */

  %let cOMRServer                 = %sesSOL_removeQuotes(&fmsetl_omrServer) ;
  %let nOMRPort                   = &fmsetl_omrPort;


  /*****************************************************************************************/

  %let cStageLib                  = &fmsetl_ConformLibrary;

  %let cStageTransTableNm         = &fmsetl_TransactionTableNm;
  %let cStageDimTypeTableNm       = &fmsetl_DimensionTypeTableNm;
  %let cSDMLib                    = &fmsetl_SDMLibrary;
  %let cSource_BaseCode           = &fmsetl_Source_BaseCode;
  %let cSource_BaseJournCode      = &fmsetl_Source_BaseJournCode;
  %let nBaseMbrSDM_Id             = &fmsetl_Source_BaseId;
  %let nBaseJournMbrSDM_Id        = &fmsetl_Source_BaseJournId;
  %let cSourceDimTypeCode         = &fmsetl_sourceDimTypeCode;
  %let nProcessedDTTM_From        = &fmsetl_processedDTTM_From;
  %let nProcessedDTTM_To          = &fmsetl_processedDTTM_To;
  %let cAcctTypeRevenue           = %qsysfunc(strip(&fmsetl_acctTypeRevenue));
  %let cAcctTypeExpense           = %qsysfunc(strip(&fmsetl_acctTypeExpense));
  %let cAcctTypeStatFlow          = %qsysfunc(strip(&fmsetl_acctTypeStatFlow));
  %let cAcctTypeNonFrequency      = %qsysfunc(strip(&fmsetl_acctTypeNonFrequency));
  %let cTmpFactSDM_Nm             = &fmsetl_tempCreateFactTable;
  %let cTmpFactWork_Nm            = WORK.sas_etl_base_fact;
  %let cTmpRemoveROFactSDM_Nm     = &fmsetl_tempRemoveROFactTable;
  %let cTmpRemoveRWFactSDM_Nm     = &fmsetl_tempRemoveRWFactTable;
  %let cSDMStatusDS               = &fmsetl_tempStatusTable;
  %let cCkIcTradingPartners       = %upcase(&fmsetl_ckIcTradingPartners);
  %let cCkNonIcTradingMembers     = %upcase(&fmsetl_ckNonIcTradingMembers);
  %let nAccountDimId              = &fmsetl_accountDimId;
  %let nAcctTypePropertyId        = &fmsetl_acctTypePropertyId;
  %let nActionType                = &fmsetl_actionType;
  %let cDLObjTypeId               = &fmsetl_dataLoadObjectTypeId;
  %let cDLObjId                   = &fmsetl_dataLoadObjectIds;
  %let nSdmROFactSetId            = &fmsetl_readOnlyFactSetId;
  %let nSdmRWFactSetId            = &fmsetl_readWriteFactSetId;

  /* Added for more than time and analysis */
  %let cETL_instanceID            = &fmsetl_instanceId;
  %let cTempFactDimMbr            = &cSdmLib..SAS_ETL_FACT_DIM_MBR;
  %let csubTempFactDimMbr         = WORK.SAS_ETL_FACT_DIM_MBR;

  /*  Assign constants  */
  %let cSCLEntryNm           =sesFMS_LoadCycleBaseFact;
  %let cLogEntry             =LoadCycleBasefact;
  %let cProduct              =BASE;

  %let cCurrentTransYtdDS    =WORK._CURRENTTRANYTD;
  %let cCurrentTransPaDS     =WORK._CURRENTTRANPA;
  %let cPriorTransYtdDS      =WORK._PRIORTRANYTD;
  %let cConvertedGlTransDS   =WORK._CONVERTEDGLTRANS;
  %let cCurrencyWorkAreaDS   =WORK._CYCLECURRENCY;
  %let cAccountWorkAreaDS    =WORK._ACCOUNT;
  %let cDimTypeWorkAreaDS    =WORK._CYCLEDIMTYPE;
  %let cDimTypeWorkAreaDS1   =WORK._CYCLEDIMTYPE1;
  %let cDimWorkAreaDS        =WORK._CYCLEDIM;
  %let cDimDimTypeDS         =WORK._DIMDIMTYPE;

  %let cAnalysisWorkAreaDS   =WORK._ANALYSIS;
  %let cAnalysisDS_SDM_RKs   =WORK._ANALYSIS_IDS;

  %let cStageDimTypeWorkAreaDS =WORK._STAGEDIMTYPE;
  %let cDimTypeTableNmDS       =WORK._STAGEDIMTYPETABLENM;
  %let cDimTypeKeyColumnNmDS   =WORK._DIMTYPERKCOL;
  %let cOutputYtdColBlankDS  =WORK._BLANKYTDFLAGTRANS;
  %let cStatusDS             =WORK._FACTSTATUS;
  %let cDLObjIdDS            =WORK._DLOBJIDS;

  %let cCurrTimeMbrId_Map_PrdId_DS    =WORK._CURRTIME_SDM_IDS;
  %let cCurrTimeMbrCode_Map_PrdId_DS  =WORK._CURRTIME_SDM_CODES;
  %let cCurrTimeMbr_StgCds_PrdID_DS   =WORK._CURRTIME_STG_CDS_PRDID;
  %let cCurrTimeMbr_StgCds_PrdID_Fmt  =$CURTMEP;

  %let cPriorTimeMbrId_Map_PrdId_DS   =WORK._PRIORTIME_SDM_IDS;
  %let cPriorTimeMbrCode_Map_PrdId_DS =WORK._PRIORTIME_SDM_CODES;
  %let cPriorTimeMbr_StgCds_PrdID_DS  =WORK._PRIORTIME_STG_CDS_PRDID;
  %let cPriorTimeMbr_StgCds_PrdID_Fmt =$PRITMEP;

  %let cIsFirstPrd_Map_PrdId_DS       =WORK._ISFIRSTPRD;
  %let cIsFirstPrd_Fmt                =FRSTPRD;

  %let cRunYTDToPA_Map_PrdId_DS       =WORK._RUN_YTD_TO_PA;
  %let cRunYTDToPA_Fmt                =RYTDPA;

  %let cCyclePrdIdsDS                 =WORK._CYCLE_PRD_IDS;

  %let cCurrencyFormat       =$CURR;
  %let cAccountFormat        =$ACCT;
  %let cDimTypeFormat        =$DTYPE;
  %let cDimTypeColFormat     =$DTPCOL;
  %let cAnalysisFormat       =ANLYS;
  %let cStageDimTypeFormat   =$SDTYPE;
  %let cDLObjIdFormat        =$DLOBJID;
  %let cCurrentAmtColNm      =convert_amt_CurrentYtd;
  %let cPriorAmtColNm        =convert_amt_PriorYtd;
  %let cTransAmtColNm        =transaction_amt;
  %let cStageJrnlDS          =GL_JRNL;
  %let cStageJrnlDetailDS    =GL_JRNL_DETAILS;
  %let cJournalDS            =WORK._STAGEJRNLS;
  %let cSourceColNm          =SOURCE_CD;
  %let cTempPrdColNm         =TEMP__PERIOD__ID__FOR__MERGE;

  /*  Make sure required values have been supplied  */
  %if "&cOMRServer" eq ""  %then %do;
    %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_omrServer, &cSCLEntryNm);
    %goto ERROR;
  %end;

  %if &nOMRPort eq . or &nOMRPort eq 0 %then %do;
    %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_omrPort, &cSCLEntryNm);
    %goto ERROR;
  %end;

  %if "&cStageLib" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_StageLibrary, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cStageTransTableNm" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_StageTransactionTableNm, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cSourceDimTypeCode" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_sourceDimTypeCode, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cSource_BaseCode" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_Source_BaseCode, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cSource_BaseJournCode" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_Source_BaseJournCode, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nProcessedDTTM_From eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_processedDTTM_From, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nProcessedDTTM_To eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_processedDTTM_To, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cAcctTypeRevenue" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypeRevenue, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cAcctTypeExpense" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypeExpense, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cAcctTypeStatFlow" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypeStatFlow, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cAcctTypeNonFrequency" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_AcctTypeNonFrequency, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cTmpFactSDM_Nm" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_tempCreateFactTable, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cTmpRemoveROFactSDM_Nm" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_tempRemoveROFactTable, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cTmpRemoveRWFactSDM_Nm" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_tempRemoveRWFactTable, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cCkIcTradingPartners" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_ckIcTradingPartners, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cCkNonIcTradingMembers" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_ckNonIcTradingMembers, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cSDMLib" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_SDMLibrary, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nAccountDimId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_accountDimId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nAcctTypePropertyId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypePropertyId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nActionType eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_actionType, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cDLObjTypeId" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_dataLoadObjectTypeId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if "&cDLObjId" eq ""  %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_dataLoadObjectIds, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nSdmROFactSetId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_readOnlyFactSetId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nSdmRWFactSetId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_readWriteFactSetId, &cSCLEntryNm);
      %goto ERROR;
  %end;


  options validvarname=any ;

  /*
   *  Create libref connections
   */

  /*
   *  Stage Libref
   */

  %bldLibNm(sesetl_sasLibRef=&cStageLib
                 , sesetl_OMRSASLibraryObjName=&FMdatatier_folder./&CONFLIBOBJ./&CONFLIBOBJ.
                 , sesetl_useType=BASE
                 , sesetl_validateLib=Y
        ) ;

  %bldLibNm(sesetl_sasLibRef=&cSDMLib
          , sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.
          , sesetl_useType=BASE
          , sesetl_validateLib=Y
           ) ;

  %bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.,
               sesetl_useType=CON
               ) ;

  /**********************************************************************************************************/
  /* subsetting target dimensions by given ETL instance ID and store it in a separate table in WORK library */
  /**********************************************************************************************************/
  %sesFMS_getDimMbrSubset(&cETL_instanceID, &cTempFactDimMbr, &cSubTempFactDimMbr);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 58, &cSCLEntryNm);
      %goto ERROR;
  %end ;


  /*
   *  We send end the Currency Codes, Dimension Type Codes and Dimension
   *  Codes in as macros.  Retrieve macro values and populate temporary
   *  work area data sets.
   */

  %sesFMS_macrosToDataSets( &cCurrencyWorkAreaDS,
                            &cDimTypeWorkAreaDS,
                            &cDimTypeWorkAreaDS1,
                            &cDimWorkAreaDS,
                            &cDimDimTypeDS,
                            &cAnalysisWorkAreaDS,
                            &cAnalysisDS_SDM_RKs,
                            &cDLObjIdDS,
                            &cCurrTimeMbrId_Map_PrdId_DS,
                            &cCurrTimeMbrCode_Map_PrdId_DS,
                            &cPriorTimeMbrId_Map_PrdId_DS,
                            &cPriorTimeMbrCode_Map_PrdId_DS,
                            &cCyclePrdIdsDS,
                            &cIsFirstPrd_Map_PrdId_DS,
                            &cRunYTDToPA_Map_PrdId_DS
                           );

  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( &cLogEntry, L, 4, &cSCLEntryNm);
    %goto Error ;
  %end ;


  /****************************************************************************************/
  /* ANALYSIS: Validate list of Analysis Codes from mid-tier exist in Staging Area.       */
  /****************************************************************************************/

  %sesFMS_validateAnalysisMembers(&cAnalysisWorkAreaDS, &cStageLib);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 4, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /****************************************************************************************/
  /*                           Current Time Period Validation                             */
  /****************************************************************************************/

  %sesFMS_createCodePrdIDTable(&cCurrTimeMbrCode_Map_PrdId_DS, &cCurrTimeMbr_StgCds_PrdID_DS,
                               &cStageLib);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 39, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_createFormat(&cCurrTimeMbr_StgCds_PrdID_DS, &cCurrTimeMbr_StgCds_PrdID_Fmt, 0, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cCurrTimeMbr_StgCds_PrdID_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /****************************************************************************************/
  /*                           Prior Time Period Validation                               */
  /****************************************************************************************/

  %sesFMS_createCodePrdIDTable(&cPriorTimeMbrCode_Map_PrdId_DS, &cPriorTimeMbr_StgCds_PrdID_DS,
                               &cStageLib);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 40, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_createFormat(&cPriorTimeMbr_StgCds_PrdID_DS, &cPriorTimeMbr_StgCds_PrdID_Fmt, 0, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cPriorTimeMbr_StgCds_PrdID_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /****************************************************************************************/
  /*                Build Map between Stage Period ID and Run YTDtoPA                     */
  /****************************************************************************************/
  %sesFMS_createFormat(&cRunYTDToPA_Map_PrdId_DS, &cRunYTDToPA_Fmt, 0, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cRunYTDToPA_Map_PrdId_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /****************************************************************************************/
  /*                 Build Map between Stage Time Mbr ID and IsFirstPrd                   */
  /****************************************************************************************/
  %sesFMS_createFormat(&cIsFirstPrd_Map_PrdId_DS, &cIsFirstPrd_Fmt, 0, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cIsFirstPrd_Map_PrdId_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*
   *  Retrieve a list of distinct accounts with their associated account types
   */
  %sesFMS_getAccountTypes( &cAccountWorkAreaDS, &cSDMLib, &nAccountDimId, &nAcctTypePropertyId, &nProcessedDTTM_To);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 3, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  Build format of DataLoad Object Ids
     */
  %sesFMS_createFormat( &cDLObjIdDS, &cDLObjIdFormat, 0, lDLObjId, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 51, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For Currency leaf nodes, build a SAS Format and return a named list.  There should be a specific
     *  currency hierarchy associated to the Cycle, we are importing to.
     *
     */
  %sesFMS_createFormat( &cCurrencyWorkAreaDS, &cCurrencyFormat, 0, lCurrency, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 5, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For Account and account types, build a SAS Format and return a named list.
     */
  %sesFMS_createFormat( &cAccountWorkAreaDS, &cAccountFormat, 0, lAccount, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 6, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For the Cycle's Dimension Types, build a SAS Format and return a named list.
     */
  %sesFMS_createFormat( &cDimTypeWorkAreaDS, &cDimTypeFormat, 0, lCycleDimTypes, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 7, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For the Cycle's Dimension Types, build a SAS Format and return a named list.
     */
  %sesFMS_createFormat( &cDimTypeWorkAreaDS1, &cDimTypeColFormat, 0, lCycleDimTypes1, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 7, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Build a list of dim types in fact set order.
     */
  %sesFMS_createDimTypeFactSetOrd( &cDimTypeWorkAreaDS, lDimTypeFactSetOrder );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 7, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Note:  the SOURCE dimension type does not exist in Stage, but will in the mart, as such,
     *         we need to remove the SOURCE dimension type from this list
     */

  proc sql noprint;
    create table lOrigCycleDimTypes
      as select *
         from lCycleDimTypes
        where CODE NE "&cSourceDimTypeCode";
    quit;


    /*
     *  Finds the currently active records from the Stage Dimension Type table and loads into a work area data set
     */
  %sesFMS_getStageDimensionTypes( &cStageLib, &cStageDimTypeTableNm,
                                  &cStageDimTypeWorkAreaDS, &cDimTypeTableNmDS,
                                  &cDimTypeKeyColumnNmDS );

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 8, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*
   *  For the Stage Dimension Types, build a SAS Format and return a named list.  This gives us an association
   *  between the Mart's dimension type name and the column name of the Stage gl_transaction_sum table.
   *
   *  It is worth noting that this list is/should be a comprehensive list of dimension types in Stage, and
   *  not necessarily the list of dimension types in the Mart's Cycle.
   */
  %sesFMS_createFormat( &cStageDimTypeWorkAreaDS, &cStageDimTypeFormat, 0, lStageDimTypes, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cStageDimTypeWorkAreaDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*
   *  There will be a specific list of Dimension Types associated to the Mart's Cycle.  We need to make
   *  sure that those dimension types also exist in the Stage Dimension Type table.  Otherwise, we have no
   *  hope of matching Cycle dimension types to columns in the Stage gl_transaction_sum table.
   *
   */

  %sesFMS_verifyDimensionTypes( lCycleDimTypes, lStageDimTypes, lCycleDimTypesCols );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 10, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Verify that the Stage GL Transaction table contains the stardard model columns.  Customer can add columns
     *  to the model without any problems.  However if columns are removed or renamed, we can expect to have
     *  errors in this and other processes.
     */
  %sesFMS_verifyStageTransModel( &cStageLib, &cStageTransTableNm, lCycleDimTypesCols, lStageTranCols );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 11, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*
   *  Create a column list to sort, summarization and merging transaction tables.  We need to remove
   *  the time period column from the list because we will be sorting and merging columns between
   *  current and prior period and we want the transactions to match.
   */

  proc sql noprint;
    create table lTranCols as
    select value
      from lCycleDimTypesCols
     where upcase(value) NE "AFFECTED_TIME_PERIOD_ID";
  quit;

  /*
   *  Get a subset of transaction from Stage and dump into two work area data sets.  One data set with
   *  transactions flagged as YTD amounts, other flagged as either (Period activity amounts or Statistical
   *  accounts).
   *
   *  Subset by:
   *     > range of Process_dttm
   *     > list of current time period(s)
   *     > list of analysis(s)
   *     > currency hierarchy leaf nodes
   *
   *  Where the following is true, Transactions are dumped to a work area YTD data set:
   *     > transaction_amt_ytd_flg is 0 (false)
   *     > account type is statistical
   *     > current period is the first period of the fiscal year and account type is I/S
   *  Otherwise transactions are dumped to a work area Period Activity data set
   *     (i.e. transactions are imported but not converted)
   *
   */

  /****************************************************************************************/
  /* CREATE LIST CONSISTING OF DIMENSION TYPE AND MEMBER SEPERATED BY COLON               */
  /****************************************************************************************/
  data work.factDimMbr;
     set &cSubTempFactDimMbr;
  run;

  data work.dimDimType;
     set &cdimDimTypeDS;
     dimension_cd = dim_cd;
     drop dim_cd;
  run;

  proc sort data = work.factDimMbr;
     by dimension_cd;
  run;

  proc sort data = work.dimDimType;
     by dimension_cd;
  run;

  /****************************************************************************************/
  %sesFMS_getTransFromStage( &cStageLib, &cStageTransTableNm, &cCurrTimeMbrCode_Map_PrdId_DS,
                             &cCurrentTransYtdDS, &cCurrentTransPaDS, &cOutputYtdColBlankDS,
                             &cAccountFormat, &cCurrentAmtColNm, &cIsFirstPrd_Fmt,
                             lCycleDimTypesCols, &cAcctTypeRevenue, &cAcctTypeExpense,
                             &cAcctTypeStatFlow, &cAcctTypeNonFrequency, &cRunYTDToPA_Fmt,
                             &cCurrTimeMbr_StgCds_PrdID_Fmt, &cTempPrdColNm, &csubTempFactDimMbr,
                             &cdimDimTypeDS);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 12, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  Get a subset of the prior period GL transactions from Stage and dump into a work area data set.
     *  We only want those transactions where the YTD amount flag is true.
     *
     *  Subset by:
     *     > prior time period
     *     > transaction_amt_ytd_flg eq true
     *     > analysis hierarchy leaf nodes
     *     > currency hierarchy leaf nodes
     */
  %sesFMS_getPriorTransFromStage( &cStageLib, &cStageTransTableNm,
                                cPriorTimeMbrCode_Map_PrdId_DS, &cAnalysisFormat,
                                &cCurrencyFormat, &cPriorTransYtdDS, &cPriorAmtColNm,
                                &cPriorTimeMbr_StgCds_PrdID_Fmt, &cTempPrdColNm);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 16, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*  Build a list that include PERIOD_ID Column  */
  /* Copy */
  data lTranCols_PrdID;
    set lTranCols;
  run;

  /* and add */
  proc sql noprint;
    insert into  lTranCols_PrdId values ("&cTempPrdColNm");
  quit;

  /*  Sort and summarize the Prior Period YTD transactions  */
  %sesFMS_sortDataSet( &cPriorTransYtdDS, lTranCols_PrdId);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 18, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_summarizeDataSet( &cPriorTransYtdDS, lTranCols_PrdId, &cPriorAmtColNm, 1);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 19, &cSCLEntryNm);
      %goto ERROR;
  %end ;

   /*  Sort and summarize the Current Period YTD subsetted work area data set  */
  %sesFMS_sortDataSet( &cCurrentTransYtdDS, lTranCols_PrdId);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 20, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_summarizeDataSet( &cCurrentTransYtdDS, lTranCols_PrdId, &cCurrentAmtColNm, 1);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 21, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /***************************************************************
  *
  *  Do the math on YTD transactions (current - prior period)
  *
  ***************************************************************/

  %sesFMS_calcPeriodActivity( &cCurrentTransYtdDS, &cPriorTransYtdDS, lTranCols_PrdId,
                              &cCurrentAmtColNm, &cPriorAmtColNm, &cConvertedGlTransDS,
                              &cTempPrdColNm);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 22, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*
   *  Append newly derived Period Activity trans. to non-derived Period Activity trans.
   */
  %sesFMS_appendDS(&cConvertedGlTransDS, &cCurrentTransPaDS);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 15, &cCurrentTransPaDS, &cConvertedGlTransDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*
   *  Add the Source column with the Base Code value.  We add this here because this dimension
   *  only exists in the FM Mart.
   */

  %sesFMS_addSourceCode( &cConvertedGLTransDS, &cSourceColNm, &cSource_BaseCode );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 29, &cConvertedGlTransDS, SOURCE_CD, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Fetch Journal Transactions
     *
     *  Subset by:
     *     > range of Process_dttm
     *     > list of current time period(s)
     *     > list of analysis(es)
     *     > part of listed currencies
     *
     */

  %global nJrnlCnt;
  %let nJrnlCnt=0;
  %sesFMS_getJournsFromStage( &cStageLib, &cCurrTimeMbrCode_Map_PrdId_DS, &cJournalDS,
                              lCycleDimTypesCols, &cStageJrnlDS, &cStageJrnlDetailDS,
                              &cSourceColNm, &cSource_BaseJournCode, &csubTempFactDimMbr,
                              &cdimDimTypeDS);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 44, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Append Journal DS to Basefact DS, only if we have found journals
     */
  %if &nJrnlCnt ne 0 %then %do;
    %sesFMS_appendDS( &cConvertedGLTransDS, &cJournalDS );
    %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 15, &cJournalDS, &cConvertedGLTransDS, &cSCLEntryNm);
      %goto ERROR;
    %end ;
  %end ;

  /*  Does our resulting table have any records in it?  */

  %if %sysfunc(exist(&cConvertedGLTransDS)) %then %do;
    proc sql noprint;
      select count(*)
        into :nCnt
        from  &cConvertedGLTransDS;
      quit;
  %end;
  %else %do;
    %sesSOL_logError( &cLogEntry, L, 24, &cConvertedGlTransDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %if &nCnt eq 0 %then %do;
    /*  Create a table of all Warnings */
    %sesFMS_buildStatusDS(&cCurrTimeMbrId_Map_PrdId_DS, &cAnalysisDS_SDM_RKs, &cStatusDS, 3);
    %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 47, &cSCLEntryNm);
      %goto ERROR;
    %end ;

    /*  No records found to update, set status to warning  */
    %let fmsetl_statusId=3;  /* success=0, critical error=1, validation error=2, warning=3  */
    %sesSOL_logError( &cLogEntry, L, 25, &cSCLEntryNm);
  %end;
  %else %do; /* Very long else */
    /*  For this set of sort and summaries, we want to include the SOURCE_CD column  */
    proc sql noprint;
      create table lTranCols_with_SOURCE as select * from lTranCols;
      insert into  lTranCols_with_SOURCE values( "AFFECTED_TIME_PERIOD_ID" );
      insert into  lTranCols_with_SOURCE values( "&cSourceColNm" );
    quit;

    /*  Sort and summarize work area data set  */
    %sesFMS_sortDataSet( &cConvertedGLTransDS, lTranCols_with_SOURCE );
    %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 26, &cConvertedGlTransDS, &cSCLEntryNm);
      %goto ERROR;
    %end ;

    %sesFMS_summarizeDataSet( &cConvertedGLTransDS, lTranCols_with_SOURCE, &cTransAmtColNm, 0 );
    %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 27, &cConvertedGlTransDS, &cSCLEntryNm);
      %goto ERROR;
    %end ;

    %sesFMS_renameTransCols( &cConvertedGLTransDS, lTranCols );

/*******************************************************************************************/
  %macro mbrConcat ;
     %global fmsetl_concatMbrStmt ;
     %let fmsetl_concatMbrStmt= ;
     data _null_ ;
       length CONCATSTMT $32767 ;
       retain CONCATSTMT ;
       set &cDimTypeWorkAreaDS end=nEnd ;
       if _n_ = 1 then do ;
         CONCATSTMT = strip(CONCATSTMT) ||
                      "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
       end ;
       else do ;
         CONCATSTMT = strip(CONCATSTMT) || " || ',' || " ||
                      "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
       end ;
       if nEnd then do ;
         call symput("fmsetl_concatMbrStmt", CONCATSTMT) ;
       end ;
     run ;
     %put FMSETL_CONCATMBRSTMT = &fmsetl_concatMbrStmt;
  %mend ;

  %mbrConcat ;

    %let cBfDtypes=;
    %let ext_mem = 0;


  %let format_missing = 0;

  %let entry = imptdata;
  %let putlog = Y;

  /*
   *  Run validation and import into the Cycles basefact table
   *  depending on TRADER in cycle input
   */

  /* Check if Trader exsits */
  %let TRADER_EXIST = 0;
  proc sql noprint;
       select count(*)
       into :TRADER_EXIST
       from &cDimDimTypeDS
       where kupcase(strip(dim_type_cd)) = 'TRADER'
       ;
  quit;

  /* Perform setup form imptdata macros */

  /* Check if &cSDMLib..SAS_DIM_TYPE exists */
  %let _tmpDSN = &cSDMLib..SAS_DIM_TYPE;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=13
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  %if  &TRADER_EXIST > 0 %then %do;
     %let trader_dim_type_rk = 0;
     proc sql noprint;
       select DIM_TYPE_ID into :trader_dim_type_rk from &_tmpDSN
       where upcase(DIM_TYPE_CD) = 'TRADER' and VALID_TO_DTTM  >= &nProcessedDTTM_To;
     quit;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     %if &trader_dim_type_rk = 0 %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=4,
                 msgTxt1=<TRADER>, msgPutlog=&putlog);
        %goto exit;
     %end;
  %end;

  /* Check if &cSDMLib..SAS_DIMENSION exists */
  %let _tmpDSN = &cSDMLib..SAS_DIMENSION;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=14
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  /* Check for the existence of active records in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from &_tmpDSN
     ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  %if  &TRADER_EXIST > 0 %then %do;
     %let _tmpWhere = where=(DIM_TYPE_ID = &trader_dim_type_rk and
                             VALID_TO_DTTM >= &nProcessedDTTM_To);
     data WORK._TMPTRADERDIMS;
        set &_tmpDSN (&_tmpWhere);
        CODE = DIMENSION_CD;
        keep CODE INTERNAL_DIMENSION_ID;
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     /* Check for the existence of active trader dimensions in the dimension table */
     proc sql noprint;
        select count(*) INTO :_rec_cnt
        from WORK._TMPTRADERDIMS
        ;
     quit;
     %checkErr;
     %if &_tmpRC %then %goto exit ;

     %if (&_rec_cnt < 1) %then %do;
       %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
               , msgTxt1=<TRADER>, msgPutlog=&putlog);
       %goto exit ;
     %end;

     proc sort data=WORK._TMPTRADERDIMS;
        by CODE;
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit;
  %end;

  /* Check if cycle's dimension table exists */
  %let _tmpDSN = &cDimWorkAreaDS;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=16
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto exit;
  %end;

  proc sort data=&_tmpDSN;
     by CODE;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if  &TRADER_EXIST > 0 %then %do;
     %let trader_int_rk = 0;
     data _null_;
        merge WORK._TMPTRADERDIMS (in = inA)
              &_tmpDSN (in = inB);
           by CODE;
        if (inA and inB) then
           call symput("trader_int_rk",INTERNAL_DIMENSION_ID);
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     %if &trader_int_rk = 0 %then %do;
        %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
                , msgTxt1=<TRADER>, msgPutlog=&putlog);
        %goto exit ;
     %end ;
  %end;

  /* Check if &cSDMLib..SAS_MEMBER exists */
  %let _tmpDSN = &cSDMLib..SAS_MEMBER;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=17
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=member, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;

  /* Check for the existence of active records in the member table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from &_tmpDSN
     ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  %if  &TRADER_EXIST > 0 %then %do;
     %if (&cCkIcTradingPartners ne Y) and (&cCkIcTradingPartners ne N) %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
                 msgTxt1=<ic_check_ind : &cCkIcTradingPartners>, msgPutlog=&putlog);
        %goto exit;
     %end;

     %if (&cCkNonIcTradingMembers ne Y) and (&cCkNonIcTradingMembers ne N) %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
                 msgTxt1=<non_ic_check_ind : &cCkNonIcTradingMembers>, msgPutlog=&putlog);
        %goto exit;
     %end;
  %end;

  /*
   * build a list of the basefact dimension types in a macro var
   *
   * Name: cBfDtypes & dtype_rks
   * Contents: list of dimensions and dimension types separated by commas
   * Example: TIME, INTORG, ANALYSIS, CURRENCY, ....
   *
   * Name: dimcnt
   * Contents: number of items in above lists
   */

  %let _tmpSortVar = DIMENSION_CD;
  data WORK._TMPALLDIMS;
     set &cDimWorkAreaDS;
     &_tmpSortVar = CODE;
     keep &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPALLDIMS;
     by &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %let _tmpWhere = where=(VALID_FROM_DTTM <= &nProcessedDTTM_To and
                          VALID_TO_DTTM >= &nProcessedDTTM_To);

  data WORK._TMPSDMDIM;
     set &cSDMLib..SAS_DIMENSION(&_tmpWhere);
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPSDMDIM;
     by &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  data WORK._TMPALLDIMS
       WORK._TMPMISSINGDIMS;
     merge WORK._TMPALLDIMS (in = inA)
           WORK._TMPSDMDIM (in = inB);
        by &_tmpSortVar;
     keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID;
     if (inA and ^inB) then
        output WORK._TMPMISSINGDIMS;
     else if (inA) then
        output WORK._TMPALLDIMS;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc sql noprint;
     select count(*) INTO :missingCnt FROM WORK._TMPMISSINGDIMS;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if (&missingCnt > 0) %then %do;
     data _null_;
        set WORK._TMPMISSINGDIMS (obs=1);
        call symput("missing_dim", strip(DIMENSION_CD));
     run;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
             , msgTxt1=<&missing_dim>, msgPutlog=&putlog);
     %goto exit ;
  %end;

  %let _tmpSortVar = DIM_TYPE_ID;
  proc sort data = WORK._TMPALLDIMS;
     by &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %let _tmpWhere = where=(VALID_FROM_DTTM <= &nProcessedDTTM_To and
                          VALID_TO_DTTM >= &nProcessedDTTM_To);

  data WORK._TMPSDMDIMTYPE;
     set &cSDMLib..SAS_DIM_TYPE (&_tmpWhere);
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPSDMDIMTYPE;
     by &_tmpSortVar;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  data WORK._TMPALLDIMS;
     merge WORK._TMPALLDIMS (in = inA)
           WORK._TMPSDMDIMTYPE;
        by &_tmpSortVar;
     keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID DIM_TYPE_CD;
     if (inA) then
        output;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc sql noprint;
     select count(*) INTO :dimcnt FROM WORK._TMPALLDIMS;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  data _null_;
     set WORK._TMPALLDIMS;
     if (_n_ = 1) then do;
        call symput("cBfDtypes",DIM_TYPE_CD);
        call symput("dtype_rks",DIM_TYPE_ID);
     end;
     else do;
        call symput("cBfDtypes",strip(symget("cBfDtypes"))||"|"||DIM_TYPE_CD);
        call symput("dtype_rks",strip(symget("dtype_rks"))||","||DIM_TYPE_ID);
     end;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  /* Check if &cSDMLib..SAS_PROPERTY exists */
  %let _tmpDSN = &cSDMLib..SAS_PROPERTY;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=19
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=property, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;

  %if  &TRADER_EXIST > 0 %then %do;
     %let ic_prop_rk = 0;
     proc sql noprint;
       select PROPERTY_ID into :ic_prop_rk from &_tmpDSN
       where upcase(PROPERTY_CD) = 'INTERCOMPANY';
     quit;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     %if &ic_prop_rk = 0 %then %do;
        %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=9
                , msgTxt1=<INTERCOMPANY>, msgPutlog=&putlog);
        %goto exit ;
     %end;
  %end;

  /* Retrieve INTERNAL_DIMENSION_ID for the ACCOUNT dimension in the cycle_dim_table */
  %let _tmpDSN = &cSDMLib..SAS_DIM_TYPE;
  %let acct_dim_type_rk = 0;
  proc sql noprint;
    select DIM_TYPE_ID into :acct_dim_type_rk from &_tmpDSN
    where DIM_TYPE_CD = 'ACCOUNT' and VALID_TO_DTTM  >= &nProcessedDTTM_To;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if &acct_dim_type_rk = 0 %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=4,
              msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Create table containing codes and dimension internal rk's for the account dimension */
  %let _tmpDSN = &cSDMLib..SAS_DIMENSION;
  %let _tmpSortVar = DIMENSION_CD;
  %let _tmpWhere=where=(DIM_TYPE_ID = &acct_dim_type_rk and
                        VALID_TO_DTTM >= &nProcessedDTTM_To);

  data WORK._TMPSORT;
     set &_tmpDSN(&_tmpWhere);
     keep &_tmpSortVar INTERNAL_DIMENSION_ID;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc sort data=WORK._TMPSORT;
     by &_tmpSortVar;
  run;

  data WORK._TMPACCTDIMS;
     set WORK._TMPSORT;
     CODE = &_tmpSortVar;
     keep CODE INTERNAL_DIMENSION_ID;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  /* Check for the existence of active account dimensions in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from WORK._TMPACCTDIMS
     ;
  quit;
  %checkErr;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
            , msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
    %goto exit ;
  %end;

  proc sort data=WORK._TMPACCTDIMS;
     by CODE;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  /* Retrieve the Dimension Internal Id for the account dimension in cycle dimension table */
  %let acct_int_rk = 0;
  data _null_;
     merge WORK._TMPACCTDIMS (in = inA)
           &cDimWorkAreaDS (in = inB);
        by CODE;
     if (inA and inB) then
        call symput("acct_int_rk", INTERNAL_DIMENSION_ID);
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if &acct_int_rk = 0 %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
             , msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
     %goto exit ;
  %end ;

  %if  &TRADER_EXIST > 0 %then %do;
     /* Create FMT_INTERCOMPANY -- format needed for determining intercompany accounts */
     %let _tmpDSN = &cSDMLib..SAS_MEMBER_PROPERTY;
     %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
        %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=20
                , msgPutlog=N);
        %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
                , msgTxt1=member_property, msgTxt2=&sesgmsg_msgReturn
                , msgPutlog=&putlog);
        %goto exit ;
     %end ;

     %let _tmpFormat=FMT_INTERCOMPANY;
     %let _tmpStart=MEMBER_ID;
     %let _tmpLabel=PROPERTY_VALUE;
     %let _tmpWhere=where=(PROPERTY_ID = &ic_prop_rk and
                           INTERNAL_DIMENSION_ID = &acct_int_rk and
                           VALID_FROM_DTTM <= &nProcessedDTTM_To and
                           VALID_TO_DTTM >= &nProcessedDTTM_To);

     data work.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
        retain FMTNAME "&_tmpFormat";
        retain TYPE "N";
        retain HLO "";
        set &_tmpDSN(&_tmpWhere rename=(&_tmpStart=START &_tmpLabel=LABEL));
        if (LABEL = 'True') then
           LABEL = 'Y';
        else
           LABEL = 'N';
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     /* Check for the existence of values for the "INTERCOMPANY" property */
     proc sql noprint;
        select count(*) INTO :_rec_cnt
        from WORK.&_tmpFormat
        ;
     quit;
     %checkErr;
     %if &_tmpRC %then %goto exit ;

     %if (&_rec_cnt < 1) %then %do;
       %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
               , msgPutlog=&putlog);
       %goto exit ;
     %end;

     proc format cntlin=&_tmpFormat library=WORK;
     run;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
                 msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
        %goto exit;
     %end;
  %end;

  /* FMT_TO_DIM */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_TO_DIM;
  %let _tmpStart=DIM_TYPE_CD;
  %let _tmpLabel=DIMENSION_CD;
  data WORK.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
     retain FMTNAME "&_tmpFormat";
     retain TYPE "C";
     retain HLO "";
     set &_tmpDSN(rename=(&_tmpStart=START &_tmpLabel=LABEL));
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_DIMS */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DIMS;
  %let _tmpStart=DIM_TYPE_ID;
  %let _tmpLabel=INTERNAL_DIMENSION_ID;
  data work.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN(rename=(&_tmpStart=START &_tmpLabel=LABEL));
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_DTYPE_ID */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DTYPE_ID;
  %let _tmpStart=DIMENSION_CD;
  %let _tmpLabel=DIM_TYPE_ID;

  data work.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN(rename=(&_tmpStart=START &_tmpLabel=LABEL));
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %checkErr;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

    /* FMT_MEMTYPE -- format to be used for data validation */

    %let _tmpFormat=FMT_MEMTYPE;
    %let _tmpStart=MEMBER_ID;
    %let _tmpLabel=DIM_TYPE_CD;
    %let _tmpWhere=where=(VALID_FROM_DTTM <= &nProcessedDTTM_To and
                          VALID_TO_DTTM >= &nProcessedDTTM_To);

    %let _tmpDSN=WORK._TMPALLDIMS;
    %let _tmpSortVar=INTERNAL_DIMENSION_ID;
    proc sort data = &_tmpDSN nodupkey;
       by &_tmpSortVar;
    run;
    %checkErr;
    %if &_tmpRC %then %goto exit;

    %let _tmpDSN=&cSDMLib..SAS_MEMBER;

    data WORK._TMPMEMBER;
       set &_tmpDSN(&_tmpWhere);
    run;
    %checkErr;
    %if &_tmpRC %then %goto exit;

    proc sort data = WORK._TMPMEMBER;
       by &_tmpSortVar;
    run;
    %checkErr;
    %if &_tmpRC %then %goto exit;

    data work.&_tmpFormat(keep=START LABEL FMTNAME TYPE HLO);
       retain FMTNAME "&_tmpFormat";
       retain HLO "";
       retain TYPE "N";
       merge WORK._TMPALLDIMS
            (in=inA rename=(&_tmpLabel=LABEL))
             WORK._TMPMEMBER
            (in=inB rename=(&_tmpStart=START));
          by &_tmpSortVar;

       if (inA and inB) and (DIM_TYPE_CD ^= 'TRADER') then
          output;
    run;
    %checkErr;
    %if &_tmpRC %then %goto exit;

    data WORK.&_tmpFormat;
       set WORK.&_tmpFormat end=last;
       output;
       if (last) then do;
          HLO = 'O';
          START = .;
          LABEL = '*** ERROR ***';
          FMTNAME = "&_tmpFormat";
          TYPE = "N";
          output;
       end;
    run;
    %checkErr;
    %if &_tmpRC %then %goto exit;

    proc format cntlin=&_tmpFormat library=WORK;
    run;
    %checkErr;
    %if &_tmpRC %then %goto exit;

    %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
       %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
                msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
       %goto exit;
    %end;

  /*******************************************************************************/
  /* CREATION OF HASH TABLE TO REPLACE INEFFICIENT FORMATING FOR EACH INDIVIDUAL */
  /* DIMENSION TYPE.                                                             */
  /*******************************************************************************/
  %macro createHashTable;

    %let _tmpDSN=&cSDMLib..SAS_MEMBER;

    proc sql;
      create table work.memberHashTable as select internal_dimension_id, member_cd, member_id
         from &_tmpDSN
         where internal_dimension_id in (select internal_dimension_id from WORK._TMPALLDIMS) and
               VALID_FROM_DTTM <= &nProcessedDTTM_To and VALID_TO_DTTM >= &nProcessedDTTM_To;
    quit;
    %checkErr;

  %mend createHashTable;

  %createHashTable ;

  %if &_tmpRC %then
     %goto exit;

  /* Check if hash table has been successfully created */
  %let _tmpDSN = work.memberHashTable;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=22
             , msgPutlog=N);
     %goto exit;
  %end;

  /*******************************************************************************/
  %if  &TRADER_EXIST > 0 %then %do;
     /* Determine the member_id of the EXT member of the TRADER dimension */

     proc sql noprint;
       select MEMBER_ID into :ext_mem from &_tmpDSN
       where INTERNAL_DIMENSION_ID = &trader_int_rk and MEMBER_CD = 'EXT';
     quit;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     %if &ext_mem = 0 %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=12,
                 msgTxt1=<EXT>, msgPutlog=&putlog);
        %goto exit;
     %end;

     /* Determine the member_id of the ALL member of the TRADER dimension */

     %let all_mem = 0;
     proc sql noprint;
       select MEMBER_ID into :all_mem from &_tmpDSN
       where INTERNAL_DIMENSION_ID = &trader_int_rk and MEMBER_CD = 'ALL';
     quit;
     %checkErr;
     %if &_tmpRC %then %goto exit;

     %if &all_mem = 0 %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=12,
                 msgTxt1=<ALL>, msgPutlog=&putlog);
        %goto exit;
     %end;
  %end;
/****************************************************************************************/
/*----------------Added by TS --------------------*/
    %macro mbrLst;
       %global fmsetl_dimTypeList;
       %let    fmsetl_dimTypeList=;
       data _null_;
           length dimTypeList_var $32767;
     retain dimTypeList_var;
           set &cdimTypeWorkAreaDS end=nEnd;
           dimTypeList_var = strip(dimTypeList_var) || ' ' ||quote(strip(CODE))||'n';

    if nEnd then do;
      call symput ("fmsetl_dimTypeList", strip(dimTypeList_var));
    end;


       run ;

       filename _memForm  TEMP;
       data _null_;
           set &cdimTypeWorkAreaDS;
           file _memForm;
           put '"' code +(-1) '"n = "dim' _N_ + (-1) '"n'  ;
       run;

    %mend mbrLst;

    %mbrLst ;
/*----------------Added by TS --------------------*/

    %if  &TRADER_EXIST > 0 %then %do;
      %sesFMS_imptdataWithTr (gl_trans_sum_table=&cConvertedGLTransDS,
                    update_tablename=&cTmpFactWork_Nm,
                    ic_check_ind=&cCkIcTradingPartners,
                    non_ic_check_ind=&cCkNonIcTradingMembers,
                    cycle_dim_table=&cDimWorkAreaDS,
                    bfdtypes=&cBfDtypes,
                    ext_mem=&ext_mem,
                    fmsetl_ProcessedDTTM_To=&nProcessedDTTM_To,
                    dataLoadObjTypeId=&cDLObjTypeId,
                    dataLoadObjId=&cDLObjId);
    %end;
    %else %do;
      %sesFMS_imptdataWithoutTr (gl_trans_sum_table=&cConvertedGLTransDS,
                    update_tablename=&cTmpFactWork_Nm,
                    cycle_dim_table=&cDimWorkAreaDS,
                    bfdtypes=&cBfDtypes,
                    fmsetl_ProcessedDTTM_To=&nProcessedDTTM_To,
                    dataLoadObjTypeId=&cDLObjTypeId,
                    dataLoadObjId=&cDLObjId);
    %end;

    /********************************************************************************
     *
     *  Gather some stats on the outcome of the job, to send back to calling code
     *
     ********************************************************************************/

      /*  Check for a Critical Error Occurance  */
    %if &fmsetl_statusId = 1 %then %do;
        %goto ERROR;
    %end ;


      /*  Create a table of all Successful imports */
    %sesFMS_buildStatusDS(&cCurrTimeMbrId_Map_PrdId_DS, &cAnalysisDS_SDM_RKs, &cStatusDS, 0);
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 47, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count BadRecs */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.badrecs, INVALID_ERROR_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.badrecs, &cSCLEntryNm);
        %goto ERROR;
    %end ;

      /*  Count Org eq Trader errors */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.org_eq_trader, ORG_EQ_TRADER_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.org_eq_trader, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count IC errors */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.icrecs, IC_ERROR_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.icrecs, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count Non IC errors */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.nonicrecs, NON_IC_ERROR_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.nonicrecs, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count successful Imports  */
    %sesFMS_updateValidStatusCnt( &cStatusDS, &cTmpFactWork_Nm, IMPORTED_TRANS_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 49, &cTmpFactWork_Nm, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count Zero values  */
    %sesFMS_updateZeroStatusCnt( &cStatusDS, &cTmpFactWork_Nm, ZERO_TRANS_CNT);
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 56, &cTmpFactWork_Nm, &cSCLEntryNm);
        %goto ERROR;
    %end ;


    /*  Print Error Report(s)  */

    %sesFMS_generateErrorReports( &cStatusDS, lDimTypeFactSetOrder, WORK.BADRECS,
                                            WORK.ORG_EQ_TRADER, WORK.ICRECS,
                                            WORK.NONICRECS, &cDLObjId);
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 36, &cSCLEntryNm);
        %goto ERROR;
    %end ;

    %if &nActionType NE 4 and &nActionType NE 5 %then %do;
        %sesFMS_uploadTempFactToSDM( lDimTypeFactSetOrder, &cTmpFactSDM_Nm, &cTmpFactWork_Nm, &cStatusDS );
        %if &_FMErrorCode = 0 %then %do;
            %sesSOL_logError( &cLogEntry, L, 52, &cTmpFactWork_Nm, &cTmpFactSDM_Nm, &cSCLEntryNm);
            %goto ERROR;
        %end ;
    %end;
    %else %do;
      /*  Create temporary REMOVE table for matching actions  */
        %sesFMS_replaceMatching( &cTmpRemoveROFactSDM_Nm, &cTmpRemoveRWFactSDM_Nm,
                                 &cStatusDS, &nActionType, lDimTypeFactSetOrder,
                                 &nBaseMbrSDM_Id, &nBaseJournMbrSDM_Id,
                                 &cSdmLib, &nSdmROFactSetId, &nSdmRWFactSetId, &cDLObjTypeId,
                                 &cTmpFactWork_Nm, &cTmpFactSDM_Nm, &cETL_instanceID);
        %if &_FMErrorCode = 0 %then %do;
            %sesSOL_logError( &cLogEntry, L, 53, &cSCLEntryNm);
            %goto ERROR;
        %end ;
    %end;
  %end;

  /*  Upload Status Report to SASSDM  */
  %sesFMS_uploadStatusTableToSDM(&cStatusDS, &cSDMStatusDS);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 52, &cSDMStatusDS, &cStatusDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*  Dump the Time  */
  %let nTrackTime=%sysevalf(%qsysfunc(datetime()) - &nTrackTime); ;
  %let TimeData=%sesFMS_getTimer;
  %put TimeData:&TimeData;

  %if &TimeData > 0  %then %do;
      %put TIMER: total time to run YTD to Period Activity conversion: &nTrackTime ;
  %end ;

  %goto THEEND;

  %ERROR:
    %let fmsetl_statusId=1;  /* success=0, critical error=1, validation error=2, warning=3  */

  %THEEND:

%mend sesFMS_LoadCycleBaseFact;

/**********************************************************
 * sesFMS_macrosToDataSets.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_macrosToDataSets.sas                                                           */
/*  Purpose: Parse information from macro variables into data sets.                          */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_macrosToDataSets (cCurrencyDS,
                                 cDimTypeDS,
                                 cDimTypeDS1,
                                 cDimDS,
                                 cDimDimTypeDS,
                                 cAnalysisDS,
                                 cAnalysisDS_SDM_RKs,
                                 cDLObjIdDS,
                                 cCurrTimeMbrIdsSDM,
                                 cCurrTimeMbrCodes,
                                 cPriorTimeMbrIdsSDM,
                                 cPriorTimeMbrCodes,
                                 cCyclePrdIds,
                                 cIsFirstPrdYr,
                                 cRunYTDToPA
                                 );

  %let cMethodNm = sesFMS_macrosToDataSets ;
  %let nTrackTime= %sysfunc(datetime());

  data &cCurrencyDS       (keep=CODE VALUE)
     &cDimTypeDS          (keep=CODE VALUE)
     &cDimTypeDS1         (keep=CODE VALUE)
     &cDimDS              (keep=CODE VALUE)
     &cDimDimTypeDS       (keep=DIM_TYPE_CD DIM_CD)
     &cAnalysisDS         (keep=CODE VALUE)
     &cAnalysisDS_SDM_RKs (keep=CODE VALUE)
     &cDLObjIdDS          (keep=CODE VALUE)
     &cCurrTimeMbrIdsSDM  (keep=CODE VALUE)
     &cCurrTimeMbrCodes   (keep=CODE VALUE)
     &cPriorTimeMbrIdsSDM (keep=CODE VALUE)
     &cPriorTimeMbrCodes  (keep=CODE VALUE)
     &cCyclePrdIds        (keep=CODE VALUE)
     &cIsFirstPrdYr       (keep=CODE VALUE)
     &cRunYTDToPA         (keep=CODE VALUE)
     ;

     length CODE $ %eval(32*&FM_DBCSMULT);
     length DIM_TYPE_CD $ %eval(32*&FM_DBCSMULT);
     length DIM_CD $ %eval(32*&FM_DBCSMULT);
     length VALUE $ %eval(32*&FM_DBCSMULT);
     length TMPCODE $254;
     length TIME_MBR_SDM_ID 8;
     length ANALYSIS_MBR_SDM_ID 8;


     do nXX = 1 to 9999999999;
       CODE = strip( kscan("&fmsetl_dimensionTypeCodes", nXX, "09"x) );
       if CODE eq "" then leave;
       VALUE = "OK";
       output &cDimTypeDS;
       VALUE = CODE;
       if (klength(VALUE) gt 26) then do;
         VALUE = kcompress("_" !! put(nXX, 12.) !! "_" !! strip(VALUE));
         VALUE = ksubstr(VALUE, 1, min(klength(VALUE), 26));
       end;
       output &cDimTypeDS1;
     end;

     do nXX = 1 to 9999999999;
       CODE = strip( kscan("&fmsetl_dimensionCodes", nXX, "09"x) );
       if CODE eq "" then leave;
       VALUE = "OK";
       output &cDimDS;
     end;

     do nXX = 1 to 9999999999;
       DIM_CD      = strip( kscan("&fmsetl_dimensionCodes", nXX, "09"x) );
       DIM_TYPE_CD = strip( kscan("&fmsetl_dimensionTypeCodes", nXX, "09"x) );
       if DIM_TYPE_CD eq "" then leave;
       if DIM_CD eq "" then leave;
       output &cDimDimTypeDS;
     end;

     do nXX = 1 to 9999999999;
       CODE = strip( kscan("&fmsetl_currencyCodes", nXX, "09"x) );
       if CODE eq "" then leave;
       VALUE = "OK";
       output &cCurrencyDS;
     end;

     do nXX = 1 to 9999999999;
       CODE = strip( kscan("&fmsetl_analysisCodes", nXX, "09"x) );
       if CODE eq "" then leave;
       VALUE = "OK";
       output &cAnalysisDS;
     end;

     do nXX = 1 to 9999999999;
       CODE = strip( kscan("&fmsetl_analysisIds_SDM", nXX, "09"x) );  /* tab delimiter */
       if CODE eq "" then leave;
       VALUE = "OK";
       output &cAnalysisDS_SDM_RKs;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_dataLoadObjectIds", nXX, "09"x) );
       if TMPCODE eq "" then leave;
       TIME_MBR_SDM_ID = input(kscan(TMPCODE, 1, ":"), 32.);
       ANALYSIS_MBR_SDM_ID = input(kscan(TMPCODE, 2, ":"), 32.);
       VALUE = input(kscan(TMPCODE, 3, ":"), 32.);
       CODE = strip(put(TIME_MBR_SDM_ID, Z30.)) || "+" || strip(put(ANALYSIS_MBR_SDM_ID, Z30.));
       output &cDLObjIdDS;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_currentTimeMbrIds_SDM", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrID  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = input(kscan(TMPCODE, 2, ":"), 32.);
       output &cCurrTimeMbrIdsSDM;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_currentTimeMbrCodes", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrCodes  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
       output &cCurrTimeMbrCodes;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_priorTimeMbrIds_SDM", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrID  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = input(kscan(TMPCODE, 2, ":"), 32.);
       output &cPriorTimeMbrIdsSDM;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_priorTimeMbrCodes", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrCodes  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
       output &cPriorTimeMbrCodes;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_cyclePrdIds", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:PeriodID  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = input(kscan(TMPCODE, 2, ":"), 32.);
       output &cCyclePrdIds;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_isFirstPeriodOfFiscalYear", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:Y or N  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
       output &cIsFirstPrdYr;
     end;

     do nXX = 1 to 9999999999;
       TMPCODE = strip( kscan("&fmsetl_runYTDtoPeriodActivity", nXX, "09"x) );
       if TMPCODE eq "" then leave; /*  Format = PeriodID:Y or N  */
       CODE = input(kscan(TMPCODE, 1, ":"), 32.);
       VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
       output &cRunYTDToPA;
     end;

  run;


  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=1;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %let _FMErrorCode=0;
    %sesSOL_logError( YTDConv, L, &error_number, &cMethodNm);

  %THEEND:

%mend sesFMS_macrosToDataSets;

/**********************************************************
 * sesFMS_replaceMatching
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_replaceMatching.sas                                                      */
/*  Purpose: Replace matching process.                                                       */
/*                                                                                           */
/*  History: 29 April 2009 by Tae Sik Han : Converted from SAS to SAS macro program.         */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_replaceMatching(cTmpROFactSDM_Nm,
                                cTmpRWFactSDM_Nm,
                                cStatusDS,           /* not used any more */
                                nActionType,
                                lCycleDimTypes,
                                nBaseMbrSDM_Id,      /* not used any more */
                                nBaseJournMbrSDM_Id, /* not used any more */
                                cSdmLib,
                                nSdmROFactSetId,
                                nSdmRWFactSetId,
                                cDLObjTypeId,
                                cSASFactsDS,
                                cSDMFactsDB,
                                cETL_instanceID);

%let cMethodNm = sesFMS_replaceMatching;
%let cSDMFactsDB_NEW=/*PUBLIC.*/ %scan(&cSDMFactsDB, 2);
%let cTmpROFactSDM_Nm=/*PUBLIC.*/ %scan(&cTmpROFactSDM_Nm, 2);
%let cTmpRWFactSDM_Nm=/*PUBLIC.*/ %scan(&cTmpRWFactSDM_Nm, 2);


%let _FMErrorCode=1; /* Just for test */

/* Initial assignment. It is changed depending on nActionType */
%let cTmp_ETL_BASE    =  _etl_sas_fact_base_&cETL_instanceID;
%let cTmp_ETL_NO_BASE =  _etl_sas_fact_no_base_&cETL_instanceID;

%let cTmp_RO_BASE    = _sas_fact_ro_base_&cETL_instanceID;
%let cTmp_RW_BASE    = _sas_fact_rw_base_&cETL_instanceID;
%let cTmp_RO_NO_BASE = _sas_fact_ro_no_base_&cETL_instanceID;
%let cTmp_RW_NO_BASE = _sas_fact_rw_no_base_&cETL_instanceID;

%let nowDTTM = %qsysfunc(datetime());
%let nTrackTime = %qsysfunc(datetime());

%let cTmpROFactSDM_Nm2=&cTmpROFactSDM_Nm._&cETL_instanceID;
%let cTmpRWFactSDM_Nm2=&cTmpRWFactSDM_Nm._&cETL_instanceID;

/*  If this is not a Replace Matching action, return , nActionType should be 4 or 5 */
%if &nActionType NE 4 and &nActionType NE 5 %then %do;
  %goto THEEND;
%end;

/* Find the location of TIME, ACCOUNT, SOURCE, and CURRENCY dimension types */
%let ord_time=;
%let ord_account=;
%let ord_source=;
%let ord_currency=;

data _null_;
    set &lCycleDimTypes end=IsLast;
    if code='SOURCE' or code='source' then do;
        call symput('ord_source', _N_-1);
    end;
    if code='CURRENCY' or code = 'currency' then do;
        call symput('ord_currency', _N_-1);
    end;
    if IsLast = 1 then
        call symput('num_dim_type', _N_);
run;


%let dim_col_def=;  /* List of dimension type column definition in DDL */
%let dim_col=;      /* List of dimension type column names             */
%let dim_col_cnt=0;
%let MAX_INDEX_COL_MYSQL=16;

%do i=0 %to &num_dim_type-1;
    %if &i = 0 %then %do;
        %let dim_col_def=DIM_&i._MEM_ID integer;
        %if &i ^= &ord_source & &i ^= &ord_currency %then %do;
              %let dim_col=DIM_&i._MEM_ID;
              %let dim_col_cnt=%eval(&dim_col_cnt + 1);
        %end;
    %end;
    %else %do;
        %let dim_col_def=&dim_col_def,DIM_&i._MEM_ID integer;
         %if (&dim_col_cnt < &MAX_INDEX_COL_MYSQL) & (&i ^= &ord_source) & (&i ^= &ord_currency)  %then %do;
            %if "&dim_col" = "" %then %do;
                %let dim_col=DIM_&i._MEM_ID;
            %end;
            %else %do;
                %let dim_col=&dim_col, DIM_&i._MEM_ID;
            %end;
         %let dim_col_cnt=%eval(&dim_col_cnt + 1);
         %end;
    %end;
%end;



/*------------------------------------------------------------------------*/
/* Decide the storage type of a temp fact table to be created in SASSDM */
/*------------------------------------------------------------------------*/
%let stg_type_tmp=myisam;

/*-----------------------------------------*/
/* Move ETL base fact from SAS into SASSDM */
/*-----------------------------------------*/
proc sql noprint;
     connect to &sesetl_EngineType as PASSTHRU ( &sesetl_BestLogin &sesetl_ConnectionOptions );

     execute (DROP TABLE IF EXISTS &cSDMFactsDB_NEW
     ) by PASSTHRU;

     execute(CREATE TABLE &cSDMFactsDB_NEW  (
               OBJECT_TYPE_ID INTEGER NOT NULL default 0,
               OBJECT_ID INTEGER NOT NULL default 0,
               &dim_col_def,
               FACT_VALUE DOUBLE PRECISION  NOT NULL default 0
             )
       ) by PASSTHRU ;

     disconnect from PASSTHRU ;
quit;


%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 50, &cMethodNm) ;
      %goto RETURNZERO;
%end;

/* Transfer new sas fact table into database area */

%sesFMS_uploadDataToSDM(&cSASFactsDS, &cSDMFactsDB_NEW);

%if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
%end ;

/*------------------------------------------*/
/* Decide storage type of two result tables */
/*------------------------------------------*/
%let stg_type_res=myisam;


%sesFMS_updateRemoveStatusCnt( &cStatusDS, &cSASFactsDS, REMOVE_CNT );


%if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
%end ;


/*  Join : extracting commonly found data idx                                                         */
/*  If nActionType =              4 : Join (etl_base x sas_base  U  etl_no_base x sas_no_base)        */
/*                                5 : Join (etl_no_base x sas_no_base)  : all splitted and joined     */
/*                   other than 4,5 : Join (etl_base x etl_base)        : Nothing splitted and joined */

/* Prepare conditional statement in where clause */

/* In case comparing all dimensions : no_split */
%do i=0 %to &num_dim_type-1;
    %if (&i = 0) %then %do;
        %let where_no_split=S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
    %end;
    %else %do;
        %let where_no_split=&where_no_split and S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
    %end;
%end;

/* In case comparing all except source and currency */
%let where_split=;
%do i=0 %to &num_dim_type-1;
    %if ("&where_split" = "") %then %do;
        %if (&i = &ord_currency or &i = &ord_source) %then %do;
      %let where_split=;
        %end;
        %else %do;
            %let where_split=S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
        %end;
    %end;
    %else %do;
        %if (&i ^= &ord_currency and &i ^= &ord_source) %then %do;
            %let where_split=&where_split and S.DIM_&i._MEM_ID = M.DIM_&i._MEM_ID;
        %end;
    %end;
%end;

%let _tmpindex = idx_dims_%sysfunc(kcompress(%sysfunc( datetime() ), .));             
proc sql noprint;
     connect to &sesetl_EngineType as PASSTHRU ( &sesetl_BestLogin &sesetl_ConnectionOptions );

     /* Create a composite index on sas etl fact table */
     execute(CREATE INDEX &_tmpindex on &cSDMFactsDB_NEW (&dim_col)
      ) by PASSTHRU ;

    /*-----------------------------------*/
    /* Prepare tables containing results */
    /*-----------------------------------*/

    /* Create result tables and temporary tables */
    execute (DROP TABLE IF EXISTS &cTmpROFactSDM_Nm
         ) by PASSTHRU;

    execute (CREATE TABLE &cTmpROFactSDM_Nm (
               FACT_IDX INTEGER NOT NULL,
               PRIMARY KEY (FACT_IDX)
             )
       ) by PASSTHRU;

    execute (CREATE RULE "ROFact_on_duplicate_ignore" AS ON INSERT TO &cTmpROFactSDM_Nm
               WHERE EXISTS (SELECT 1 FROM &cTmpROFactSDM_Nm 
                             WHERE (FACT_IDX)=(NEW.FACT_IDX))
               DO INSTEAD NOTHING
       ) by PASSTHRU; 

    execute (DROP TABLE IF EXISTS &cTmpRWFactSDM_Nm
       ) by PASSTHRU;

    execute (CREATE TABLE &cTmpRWFactSDM_Nm (
               FACT_IDX INTEGER NOT NULL,
               PRIMARY KEY (FACT_IDX)
             )
       ) by PASSTHRU;

    execute (CREATE RULE "RWFact_on_duplicate_ignore" AS ON INSERT TO &cTmpRWFactSDM_Nm
               WHERE EXISTS (SELECT 1 FROM &cTmpRWFactSDM_Nm 
                             WHERE (FACT_IDX)=(NEW.FACT_IDX))
               DO INSTEAD NOTHING
       ) by PASSTHRU; 

    /*---------------------------*/
    /* Join by concat_member_ids */
    /*---------------------------*/

    %if &nActionType = 4 %then %do;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where M.object_type_id=&cDLObjTypeId
                     and &where_no_split
        ) by PASSTHRU;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where M.object_type_id<>&cDLObjTypeId
                     and &where_split
        ) by PASSTHRU;

        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where M.object_type_id=&cDLObjTypeId
                     and &where_no_split
        ) by PASSTHRU;

        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where M.object_type_id<>&cDLObjTypeId
                     and &where_split
        ) by PASSTHRU;
    %end;
    %else %if &nActionType = 5 %then %do;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where &where_split
        ) by PASSTHRU;
        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where &where_split
        ) by PASSTHRU;
    %end;
    %else %do;
        execute ( insert into &cTmpROFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmROFactSetId. M
                   where &where_no_split
        ) by PASSTHRU;

        execute ( insert into &cTmpRWFactSDM_Nm
                  select distinct M.FACT_IDX
                    from &cSDMFactsDB_NEW S, SAS_FACT_&nSdmRWFactSetId. M
                   where &where_no_split
        ) by PASSTHRU;

    %end;

    execute (DROP RULE "ROFact_on_duplicate_ignore" ON &cTmpROFactSDM_Nm
    ) by PASSTHRU;

    execute (DROP RULE "RWFact_on_duplicate_ignore" ON &cTmpRWFactSDM_Nm
    ) by PASSTHRU;

    /* Rows of fact_value =0 in the etl_fact should be removed after Matching process */
    execute ( delete
                from &cSDMFactsDB_NEW
               where FACT_VALUE = 0;
    ) by PASSTHRU;

    disconnect from PASSTHRU ;

quit;


%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 50, &cMethodNm) ;
      %goto RETURNZERO;
%end;



/*  Dump the Time  */
%sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

%goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

%ERRORRETURN:
  %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

%RETURNZERO:
  %let _FMErrorCode=0;

%THEEND:

%mend sesFMS_replaceMatching;





/**********************************************************
 * sesFMS_sortDataSet.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_sortDataSet.sas                                                         */
/*  Purpose: Sort data set.                                                                  */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_sortDataSet(cDSName, sortKey);

  %let cMethodNm = sesFMS_sortDataSet;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cDSName)) %then %do;
    %let error_number=15;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;

  /*  Build the column sort by list  */
  filename _col temp;
  data _null_;
      file _col;
    set &sortKey end=eof;
    put '"' value + (-1) '"n ';
      if eof then put ';';
  run;

  /* Sort using cCols */
  proc sort in=&cDSName out=&cDSName ;
       by %inc _col ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=17;
    %let errObj=&cDSName;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_sortDataSet;


/**********************************************************
 * sesFMS_summarizeDataSet.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_summarizeDataSet.sas                                                    */
/*  Purpose: Summarize data set.                                                             */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_summarizeDataSet(cDSName, lTranCols, cAmtColNm, nIDTimePeriod);

  %let cMethodNm = sesFMS_summarizeDataSet;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cDSName))  %then %do;
    %let error_number=15;
    %let errObj=&cDSName;
    %goto ERRORRETURN;
  %end ;

  /*  Build the column summary by list  */
  filename _col temp;
  data _null_;
      file _col;
    set &lTranCols end=eof;
    put '"' value + (-1) '"n ';
      if eof then put ';';
  run;

  /* conditional code */
  %let subset_org_code=;
  %if &nIDTimePeriod %then %do;
    %if &cProduct eq BASE %then %do;
       %let subset_org_code= %str(id AFFECTED_TIME_PERIOD_ID;);
    %end;
    %else %do;
       %let subset_org_code= %str(id TIME_PERIOD_ID;);
    %end;
  %end ;

  /*  Run the summarization  */
  proc summary data=&cDSName nway ;
       by %inc _col ;
      &subset_org_code
          var &cAmtColNm ;
          output out=&cDSName (drop=_type_ _freq_) sum= ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=18;
    %let errObj=&cDSName;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_summarizeDataSet;

/**********************************************************
 * sesFMS_renameTransCols
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_renameTransCols                                                          */
/*  Purpose: Rename Transaction Table Columns                                                */
/*                                                                                           */
/*  History    13JAN2010 by Bill Jones                                                       */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_renameTransCols( cConvertedGLTransDS, lTranCols );

  proc sort in=&lTranCols out=work.tranCols;
    by value;
  run;

  %let _tmpDSN = &cStageLib..DIMENSION_TYPE;

  data work.dimTypeCols;
     set &_tmpDSN;
     value = base_fact_column_nm;
     tranColNm = strip(dimension_type_cd)||'_CD';
     keep value tranColNm;
  run;

  proc sort data = work.dimTypeCols nodupkeys;
     by value;
  run;

  filename _dropcol temp;
  data _null_;
    file _dropcol;
    set &lTranCols end=eof;
    if _N_ = 1 then
       put 'drop AFFECTED_TIME_PERIOD_ID ';
    if value ne 'CURRENCY_CD' then
       put '"' value + (-1) '"n ';
    if eof then
       put ';';
  run;

  filename _renmcol temp;
  data _null_;
    file _renmcol;
    merge work.tranCols(in=inA) work.dimTypeCols;
       by value;
    if _N_ = 1 then
         put 'rename AFFECTED_TIME_PERIOD_ID = TIME_CD;';
    if inA and value ne 'CURRENCY_CD' then
         put 'rename "' value +(-1) '"n = "' tranColNm +(-1) '"n ;';
  run;

  proc datasets library=WORK nolist;
     modify _CONVERTEDGLTRANS;
        %inc _renmcol;
  quit;

%mend sesFMS_renameTransCols;

/**********************************************************
 * sesFMS_updateInvalidStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateInvalidStatusCnt.sas                                               */
/*  Purpose: Update Invalid Status Count.                                                    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateInvalidStatusCnt(cStatusDS, cErrorDS, cErrCntColNm);

  %let cMethodNm = sesFMS_updateInvalidStatusCnt;
  %let cSummaryDS = work._tempFact1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cErrorDS
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %if &_nobs gt 0 %then %do;
    proc sql noprint;
      update &cStatusDS
      set &cErrCntColNm = &_nobs
        , status_id = 2
      ;
    quit;
  %end;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateInvalidStatusCnt;


/**********************************************************
 * sesFMS_updateRemoveStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateRemoveStatusCnt.sas                                                */
/*  Purpose: Update Valid Status Count.                                                      */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateRemoveStatusCnt(cStatusDS, cDS, cImpCntColNm);

  %let cMethodNm = sesFMS_updateRemoveStatusCnt;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %let _nobs = 0;
  data _null_;
    set &cDS nobs=_nobs;
    call symput('_nobs', _nobs);
    stop;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm = &cImpCntColNm +
       case when status_id ne 0
         then 0
         else &_nobs
       end
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateRemoveStatusCnt;


/**********************************************************
 * sesFMS_updateValidStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateValidStatusCnt.sas                                                */
/*  Purpose: Update Valid Status Count.                                                      */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateValidStatusCnt(cStatusDS, cValidDS, cImpCntColNm);

  %let cMethodNm = sesFMS_updateValidStatusCnt;
  %let cSummaryDS = work._tempFact1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cValidDS
    where FACT_VALUE ne 0
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm =
       case when status_id ne 0
         then 0
         else &_nobs
       end
     , status_id =
       case when status_id eq 0 and &_nobs eq 0
         then 3
         else status_id
       end
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateValidStatusCnt;


/**********************************************************
 * sesFMS_updateZeroStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateZeroStatusCnt.sas                                                           */
/*  Purpose: Update Zero Status Count.                                                       */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateZeroStatusCnt(cStatusDS, cValidDS, cImpCntColNm);

  %let cMethodNm = sesFMS_updateZeroStatusCnt;
  %let cSummaryDS = WORK._TEMPFACT1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cValidDS
    where FACT_VALUE eq 0
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm =
       case when status_id eq 2
         then 0
         else &_nobs
       end
     , status_id =
       case when status_id ne 2 and &_nobs gt 0 and IMPORTED_TRANS_CNT = 0
         then 0
         else status_id
       end
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateZeroStatusCnt;


/**********************************************************
 * sesFMS_uploadDataToSDM.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_uploadDataToSDM.sas                                                     */
/*  Purpose: Upload Table to SDM.                                                            */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_uploadDataToSDM(cSrcNm, cTrgNm);

  %let cMethodNm = sesFMS_uploadDataToSDM;
  %let nTrackTime = %qsysfunc(datetime());

  %let fmsetl_tempCheck1 = ;
  proc sql noprint ;
       select count(*)
         into :fmsetl_tempCheck1
         from &cSrcNm ;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %let nCnt = &fmsetl_tempCheck1;
  %if &nCnt eq . %then %let nCnt = 0 ;
  %if &nCnt EQ 0 %then %do;
      /*  No records found, skip the rest  */
    %goto THEEND;
  %end ;

  /*  Create output file name  */
  %let cExportCSV = %qsysfunc(strip(%qsysfunc(tranwrd(%qsysfunc(pathname(SASDATA)), \, /))))/upload%sysfunc(kcompress(%sysfunc( datetime() ), .)).txt;

  /* %let cSrcNm=%qsysfunc(&cSrcNm); */
  filename _outfile "&cExportCSV" encoding='utf-8';
  proc export data=&cSrcNm 
       outfile = _outfile
       dbms = DLM REPLACE;
       delimiter='~';
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
       connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );

       execute( copy &cTrgNm
                from %str(%')&cExportCSV.%str(%')
                with (format csv,
                      delimiter '~',
                      header,
                      NULL ' ')
              ) by &sesetl_EngineType ;

       disconnect from &sesetl_EngineType ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Delete text file  */
  data _null_;
    length fref $8;
    fref = _blank_;
    rc = filename(fref, "&cExportCSV" );

    if rc=0 and fexist(fref) then 
      rc=fdelete(fref) ;

    rc=filename(fref, '') ;
  run;  

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_uploadDataToSDM;


/**********************************************************
 * sesFMS_uploadStatusTableToSDM.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_uploadStatusTableToSDM.sas                                              */
/*  Putpose: Upload Status Table to SDM.                                                     */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_uploadStatusTableToSDM( cSrcTable, cTrgTable);

  %let cMethodNm = sesFMS_uploadStatusTable;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());


  %let tmpUploadNm = PUBLIC.%scan(&cTrgTable, 2);

  proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );

   execute(CREATE TABLE &tmpUploadNm (
             TIME_ID INTEGER,
             ANALYSIS_ID INTEGER,
             PERIOD_ID INTEGER,
             STATUS_ID INTEGER,
             IMPORTED_TRANS_CNT INTEGER,
             ZERO_TRANS_CNT INTEGER,
             INVALID_ERROR_CNT INTEGER,
             ORG_EQ_TRADER_CNT INTEGER,
             IC_ERROR_CNT INTEGER,
             NON_IC_ERROR_CNT INTEGER,
             REMOVE_CNT INTEGER
           )
     ) by &sesetl_EngineType ;

   disconnect from &sesetl_EngineType ;
  quit;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %sesFMS_uploadDataToSDM(&cSrcTable, &tmpUploadNm);
  %if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
  %end ;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_uploadStatusTableToSDM;

/**********************************************************
 * sesFMS_uploadTempFactToSDM.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_uploadTempFactToSDM.sas                                                 */
/*  Purpose: Upload Temporary Fact Table to SDM.                                             */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_uploadTempFactToSDM (lCycleDimTypes, cTmpFactSDM_Nm, cTmpFactWork_Nm, cStatusDS);

  %let cMethodNm = sesFMS_uploadTempFactToSDM;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Get the list of Time and Analysis Ids that have okay import status   */

  proc sql noprint;
    select count(*)
      into :num_data
      from &cStatusDS
      where STATUS_ID = 0;
  quit;

  %let num_data = %eval(&num_data);

  %if &num_data > 0 %then %do;
      /* Find the location of TIME, ANALYSIS and INTORG dimension types */
      %let ord_time=;
      %let ord_account=;
      %let ord_source=;
      %let ord_currency=;

      data _null_;
          set &lCycleDimTypes end=IsLast;
          if code='SOURCE' or code='source' then do;
              call symput('ord_source', _N_-1);
          end;
          if code='CURRENCY' or code = 'currency' then do;
              call symput('ord_currency', _N_-1);
          end;
          if IsLast = 1 then
              call symput('num_dim_type', _N_);
      run;


      %let dim_col_def=;  /* List of dimension type column definition in DDL */
      %let dim_col=;      /* List of dimension type column names             */

      %do i=0 %to &num_dim_type-1;
        %if &i = 0 %then %do;
            %let dim_col_def=DIM_&i._MEM_ID integer;
            %if &i ^= &ord_source & &i ^= &ord_currency %then %do;
                 %let dim_col=DIM_&i._MEM_ID;
            %end;
        %end;
        %else %do;
            %let dim_col_def=&dim_col_def,DIM_&i._MEM_ID integer;
             %if &i ^= &ord_source & &i ^= &ord_currency  %then %do;
                %if "&dim_col" = "" %then %do;
                    %let dim_col=DIM_&i._MEM_ID;
                %end;
                %else %do;
                    %let dim_col=&dim_col, DIM_&i._MEM_ID;
                %end;
             %end;
        %end;
      %end;

      data &cTmpFactWork_Nm (/*drop=TIME ANALYSIS INTORG IDXCNT*/);
          length OBJECT_TYPE_ID 8 ;
          length OBJECT_ID 8 ;
          set &cTmpFactWork_Nm(where=(FACT_VALUE ne 0));
          length FACT_VALUE 8 ;
      run ;
      %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
        %goto RETURNZERO;
      %end;

      /*------------------------------------------------------------------------*/
      /* Decide the storage type of a temp fact table to be created in SASSDM */
      /*------------------------------------------------------------------------*/
      %let stg_type_tmp=myisam;

      /*-----------------------------------------*/
      /* Move ETL base fact from SAS into SASSDM */
      /*-----------------------------------------*/
      %let tmpUploadNm = PUBLIC.%scan(&cTmpFactSDM_Nm, 2);
      proc sql noprint;
           connect to &sesetl_EngineType as PASSTHRU ( &sesetl_BestLogin &sesetl_ConnectionOptions );

           execute (DROP TABLE IF EXISTS &tmpUploadNm
           ) by PASSTHRU;

           execute(CREATE TABLE &tmpUploadNm (
                     OBJECT_TYPE_ID INTEGER NOT NULL default 0
                   , OBJECT_ID INTEGER NOT NULL default 0
                   , &dim_col_def
                   , FACT_VALUE DOUBLE PRECISION  NOT NULL default 0
                   )
             ) by PASSTHRU ;

           disconnect from PASSTHRU ;
      quit;

      %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
        %goto RETURNZERO;
      %end;

      %sesFMS_uploadDataToSDM(&cTmpFactWork_Nm, &tmpUploadNm);
      %if &_FMErrorCode = 0 %then %do;
        %goto RETURNZERO;
      %end ;

  %end;

  /*  Dump the Time  */

  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;
  %THEEND:

%mend sesFMS_uploadTempFactToSDM;

/**********************************************************
 * sesFMS_verifyStageTransModel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_verifyStageTransModel.sas                                                 */
/*  Purpose: Verify DDS trans. table contains standard model columns.                        */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_verifyStageTransModel(cStageLibRef, cStageTransTableNm, lCycleDimTypes, cStageModel);

  %let cMethodNm = sesFMS_verifyStageTransModel;
  %let cStageLibRef = %qsysfunc(strip(&cStageLibRef));
  %let cStageTransTableNm = %qsysfunc(strip(&cStageTransTableNm));
  %let cInputDS =&cStageLibRef.&cStageTransTableNm;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Build a list of columns from the physical Stage table  */
  %if (%qsysfunc(exist(&cStageLibRef..&cStageTransTableNm))) %then %do;
      proc contents data=&cStageLibRef..&cStageTransTableNm out=_lTranCols(keep=name rename=(name=code)) noprint;
      run;
      data _lTranCols;
         set _lTranCols;
         code = upcase(code);
      run;
  %end;
  %else %do;
      %put ERROR: Table(&cStageLibRef..&cStageTransTableNm) does not exist: from Macro(&cMethodNm)  ;
      %goto RETURNZERO;
  %end;

  /* Does the Stage gl_transaction_sum table contain the standard list of columns ? */
  /* Does the Exclude list of columns exist in the physical table  */
  /* lStageTranCols and lStageTranOPCols have been created by YTDConv */

  proc sort in=&cStageModel ;
    by code;
  run;

  proc sort in=_lTranCols ;
    by code;
  run;

  data _tmp_a_minus_b;
     merge  &cStageModel(in=a) _lTranCols(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(_tmp_a_minus_b))) %then %do;
    %let numRows=0;
    proc sql noprint;
      select count(*)
        into :numRows from _tmp_a_minus_b;
    quit;

    %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;

      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 13,  &cStageTransTableNm, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto RETURNZERO;
    %end;
  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto RETURNZERO;
  %end;

  /*  Does the Stage gl_transaction_sum table contain the dimension type columns the Cycle is expecting ?  */
  proc sql noprint;
    create table work._lcycle_lstage_dim as
    select  d.value as code
      from  lCycleDimTypes c, lStageDimTypes d
       where c.code=d.code;

  proc sort in=_lcycle_lstage_dim out=_lcycle_lstage_dim;
    by code;
  run;

  data _tmp_a_minus_b;
       merge  _lcycle_lstage_dim(in=a) _lTranCols(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(_tmp_a_minus_b))) %then %do;
    %let numRows=0;
    proc sql noprint;
      select count(*)
        into :numRows
          from _tmp_a_minus_b;
     quit;
      %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;

      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 14,  &cStageTransTableNm, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto  RETURNZERO;
    %end;
    /* clean the temp work table */
    /*
    proc datasets library=work nolist;
      delete _tmp_a_minus_b;
    quit;
    */

  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_verifyStageTransModel;


/**********************************************************
 * sesFMS_verifyDimensionTypes.sas
 **********************************************************/

/* ------------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.               */
/*  Name   : sesFMS_verifyDimensionTypes.sas                                                   */
/*  Purpose: Verify Marts Cycle dimension types with Stage Dimension types.                    */
/*           There will be a specific list of Dimension Types associated to the Mart's Cycle.  */
/*           We need to make sure that those dimension types also exist in the Stage Dimension */
/*           Type table.  Otherwise, we have no hope of matching Cycle dimension types to      */
/*           columns in the Stage gl_transaction_sum table.                                    */
/*           One exception:  The SOURCE Dimension type does not exist in the Stage, but will   */
/*                           in the mart.                                                      */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.            */
/* ------------------------------------------------------------------------------------------- */
%macro sesFMS_verifyDimensionTypes(lCycleDimTypes, lStageDimTypes, lCycleDimTypesCols);

  %let cMethodNm = sesFMS_verifyDimensionTypes;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Does the Exclude list of columns exist in the physical table  */
  data _tmp_a_minus_b;
       merge  &lCycleDimTypes(in=a where=(code ne 'SOURCE')) &lStageDimTypes(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(work._tmp_a_minus_b))) %then %do;
    proc sql noprint;
      select count(*)
        into :numRows
          from _tmp_a_minus_b;
     quit;
      %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;
      quit;
      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 12, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto  RETURNZERO;
    %end;
    /* clean the temp work table */
    proc datasets library=work nolist;
      delete _tmp_a_minus_b;
    quit;

  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto ERRORRETURN;
  %end;

  /*Get corresponding dimension type RK using lDDSDimTypes tables extracted from Stage */
  %if %sysfunc(exist(&lCycleDimTypesCols)) %then  %do;
    proc datasets library=work nolist;
      delete &lCycleDimTypesCols;
    quit;
  %end;

  proc sql noprint;
    create table &lCycleDimTypesCols as
    select b.code, b.value
        from &lCycleDimTypes a, &lStageDimTypes b
     where a.code=b.code;
  quit;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, 12, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend  sesFMS_verifyDimensionTypes;

%MEND sesfms;
